<?php
/**
	File name : cacheReader.php
	Author of this file : Oscar Fawer until 2019-08-15
	Contact : www.ataox.org 
	Web sites : ataox.org / ataox.com / ataox.info
	Copyrights open source GPL, GNU General Public License V3 : see ./aMainDr/code/LICENSE.txt


	The main cache reader system to speed uo the system, the algo is very simple :
		(1) need an automatic transformation from complex parametered url into matchimg file names.
		(2) if the file is there we use it as a cache if :
			- not logged as a web master ... or trying to log !!
			- not log as a forum user if the file is forum sensitive !!!
			- not defined as uncachable (bascket page of an ecommerce because it needs instant reactivity from am unknowed user)
			
	Remarks :
		(1) This is also the base for cache writing!
		(2) Deletion of the caches is a bit tricky because it concerns the current mode and the other mode 
		    (eg: deskTop / smartPhone) and it's not actually faisable to test all possibilities, therefore in 
		    some cases specific deletions may be useful + we may implement calendar base deletion.
*/
class cacheReader{
	

	function __construct(&$tagsFu){
		if ($tagsFu!==null){
			$this->dtb= &$tagsFu->dtb;
		}else{
			$this->dtb= null;
		}
		
		$this->lastCalculatedCcDfn= false;
	} // construct
	

	/**
		Return true or false if the page is cachable based only on the first characters! Other crditerion such as
		webmaster logged, forum logged etc are used in parallel.
	*/
	function cachableCateg(&$p){
		return $p!=='' && $p[0]!=='_';
	} // cachableCateg
	

	/**
	*/
	function getLastCalCcDfn(){
		return $this->lastCalculatedCcDfn;
	} // getLastCalCcDfn
	

	/**
		Determine if we need to look for a cache or not! Also  What the lambda user can use a cache without any rights is also what can be cached by anybody 
		when a cache file is not present. A specific function could have been named lambdaCachable().
	*/
	function lookForCache(){
		if ($this->getVal(_c_urlUniqEdtPfx)!==false){
			return false; // (main editor likely active .. if fake edition just computes more) unique editing prefix, standard === "uniqPfx"
		}		
		
		if ($this->getVal(_c_localPwBoxUrlVar)!==false){
			return false; // (a webmaster is trying to log) url var to ask a password (pwBoxRequest / e)
		}
		
		if ($this->getVal(_c_statsAvoidUVar)!==false){
			// Likely webmaster testing the pages .. do not want it in the stats, so can't use the cache 
			// because the cache hasn't memorized this parameter in included url
			return false;
		}
		
		$x= false;
		if (defined('_c_pwGuestLBoxR')){
			$x= $this->getVal(_c_pwGuestLBoxR);	// (a guest is trying to log)
		}
		if ($x!==false){
			return false;
		}
		
		$p= $this->getVal(_c_localUrlPageVar,_c_localPermanDefPage);	// (urlPageVar / p)
		if (!$this->cachableCateg($p)){
			return false;	// A system page that is not cachable.
		}

		$cryptHashPw= $this->ckVal('fumUsrRight');
		if ($cryptHashPw===false || $cryptHashPw===_c_ko){
			// print('Passed E'._c_);
			$forumCachable= true;	// No forum users logged --> uses the cache
		}else{
			$forumCachable= false;	// A forum user islogged --> can't use the cache
		}
		
		return  $forumCachable;
	} // lookForCache


	/**
		Computes the string path DFN for a possible cache candidate.
		
		Returns the full string path for a possible cache main candidate (.txt) or alternative 
		information file (.php). Note that the information file is necessary for forum pages because otherwise
		we can't find if it's a forum page in a reasonable time because forum pages can be accessed without any parameters.
		
		Input :
			$forcedPg : when we need to forcr a pager name.
			$fileSffxPack : the concerned FORUM file suffix pack directly given to access the correct file.
				
	*/
	function cacheCandidateDfn($forcedPg=false, $fileSffxPack=false, $imposeSmartPh=false){
		$b= $this->realCachePN($okOrigPg,$forcedPg);
		if ($fileSffxPack!==false){
			$b.= $this->divCacheFlSuffx($imposeSmartPh);
			$b.= $fileSffxPack;	// in this case NO url defined but forced parameter definition
		}else if ($fileSffxPack!==true){
			$xP= $this->getVal(_c_fxCodeIndx);		// xpetal simple post
			$xT= $this->getVal(_c_rfxPage);		// xpetal post threadid
			$postGrpId= $this->getVal(_c_rfTGroupIdx);		// id of the group of post threads
			$b.= $this->packFileSuffx($xP,$xT,$postGrpId,$imposeSmartPh);
		}else{
			print('$fileSffxPack===true');	// def program, should never happens !!
		}
		return $this->mainFlStr($b);
	} // cacheCandidateDfn
	

	/**
		Packs the file suffixes
		Inout :
			$xP : xpetal simple post
			$xT : xpetal post threadid (id of a thread : "a-b"  or id of the concerned thread pack "c")
			$postGrpId : id of the group of post threads
	*/
	function packFileSuffx($xP=false, $xT=false, $postGrpId=false, $imposeSmartPh=false){
		$b= $this->divCacheFlSuffx($imposeSmartPh);
		$b.= $this->fumCacheFlSuffx($xP,$xT,$postGrpId);
		return $b;
	} // packFileSuffx
	
	/**
		Returns diverse cache (zoom and smart phone part) file suffixes.
		Input :
			$imposeSmartPh : 
				true : impose it
				false : does not care
				jump : jump the smart phone block
	*/
	function divCacheFlSuffx($imposeSmartPh=false){
		$b= '';
		$zoomId= $this->getVal(_c_idTextsZm);
		if ($zoomId!==false){
			$b.= _c_zoomCachPfx.$this->retIfStr($zoomId);
		}
		
		if ($imposeSmartPh!=='jump'){
			$spMode= $this->getVal(_c_smartPhoneMode);
			if ($spMode===_c_ok || $imposeSmartPh===true){
				$b.= $this->buildSffx(_c_smartPhoneSffx);
				$uParag= $this->getVal(_c_smartPhUniqParag);	// Protected it with a dedined list of _c_smartPhUniqParag for each page!
				$b.= $this->retIfStr($uParag);
			}
		}
		return $b;
	} // divCacheFlSuffx
	
	
	/**
		Special function to give cache DfN for simple deletion
	*/
	function simpleCacheDfn($sPhoneVal=_c_ko){
		$b= $this->realCachePN($okOrigPg);
		
		if ($sPhoneVal===_c_ok){
			$b.= $this->buildSffx(_c_smartPhoneSffx);
		}
		
		return $b;
	} // simpleCacheDfn
	
	
	/**
		Simplified call for packFileSuffx() with a single multi-function parameter.
	*/
	function packFileSuff($xZ=false){
		return $this->packFileSuffx($xZ,false,false);
	} // packFileSuff
	
	
		
	
	/**
		To give the proper extension to the given file body name.
	*/
	function mainFlStr($bodyFN){
		return _c_curCachePath.$bodyFN.'.txt';		// the main cached information (large file)
	} // mainFlStr

	
	/**
		Uses the currently cached page if it's there and the condition are met! Then returns the full-computing
		condition!
	*/
	function tryCurrentCache(){
		if ( $this->lookForCache() ){
			$ccDfn= $this->cacheCandidateDfn();
			$this->lastCalculatedCcDfn= $ccDfn;
			if (file_exists($ccDfn)){	// Try it
				print( file_get_contents($ccDfn) );	// Uses it right here
				return false;	// Do not need a full computing
			}
		}
		return true;	// Need a full computing
	} // tryCurrentCache
	
	
	/**
		Returns the page name that needs to be cached with an existance verification, otherwise because it comes from the URL, 
		prankster could saturate the cache with boggus names! Used in cacheCandidateDfn, and also to detect and correct widgets,
		
		Input : $forcedPg
		Output : $okOrigPg
			
	*/
	function realCachePN(&$okOrigPg, $forcedPg=false){
		$dtb= &$this->dtb;
		
		if ($this->getVal(_c_idTextsZm)!==false){
			return '';
		}
		
		$okOrigPg= true;	// optimist
		if ($dtb===null){
			$p= $this->getVal(_c_localUrlPageVar);
			$defPg= _c_localPermanDefPage;
		}else{
			// Direct database connection used by the writing process that are better of with as fresh as possible data especialy if it's necessary to write the local constant file again.
			$p= $this->getVal( $dtb->getGenVar('urlPageVar') );
			$defPg= $dtb->getGenVar('permanentDefPage');
		}
		
		if ($forcedPg===false){
			if ($p!==false){
				$b= $p;
			}else{
				$b= $defPg;
			}
		}else{
			$b= $forcedPg;
		}
		
		if ( $this->cachableCateg($b) ){
			if ($dtb!==null){
				// can do the verification
				if ( !$dtb->elementExists(_c_pcId, $b, _c_PolyCellPg, $b) ){
					$b= _c_localPermanDefPage;
					$okOrigPg= false;
				}
			} // else :  can't do the verification so normally just using the cache as a reader and $b is considered as correct
		}else{
			// Wrong categ so set the default
			$b= _c_localPermanDefPage;
			$okOrigPg= false;
		}
		
		return $b;
	} // realCachePN

	
	/**
		Rem : would be good if this method was integrated inside divCacheFlSuffx(()
	*/
	function zmSmartPhoneCacheStr($zoomId, $spMode=_c_ko){
		$b= _c_zoomCachPfx.$this->retIfStr($zoomId);
		if ($spMode===_c_ok ){
			$b.= $this->buildSffx(_c_smartPhoneSffx);
		}
		
		return $b;
	} // zmSmartPhoneCacheStr
	

	/**
		Forum suffices short circuitable by not calkling this function
	*/
	function fumCacheFlSuffx($xP=false, $xT=false, $postGrpId=false){
		$b= $this->retIfStr($xP);			
		$b.= $this->retIfStr($xT);
		$b.= $this->retIfStr($postGrpId);
		return $b;
	} // fumCacheFlSuffx
	
	
	/**
		Return the input if it is a string,
	*/
	function retIfStr($s){
		if (is_string($s)){
			return $this->buildSffx($s);
		}
		return '';
	} // retIfStr

	function buildSffx($s){
		return '-'.$s;	// !! must be the same separator as for the X table Ids. Akgo fumShrdCdLgt: rfXCdPgNm() // forumAlgo::threadIdStr()
	} // buildSffx
		
	
	/**
		Idem tools::getVal() so we don't have to waste time with the whole tools.php class
	*/
	function getVal($key,$val=false){
		if ( array_key_exists($key,$_GET) ){	// or isset($_GET[$key])  fastest ??
			return $_GET[$key];
		}else{
			return $val;
		}
	} // getVal
	
	
	/**
	*/
	function ckVal($name,$val=false){
		$aSql= &$this->aSql;
		if ( isset($_COOKIE[$name]) ){
			return $_COOKIE[$name];
		}else{
			return $val;
		}
	} // ckVal
	

	/**
		Return true if we need to instanciate statsWriter.php!
		Note this function is localized inside cacheReader.php because is in fact the lighter
		toolbox when CMSing as light as possible, and it's accessible anywhere.
	*/
	function gotStatWriter(){
		$notEdt= $this->getVal(_c_urlUniqEdtPfx)===false;
		$isStatPg= $this->getVal(_c_localUrlPageVar)==='_stats';
		return _c_localStatsUseIt===_c_ok && ($notEdt || $isStatPg);
	} // gotStatWriter
	
} // cacheReader
?>