<?php
/**
	File name : centraCl.php
	Author of this file : Oscar Fawer until 2019-08-15
	Contact : www.ataox.org 
	Web sites : ataox.org / ataox.com / ataox.info
	Copyrights open source GPL, GNU General Public License V3 : see ./aMainDr/code/LICENSE.txt


	centraCl.php define the central class.mastering all other classes as far as algorithmis goes, 
	with the main method being mainExec(). More precisely, the real call order is :
		The main index (index.php) --> starter.php --> const.php --> centraCl.php (instanciated inside the starter)
*/
class centraCl extends oTop{

	function __construct(&$cacheR, &$statsWr){
		parent::__construct();
		$this->cacheR= &$cacheR;	// also can be NULL
		$this->statsWr= &$statsWr;
	} // construct

	function go(){
		$cacheR= &$this->cacheR;
		
		$dtbNectB= $this->mainExec($all);
		if ($dtbNectB){
			$tagsFu= &$this->tagsFu;
			$tagsFu->setCacheR($this->cacheR);
			
			$glob= &$this->glob;
			$fls= &$this->fls;
			$tools= &$this->tools;
			$aSql= &$this->aSql;
			$dtb= &$this->dtb;
			$rMg= &$this->rMg;	// right manager
			$rMg->checkTimeActions();
			$cacheWr= &$this->cacheWr;
			$statsWr= &$this->statsWr;
						
			/* ---------------------------------------------------------------------------------------------------------
				CACHE writing by any users! Allows progressive cache rewriting when previous caches (possibly many)
				were deleted because of large scale deprecation, such as menu deprecation!
				If no cache then cache the concerned page with a content fully calculated inside $all
			*/
			if ( !$rMg->quickWMRight() && $cacheR->lookForCache() ){
				if ( $cacheWr->cacheIsPossiblyWritable()){
					if ($cacheWr->allowToWrite() ){
						$ccDfn= $cacheWr->cacheWrCandidateDfn();						
						if ( $ccDfn!==false && !file_exists($ccDfn) ){
							$forumSignedIn= false;
							$clsForum= $tagsFu->expliClss('forum');	// get the forun class id it exists otherwise false
							if ($clsForum!==false){
								$clsShrdCd= &$clsForum->shrdCd;
								$forumSignedIn= $clsShrdCd->correctlySignedIn();
							}
							
							if (!$forumSignedIn){
								if ( $dtb->getGenVar('cacheDebugMark')===_c_ok ){
									$all= $cacheWr->debugMark($all);
								}
								
								$curPage= $tools->curPageName();
								$shortHandIdsStr= $dtb->getElem(_c_PolyCellPg, _c_pcId,$curPage,  _c_pcNecStr);
								$shortHandV= $tools->trimStrToV($shortHandIdsStr,';');
								
								$uParag= $tools->getVal(_c_smartPhUniqParag);
								
								if ($uParag===false || in_array($uParag,$shortHandV)){
									// Warning : for forums also see cacheWriter::delConcernedCache() used in forumShareCd
									$fls->strToFile($ccDfn,$all);	// WRITING THE CACHE (lambda user point of view)
								}
							}
						}
					}
				}
			}
			
			if (!$this->nakedCall()){
				$logedOrEdtFprm= $glob->wRights || $rMg->requestPwForm();			
				if ($logedOrEdtFprm){
					$tools->spcDeleteCook(_c_c3PwCook);
				}
			}
			
			if ( $statsWr!== null && !$statsWr->privateActivities() ){
				$ajaxSup= $statsWr->statsAjaxSup();
				$all= str_replace('<!-- statSupportToSubst -->',$ajaxSup,$all);
			}
			
			// print($glob->getLspTotStr().$all);	// KEEP the comment in case we need to use $glob->getLspTotStr()
			print($all);		// Main screen printing
			$aSql->close();
		}
	} // go
	
	
	/**
	*/
	function mainExec(&$all){
		$glob= new globVar();	
		$this->glob= &$glob;
		
		$tools= new tools($this);
		$glob->loadTools($tools);
		$this->tools= &$tools;
					
		$acc= new accents($tools);
		$this->acc= &$acc;
		$glob->loadAcc($acc);
		
		$tools->setAccAccess($acc);
				
		$caps= new capsHtml($this);
		$this->caps= &$caps;
		$tools->setCaps($caps);
		$glob->loadCaps($caps);
						
		$fls= new filesMg($this);
		$this->fls= &$fls;
		$caps->extraLoads($fls);
		$acc->addsFls($fls);
		$tools->addsFls($fls);
		
		
		/* ---------------------------------------------------------------------------------
			The place to recover from a lost database and other difficult cases
			Note : no relation with the undo/redo recovery.php class.
		*/
		if ( $tools->getVal(_c_recoveryActions)!==false ){
			// The place to recover from a lost database and other difficult cases. Xpetal & parsing not accessible from there
			include _c_LibPath . 'recoveryActions.php';
			$recvry= new recoveryActions($this);
			$dspl= $recvry->go();
			print($dspl);
			return false; // so do not go further !!
		}
		
		/**
			Instanciates aMySql.php or simulSql.php, 
		*/
		$className= _c_CurrentDtbAbstract;
		if ($className==='simulSql'){
			include_once _c_DtbLibPath.'fullCnd.php';
		}
		
		$dtbCmpl= null; // because the being built database is only defined inside frame<builder.
		$aSql= new $className($this,$dtbCmpl);
		
		$this->aSql= &$aSql;
		$tools->setAsqlAccess($aSql);
				
		$dtb= new localDB($this,$aSql);
		$this->dtb= &$dtb;
		
		$glob->loadDtb($dtb);
		$tools->setDtbAccess($dtb);
		$caps->setDtbAccess($dtb);
		$fls->setDtbAccess($dtb);
		
		$topH= new topHash($this);	
		$this->topH= &$topH;
		
		$pRnd= new pseudoRnd($this);
		$this->pRnd= &$pRnd;
		
		$mbCrypt= new mbCrypt($this);
		$this->mbCrypt= &$mbCrypt;
		$dtb->setMbCrypt($mbCrypt);
		$tools->setMbCrypt($mbCrypt);
		
		$xbCrp= new xbCrypt($this);
		$this->xbCrp= &$xbCrp;
		
		$hPwC= new hashPwCrypt($this);
		$this->hPwC= &$hPwC;
		$dtb->loadHPwC($hPwC);
		$tools->hPwC($hPwC);
		$fls->hPwC($hPwC);
		
		/**
			To cache variables from the genVar table. Normally always 
			initialised to true but could be usefull at false for debugging.
		*/
		define('_c_genVarCache', true);	
		
		$rMg= new rightsMg($this);
		$this->rMg= &$rMg;
		$tools->rMg($rMg);
		$caps->loadsRMg($rMg);
		$fls->loadsRMg($rMg);		
		$dtb->loadRights($rMg);
		$glob->loadRight($rMg);		
		
		$dtbNectB= $dtb->dtbServerConnection();		// return a dtb coNection Boolean
		if (!$dtbNectB){
			return false;	// no connection, abort !!
		}
		
		// The first possible use of $dtb for effective database calls in MySql !!!!!!		
		$rMg->setForceEditRights();
		$dtb->loadsDefSPhoneMode();
		
		$codedAS= $dtb->getGenVar('codedAmperSand');
		$wpCdAS= $dtb->getGenVar('wpCdAmperSand');
		
		
		// '_c_Amp' is the kind of ampersands used in urls
		if ($codedAS===_c_ok){
			define('_c_Amp',_c_realHtmlAmp);	// always '&amp;'
			define('_c_gotRealHtmlAmp',true);	// imposes a Real Html Ampersand for Urls
		}else{
			define('_c_Amp','&');
			define('_c_gotRealHtmlAmp',false);
		}
			
		
		define('_c_gotWpCdAmp', $wpCdAS===_c_ok);
		
		$trgBrdElim= $dtb->getGenVar('wpTrgBrdElim');
		define('_c_wpTrgBrdElim', $trgBrdElim===_c_ok);
		
		
		// Setting the use of Html coding : 2 memo cells for historical reasons
		define('_c_usesHtmlAmpCode', $dtb->getGenVar('usesHtmlAmpCode')===_c_ok); // the base constant
		$acc->setUsesHtmlCode(_c_usesHtmlAmpCode);
		
		define('_c_stdFoldersRights',octdec($dtb->getGenVar('foldersRights',0755)));	// standard folders rights (default is strict)
		define('_c_stdFilesRights',octdec($dtb->getGenVar('filesRights',0644)));	// standard files rights (default is strict)
		
		define('_c_acceptUtfPgN', $dtb->getGenVar('acceptUtfPgN')===_c_ok);
		define('_c_allMenusNoFollow', $dtb->getGenVar('allMenusNoFollow')===_c_ok);
		
		define( '_c_bookMrkIconPath', _c_UsrImagePath.$dtb->getGenVar('urlIcon') );
		
		
		define('_c_thumbsWidth',$dtb->getGenVar('genThumbsWidth','192'));	// Real Thumbs width
		define('_c_thumbsHeight',$dtb->getGenVar('genThumbsHeight','144'));	// Real Thumbs height
		
		define('_c_dsplThumbsW',$dtb->getGenVar('dsplThumbsW','96'));	// Displayed Thumbs width
		define('_c_dsplThumbsH',$dtb->getGenVar('dsplThumbsH','72'));	// Displayed Thumbs height

		$urlPageVar= $dtb->getGenVar('urlPageVar');
		// usually "p"	doublon _c_localUrlPageVar  (familly : _c_idTextsZm static inside the starter)
		define('_c_urlPageVar', $urlPageVar);	
		
		
		/* !!! Redondancy with cacheConst::_c_localPermanDefPage --> could solve it with a definition of 
		   _c_permanDefPage inside cacheConst.php and getting rid of _c_localPermanDefPage. */
		define('_c_permanDefPage', $dtb->getGenVar('permanentDefPage'));
		
		$curPg= $tools->curPageName();
		define('_c_replaceMostPgWithDef', $dtb->getGenVar('replaceMostPgWithDef')===_c_ok && $curPg[0]!=='_');
		$glob->loadsTranslatLg();
		
		/**
			can be chosen freely at installation, so makes sure it's different
		*/
		define('_c_pwBoxUrlVar', $dtb->getGenVar('pwBoxRequest'));		// early progr:'e' --> any entitle webmaster.
		if (_c_pwBoxUrlVar===$urlPageVar){
			$urlPageVar.= 'a';	// can be chosen freely at installation, so makes sure it's different from _c_pwBoxUrlVar
		}

		define( '_c_vaLoginRequest', $tools->getVal(_c_pwBoxUrlVar) );	// could use it a bit further !!!
		
						
		$pathCss= _c_mainCssDfn;
		
		define('_c_wToDtbVarN','wdgtToDtbVar');	// The Widget to Database variable Name
		define('_c_mainHomePage', $dtb->getGenVar('mainHomePage') );
		
		$defUId= $dtb->getGenVar('defaultUserId');
		define('_c_defRegUser',$defUId);	// default and FAKE registred user! Used to prompt for a password without giving away a user Id
		
		define('_c_mainUsrSecretDir', $dtb->getGenVar('mainUsrSecretDir') );
		define('_c_muSecretDPath', _c_zCms._c_mainUsrSecretDir.'/');
		
		$initMT= new initMultiTags();
		$this->initMT= &$initMT;
		
		
		$tagsFu= new tagsFu($this);
		$this->tagsFu= &$tagsFu;
		$dtb->loadTgFu($tagsFu);
		$aSql->setTagFu($tagsFu);
		$rMg->addTagsFu($tagsFu);
		$tools->setTagsFu($tagsFu);
		$caps->setTagFu($tagsFu);
		
		$tagsFu->setCacheR($this->cacheR);
		$tagsFu->setStatsWr($this->statsWr);

		
		$cacheWr= new cacheWriter($tagsFu);
		$this->cacheWr= &$cacheWr;
		$tagsFu->setCacheWr($cacheWr);
		$glob->anyCodeInHeader.= '<!-- statSupportToSubst -->'._c_R;
		
		$scanStr= new scanStr($tagsFu);	// Standard dynamic string for the current record		
		$this->scanStr= &$scanStr;
		
		define('_c_tgPolyglot','glot'); 
		$this->polyglot= $tagsFu->instExplitCentralCls('polyglot');
		$tagsFu->polyglot= &$this->polyglot;	// direct access
		$polyglot= &$this->polyglot;
		$glob->gMasterLanguage= $polyglot->lgVect[0];
		
		
		include _c_LibPath . 'imgProc.php';	
		$this->imgP= new imgProc($tagsFu);
		$tagsFu->setImgProc();
		
		$gDtb= null;
		$parser= new tagsParser($tagsFu,$gDtb);
		$this->parser= &$parser;
		$dtb->setParser($parser);
		$tagsFu->setParserAcces($parser);
		
		$imgTags= $tagsFu->instExplitCentralCls('imgTags'); 
		$tagsFu->setImgTags($imgTags);
		
	
		define('_c_allowNoPwMode', $dtb->getGenVar('allowNoPassWordMode'));
		define('_c_nPwModeUVar', $dtb->getGenVar('nPwModeUVar'));
		define('_c_nPwModeUVal', $dtb->getGenVar('nPwModeUVal'));
		define('_c_userIdNm', 'userId');		// user Id text box name and url variable name to request the loggin box !!				
		
		// Password manager constantsMainly for class rightsMg
		define('_c_forbidUserId', $dtb->getGenVar('forbiddenUserId'));
		
		
		
		$userId= $rMg->realTimeUserId();

		if (!defined ('_c_pwGuestLBoxR')){
			// transition relative to 'guestVarErsatzt' implementation ...
			define('_c_pwGuestLBoxR', $dtb->getGenVar('pwBoxGuestReq'));	// Guest loggin url variable name
		}
		
		define('_c_pwGuestBoxRVal', $dtb->getGenVar('pwReqGuestVal'));	// Guest loggin url value name
		

		define('_c_archiveDirNm', 'backups');	// standard archive name
		define('_c_stdAchiveDirPath', _c_ProjectsPath._c_archiveDirNm.'/');	// standard archive dir path, used systematically for downloads
		
		$sADN= $dtb->getGenVar('secretArchivDirN');
		define('_c_forbArchDirPath', _c_ProjectsPath.$sADN.'/');	// guest forbidden archive dir path, so secret standard webmaster archive path!
		define('_c_mainDumpDrPth',_c_forbArchDirPath);	// dirpath
		
		$guestLBR= $tools->gostVal( _c_pwGuestLBoxR );	// only valid at login box stage
		// also see rightsMg::$glob->cndtWrRghts
		if ($guestLBR!==false || $userId===_c_guestUserId){
			// guest only access !!	
			define('_c_archiveDirPath', _c_ProjectsPath._c_archiveDirNm.'/');
					
			define('_c_guestOnlyAccess', true);
			define('_c_pwLogBoxReqst',_c_pwGuestLBoxR);	// url variable to request a password box
			define('_c_pwLogReqstVal', _c_pwGuestBoxRVal );	// url value value for the request.
		}else{
			// classical access !
			$guestExists= $dtb->recordExists(_c_UsersTbl,$dtb->condId(_c_guestUserId));
			
			define('_c_archiveDirPath', _c_forbArchDirPath);		// Where archives are built by fully allowed user to avoid guest peering!
			
			define('_c_guestOnlyAccess', false);
			define('_c_pwLogBoxReqst', _c_pwBoxUrlVar);	// url variable to request a password box
			define('_c_pwLogReqstVal', $dtb->getGenVar('pwRequestVal'));	// url value value for the request.
		}
		
		
		$pwForm= '';
		$pwCheck= $rMg->genPwCheck($pwForm);
		$rMg->condUnlogClnUp();
		
		if ($glob->cndtWrRghts===_c_unspecified){
			// simple user, set it to true so hash captchas and forum staff can be written
			$glob->cndtWrRghts= true;
		}
		$glob->wRights= $rMg->pwCheck3();
		
		
		/**
			Header Xpetal additions. Came from the need to use the class "adverts.php" in a few sites.
		*/
		$xPetalDeclar= $dtb->getGenVar('addXPetalGenDecl');	// site specific xPetal declarations
		$parser->parseStdStr($xPetalDeclar);
		
		$xPetPrfx= $dtb->getGenVar('addXPetGnPfxBuff');
		$tagsFu->hdSupParsedXPP= $parser->parseStdStr($xPetPrfx);
		
		$imgTags->setAds($tagsFu->mainAdvertPlugIn);
		
		$this->simpleMenStr= $tagsFu->instExplitCentralCls('simpleMenStr');
		$tagsFu->simpleMenStr= &$this->simpleMenStr;	// direct access
		
		include _c_LibPath.'genAssoc.php';
		$this->assoc= new genAssoc($tagsFu);
		$assoc= &$this->assoc;
		$tagsFu->setGenAssoc($assoc);
		$tools->setGenAssoc($assoc);
		
		if ($this->nakedCall()){
			// Naked call : fast call without : special right/editor && page support
			$methLoc= $tools->getVal(_c_urlMethLoc);
			$v= $fls->trunkSplit($methLoc,'.');
			$clN= $v[1];
			$path= $v[0].$clN._c_slash;
			$fuN= $v[2];
			
			$idMeth= substr($clN.'.'.$fuN,0,_c_largestIdSz);
			if ($dtb->getGenVar($idMeth)===_c_ok){
				// the method is allowed (registred in the database)
				$cl= $tagsFu->instExplitCl($path,$clN,true);
				$all= $cl->$fuN();	// eventual parameters must be taken directly from the url.
			}else{
				$all= $glob->el('Error : the method ','kmgu').$clN.'::'.$fuN.$glob->el(' is not registred as : ','kmgv').$idMeth;
			}
			return true;
		} // $nakedCall
		
		define('_c_archiveDirPathNS', $tools->rmLstByte(_c_archiveDirPath));	// ... no slash
		define('_c_archiveDirPthPlus', _c_archiveDirPath._c_fullAspBkpPlus);

		/*
			Special instanciation was only in logged state but access is problematic without these
			assignations in the case of old site upgrade, so leave it accessible all the time.
		*/
		$tagsFu->setWmRights();			// installs cntrXRight
		$this->wmRights= $tagsFu->instExplitCentralCls('cntrXRight');	// and direct link
		$wmRights= &$this->wmRights;
		$fls->loadsWmRights($wmRights);
		$caps->setWmRight($tagsFu);		// wmRights
		$glob->loadTagsFu($tagsFu);		// + wmRights
		$fls->loadsTagsFu($tagsFu);		// + wmRights
		
		if ($pwCheck!==false && $glob->wRights){			
			$eecTblNm= $tools->getVal('extractElemCurTblNm');
			if ($eecTblNm!==false){
				$xElemDtb= $tagsFu->instExplitExternalCls('extractElemDtb');
				if ($xElemDtb->extract()){
					return;
				}
			}
			
			include _c_KernelPath . 'recovery.php';
			$rec= new recovery($this);
			$this->rec= &$rec;
			$dtb->loadRecov($rec);
			$tagsFu->setRecov($rec);			
					
			$editTools= new editTools($tagsFu,$rec);
			$this->editTools= &$editTools;
			$editTools->linkTagsFu($tagsFu);					// -- could have direct parameters
			$editTools->setParserAcces($parser);				// -- could have direct parameters

			$rec->loadsEdtTools($editTools);
			// Eventually does the recovery action
			
			$undoVal= $tools->getVal(_c_undoDtb);
			if ($undoVal!==false){
				if ($glob->cndtWrRghts){
					switch ($undoVal){
						case 'clr':
							$rec->resetRecovTbl();
						break;
						
						case 'undo':
							$rec->undo();
						break;
						
						case 'redo':
							$rec->redo();
						break;
						
						default:;
					} // switch
				}else{
					$glob->setGuestDisplay($undoVal);
				}
			}
			
		}
						
		$tblName= $tools->getVal(_c_tableUrlVar,_c_PolyCellPg);
		$editorPos= 'right'; // initialised just to keep the interpreper happy	
		
		if ($glob->wRights){
			include _c_KernelPath . 'tpEdParsB.php';
			$topEdtBar= new tpEdParsB($tools,$caps,$parser,$dtb,$rMg,$tagsFu);
			$this->topEdtBar= &$topEdtBar;			
						
			$tEd= new textEditor($this);
			$this->tEd= &$tEd;
			$editorPos= $tools->gostVal(_c_editorPos,'right');
			
			if ( $dtb->getGenVar('cleaningAfterInstall')===_c_ok && _c_IndexFN!=='installer.php' ){
				// does some post insallation cleaning (could be more compact and better situated but fine for now!)
				$dtb->setGenVar('fastMbStrToArray',_c_ok);
				$fls->secUnlink('./masterFlag.php');
				
				$dtb->setGenVar('cleaningAfterInstall',_c_ko);
			}
		}
		
		$checkForm= $tools->postVal('checkForm');
		$curPage= $tools->curPageName();
		$tagsFu->tagInitialisation();	// process the init-xpetal-tag file
		if ( $rMg->absoluteSurfMode() ){
			// no edition right
			$mainStr= $dtb->getMainStr($sz,$curPage);
			if ($mainStr===false){
				$mainStr= $dtb->getMainStr($sz,_c_mainHomePage);
			}
			$all= $this->buildMainBody($scanStr,$mainStr,$sz,$tblName,$curPage,$pathCss,$editorPos);
			return true;
		}
		
		if ($pwCheck==='fm'){
			// Display the password form
			$all= $caps->formatAll( $pwForm, $tagsFu->globHtmlMTitle, $tagsFu->globHtmlMDescr, $tagsFu->globHtmlMKw, 
					$tagsFu->globHtmlMLang, '7 days', 'all', 'index, follow', $pathCss);
		}else{
			// Display the site with or without editing facilities depending on logging state
			$pwCh3= $glob->wRights;
			if ($pwCh3){
				$curPage= $tools->curPageName(true);
				$mainStr= $tEd->analyseAndChgDtb($sz,$tblName,$curPage); // STANDARD PAGE CASE --------
			}else{
				// no edition right
				$mainStr= $dtb->getMainStr($sz,$curPage);
			}
			
			$all= $this->buildMainBody($scanStr,$mainStr,$sz,$tblName,$curPage,$pathCss,$editorPos);
			return true;
		}
		
		return true;	
	} // mainExec	


	/*
	*/
	function buildMainBody(&$scanStr,&$origMainStr,$sz,$tblName,$curPage,$pathCss,$editorPos){
		$glob= &$this->glob;
		$tools= &$this->tools;
		$dtb= &$this->dtb;
		$rMg= &$this->rMg;
		$caps= &$this->caps;
		$fls= &$this->fls;
		$parser= &$this->parser;
		$tagsFu= &$this->tagsFu;
		$tEd= &$this->tEd;
		$topEdtBar= &$this->topEdtBar;
		
		
		$mainStr= $origMainStr;
	
		/**
			PARSING the main string of _c_PolyCellPg
				... so it executes the content, and if that content holds dtb written instructions we
				don't want theses write actions to hold the "special display structures", so the 
				--> crypted && surfing test.
		*/
		$count= 0;
		$topEditBar= '';
		$editPanel= '';
		$supSrfPanel= '';
		
		$successWithNPg= $rMg->quickWMRight() && $tEd->successWithNewPgFrom();
		if ($successWithNPg!==false){
			// got an insertion from a mother page --> force the surf mode
			$surfEdt= _c_seSurfVal;
		}else{
			$surfEdt= $tools->gostVal(_c_surfEdt,_c_seSurfVal);
		}
		
		if ($rMg->quickWMRight()){
			if ($curPage[0]==='_'){
				$globSurfStl= $dtb->getGenVar('surfStylEdtWhite');	// get a white background here or no background depending on site, to avoid unreadable technical pages!!
			}else{
				$globSurfStl= $dtb->getGenVar('globSurfStyleEdt');
			}
		}else{
			if ($tools->isInSmartPM()){
				$globSurfStl= $dtb->getGenVar('globSurfStyleSPh');
			}else{
				$globSurfStl= $dtb->getGenVar('globSurfStyle');
			}
		}
		
		$this->forumBar= $tagsFu->instExplitCl('central/forumBar/','forumBar',true); // also call in tagsFu::tg_extplugin()
		if ($rMg->quickWMRight()){
			if ($surfEdt===_c_seEditVal){
				$surfPanel= '';
			}else{
				$surfPanel= $this->localParsePg($scanStr,$mainStr,$count);
				$curPage= $tools->curPageName();
				
				/**
					Saves the tg_mobidselect() selection attribute(s) for cache (special mobile shortening) validity check
					!! If SmartPh short hands are used inside zoomed images idText we'll have many more valid ids, so saving them will require to recover the ones already saved in _c_PolyCellPg!!!
				*/
				$shortHandIdsStr= $dtb->getElem(_c_PolyCellPg, _c_pcId,$curPage,  _c_pcNecStr);
				if ($shortHandIdsStr===false){
					$shortHandIdsStr= '';
				}else{
					$shortHandIdsStr= trim($shortHandIdsStr);
				}
				
				
				if ($shortHandIdsStr!==''){
					$shortHandV= $tools->trimStrToV($shortHandIdsStr,';');
					
					// Combine $shortHandVwith $glob->allowebSmartPhParagrIds keep only different values!
					$newIdsV= $tools->arrayUnion($shortHandV,$glob->allowebSmartPhParagrIds);
					$newSmartIdsStr= $tools->arrayToStr($newIdsV,'; ');
					
					/**
						Warning in simulSql and in some installations a visualization of the page
						in surf mode less than 2 or 3 seconds after editing it will prevent 
						the modifications to take place!! $dtb->setElem(..) saves indeed the whole
						record. Php7 early installation seem to have a few delay issues.
					*/
					$dtb->setElem(_c_PolyCellPg,  _c_pcNecStr,$newSmartIdsStr,  _c_pcId,$curPage);
				}
				
				$globSurfStl.= $glob->supStyleForMenus();
			}
			
			$forceToSurf= $successWithNPg!==false;
			if ($rMg->gotToDsplEditor($forceToSurf)){
				if ($surfPanel!==''){
					$supSrfPanel= $caps->sDiv($surfPanel,$globSurfStl); // editor included --> was !!! asked to simulate a 100% width.
				}
				$editPanel= $caps->div($tEd->displayWidgets($origMainStr,$sz,$tblName,$curPage),'editPanel');
			}else{
				$supSrfPanel= $caps->sDiv($surfPanel,$globSurfStl); // invisible or no editor surrounding --> allow a 100% on the page
			}
		}else{
			$surfPanel= $this->localParsePg($scanStr,$mainStr,$count);
			$globSurfStl.= $glob->supStyleForMenus();
			$supSrfPanel= $caps->sDiv($surfPanel,$globSurfStl);
		}
		
		$uMess= '';
		if ($glob->wRights){
			$uMess= $glob->returnUserMess()._c_BrFlux;
		}
		

		if ( $glob->wRights && $tools->gostVal(_c_hideTopEditBar,_c_ko)!==_c_ok ){
			$topEditBar= $topEdtBar->displayBar();
		}		
		
		if ($editPanel===''){
			// Got to set this one inside the CSS
			// + actually the other too because they need mutual width adaptation
			$advertPanel= '';
			$usePPAdvert= $dtb->getGenVar('usesPreProgrAdvert');
			if ($usePPAdvert!==_c_ko && !$rMg->quickWMRight() && $curPage[0]!=='_') {
				/* 
					&& !$rMg->quickWMRight()  ... don't want any advert for the webmaster, because too much distraction
					+ generally no right to click on them !!!
				*/
				$stl= $dtb->getGenVar('flexAdvertFrame');
				$advrtLC= $tagsFu->expand($dtb->getGenVar('advertContent'));
				$advertPanel= $caps->sDiv($advrtLC,$stl);
			}

			$mainContent= $topEditBar.$uMess.$supSrfPanel;
			if ($rMg->quickWMRight()){
				$stlAllIfNect= $dtb->getGenVar('nectMainPanelSurfStl');	// surround all if connected
				$mainContent= $caps->sDiv($mainContent, $stlAllIfNect);
			}
		}else{
			$stlAllIfNect= $dtb->getGenVar('nectMainPanelEdtStl');	// surround all if connected
			if ($editorPos==='right'){
				$dspl= $supSrfPanel.$editPanel;
			}else{
				$dspl= $editPanel.$supSrfPanel;
			}
			
			$mainContent= $caps->sDiv($topEditBar.$uMess.$dspl, $stlAllIfNect);
		}
		
		$allVsblLblClc= '';
		if ( $surfPanel==='' && $editPanel==='' && $rMg->quickWMRight() ){
			$urlPlus= $tools->addUrlVar(_c_surfEdt,_c_bthEditVal);
			// to make it all visible again!
			$allVsblLblClc= $tools->linkCurrentPage($glob->el('The whole display disappeared...','karm'),'sf',$urlPlus);
			$allVsblLblClc= _c_BrFlux._c_BrFlux._c_BrFlux.$caps->alignCenter($allVsblLblClc,'600','10','10',' border: 3px dashed red;');
		}		
				
		if ($tagsFu->cRobots===false){
			$cRobots= $dtb->getGenVar('defRobotContent');
		}else{
			$cRobots= $tagsFu->cRobots;
		}
				
		$fls->fileDumpStr();
		return $caps->formatAll($allVsblLblClc.$glob->getGuestDisplay().$mainContent, 
			$tagsFu->globHtmlMTitle, $tagsFu->globHtmlMDescr, $tagsFu->globHtmlMKw, $tagsFu->globHtmlMLang,
			'7 days', 'all', $cRobots, $pathCss);
	} // buildMainBody	
		
			
	/**
		Regroup the parsing functions locally as a way to include the global page code inside
		any external predetermined xpetal code.
		---> implement idCode::mainContentSupCode code along the modeling conditions!!
	*/
	function localParsePg(&$scanStr,&$mainStr,&$count){
		$tagsFu= &$this->tagsFu;
		$tools= &$this->tools;
		$dtb= &$this->dtb;
		$parser= &$this->parser;
		
		$cPgNm= $tools->curPageName();
		$gotCatWalk= $dtb->getGenVar('galCatWalk')===_c_ok;	// the cat walk is possible
		$strPgLst= $dtb->getElem(_c_idCodeTbl, _c_ictId,'mainContSupCdPgList', _c_ictCode);
				
		$parseCatWalk= false;
		if ($gotCatWalk){
			if ($tools->isInCatWalk()){
				if ($tools->inVString($strPgLst,$cPgNm)){
					/*
						Difference with a standard page : it parses an x-petal code (#1 & #2) especially designed to
						surround the parsing of the main page. The first need appeared in a fashion context, so the variables names.
						Possible evolution for different mainContentSup... parsing code for the same web site
						-------> could define a new table to link page-names with a specific general encapsuler to connect to!
						But this is not to be abused because it would be more prctical to have everythings held within the
						current page.
					*/
					$parseCatWalk= true;
					$code= $dtb->getElem(_c_idCodeTbl, _c_ictId,'mainContentSupHeadCode', _c_ictCode);	// #1
					$code.= $mainStr;
					$code.= $dtb->getElem(_c_idCodeTbl, _c_ictId,'mainContentSupTailCode', _c_ictCode);	// #2
					$scanStr->setScGenVar($tagsFu,$code,0,false,$cPgNm);
				}
			}
		}
		
		if(!$parseCatWalk){
			// The standard case just straight parsing
			$scanStr->setScGenVar($tagsFu,$mainStr,0,false,$cPgNm);
		}
		
		return $parser->parsePages($scanStr,$count);
	} // localParsePg
	

	/*
		Return true when we have a call to display something asap 
		wihout anything surrounding it (editor, top-page etc.)
	*/
	function nakedCall(){
		return $this->tools->getVal(_c_urlMethLoc)!==false;
	} // nakedCall
	
} // centraCl
?>