<?php
/**
	File name : editTools.php
	Author of this file : Oscar Fawer until 2019-08-15
	Contact : www.ataox.org 
	Web sites : ataox.org / ataox.com / ataox.info
	Copyrights open source GPL, GNU General Public License V3 : see ./aMainDr/code/LICENSE.txt


	A class to care about primary editing, mainly to manage the main Xm2 text areas.
*/
class editTools extends oTop{
	

	function __construct(&$tagsFu,&$rec){
		parent::__construct();
		$this->glob= &$tagsFu->glob;
		$this->acc= &$tagsFu->acc;
		$this->fls= &$tagsFu->fls;
		$this->tools= &$tagsFu->tools;
		$this->dtb= &$tagsFu->dtb;
		$this->polyglot= &$tagsFu->polyglot;
		$this->rec= &$rec;
	} // construct

	/**
	*/
	function linkTagsFu(&$tagsFu){
		$this->tagsFu= &$tagsFu;
	} // linkTagFu
	
	function setParserAcces(&$parser){
		$this->parser= &$parser;
	} // setParserAcces
	
		
	/**
		Build squared tag (the symetric of the JS edit-version), from the tag name, a normalised vectors and the body if given.
		
		Input :
			$tgName : the given tag name
			$nVect : normalised associative array
			$body : the body for bodied tags
		
	*/
	function buildSqrTag($tgName,$nVect,$body=false){
		// Build parameters list
		
		$pmStr=''; // parameters string to load
		if ($nVect!==false){
			reset($nVect);
			foreach ($nVect as $key=>$val){
				if ($val!==false){
					$pmStr.= _c_space.$key._c_equ._c_dblQ.$val._c_dblQ;
				}
			}
		}
		
		if ($body===false || $body===''){
			return _c_lBrk.$tgName.$pmStr._c_space._c_slash._c_rBrk;
		}else{
			return _c_lBrk.$tgName.$pmStr._c_rBrk. $body ._c_lBrk._c_slash.$tgName._c_rBrk;
		}
	} // buildSqrTag
	
	/**
		Idem buildSqrTag(), just add spaces arround consistent $body.
	*/	
	function spacedSqrTag($tgName,$nVect,$body=false){
		if ($body===false || $body===''){
			return $this->buildSqrTag($tgName, $nVect,$body);
		}else{
			return $this->buildSqrTag($tgName, $nVect, _c_space.$body._c_space);
		}
	} // spacedSqrTag
	
	/**
		Idem spacedSqrTag() without the spaces, so idem buildSqrTag()!! Just easier to call and remember.
	*/		
	function sqrTag($tgName,$nVect,$body=false){
		return $this->buildSqrTag($tgName, $nVect,$body);
	} // sqrTag
	
	
	/**
		Computes the new page name and the exact label in the case of the creation of a new page!
			---> newPageInMenu() is in full charge of the label convertion into a valif page name
			so this trivial method! Keep it as a method : (1) to avoid parameters confusion 
			with ->newPageInMenu($label,$label,...), in case we need a special checking for the simple calls.
		Input :
			$label : raw given label
			$addLgSffx : an eventual language suffix to add!
		Output :
			$newLabel : the exact new label
			RETURN the exact page name !!
	*/
	function finalNewPageStr(&$newLabel, $label, $addLgSffx=false, $dontCollide=true){
		$tagsFu= &$this->tagsFu;
		$glob= &$this->glob;
		$fls= &$this->fls;
		$dtb= &$this->dtb;

		if ($label===false || trim($label)===''){
			return false;	// Error inside finalNewPageStr() || blank user input
		}
		
		$tagsFu->curDiffPgName= false;
		$tagsFu->usesUtfUrl= false;
		
		$pageName= trim($tagsFu->expandX($label));	// Need a trim at this stage because of possible between tag spaces.
		$newLabel= trim($pageName); // Computes the real desired label by removing/executing the eventual tags (expandX) and triming!
		
		if ($tagsFu->curDiffPgName!==false){
			$pageName= $tagsFu->curDiffPgName;
		}
		
		$pageName= $fls->safeIdString($pageName);
		if ($dtb->getGenVar('builderSafeIdsLowerCases')===_c_ok){
			$pageName= strtolower($pageName);
		}
		
		if ($dontCollide){
			// Check if $pageName is already an index, if so adapt it!
			$i=0;
			while ( $dtb->elementExists(_c_pcId, $pageName, _c_PolyCellPg) ){
				if ($i===0){
					if ($addLgSffx!==false){
						$pageName= $pageName.'-'.$addLgSffx;
					}
					
					$i= 2;	// start numerotation at 2
				}else{
					if ($addLgSffx===false){
							$pageName= $pageName.'-'.$i;
						}else{
							$pageName= $pageName.'-'.$i.$addLgSffx;
						}
					$i++;
				}
			}
		}
		
		$pgNmSz= strlen($pageName);
		if ($pgNmSz+3>_c_largestIdSz){
			$glob->userError($pageName.$glob->el(' is above the maximum size of : ','kmci').(_c_largestIdSz-3).$glob->el('characters!','kmcj'));
			return false;
		}
		
		return $pageName;
	} // finalNewPageStr
		
	
	/**
		Integrate and create a full page given by its body content and menu label at the current menu position.
		Input:
			$pageName : The current page name to create ---> the name must have passed all tests for DTB compatibility 
			and Index uniqueness, OR be set at false (to avoid cumbersome external testing).
				
			$label : the label going to be displayed.
			$body : the body of the page
			$parentPgName : if false --> independant page
			$addLgSffx : the language used to suffix the page name and to create the polyglot table!
			$cellType : reconverted to language storage inside the _c_PolyCellPg table
		Output :
			- integrate the new page where and how it's specified (4 possibilities : dependant | $cellType)
			$pageName : created as a child of $parentPgName.
	*/
	function newPageInMenu($pageName,$label, $body, $parentPgName=false, $addLgSffx=false, $cellType=_c_pcTypAny){
		$tagsFu= &$this->tagsFu;
		$glob= &$this->glob;
		$tools= &$this->tools;
		$dtb= &$this->dtb;
		
		if ($pageName===false){
			return false;
		}
		
		/** ---------------------
			Creates the CURRENT MENU if not already created	
		*/	
		if (  !$dtb->recordExists(_c_PolyCellPgM, $dtb->condId($pageName) )  ){
			$dtb->createFullCell(_c_PolyCellPgM,$pageName,'','','');
		}
		
			
		// Adds the menu xml label.
		$curLbl= $dtb->getCllStr($pageName,_c_PolyCellPgM);	 // could just use $pageName ???
		if (!$tools->sIsInB('"'.$pageName.'"',$curLbl)){
			$nVect1['pg']= $pageName;
			$xmlLabel= $this->buildSqrTag(_c_tgMenBut,$nVect1, $label);
			$dtb->chgPolyCell(_c_PolyCellPgM,$pageName,$xmlLabel);
		}

			
		/** 
			Adds the menu up tag before whatever info is already in there
		*/
		$curNectStr= $dtb->getCllStr($pageName,_c_PolyCellPgM,_c_pcNecStr);
		$nVect2['ref']= $parentPgName;
		
		if (!$tools->sIsInB('"'.$parentPgName.'"',$curNectStr)){
			$curNectStr= $this->buildSqrTag(_c_tgMenUp,$nVect2)._c_R.$curNectStr;
			$dtb->chgPolyCell(_c_PolyCellPgM,$pageName,$curNectStr,_c_pcNecStr);
		}

		
		/** ---------------------
			Access/Create ABOVE/PARENT MENU
			Adds $pageName in only if not already done, so can run the algo many times without mess.
		*/
		$parentMenList= $dtb->getCellStr($sz,$parentPgName,_c_PolyCellPgM,_c_pcNecStr);			
		if (!$tools->sIsInB('"'.$pageName.'"',$parentMenList) && $parentPgName!==false){
			$nVect3['pg']= $pageName;
			$nVect3['down']= 'open';
			$parentMenList.= _c_R.$this->buildSqrTag(_c_tgMenTree,$nVect3);
			$dtb->chgPolyCell(_c_PolyCellPgM,$parentPgName,$parentMenList,_c_pcNecStr);	
		}

					
		/** ---------------------
			Create and adapt the page
		*/
		if (  !$dtb->recordExists(_c_PolyCellPg, $dtb->condId($pageName)) ){
			$dtb->createFullCell(_c_PolyCellPg,$pageName,'','',$cellType);
			
			if ($tagsFu->usesUtfUrl!==false && $tagsFu->curDiffPgName!==false){
				// For risky kamicase utf8 URL, or just longer than _c_largestIdSz
				$auV= array($pageName,$label);
				$dtb->newRecFromV(_c_ascUtfTN,$auV);
			}
			$tools->lastCreatedPgId= $pageName;	
			
			$content= '';
			$firstUpTag= $tools->postVal('firstUpTag');
			if ( $firstUpTag===false || $firstUpTag==='' ){
				$firstUpTag= 't0pAll'; // ##t0pAll to recover if the used model is biased
			}
			
			$nVect4['ref']= $firstUpTag;	// up value copied from the current page
			$content.= $this->buildSqrTag(_c_tgUp,$nVect4)._c_R;
			$content.= $this->spacedSqrTag(_c_tgTitle,false,$label)._c_R;
			$content.= $this->spacedSqrTag(_c_tgDescr,false,$label)._c_R;						
			$content.= $this->spacedSqrTag(_c_tgKw,false,' ')._c_R;
			
			$valTopic= $tools->postVal(_c_tgTopic);
			if ($valTopic!==false){
				$v['name']= $valTopic;
				$content.= $tagsFu->rebuildTagStr(_c_tgTopic,$v)._c_R;
			}
		
			if ($addLgSffx===false){
				$curLg= $tools->postVal(_c_tgLang,$glob->gMasterLanguage);
			}else{
				$curLg= $addLgSffx;
			}
			
			$content.= $this->sqrTag(_c_tgLang,false,$curLg)._c_R._c_R;
			$content.= $body;
			
			$pgFillerId= $tools->postVal('pgFillerCb');
			$content.= $glob->sl('#DefaultFiller',$pgFillerId);
			
			// correct a zealous process that has a bias for &quot; ... could lok at the source of the problem instead
			$content= str_replace('&quot;','"',$content); 
			$dtb->chgPolyCell(_c_PolyCellPg,$pageName, _c_stopSmplSrf.$content);
		}
		
		return true;
	} // newPageInMenu
	
	
	/**
		Multi language version. Add the same page with a language page name suffix to all the
		siblings of $parent.
		
		Input :
			$label : the given label :
				standard case --> one line
				expert --> multi-line labels one for each language.
			$body : what we fill the new page body with to make it understandable
			$parent : the parent page for menu insertion.
		Output :
			Building the pages menu structure
	*/
	function newNLgPgInMenu($label,$body,$parent,$cellType=_c_pcTypAny){
		$tagsFu= &$this->tagsFu;
		$glob= &$this->glob;
		$tools= &$this->tools;
		$dtb= &$this->dtb;
		$glot= &$this->polyglot;

		// $parent : the current language parent is passed from textEditor
		$lgSet= $dtb->languageFamVect($parent);
		$mixVect= $glot->polyglotizeGivenPg($parent,$lgSet);
		$lgSet= $mixVect;
		
		$label= trim($label);
		if ( $tools->sIsInB(_c_nL,$label) ){
			// MULTILINE WERE GIVEN, so EXPERT MODE with strict definition of each page name
			$expertMd= true;
			$lblVect= $tools->mbExplode($label,_c_nL);
			$canLblVect= $tools->trimFVNES($lblVect);	// canonic label vector
			
			$nbOfLbl= count($canLblVect);
			$nbLangMum= count($lgSet);	// number pf language of the parent
			if ($nbOfLbl!==$nbLangMum){
				// return an error message with specification of the two numbers!
				$glob->userError($glob->el('The number of specified menu is different ...','kmch').$nbOfLbl.'!='.$nbLangMum);
				return '';
			}
		}else{
			$expertMd= false;
		}
				
		if ($lgSet!==false){
			$i= 0;
			reset($lgSet);
			foreach ($lgSet as $lg=>$curParent){
				if ($expertMd){
					$curLabel= $canLblVect[$i];
					// n finalNewPageStr(
					$pageName= $this->finalNewPageStr($newLabel,$curLabel,false,false);
					print('$pageName='.$pageName._c_);
				}else{
					// non expert ----> a single line for the k-board inputed string ---> $label
					$pageName= $this->finalNewPageStr($newLabel,$label,$lg);
				}
				
				$resNPg= $this->newPageInMenu($pageName,$newLabel,  $body.'---'.$lg,  $curParent,$lg,$lg);
				if ($resNPg===false){
					return false;
				}
				
				$polyglotV[$lg]= $pageName;
				if ($glob->gMasterLanguage===$lg){
					$masterPg= $pageName;
				}
				
				$i++;
			} // foreach
			
			reset($polyglotV);
			foreach ($polyglotV as $lg=>$pageName){
				if ($glob->gMasterLanguage!==$lg){
					$dtb->createPolyDefCell($pageName,$masterPg);	// Save the current polyglot record
				}
			}
			
			$taggedStr= $tagsFu->rebuildTagStr(_c_tgPolyglot,$polyglotV);
			$dtb->createPolyDefCell($masterPg,$taggedStr);		// Save the master record
			
			$cLg= $dtb->currentLg();
			$pageName= $polyglotV[$cLg];				
		}else{
			// did not find any other language in the parent glot set, so just a normal creation.
			$pageName= $this->finalNewPageStr($newLabel,$label);
			$this->newPageInMenu($pageName,$newLabel,$body,$parent,false,$cellType);
		}
	} // newNLgPgInMenu

	
	/**
		Delete the full page without caring about other languages.
				Input :
					$page : page Id
				Output ; the deletion.
	*/
	function deletePage($pgId){
		$dtb= &$this->dtb;
	
		$dtb->delete(_c_PolyCellPg,$pgId);		// polyvalent cell page
		$dtb->delete(_c_PolyCellPgM,$pgId);		// polyvalent cell menu
	} // deletePage


	/**
		Delete the full page accross all languages possibilities : _c_PolyCellPg, _c_PolyCellPgM, _c_PolyglotTbl
		Deletion accross the ful spectrum of language is much easier that partial deletion and rebuilding a partial 
		language set not even sure to bused.
			Really more confortable to have an undo associated with tha function!
				Input :
					$page : page Id
				Output ; the deletion.
	*/
	function deletePolyglotPage($pgId){
		$dtb= &$this->dtb;
		
		if (!$dtb->elemExistSameCase(_c_pcId, $pgId)){
			return false;
		}
		
		$lgSet= $dtb->languageFamVect($pgId);		
		if ($lgSet!==false){
			// Full deletion of the page and the language relatives
			if (true){
				// Delete only the page matching the current language!
				$parentPage= $this->getParentPage($pgId);
				$this->deletePage($pgId);			// delete the specific language page main info & menus
				$dtb->delete(_c_PolyglotTbl,$pgId);	// delete the corresponding polyglot record.
				$this->deleteMenuCall($parentPage, $pgId);
			}else{
				/*
					WARNING this is a visible VAULT : Whole deletion of pages accross languages, this cannot apply
					on a left over page after deletion of the used part, because it would also delete the active page
					that has been replaced inisde its menu! A way arround it would be to erase completyl the matching 
					old menus record when renaming!
				*/
				reset($lgSet);
				foreach ($lgSet as $lg=>$pgId){
					if ($pgId!==false){
						$parentPage= $this->getParentPage($pgId);
						$this->deletePage($pgId);			// delete the specific language page main info & menus
						$dtb->delete(_c_PolyglotTbl,$pgId);	// delete the corresponding polyglot record.
						$this->deleteMenuCall($parentPage, $pgId);
					}
				}
			}
		}else{
			// no languages relatives found, so just delete the given page.
			$parentPage= $this->getParentPage($pgId);			
			$this->deletePage($pgId);
			$this->deleteMenuCall($parentPage, $pgId);
		}
	} // deletePolyglotPage
	
	
	/**
	*/
	function delSpecifiedRec($pgId,$tblName){
		$dtb= &$this->dtb;
		$dtb->delete($tblName,$pgId);
	} // delSpecifiedRec
	
	/**
	*/
	function createSpecifiedRec($pgId,$tblName){
		$acc= &$this->acc;
		$tools= &$this->tools;
		$dtb= &$this->dtb;
		
		$pgId= $acc->safeIdStr($pgId);
		if ($tblName===_c_PolyCellPg){
			$lgTyp= _c_pcTypAny;
		}else{
			$lgTyp= '';
		}

		$dtb->createFullCell($tblName,$pgId,'Created with the option : "Create Specified Combo Record"','',$lgTyp);
		$tools->lastCreatedPgId= $pgId;	
	} // createSpecifiedRec
	
	
	
	/**
		Delete the mtree line $pageId matching $oldId. The used algo is primitive :
			- 1st match
			- then scans symetrically until it finds 1st left _c_lBrk and right _c_rBrk to remove the string. Could
			  generate the string and remove it at once, but too many forms possible because of user modification.
				Rem : with a more sophisticated xml algorithm, it would extract a hierachical tree, then the concerned node would
				be easy to remove.
	*/
	function deleteMenuCall($pageId, $oldId){
		$dtb= &$this->dtb;
	
		$nectStr= $dtb->getCellStr($sz,$pageId,_c_PolyCellPgM,_c_pcNecStr);
		$pos= strpos($nectStr,$oldId);
		if ($pos!==false){
		
			// look for tag extreme left position
			$i= $pos;		
			$c1= $nectStr[$i];
			while($i>0 && $c1!==_c_lBrk){
				$i--;
				$c1= $nectStr[$i];
			}
		
			// look for tag extreme right position
			$j= $pos;		
			$c2= $nectStr[$j];
			while($j<$sz && $c2!==_c_rBrk){
				$j++;
				$c2= $nectStr[$j];
			}
			
			if ($c1===_c_lBrk && $c2===_c_rBrk){
				// got the tag extreme positions, so just got to take the 2 extremities and glue them
			
				if ($i>0){
					$i--; // shifts left one char from _c_lBrk
					$leftStr= substr($nectStr, 0,$i);
				}else{
					$leftStr= '';
				}
			
				if ($j>0){
					$j++; // shifts right one char from _c_lBrk
					$rightStr= substr($nectStr,$j);
				}else{
					$rightStr= '';
				}

				$dtb->chgPolyCell(_c_PolyCellPgM,$pageId,trim($leftStr)._c_R.trim($rightStr),_c_pcNecStr);
			}else{
				$this->oError('deleteMenuCall : left or right bracket missing');
				return false;
			}
		}else{
			// Does not need this error because deletion likely done else-where!
			// $this->oError('deleteMenuCall : Id '. $oldId .' not found in '.$nectStr);
			return false;
		}
	} // deleteMenuCall
	
	
	/**
		Adapts the menu button.
			Input ;
				$id : the id of the button to adapt.
	*/
	function adaptMenButton($oldId,$id){
		$dtb= &$this->dtb;
		$parser= &$this->parser;
	
		// Adapt the button definition
		$buttonStr= $dtb->getMainStr($szM,$id,_c_PolyCellPgM);
		$parser->parseStdStr($buttonStr, _c_PolyCellPgM._c_dbSep.$id);
		$buttonStr= str_replace(_c_rBrk.$oldId._c_lBrk, _c_rBrk.$id._c_lBrk, $buttonStr);
		$dtb->chgPolyCell(_c_PolyCellPgM,$id,$buttonStr);
	} // adaptMenButton
	
	
	/**
		Changes the old Id accross language sets.
			Input :
				$actId : actual Id.
				$newId : new Id.
	*/
	function chgPolyglotTbl($actId,$newId){
		$dtb= &$this->dtb;
		
		$lgSet= $dtb->languageFamVect($actId);	
		if ($lgSet!==false){
			reset($lgSet);
			foreach ($lgSet as $lg=>$curId){
				$pStr= $dtb->getPolyglotStr($curId);	// get the current polyglot string
				
				if ($pStr===$actId){
					// simple redirection case of the polyglot table
					$pStr= $newId;
				}else{
					$pStr= str_replace(_c_dblQ.$actId._c_dblQ, _c_dblQ.$newId._c_dblQ, $pStr);		// changes $pStr
				}
				
				$dtb->setPolyglotStr($curId,$pStr);
			}
			
			$dtb->chgCellIndex(_c_PolyglotTbl,$actId,$newId);	// change id access itself			
		}
	} // chgPolyglotTbl
	

	/**
		Give back the menu parent Id.
	*/
	function getParentPage($selectedRecId){
		$glob= &$this->glob;
		$dtb= &$this->dtb;
		$tagsFu= &$this->tagsFu;
		$parser= &$this->parser;
	
		$menuNectStr= $dtb->getCllStr($selectedRecId, _c_PolyCellPgM,_c_pcNecStr);
		$glob->justMenUpVal= true;		
		$parser->upMenRecName= false;
		$tagsFu->goingDownMenu= false;
		$parser->parseStdStr($menuNectStr, _c_PolyCellPgM._c_dbSep.$selectedRecId); // to load the parent $parser->upMenRecName.
		return $parser->upMenRecName;
	} // getParentPage
	
		
	/**
		Index change at the condition that the new index does not exists.
	*/
	function condIdChg($newIdName,$ectId, $tblName){
		$glob= &$this->glob;
		$dtb= &$this->dtb;
		
		$glob->errorOkToDisplay= false;
		$eesc= $dtb->elemExistSameCase(_c_pcId,$newIdName,$tblName);
		$glob->errorOkToDisplay= true;
	
		if ($eesc){
			$glob->userError($glob->el('The id : ','kmck').$newIdName.$glob->el(' exists already in ','kmcl').$tblName);
			return false;
		}else{
			$dtb->chgCellIndex($tblName,$ectId,$newIdName);
			return true;
		}
	} // condIdChg
	
		
	/**
		Changes the current page Id with adaptation of menus and language sets.
			$newIdName : new id name in a normalized-string.
			$ectId : selected Id to change.
			$hlRecov : to allow or block the high level recovery.
			$fastRecov : fast recovery or not. Practically always true, but just useful for testing!
			
		Note on general recovery processes : always high level recovery, so needs also high level reading.
	*/
	function changePageId($newName, $ectId, $hlRecov=true, $fastRecov=true){
		$dtb= &$this->dtb;
		$glob= &$this->glob;
		$acc= &$this->acc;
		$tagsFu= &$this->tagsFu;
		$rec= &$this->rec;
		
		
		$newIdName= $acc->safeIdStr($newName);
		if ($newIdName===''){
			return;
		}
		
		$parentId= $this->getParentPage($ectId);
		if ($fastRecov){
			$glob->stopAllRecov();	// stopAllRecov() ... shield associated with storesHlRecov() recovery works best+faster
			
			if ($hlRecov){
				// specialised fast (high level) recovery. Also see recovery::forChgRec()
				$rec->storesHlRecov('changePageId', $ectId,$newIdName);		// Direct call for high level undo/redo
			}
		}	
		
		$chgPg= $this->condIdChg($newIdName,$ectId,_c_PolyCellPg);		// changes page access Id
		if ($chgPg===false){
			if ($fastRecov) $glob->prevAllRecov();
			return;
		}
						
		$chgMen= $this->condIdChg($newIdName,$ectId,_c_PolyCellPgM);	// changes menu access Id
		if ($chgMen===false){
			if ($fastRecov) $glob->prevAllRecov();
			return;
		}
		
		if ($chgPg && $chgMen){
			$this->adaptMenButton($ectId,$newIdName);
						
			// Changes old Id inside the mother menu.
			$nectStr= $dtb->getCllStr($parentId,_c_PolyCellPgM,_c_pcNecStr);
			$nectStr= str_replace( _c_dblQ.$ectId._c_dblQ, _c_dblQ.$newIdName._c_dblQ, $nectStr);
			$dtb->chgPolyCell(_c_PolyCellPgM,$parentId,$nectStr,_c_pcNecStr);
			
			// Changes old Id of the main button
			$nectStr= $dtb->getCllStr($newIdName,_c_PolyCellPgM,_c_pcMainStr);
			$nectStr= str_replace( _c_dblQ.$ectId._c_dblQ, _c_dblQ.$newIdName._c_dblQ, $nectStr);
			$dtb->chgPolyCell(_c_PolyCellPgM,$newIdName,$nectStr,_c_pcMainStr);
			
			// Changes old Id in the redirection table
			if ($dtb->elementExists(_c_redirectPgId,$ectId,_c_pgRedirectTbl)){
				$dtb->setElem(_c_pgRedirectTbl, _c_redirectPgId,$newIdName, _c_redirectPgId,$ectId);
			}
			
			// Changes targets of the redirection table
			$v= $dtb->subTable(_c_pgRedirectTbl,_c_redirectPgTrgt,$ectId);
			if ($v!==false){
				reset($v);
				foreach ($v as $key=>$row){
					$id= $row[_c_redirectPgId];
					$dtb->setElem(_c_pgRedirectTbl, _c_redirectPgTrgt,$newIdName, _c_redirectPgId,$id);
				}
			}
					
			// Changes old Id accross language sets.		
			$this->chgPolyglotTbl($ectId,$newIdName);
			
			$glob->oldPgInChgPgId= $ectId;
			$glob->newPgInChgPgId= $newIdName;
			
			// Replacement tag having for purpose to : get the pg param from mTree and update the corresponding page.
			// Once the page tag-extracted by tg_mtreepg(), just does a string substitution with the "" 
			$tagsFu->tagSubstitOn('mtree','mtreepg');
			$tagsFu->includeRec($newIdName,_c_PolyCellPgM,_c_pcNecStr,false); // Executes the mtree(s) as mtreepg(s) 
			$tagsFu->tagSubstitOff('mtree');
		}		
		
		if ($fastRecov){		
			$glob->prevAllRecov();
		}
	} // changePageId	
	
	
} // editTools
?>
