/**
	File name : genAsa.js
	Author of this file : Oscar Fawer until 2019-08-15
	Contact : www.ataox.org 
	Web sites : ataox.org / ataox.com / ataox.info
	Copyrights open source GPL, GNU General Public License V3 : see ./aMainDr/code/LICENSE.txt


*/

/**
	File name and object name : genAsa (general javascript for this cms). Note kk.js is also general but intend to support
	a small number of basic methods with a tiny mnemonic access (kk), when genAsa holds mainly the specialised password
	methods.
								
	Generalities :
		localStorage  (handy)
		
		genAsa / xbCrypt
*/
function genAsa(){

	/**
		Public variables
	*/

	/**
		Methods interface.
	*/
	this.createsC2PwCook= createsC2PwCook;
	this.createsC3PwCook= createsC3PwCook;
	this.unlog= unlog;
	
	this.cryptPwBoxes= cryptPwBoxes;
	this.createCookie= createCookie;
	this.readCookie= readCookie;
	
	this.toggleNicEditor= toggleNicEditor;
	this.setUpNiceEdIfCreated= setUpNiceEdIfCreated;
	this.setUpNiceEdit= setUpNiceEdit;
	this.removeNicEditor= removeNicEditor;
	
	this.doCrypt= doCrypt;
	this.doUnCrypt= doUnCrypt;
	
	this.doMbCrypt= doMbCrypt;
	this.doMbUnCrpt= doMbUnCrpt;
	
	this.hexaCrpt= hexaCrpt;
	this.hexaUnCrpt= hexaUnCrpt;
	
	this.hexEncode= hexEncode;	
	this.hexDecode= hexDecode;
		
	this.debWriteLn= debWriteLn;
	this.resetWriteLn= resetWriteLn;
	this.debWrite= debWrite;
	this.setSortOfInteger= setSortOfInteger;
	
	this.changeEdtLanguage= changeEdtLanguage;	
	
	this.pwBoxCrypt= pwBoxCrypt;
	
	this.globUniqPrefix= '';	// see jsEncryption (could find another way) 
	
	

	this.niceEditWasCreated= false;		// Public global variables

	var niceEditObj= null;	// Public global variables


	/**
		Methods definitions
	*/	


	/* ---------------------------------------------------------------------------------------------------
		Got to do a multiplicative congruential crypt
			Input :
				str : string to crypt.
				crptK : crypt key.
	--------------------------------- */
	
		/* ---------------------------------------------------------------------------------------------------
			Multibytes or single bytes encryption depending on _c_multiByteCrypting (by defailt mutlti-byzes
			may revert to single bytes for speed !! )
		*/	
		function doCrypt(str,crptK){
			return xbCrypt.xbCrpt(str,crptK);
		} // doCrypt
		
		function doUnCrypt(str,crptK){
			return xbCrypt.xbUnCrpt(str,crptK);
		} // doUnCrypt
		
		
		/* ------------------------------------------------------------------------------------------------------
			Only Multibytes and only in Javascript so dont have the PHP-Javascript  8 / 16 conversion problem
			Actualy would be faster and acceptable to call xbCrypt directly
		*/	
		function doMbCrypt(str,crptK){
			return xbCrypt.mbCrpt(str,crptK);
		} // doMbCrypt
		
		function doMbUnCrpt(str,crptK){
			return xbCrypt.mbUnCrpt(str,crptK);
		} // doMbUnCrpt
		
		
		function hexaCrpt(str,crptK){
			return xbCrypt.hexaCrpt(str,crptK);
		} // hexaCrpt
		
		function hexaUnCrpt(str,crptK){
			return xbCrypt.hexaUnCrpt(str,crptK);
		} // hexaUnCrpt
		
	/*		
	--------------------------------- */
		
	/**
	*/
	function hexEncode(mbStr){
		return xbCrypt.hexEncode(mbStr);
	} // hexEncode
	
	/**
	*/
	function hexDecode(hxStr){
		return xbCrypt.hexDecode(hxStr);
	} // hexDecode


	
	/**
		Called for a general initialisation inside the php class capsHtml::formatAll()
	*/
	function setSortOfInteger(iSort){
		return xbCrypt.setSortOfInteger(iSort);
	}  // setSortOfInteger
	


	/**
		Debug by writing on the screen inside the 'JsDebug' HEAD defined <DIV> tag :
			look at capsHtml::formatAll()
	*/
	function debWriteLn(str){	
		var elem= document.getElementById('JsDebug');
		if (elem !== null){
			var strContent= elem.innerHTML;
			var suffx= '<br />'+'-----'+"\r\n";	
			elem.innerHTML= strContent + '<br />' + "\r\n" + str + suffx;
		}
	}	// debWriteLn


	/**
	*/
	function resetWriteLn(){
		var elem= document.getElementById('JsDebug');
		
		if (elem !== null){
			elem.innerHTML= '';
		}	
	}  // resetWriteLn


	/**
		Debug by writing on the screen inside the 'JsDebug' HEAD defined <DIV> tag
	*/
	function debWrite(str){	
		var elem= document.getElementById('JsDebug');
		if (elem !== null){
			elem.innerHTML= elem.innerHTML + str;
		}	
	}  // debWrite
	

	/**
		Creates the session crypting key At Login Time :
			c2PwCook= hashI*cpw === hashI*pw*pw;
		and crteate the 1st cookName3
			
		Input :
			pwBN : password box name!
			uniqPrfx : unique prefix.
			ppHash ; user primary primary hash to go from cpw to c2pw (storable safe).
			pHsh : user primary hash.
			cookName3 : the name for the computed cookie that is line safe.
		Output :
			localStorage.uniqPrfx+'sessionKey' : the session key
			the cookie cookName3 : the one use hash for login verification.
	*/
	function createsC2PwCook(pwBN,uniqPrfx,ppHash,pHsh,cookName3){		
		var ptUserId= document.getElementById('userId');
		
		if (ptUserId!==null){
			var wUserId= ptUserId.value;
		}else{
			var wUserId= window.winUserId;
		}
		
		var pw= document.getElementById(pwBN).value;
		if (wUserId==='guest'){
			createCookie(cookName3,pw,1);
			localStorage.guestPw= pw;
		}else{
			localStorage.ppHashVal= ppHash;
			debWriteLn('#ppHash='+ppHash);
			debWriteLn('#pHsh='+pHsh);

			debWriteLn('#pw='+pw);
			
			// Autocrypt of pw
			document.getElementById(pwBN).value= 'ko';
			
			var cpw= doCrypt(pw,pw);
			debWriteLn('#cpw='+cpw);
			
			//  Crypt the ppHash with cpw
			var cTwoPw= doCrypt(ppHash,cpw);	// Local C2 pw
					
			localStorage.setItem(uniqPrfx+'sessionKey',cTwoPw);
				
			debWriteLn('#cTwoPw='+cTwoPw);
						
			var c3PwCook= doCrypt(pHsh,cTwoPw);
			debWriteLn('###pHsh='+pHsh);
			debWriteLn('###cTwoPw='+cTwoPw);
			debWriteLn('###c3PwCook='+c3PwCook);
			
			createCookie(cookName3,c3PwCook,1);
		}
	}  // createsC2PwCook


	/**
		Manages the password encryption with hash to generate a network safe
		single use edition hash.
		
		Input :
			uniqPrfx : unique prefix.
			hash : user primary hash.
			cookName3 : the name for the computed cookie that is line safe.
		Output :
			the cookie cookName3 : the one use hash for login verification.		
	*/
	function createsC3PwCook(uniqPrfx,hash,cookName3){
		this.globUniqPrefix= uniqPrfx;	// to be used by jsEncryption for password key crypting.
		
		if (window.winUserId==='guest'){
			createCookie(cookName3,localStorage.guestPw,1);
		}else{
			var c2PwCook= localStorage.getItem(uniqPrfx+'sessionKey');
			localStorage.c2PwCook= c2PwCook;
			var c3PwCook= doCrypt(hash,c2PwCook); // crypts the current hash with c2PwCook
			createCookie(cookName3,c3PwCook,1);
		}
	}  // createsC3PwCook


	/**
		genAsa.unlog()
	*/
	function unlog(uniqPrfx,cookName3){
		createCookie(cookName3,'ko',-1);
		localStorage.setItem(uniqPrfx+'sessionKey',null);
		createCookie(uniqPrfx+'ckSessOrdId','ko',-1);
		createCookie(uniqPrfx+'ckTopHashId','ko',-1);
	} // unlog


	/**
		To change the password, need to be checked again and may be placed somewhere else
	
		Check if the given values are correct :
			new pw correct size
			old pw correct
			nPw1 === nPw2
		Then crypt nPw1 for password change and pw for 2nd comparison with stored original password.
			
			Input :
				old / new passwords text box ids
	*/
	function cryptPwBoxes(userId,uniqPrfx,cookName2,cookName3, pwId,nPwId1,nPwId2,minSz,maxSz){
		var primHashVal= localStorage.ppHashVal;
		var pw= document.getElementById(pwId).value;
		var pw1= document.getElementById(nPwId1).value;
		var pw2= document.getElementById(nPwId2).value;
		var szPw1= pw1.length;
		var szPw2= pw2.length;
		
		
		
		
		if (szPw1 < minSz || szPw1 > maxSz){
			alert('The first new password is not of the correct size: ' + minSz + ' < size < ' + maxSz);
			return false;
		} // else 1st given password got the correct size
		
		
		if (userId===localStorage.serverGuestName){
			document.getElementById('formPw').submit();
			return true;
		}
		
		
		var c2Pw= localStorage.getItem(uniqPrfx+'sessionKey');
		var szCPw= c2Pw.length;
				
		var pwCrpt= doCrypt(pw,pw);
		var hPwCrypt= doCrypt(primHashVal,pwCrpt);
		
		var szPwCrpt= hPwCrypt.length;
		
		
		if ( c2Pw.substring(0,szCPw-1) !== hPwCrypt.substring(0,szPwCrpt-1) ){		
			alert('The actual password is not correct!');
			return false;
		} // else old password is correct
		
		
		// To test if the 2 new pw are equal	
		if ( pw1.substring(0,szPw1-1) !== pw2.substring(0,szPw2-1) ){
			alert('The new passwords are not equal!');
			return false;
		}
		
		document.getElementById(nPwId2).value= '';	// supresses the going to be sent information

		var c3Pw= readCookie(cookName3);			
		var c4Pw= doCrypt(c3Pw,c2Pw);			// special crypting key used in that case
		
		var c3yptOldPw= doCrypt(pw,c4Pw);		// pw already taken!
		var c3yptNewPw= doCrypt(pw1,c4Pw);
			
		document.getElementById(pwId).value= c3yptOldPw;
		document.getElementById(nPwId1).value= c3yptNewPw;
		
		document.getElementById('formPw').submit();
	}  // cryptPwBoxes


	/**
		To create a specific cookie, remark : the browser does not concatenates values and date in the same string
		as the actual document.cookie= concatenation seems to indicate! --> hidden steps of separating strings for efficiency.
		Why did we used cookies at all ? For connected webmaster dynamic hash Id, the choice was url values or cookies! 
		Url values could become quite long and can't be crypted with https !
		
		Input :
			ckname : name of the cookie (string)
			value : value of the cookie (string)
			days : number of days the cookie is valid for (integer)
		Output : stores the cookie.
	*/
	function createCookie(ckname,value,days){
		var date = new Date();
		var present= date.getTime();	// [u sec]
			
		if (days>=1){
			date.setTime( present + days*24*60*60*1000 );
		}else{
			date.setTime( present - 3600000 );
		}
		
		var expires = '; expires='+date.toGMTString();
		document.cookie= ckname+"="+value+expires+"; path=/";
	};

	/**
		Read the specified cookie. Rem, code to display the actual cookie full string of values : writeLn('document.cookie='+document.cookie);
		Input :
			ckname : name of the cookie (string).
		Output :
			return the value of the cookie.
	*/
	function readCookie(ckName){
		var ca= document.cookie.split(';');
		for (var i=0; i<ca.length; i++){
			var cook= ca[i];	// current cookie str
			var leftS= kk.strLeftOfC(cook,'=');
			if (kk.trim(leftS)===ckName){
				return kk.strRightOfC(cook,'=');
			}
		}
		return '';
	}; // readCookie	


	/**
		Input :
			edtN : the name/id of the edit-box (arraybox)
			ckName : the name of the cookie to fix the choice of the toggle button, either with or without nicEdit
	*/
	function toggleNicEditor(edtN, ckName){
		if (niceEditObj===null) {
			setUpNiceEdit(edtN, ckName);
		}else{
			removeNicEditor( edtN, ckName,'ok');
		}
	} // toggleNicEditor

	
	/*
	*/
	function setUpNiceEdIfCreated(edtN, ckName){
		if (this.niceEditWasCreated){
			setUpNiceEdit(edtN, ckName);
		}
	}; // setUpNiceEdIfCreated
	
	
	/*
	*/
	function setUpNiceEdit(edtN, ckName){
		if (niceEditObj===null) {
			niceEditObj = new nicEditor({fullPanel : true, maxHeight : 430}).panelInstance(edtN);
			this.niceEditWasCreated= true; // was created at least ones !!
			createCookie(ckName,'on',1);
			var xPetalDiv= document.getElementById('polInserDiv');
			xPetalDiv.innerHTML= '';
			forumBar.emoSwitchOff();
		}
	}; // setUpNiceEdit
		
	/**
	*/
	function removeNicEditor(edtN, ckName, initLocStor){
		if (niceEditObj!==null){
			niceEditObj.removeInstance(edtN);
			niceEditObj= null;
			createCookie(ckName,'off',1);
			if (initLocStor==='ok'){
				kk.setInnerHtml(localStorage.fumStyleInsrtTags,'polInserDiv','Z');
			}
		}
	} // removeNicEditor
		
	
	/* -----------------------------------------------------------------------
		Methods definitions
	*/
	function changeEdtLanguage(selId,curUrl,edtLangUV){
		list= document.getElementById(selId);
		curLg= list.options[list.selectedIndex].value;
		
		location.href= curUrl + '&'+edtLangUV+'='+curLg;
	} // changeEdtLanguage
	

	/**
		Autocrypt a given text box
			Input:
				uniqPrfx : sesion key prefix
				tbxId : text box id.
		Application : brand new installation of a new site.
		
		php  _c_ckUniqPrfx  ===  uniqPrfx
	*/
	function pwBoxCrypt(uniqPrfx, tbxId){
		var cryptKey= localStorage.getItem(uniqPrfx+'sessionKey');
		var pw= document.getElementById(tbxId).value;
		var cpw= doCrypt(pw,cryptKey);
		document.getElementById(tbxId).value= cpw;		
	}  // pwBoxCrypt


	/**
	*/
	function writeLn(s){
		kk.writeLn(s);
	};

	/**
	*/
	function write(s){
		kk.write(s);
	};	
	

} // genAs