<?php
/**
	File name : recovery.php
	Author of this file : Oscar Fawer until 2019-08-15
	Contact : www.ataox.org 
	Web sites : ataox.org / ataox.com / ataox.info
	Copyrights open source GPL, GNU General Public License V3 : see ./aMainDr/code/LICENSE.txt


Recovery : two tables are necessary :
	
	recovData : stores the recovery information/main data with the main following fields :
		- asaStep is a unique id for the cms step that is made of the concatenation : 
				userId.asaAbsRecStep (cms absolute recovery step).
		- asaSubS : sub step to sort the current asaStep in the correct order. The sort is done
		on a php vector, eventually with re-interogation of the Dtb (not time critical + few user).
		
	
		
	recovCirc : circular buffer used to access storRecov (each users have their own space
		so their circular table do not interfer) :
		- id ; local id of the table.
		- asaStep : the id to look for in storRecov.
		
	Basic algos (origAction : original action) :
		- Normal Recovery Storage (NRS : prepares the undo) : 
			- if a record is changed --> stores the old record in the recovery tables.
			- if a record is created just stores 'origAction', valuesStr===''
			- if a record is deleted --> stores the old record in the recovery tables.
				! stores the old record seems valid for all actions if we consider that the old rec of new one is
				an empty string, but that won't be good enough to tell the difference betw. a new rec. creation
				and a modif. of an empty rec. so it's necessary to store the original action.
		- Undo : stores the old state inside the main database -->
			- do not apply normal NRS to these operations, but Inside-stack RS (IRS)
			- re-establish the original state invert the 'origAction' with the concerned data :
				- does an IRS (in most cases exchange the informations). If a record creation was done before, :
				  delete the recently created record and store it at the exact stack place the undo action happend.
			- ---> we can completly re-use NRS function at the condition of having the writing target as a parameter.
		
		- Redo :
			- just scan the stack the other direction, and again just apply IRS.
			
		- When normal edition (normal recovery) start again after a string of undo/redo undoing after last step
		  may not be intuitive! In fact : redoing information in front (and below) the new inserted data is lost!
		  --> So need to re-asset the stack limit at the head of the last undo/redo operation, otherwise previous
		  redo would become undo because passing from front to behind.
		- Future improvment, instead of ecrasing the discarder redo steps, could extract the steps and propose them
		  as steps self-service actions (easier just for copy paste actions).
		  
		  
	Two main categories of functions :
	
		1) Recovery storage : the function that extend the quantity of information ready to be recovered :
			forChgRec()
			forNewRec()
		  	forDelRec()
		  	
		2) Recovery use : function to instanciate a recovery.
			undo(true): an undo.
			redo() :	a redo..
			
	At some stage : could use it for incremenmtal backups, especially because a stream of change on
	the same element can be compressed just by taking the last one.
	
*/
class recovery extends oTop{
	

	// function __construct(&$glob,&$acc,&$dtb,&$rMg,&$fls,&$iBkp){
	function __construct(&$asa){
		parent::__construct();		
		$this->glob= &$asa->glob;
		$this->acc= &$asa->acc;
		$this->dtb= &$asa->dtb;
		
		$this->rMg= &$asa->rMg;
		$rMg= &$this->rMg;

		$this->fls= &$asa->fls;
		$tagsFu= &$asa->tagsFu;
		
		$this->wmR= &$tagsFu->wmRights;
		$wmR= &$this->wmR;
		
		/*
			Data table to access datas using the circular buffer.
			Note : 
				- this buffer is linear, but is explicitelly deleted when _c_RecCircTbl bites on its own active
				  fields.
				- don't need to worry about space between users, because linear or huge loop, could fix an autoamtical
				  reset at a few billions steps.
				- values may not be contiguous if muti-threaded between users.
		*/
		if ($wmR->wmUndoTwo()){
			define('_c_RecDataTbl','recovD2');	// the current user is forced on the 2nd recovery table (avoid possible interferences with the 1st set of recov tables)
		}else{
			define('_c_RecDataTbl','recovData');
		}		
		define('_c_recDatId',_c_genId);			// standard table id.
		define('_c_recDatUid','userId');						
		define('_c_recDatAStp','asaStep');		// one asaStep = one or many records saved with one user manip.
		define('_c_recDatSubS','asaSubS');		// to have the exact order
		define('_c_recDatAct','origAction');			
		define('_c_recDatTbl','tableName');				
		define('_c_recDatWhr','whereStr');
		define('_c_recDatVal','valuesStr');
			
		/*
			Special definition for high and low level remaming recovery.
			Note : the information normally held by the where and value string are concentrated
			with the pair OldName / NewName, so in this case the convention of the recovery table 
			changes as defined below.
		*/
		define('_c_recOldName','whereStr');	// Before action string
		define('_c_recNewName','valuesStr');	// After action string
			
			
		/*
			Storage in _c_GenVarTbl of the next free id of _c_RecDataTbl! Indeed a single free head value for all
			users because the recovery of the correct field is done with a search on a unique Id common to the 
			cms step. Heavy parallel ---> need to avoid parallel use.
			
			define('_c_recDataFreeId','recDataFreeId');
		*/
		
		define('_c_mxSzAStep',100);			// max size cms Step (just used below !!)
		define('_c_recCrcUsrSpan',50);		// user id span in _c_RecCircTbl	
				
		define('_c_recDataUserSpan', _c_recCrcUsrSpan*_c_mxSzAStep);	// Maximum span for a given user inside the large recvery table _c_RecDataTbl
		define('_c_rdFreeIdMax',999999999);
				
		/*
			Circular table for the circular buffer : one record === one asaStep.
		*/
		if ($wmR->wmUndoTwo()){
			define('_c_RecCircTbl','recovC2');
		}else{
			define('_c_RecCircTbl','recovCirc');
		}		
		define('_c_recCrcId','id');
		define('_c_recCrcUid','userId');			
		define('_c_recCrcAStp','asaStep');
		define('_c_recCrcStSz','aStepSz');	// sizes of the asaStep, so when going back can check that all the records are accessible
			
		
		$this->gCircFreeId= false;
		
		define('_c_asaSubStepIni',0);
		$this->asaSubStep= _c_asaSubStepIni;
	
		define('_c_undoOnStop','stop');
		
		$this->firstRecovCall= true;
		
		$this->realMin= false;
		$this->realMax= false;
	} // construct

	/**
	*/
	function loadsEdtTools(&$eTools){
		$this->eTools= &$eTools;
	} // loadsEdtTools
	

	/**
		Absolute minimum and maximum for the circular buffer.
	*/
	function minCircId($userId){
		$dtb= &$this->dtb;
		// got to return an int for coherent type/calculation
		return (int)$dtb->getElem(_c_UsersTbl,_c_userId,$userId,_c_usrStrtRecId);
	} // minCircId
				
	function maxCircId($userId){
		return $this->minCircId($userId) + _c_recCrcUsrSpan;
	} // maxCircId
	
	function minMaxCrcId($userId,&$min,&$max){
		$min= $this->minCircId($userId);
		$max= $this->maxCircId($userId);
	} // minMaxCrcId


	/**
		Real minimum and maximum, calculated as the limit of the may be uncomplete circular buffer, so the 
		limits of the linear "short" list of record.
		Warning : $this->realMin should work if called either before or after the cms circ modif.
	*/
	function realMinMaxCircId($userId,&$min,&$max){
		$glob= &$this->glob;
		$dtb= &$this->dtb;
		
		$cond= $dtb->condStr(_c_recCrcUid,$userId);		// To take the $userId part !! old bug correted
		$v= $dtb->vectIdNames(_c_RecCircTbl,$cond,_c_recCrcId); // Vector of ids !
			
			
		$idMin= 0;
		$idMax= sizeof($v)-1;
			
		$kExist= is_array($v) && array_key_exists($idMin,$v) && array_key_exists($idMax,$v);
			
		if ($v!==false && $kExist && sort($v)){
			$min= $v[$idMin];
			$max= $v[$idMax];
		}else{
			// not a single element (conventional assignation)
			$min= $this->freeRecovId($userId);
			$max= $min;
		}
		
		if ($max<$this->maxCircId($userId)){
			$max++; // so take account for the free head
		}
	} // realMinMaxCircId
	
	
	/**
		Returns the index of the following redo, to eventually do a redo if possible (not bumping in the free head limit)	
	*/
	function nextRedoPos($userId,$udPos){
		$this->realMinMaxCircId($userId,$min,$max);	// Relative min/max 
		if ($udPos<$max){
			$udPos++;
		}else{
			$udPos= $min;
		}
		return $udPos;
	} // nextRedoPos
		
	/**
	*/
	function okToRedo($userId,$pos,$freePos){	
		$fPos= $this->nextRedoPos($userId,$pos); // calculates the possible future position
		return $fPos!=$freePos;
	} // okToRedo

	/**
		Returns the index of the following undo, to eventually do an undo if possible (not bumping in the free head limit)
	*/
	function prevUndoPos($userId,$udPos){
		$this->realMinMaxCircId($userId,$min,$max);	// Relative min/max		
		if ($udPos>$min){
			$udPos--;
		}else{
			$udPos= $max;
		}
		return $udPos;
	} // prevUndoPos

	/**
	*/
	function okToUndo($userId,$pos,$freePos){
		$fPos= $this->prevUndoPos($userId,$pos); // calculates the possible future position
		return $fPos!=$freePos;		
	} // okToUndo

	/**
		Give the free cell head of the "small" circular recovery buffer.
	*/
	function freeRecovId($userId){
		$dtb= &$this->dtb;
		return $dtb->getElem(_c_UsersTbl,_c_userId,$userId,_c_usrFreeRecId);
	} // freeRecovId

	/**
		Return true if the given undo position exists in the recovery stack. Only useful
		to test down position, because right/head are checked through the free head / freeRecovId().
	*/
	function undoPosExists($udPos){
		$dtb= &$this->dtb;	
		return $dtb->elementExists(_c_recCrcId,$udPos,_c_RecCircTbl);
	} // undoPosExists

		
	/**
	*/
	function setUndoPos($userId,$udVal){
		$dtb= &$this->dtb;	
		$dtb->setElem(_c_UsersTbl,_c_usrUndoPos,$udVal, _c_userId,$userId);
	} // setUndoPos		
		
				
	/**
		Next pos for the whole loop independantly if the fields are set or not.
		There is no need for a symetrical prevCircPos() function because we are creating new
		recovery fields in only one direction and deletion is done in full sets.
	*/
	function nextCircPos($userId,$pos){
		$this->minMaxCrcId($userId,$min,$max);		
		if ($pos<$max){
			$pos++;
		}else{
			$pos= $min;
		}
		
		return $pos;
	} // nextCircPos
	
	
	/**
		Just got to take it, but limits are set at wrttting time, there : 
			$this->decrUndoPos() : does not ecrases anything and stop on the free head limit (okToUndo())
			$this->adaptFreeHead() : ecrases infos : brute force looping in the circular buffer
	*/
	function getUndoPos($userId){
		$dtb= &$this->dtb;	
		return $dtb->getElem(_c_UsersTbl,_c_userId,$userId,_c_usrUndoPos);
	} // getUndoPos	
	
	/*
		Get the previous undo position or return false if out of limit.
	*/
	function undoPosOrFalse($userId){
		$pos= $this->getUndoPos($userId);
		$freePos= $this->freeRecovId($userId);
		
		if ($this->okToUndo($userId,$pos,$freePos)){
			return $pos;
		}else{
			return false;
		}
	} // undoPosOrFalse	
	
		
			
	/**
		More complicated than getUndoPos(). Because the redo is calculated, not just taken, so the limit
		must be taken into account, but for the getUndoPos(), limits were taken into account when setting it.
	*/	
	function getRedoPos($userId){				
		$udPos= $this->getUndoPos($userId);
		$freePos= $this->freeRecovId($userId);
				
		/*
			Redo --> in the record list, moves toward the right, so must not bump into $freePos when it's
			on the right, being already on the other side imply that it looped back to the beginning, so still
			possible to move "right". The strict limit is obtained by knowing that undo pos and free pos must not overlap.
		*/
		if ($this->okToRedo($userId,$udPos,$freePos)){
			return $this->nextRedoPos($userId,$udPos); // the right-next undo pos matches the redo
		}else{
			return false;	// no redoing possibilities ahead
		}
	} // getRedoPos
		
	/**
		Decreases the undo position if possible (a real undo).
	*/
	function decrUndoPos($userId){
		$udPos= $this->getUndoPos($userId);
		$freePos= $this->freeRecovId($userId);
		
		if ($this->okToUndo($userId,$udPos,$freePos)){
			$pos= $this->prevUndoPos($userId,$udPos);
			$this->setUndoPos($userId,$pos);
		}else{
			// nothing
		}
	} // decrUndoPos
	
		
	/**
		Increases the undo position, this matches doing a REDO.
	*/
	function incrUndoPos($userId){
		$pos= $this->getRedoPos($userId);
		
		if ($pos!==false){
			$this->setUndoPos($userId,$pos);
		}
	} // incrUndoPos
				
	
	/**
		Returns the recovery limits or false if they are not accessible.
		Input :
			$undo : sort of undo. True--> undo; False--> redo
						
		Call : $this->getRecovPos($userId,$undo)
	*/
	function getRecovPos($userId,$undo){
		if ($undo){
			$undoPos= $this->undoPosOrFalse($userId);
		}else{
			$undoPos= $this->getRedoPos($userId);
		}
		
		if (!$this->undoPosExists($undoPos)){
			$undoPos= false;
		}
		return $undoPos;
	} // getRecovPos
		
		
		
	/**
		$asaStep= $this->calcAsastep($userId,$freeId)
	*/
	function calcAsastep($userId,$id){
		return $userId.'_'.$id;
	} // calcAsastep
	

	/**
		Recover Circular Free Id ++ : loads (mainly) the free user-relative Id ( $this->gCircFreeId ) for the circular table and increment it.
		Remark : initialisation for the construction of the recovery table.
		Input : 
			_c_usrFreeRecId & _c_usrStrtRecId form the user table to use the circular table.
		Output :
			_c_usrFreeRecId & _c_recCrcStSz
			$this->gCircFreeId
			$this->gCircStepSz
			
		Need to be called within a pair stopAllRecov() / prevAllRecov()		!!
	*/
	function recCircFreeId(){
		$dtb= &$this->dtb;
		$rMg= &$this->rMg;
		
		$userId= $rMg->realTimeUserId();		
		$this->minMaxCrcId($userId,$startId,$maxId);		
						
		// Got to save or update the new circular record.
		if ($this->gCircFreeId===false){
			// 1st recovery operation, so read the freeId value and update the user table
		
			// Get and save the free head and undo position of the circular recovery table inside the user table.
			$freeId= $this->freeRecovId($userId);
			
			if ($freeId<$maxId){
				$dtb->setElem(_c_UsersTbl, _c_usrFreeRecId,$freeId+1, _c_userId,$userId);
				$dtb->setElem(_c_UsersTbl, _c_usrUndoPos,$freeId, _c_userId,$userId);
			}else{
				$dtb->setElem(_c_UsersTbl, _c_usrFreeRecId,$startId, _c_userId,$userId);
				$dtb->setElem(_c_UsersTbl, _c_usrUndoPos,$maxId, _c_userId,$userId);
			}
					
			$asaStep= $this->calcAsastep($userId,$freeId);
		
			// creates a new circular record.
			$valV= array($freeId,$userId,$asaStep,1);
			
			if ($dtb->elementExists(_c_recCrcId,$freeId,_c_RecCircTbl)){
				$dtb->deleteTableRecord(_c_RecCircTbl,_c_recCrcId,$freeId);		// so can write it just after.
			}
			
			$dtb->newRecFromV(_c_RecCircTbl,$valV);		
			$this->gCircFreeId= $freeId;
			$this->gCircStepSz= 1;
		}else{
			// 2nd etc recovery operation of the same cms command
			$asaStep= $userId.'_'.$this->gCircFreeId;
			
			// update _c_recCrcStSz
			$this->gCircStepSz++;
			$dtb->setElem(_c_RecCircTbl,_c_recCrcStSz,$this->gCircStepSz, _c_recCrcId,$this->gCircFreeId);
		}
		$this->asaStepId= $asaStep;
	} // recCircFreeId
	
	
	/**
		Returns the current recDataFreeId and increment it + loads the cms sub step $asaSubS.
		Remark : initialisation for the construction of the recovery table.
			Input :
				$this->asaSubStep
			Output :
				$asaSubS
				$freeId : returns it as the absolute free id.
				
		Need to be called within a pair stopAllRecov() / prevAllRecov()		!!
		
		Got to use :
			getFreeIndx() & incrFreeIndx()
	*/
	function recDataFreeId(&$asaSubS){
		$dtb= &$this->dtb;
		$rMg= &$this->rMg;
		
		$asaSubS= $this->asaSubStep;
		$this->asaSubStep++;
	
		$userId= $rMg->realTimeUserId();
		$curUsrCond= $dtb->condStr(_c_userId,$userId);
		
		$smallestId= (int)$dtb->selectElem(_c_UsersTbl, $curUsrCond, _c_recDtSmlstId);
		$freeId= $dtb->selectElem(_c_UsersTbl, $curUsrCond, _c_recDataFreeId);
		
		if ($freeId!==false && (int)$freeId < $smallestId + _c_recDataUserSpan){
			$nextId= $freeId+1;
			$dtb->setElem(_c_UsersTbl,  _c_recDataFreeId,$nextId,  $curUsrCond);	
			$retFreeId= (int)$freeId;
		}else{
			// 1st one or new initialisation of the current head
			$dtb->setElem(_c_UsersTbl, _c_recDataFreeId,$smallestId, $curUsrCond);
			$retFreeId= $smallestId;
		}
		
		return $retFreeId;
	} // recDataFreeId
	
	
	/**
	*/
	function generalInit(&$asaSubS){
		$glob= &$this->glob;

		$glob->stopAllRecov();
		$this->recCircFreeId();		
		$freeId= $this->recDataFreeId($asaSubS);	// new _c_recDatId + loads $asaSubS
		$glob->prevAllRecov();
		return $freeId;
	} // generalInit
	
	
	
	/* -----------------------------------------------------------
		Recovery external calls
	*/
	
		
	/**
		Return true (once) if at first recall call
	*/
	function asastepFrstRecovCall(){
		if ($this->firstRecovCall){
			$this->firstRecovCall= false;
			return true;
		}else{
			return false;
		}
	} // asastepFrstRecovCall
		
	
	/**
		Adapt the free head to the current undo position.
	*/
	function adaptFreeHead($userId){
		$glob= &$this->glob;
		$dtb= &$this->dtb;
		
		$glob->stopAllRecov();
		$undoPos= $this->getUndoPos($userId);
		$freeHead= $this->freeRecovId($userId);
		
		// Erases whatever is between $undoPos -- > -- and ----$freeHead
		$pos= $this->nextCircPos($userId,$undoPos);
		if ($pos==$freeHead){	// only 2 == ... not the same type! Indem below !=
			$glob->prevAllRecov();
			return;	 // standard case after stack updates.
		}
		
		
		$i=0;
		// To delete the steps in front of the new modification
		while ($pos!=$freeHead && $i<=_c_recCrcUsrSpan){
			$asaStep= $dtb->getElem(_c_RecCircTbl, _c_recCrcId,$pos, _c_recCrcAStp);
			
			$dtb->deleteAll(_c_RecDataTbl, _c_recDatAStp,$asaStep);		// Delete all $asaStep of _c_RecDataTbl
			$dtb->deleteTableRecord(_c_RecCircTbl,_c_recCrcId,$pos);	// Delete the concerned _c_recCrcId (front part)
			
			$pos= $this->nextCircPos($userId,$pos);
			$i++;
		}
		
		
		if ($i>_c_recCrcUsrSpan){
			$this->topErr('recovery::adaptFreeHead');
		}
		
		// Reset the head
		$dtb->setElem(_c_UsersTbl, _c_usrFreeRecId,$undoPos+1, _c_userId,$userId);
		$glob->prevAllRecov();
	} // adaptFreeHead	
	
	
	/**
		Stores a high level record. This is hight (naturally compressed level), so it short circuit a pure sql recovery system!
		So using it imply that partial recovery is done using this CMS itself ---> would be good for that
		to be able to install a core version database-independant so we can bootstrap easily. For purists, it
		should be possible to short circuit this type of recovery to allow pure Sql.
	*/
	function storesHlRecov($origAction, $oldStr,$newStr){
		$set= $oldStr.'='.$newStr;
		$this->forChgRec(_c_ko, _c_ko, $origAction, $set);
	} // storesHlRecov		
		
	/**
		Loads the recovery table following an action. For the oposite operation, need to extract the record 
		of $tableName at the $whereStr position.
			Input :
				$tableName : name of the table holding the record to save..
				$whereStr : access to the concerned record of $tableName,	
				$origAction : origin kind of database action
				$nStrInf : new storable info, to use for the stream back-up (exact inverse of the undo table)	
	*/
	function forChgRec($tableName,$whereStr, $origAction,&$nStrInf){
		$glob= &$this->glob;
		$acc= &$this->acc;
		$fls= &$this->fls;
		$dtb= &$this->dtb;
		$rMg= &$this->rMg;
		
				
		$glob->stopAllRecov();
		$userId= $rMg->realTimeUserId();
		$this->adaptFreeHead($userId); 	// For in-between updates : make sure nothing is ahead of the undo pointor if we are somewhere in the middle of the recovery info
		
		// Conditional deletion of eventual old cms steps when looping (check that on 1st asastep recov call)
		if ($this->asastepFrstRecovCall()){
			$freeHead= $this->freeRecovId($userId);
			$freeId= $this->freeRecovId($userId);
			$asaStep= $this->calcAsastep($userId,$freeId);	// Future asastep
			$dtb->deleteAll(_c_RecDataTbl,_c_recDatAStp,$asaStep);	// Delete all parasite $asaStep of _c_RecDataTbl
		}

		$id= $this->generalInit($asaSubS);
		$where= $dtb->condStr(_c_recDatId,$id);
		
		if ($origAction==='chgIdx' || $origAction==='changePageId'){
			// The change was done on the main index need to store the value of the original index for deletion when undoing.
			$fls->splitOnFirstC($nStrInf,$oldStr,$newStr);
			$valV= array($id,$userId,$this->asaStepId,$asaSubS,$origAction,$tableName,$oldStr,$newStr);		
			$dtb->newElem(_c_RecDataTbl,$where,false,$valV,true);
		}else{		
			if ($origAction==='new'){
				$valuesStr=''; // original action was a New record. so no string to save! But still the action New to store so as to be able to undo!
			}else{
				$rec= $dtb->getElem($tableName,$whereStr);
				$valuesStr= $acc->bSlhQP($rec);		// Total original \ strings to save.

				// #5June2017
				
				/*
				$rec= $dtb->getElem($tableName,$whereStr);
				$valuesStr= $rec[_c_pcMainStr];
				*/
			}	
			
			// Save recovery tbl :
			$valV= array($id,$userId,$this->asaStepId,$asaSubS,$origAction,$tableName,$whereStr,$valuesStr);
			$dtb->newElem(_c_RecDataTbl,$where,false,$valV,true);
		}
				
		$glob->prevAllRecov();
	} // forChgRec
	

	/* -------------------------------------------------------------------------------------------------
		On this level, is able to store all operations inside the recovery table, so no needs to
		use the recovered information to INSTANCIATE AN UNDO (do an undo, so just undo). Generallities :
			1) look inside the circular one step table for _c_recDatAStp.
			2) Uses _c_recDatAStp to extract the corresponding steps inside _c_RecDataTbl.
			3) "Exchange" the undo/redo buffer with the corresponding information :
					should have a one to one (*1) match, so exchange can be done step by step, using local index
					step by step forward to rebuild in the exact order (respect the origibal order).
					(*1) even if the 1 to 1 match is broken, it's still possible to do the exchange because list of
					record can inflate or deflate, but logically should not happend, all stored operation having
					an image inside the undo/redo table.
	*/
	
			
	/**
	*/
	function getAsaStep($cirPos){
		$dtb= &$this->dtb;
		return $dtb->getElem(_c_RecCircTbl,_c_recCrcId,$cirPos,_c_recCrcAStp);		
	} // getAsaStep

	/**
	*/
	function getAsaStepSz($cirPos){
		$dtb= &$this->dtb;
		return $dtb->getElem(_c_RecCircTbl,_c_recCrcId,$cirPos,_c_recCrcStSz);		
	} // getAsaStepSz


	
	/**
		$this->adaptedSort($subTbl,$undo);
	*/
	function adaptedSort(&$v,$undo){
		if ($undo){
			return krsort($v);	// reversed key sort
		}else{
			return ksort($v);
		}
	} // adaptedSort
	

	/**
		Does not seem to be an error, renaming generates the message but works fine!
	*/
	function undoWarning($pos,$tableName,$whereStr){
		$glob= &$this->glob;
		$glob->userError('recovery::undo/'.$pos.', : $tableName='.$tableName.'; $whereStr='.$whereStr._c_);
	} // undoWarning
	
	
	
	/**
	*/
	function smartStopRecov(){
		$glob= &$this->glob;
		$dtb= &$this->dtb;
	
		if ($this->dissoGrIs()){
			$glob->stopAllRecov();	// dissoc, so does not recover anything	
		}else{
			$glob->stopRecv();	// Normal state : only stopRecv(), so can save the moves inside the incremental backup
		}
	} // smartStopRecov
	
	/**
	*/
	function smartPrevRecov(){
		$glob= &$this->glob;
		$dtb= &$this->dtb;
	
		if ($this->dissoGrIs()){
			$glob->prevAllRecov();
		}else{
			$glob->prevRecv();	// Normal state
		}
	} // smartPrevRecov
	
	/**
		A hand manipulated function to dissociate or not the
		General Recovery from the Incremental Savings.
	*/
	function dissoGrIs(){
		// return false;
		return true;
	} // dissoGrIs

			
	
	
	/* -------------------------------------------------------------------------------------------------------
		Undo the last cms step/operation.
		Input :
			// $asaStep : _c_recDatAStp (the unique string matching the whole cms step). Given through the 
			undo button url variable. Start state : ----> for the start call : 	$dtb->lastAsaStep()
	*/
	function undo($undo=true){
		$glob= &$this->glob;
		$acc= &$this->acc;
		$fls= &$this->fls;
		$dtb= &$this->dtb;
		$rMg= &$this->rMg;
		$eTools= &$this->eTools;
		
		$this->smartStopRecov();
		
		$userId= $rMg->realTimeUserId();
		$undoPos=$this->getRecovPos($userId,$undo);	// undo position
		
		
		if ($undoPos===false){
			$glob->userError('Recovering operation out of limits!');
			$this->smartPrevRecov();
			return false;
		}
		
		
		$asaStep= $this->getAsaStep($undoPos);
		$subTbl= $dtb->kSubTable(_c_RecDataTbl,_c_recDatAStp,$asaStep,_c_recDatSubS);
		
		
		if ($subTbl===false){
			$glob->userError('Did not find the fields '.$asaStep.' to recover! $undoPos='.$undoPos);
			$this->smartPrevRecov();
			return false;
		}
		
		if ( $this->adaptedSort($subTbl,$undo) ){
			reset($subTbl);
			foreach ($subTbl as $key=>$curRecovRec){
				$origAction= $curRecovRec[_c_recDatAct];
				$tableName= $curRecovRec[_c_recDatTbl];
				$whereStr= $curRecovRec[_c_recDatWhr];		// data Where
				$valuesStr= $curRecovRec[_c_recDatVal];		// $curRecovRec['valuesStr']
				$curRecId= $curRecovRec[_c_recDatId];
				$where= $dtb->condStr(_c_recDatId,$curRecId);	// recovery Where
				
				/*
					Instanciates the undo using the above variables.
				*/
				switch ($origAction){
					/*
						Recovery of a changed dtb action : swap values between the mentionned record and the recovery table.
					*/
					case 'chg':
						// Get the standard record before to ecrase it
						$lastRec= $dtb->getElem($tableName,$whereStr);
						
						/*
							Ecrases the standard record with the recovery value :
								Rem : would have been nicer to do an update (only one operation), but
								(1) much easier (compacter & more practical for a new record) to store 
								only the values in the recovery structures, so here it is!
								(2) vectorising the string is difficult because of multiple \\\" imbrications.
						*/
						$dtb->deleteTableRecord($tableName,$whereStr);
						$dtb->newRecordStr($tableName,$valuesStr,$whereStr);						
						
						if ($lastRec!==false){
							$glob->stopIBkp();
							// Replace the vacant place of the recovery table with the previously ecrased element, so  can do a redo if asked for.
							$dtb->setElem(_c_RecDataTbl, _c_recDatVal,$acc->bSlhQP($lastRec), $where);
							$glob->prevIBkp();
						}else{
							// Case when the where string pointed to nowhere !!  May have to check if it's normal that it pointed to nowhere !!
							$this->undoWarning('chg',$tableName,$whereStr);
						}
					break;
				
					/*
						To change the main index implies some important difference with a general change (above) : 
							- no need for creating and deleting a whole file.
							- much easier to just memorise the change.
							- even so, most if the time much faster to memorize the upper level, eg: at the page level.
					*/
					case 'chgIdx':
						$oldId= $curRecovRec[_c_recOldName];
						$newId= $curRecovRec[_c_recNewName];
						
						// Rename the visible table ids.
						$dtb->renameRec($tableName,$newId,$oldId);	  
						
						// ids are reversed so the redo can be done.
						$dtb->setElem(_c_RecDataTbl, _c_recOldName,$newId, $where);	
						$dtb->setElem(_c_RecDataTbl, _c_recNewName,$oldId, $where);	
					break;
					
				
					case 'new':
						/*
							The action to reverse $undo was a instanciation! ---> Need to 
							- Need to save CRecord in the recoVery table.
							- Remove the concerned record CRecord from the std table
						*/
						if ($undo){
							$lastRec= $dtb->getElem($tableName,$whereStr);
							$dtb->deleteTableRecord($tableName,$whereStr);
							
							if ($lastRec!==false){	
								$assVal= $acc->bSlhQP($lastRec);
								$dtb->setElem(_c_RecDataTbl,_c_recDatVal,$assVal,$where);
							}else{
								$this->undoWarning('new',$tableName,$whereStr);
							}
						}else{
							$dtb->newRecordStr($tableName,$valuesStr,$whereStr);
							$dtb->setElem(_c_RecDataTbl,_c_recDatVal,'',$where);
						}
					break;
					
				
					case 'del':
						/*
							The action to reverse was a deletion! ---> Need to 
								- adds the concerned recovery record to the place it belonged to.
								- replaces the content of the recovery record with an empty string.
								- when reversing the undo order $undo===false ---> need to delete actual recod
								  ... not just swapping
						*/
						if ($undo){
							$dtb->newRecordStr($tableName,$valuesStr,$whereStr);
							$dtb->setElem(_c_RecDataTbl,_c_recDatVal,'',$where);
						}else{
							$lastRec= $dtb->getElem($tableName,$whereStr);
							$dtb->deleteTableRecord($tableName,$whereStr);
							
							if ($lastRec!==false){
								$assVal= $acc->bSlhQP($lastRec);
								$dtb->setElem(_c_RecDataTbl,_c_recDatVal,$assVal,$where);
							}else{
								$this->undoWarning('del',$tableName,$whereStr);
							}
						}
					break;
					
					case 'changePageId':
						$originalName= $curRecovRec[_c_recOldName];
						$replacingName= $curRecovRec[_c_recNewName];
						
						$eTools->changePageId($originalName,$replacingName, !$this->dissoGrIs() ); // change page and menu inside the corresponding language				
						
						$glob->stopIBkp();
						// invert the above change inside the left over (ready to redone) recovery cell
						
						$dtb->setElem(_c_RecDataTbl,  _c_recOldName,$replacingName,  _c_recDatId,$curRecId);
						$dtb->setElem(_c_RecDataTbl,  _c_recNewName,$originalName,  _c_recDatId,$curRecId);
						
						$glob->prevIBkp();
					break;
					
					
					default: 	$this->topErr('recovery::undo');
				} // switch			
							
			} // for each
			
			
			$glob->stopIBkp();
			if ($undo){
				$this->decrUndoPos($userId);
			}else{
				$this->incrUndoPos($userId);
			}
			$glob->prevIBkp();
			
		}else{
			$this->topErr('recovery::undo(), can\'t sort!');
		}
		
		$this->smartPrevRecov();
		return true;	
	} // undo
	
	/**
	*/
	function redo(){
		return $this->undo(false);
	} // redo
	
		
	/**
		Reset all the recovery table to a virgin state.
	*/
	function resetRecovTbl(){
		$glob= &$this->glob;
		$dtb= &$this->dtb;
		$rMg= &$this->rMg;
		$glob->stopAllRecov();
		
	
		$userId= $rMg->realTimeUserId();	
		$strtRecId= $dtb->getElem(_c_UsersTbl, _c_userId,$userId, _c_usrStrtRecId);
		$smallestDataId= $dtb->getElem(_c_UsersTbl, _c_userId,$userId, _c_recDtSmlstId);		
		
		
		// Reset recover elements of the user table.
		$curConD= $dtb->condStr(_c_userId,$userId);
		$dtb->setElem(_c_UsersTbl,  _c_usrFreeRecId,$strtRecId+1, $curConD);
		$dtb->setElem(_c_UsersTbl,  _c_usrUndoPos,$strtRecId, $curConD);
		$dtb->setElem(_c_UsersTbl,  _c_recDataFreeId,$smallestDataId, $curConD);		
		
		
		// Reset the part of the main memory recover tables belonging to the current user.
		$dtb->deleteAll(_c_RecDataTbl,_c_recDatUid,$userId);
		$dtb->deleteAll(_c_RecCircTbl,_c_recCrcUid,$userId);
		
		/*
			Add a gimmick value at _c_RecCircTbl, matching (_c_usrUndoPos,$strtRecId).
				1) Won't get integrated in visible datas because freePos is just folowing.
				2) Get the same logic as in middle work : undopos just before freePos
		*/
		
		$gimV= array($strtRecId,$userId,$this->calcAsastep($userId,$strtRecId),0);
		$dtb->newRecFromV(_c_RecCircTbl,$gimV);
		
		$glob->prevAllRecov();		
	} // resetRecovTbl
} // recovery
?>