<?php
/**
	File name : scanStr.php
	Author of this file : Oscar Fawer until 2019-08-15
	Contact : www.ataox.org 
	Web sites : ataox.org / ataox.com / ataox.info
	Copyrights open source GPL, GNU General Public License V3 : see ./aMainDr/code/LICENSE.txt


	A class to group all attributes directly linked to the record strings to scan.
*/
class scanStr extends oTop{
	
	/* 
		Input :
			$tgF: class tagsFu
			$s : the given string to be converted into the scanStr format.
			etc...
	*/
	function __construct(&$tgF, &$s=NULL, $sz=0, $i=0, $stop=false, $cellId='nop', $user=_c_defRegUser){
		parent::__construct();	
		
		$this->tools= &$tgF->tools;		
		$this->dtb= &$tgF->dtb;		
		
		$this->setScGenVar($tgF,$s,$i,$stop,$cellId,$user);	
		$this->setTrackers();
	} // construct

	/**
	*/
	function setScGenVar(&$tgF, &$s=NULL,$i=0,$stop=false,$cellId='nop',$user=_c_defRegUser){
		$tools= &$this->tools;		
		$dtb= &$this->dtb;
		
		// $this->str is an ARRAY !! (for easiness of use of multi-bytes "strings")
		if ( $dtb->getGenVar('multiByteParsing')!==_c_ok ){
			$this->str= $tools->stdStrToAry($s,$sz);
		}else{
			$this->str= $tools->mbStrToAry($s,$sz);
		}
		
		$this->sz= $sz;			// its size
		$this->i= $i;			// current scanning index, matching the character $this->cf
		$this->stop= $stop;		// to stop the scanning process mostly to change the level		
		$this->cellId= $cellId;
		
		$this->user= $user;
	} // setScGenVar
	
	
	/**
		Convert back to a standard string for easy debug-display
	*/
	function toStandardStr(){
		$tools= &$this->tools;		
		return $tools->arrayToStr($this->str);
	} // toStandardStr
		
	
	/**
		True if the end is not reached.
	*/
	function enoughSpace(){
		return $this->i < $this->sz;
	} // enoughSpace

		
	/**
	*/
	function cellId(){
		return $this->cellId;
	} // cellId	
		
		
	/**
		Tracking characters to allow an efficient vision of the current spot.
	*/
	function setTrackers($cp2='',$cp='',$cn='',$cf=''){
		$this->cp2= $cp2;	// older past (one step)
		$this->cp= $cp;		// past (now-1)
		$this->cn= $cn;		// now (current character)
		$this->cf= $cf;		// future (now+1)
	} // setTrackers
	
	
	/**
		Get the tracking characters (mainly debug purpose)
	*/
	function getTrackers(&$cp2,&$cp,&$cn,&$cf){
		$cp2= $this->cp2;
		$cp= $this->cp;
		$cn= $this->cn;
		$cf= $this->cf;
	} // getTrackers
	
	
	/**
		$this->scanStr->printTrackers('aaa');
	*/	
	function printTrackers($marker=''){
		$i= $this->i;
		print($marker.' Four Trackers= '.$this->cp2.$this->cp.$this->cn.$this->cf._c_Ns._c_Ns._c_Ns._c_Ns.
				' sz='.$this->sz);
				
		if ($i<$this->sz){		
			print(' $this->str[$i]='.$this->str[$i]._c_);
		}
		
		print(' $i='.$i._c_);
	} // printTrackers
	
	function pstr(){
		print('$this->str='.$this->str._c_);
	} // p
	
	
	/**
		Update the 1st tracker character (the future char).
	*/
	function updateFirstTracker(){
		$this->cf= $this->str[$this->i];
	} // updateFirstTracker
			
	/**
	*/
	function next(){
		$this->cp2= $this->cp;
		$this->cp= $this->cn;
		$this->cn= $this->cf;
		
		$this->i++;
	} // next
	
	/**
		True if we are On The End.
	*/
	function passedLastChar(){
		return $this->i>$this->sz-1;
	} // passedLastChar
	
	
	/**
	*/
	function currentIndex(){
		return $this->i;
	} // currentIndex
	
		
	
	/**
		Give the word or the end of the word starting just after the current index.
		Input :
			$lim : local scan limit
			$offset : standard case just in from of $this->i, so offfset === 0
			$sep : separator to define the main ending condition.
	*/
	function currentWord($lim,$offset=0){
		$word='';
		$i= $this->i+$offset;
		$c= $this->str[$i];
		
		/*
			Recent addition  $c!==_c_carRet && $c!==_c_nL && $c!==_c_rT
			Otherwise not recognizing tags when separated wit that! Very annoying bug !!
		*/
		while ( $c!==_c_space && $c!==_c_slash && $c!==_c_rBrk && $c!==_c_carRet && $c!==_c_nL && $c!==_c_rT && $this->enoughSpace() && $i-$this->i<=$lim){
			$word.= $c;
			$i++;
			$c= $this->str[$i];
		}
		return $word;
	} // currentWord

		
} // scanStr
?>
