<?php
/**
	File name : statsWriter.php
	Author of this file : Oscar Fawer until 2019-08-15
	Contact : www.ataox.org 
	Web sites : ataox.org / ataox.com / ataox.info
	Copyrights open source GPL, GNU General Public License V3 : see ./aMainDr/code/LICENSE.txt



	Independant code to write summarized traffic statistics of the current website.
	It must be independant from the main Php clusters (Xpetal part etc.) and any preformated
	database structure because it must also be called when running in mode cache therefore
	extra-light otherwise caching does not make any sense!
	
	Most important :
		$this->currentStatsDfN($ext='.txt') : path and name of the current file couple
		External Calls :
			$this->writeNewStatsEvent();
			$this->writeNotBotEvent($ajaxId);
		$this->buildStatsEventStr($indexEvent) : define all interesting $_SERVER[] values
		$this->statsPhpStr($i,$eti,$value) : creates the php strings
		Key variable : $_SERVER['HTTP_REFERER'], search : $_SERVER HTTP_REFERER
		Controversy : look at strip_tags()
*/
class statsWriter{
	
	function __construct(&$cacheReader){
		$this->cacheR= &$cacheReader;
				
		if (!file_exists(_c_statsEventNextIdxDfn)){
			// Successive check and makedir
			
			if (!is_dir(_c_genStatsPathNS)){
				// in that case does not exists
				mkdir(_c_genStatsPathNS,_c_statsFoldersRights);
			}
			
			if (!is_dir(_c_curStatsPathNS)){
				// in that case does not exists
				mkdir(_c_curStatsPathNS,_c_statsFoldersRights);
			}
			
			// Sets the 1st default _c_statsEventNextIdxDfn
			$phpNextStr= $this->phpFileCaps('$this->idxNextEvent=100;');
			$this->strToFile(_c_statsEventNextIdxDfn,$phpNextStr,'w');
		}

		include _c_statsEventNextIdxDfn; // now it should exist

		/**
			"Cloned" from hashPwCrypt::zArrCons (doesn't need to be the same, but need to be independant
			--> could include a common file).
		*/
		$this->zArrCons= array(
		'C','j','k','l','m','T','V','W','X','B','3','h','r','2','Z','p','q',
		'G','H','N','0','1','9','F','P','J','K','L','M','n','Q','R','S','D', 
		'b','c','d','f','g','4','8','s','t','6','7','v','w','5','x','z');
		$this->zLnArrCons= sizeof($this->zArrCons);
		
		// Pure consons
		$this->zArrC= array(
		'C','j','k','l','m','T','V','W','X','B','h','r','Z','p','q',
		'G','H','N','F','P','J','K','L','M','n','Q','R','S','D', 
		'b','c','d','f','g','s','t','v','w','x','z');
		$this->zLnArrC= sizeof($this->zArrC);
	} // construct
	
	
	function getVal($key,$val=false){
		$cacheR= &$this->cacheR;
		return $cacheR->getVal($key,$val);
	} // getVal
	
	
	/**
		By default append a string to an existing file (auto creation at the start) and return the number of byte written or false. 
		Input :
			$dfn : directory and file name.
			$gStr : given string.
			$mode : to define the writing mode
	*/
	function strToFile($dfn,$gStr,$mode='a'){
		$fd= fopen($dfn,$mode);
		$res= fwrite($fd, $gStr);
		fclose($fd);
		chmod($dfn,_c_statsFilesRights);	// in fact change the file access rights ... mode ...
		return $res; // for an eventual checking
	} // strToFile
	
	/**
		Encapsulates the given string into the standard Php tags (model defined in filesMg.php ... the structure/work
		would become too complex if code redondancy was to be avoidrd at any cost), can't use filesMg.php in cache mode.
	*/
	function phpFileCaps($s){
		return	'<?php'._c_R.$s._c_R.'?>';
	} // phpFileCaps
	
	
	/**
		Backslash the given char, so it can be capsulated between two of these chars (model in accents.php)
	*/
	function backSlashC($s,$c='\''){
		$s= str_replace (_c_bkSlash, _c_dblBkSlash, $s); // --> big mess when using the undoing editing options, but ok here
		return str_replace ($c, _c_bkSlash.$c, $s);
	} // backSlashC

	
	/**
		Define a $s[event Index][category]= array for each relevant http request.
		Input :
			$i : index visit (incremented for each http request).
			$eti : string to index the concerned entity/category (time, kWords, pageName etc.)
			$value : matching accessible value during the concerned http request.
	*/
	function statsPhpStr($i,$eti,$value){
		// $value= strip_tags($value);
		return _c_statsVect.'['._c_snglQ.$i._c_snglQ.']['._c_snglQ.$eti._c_snglQ.']='.
			_c_snglQ.$this->backSlashC($value,_c_snglQ)._c_snglQ.';'._c_R;
	} // statsPhpStr
		
	function notBotsPhpStr($ajaxId){
		return $this->statsPhpStr($ajaxId,'notBot',_c_ok);
	} // notBotsPhpStr
	
	
	
	/**
		Easy to read time.
	*/
	function timeStamp(){
		$t= time();
		return date('YMd',$t).'-'.date('H',$t).':'.date('i',$t).':'.date('s',$t);
	} // timeStamp
	
	
	/**
		Return true if we don't want to register the stats because of probable webmaster connection
		statsWriter::privateActivities()
	*/
	function privateActivities(){
		// Standard log content, page name irrelevant --> stats irrelevant (except if want to monitor login trials)
		$eCall= $this->getVal(_c_localPwBoxUrlVar);
		
		/**
			Maintaining secret the 2 following parameters would have the advantage of avoiding external
			avoidance of local statistic storeage that also avoid the cache and slow down the system.
		*/
		$uniqPfx= $this->getVal(_c_urlUniqEdtPfx); // Classic edition situation
		
		// Must avoid using the cache to keep this flag going! Implemented in lookForCache()
		$statAvoids= $this->getVal(_c_statsAvoidUVar)===_c_ok;
		
		
		return $eCall!==false || $uniqPfx!==false || $statAvoids;
	} // privateActivities

		
	/**
		The main information : pages and zooms are groupped with their prefix.
		Note about images : can't trigger images alone because Php is not activated.
	*/
	function mainInformationLabel(){
		$s= $this->getVal(_c_localUrlPageVar);
		if ($s!==false){
			return _c_localUrlPageVar.'='.$s;
		}
		
		$s= $this->getVal(_c_idTextsZm);
		if ($s!==false){
			return _c_idTextsZm.'='.$s;
		}else{
			return _c_localUrlPageVar.'='._c_localPermanDefPage;
		}
	} // mainInformationLabel
	
	
	/**
	*/
	function buildStatsEventStr($indexEvent){
		$buff= '';
		$buff.= '';
				
		$str= $this->timeStamp();
		$buff.= $this->statsPhpStr($indexEvent,'hTm',$str);	// human readable date-time
		
		$str= time();
		$buff.= $this->statsPhpStr($indexEvent,'xTm',$str); // unix time
		
		$str= $_SERVER['REMOTE_ADDR'];
		$buff.= $this->statsPhpStr($indexEvent,'ip',$str); // Address IP of the client
		
		
		$miLbl= $this->mainInformationLabel();	
		$buff.= $this->statsPhpStr($indexEvent,'main',$miLbl); // currently defined page name
		
		$str= $_SERVER['QUERY_STRING'];
		$buff.= $this->statsPhpStr($indexEvent,'query',$str); // Own query!
	
		
		if (array_key_exists('HTTP_REFERER',$_SERVER)){
			$httpRef= $_SERVER['HTTP_REFERER']; // Eventually hold search engines key words
			$buff.= $this->statsPhpStr($indexEvent,'ref',$httpRef); // 
		}else{
			$buff.= $this->statsPhpStr($indexEvent,'ref',''); // no referer, the user typed the url inisde the browser bar, or used a bokmark
		}
		$str= $this->extractUrlKeyW($httpRef);
		$buff.= $this->statsPhpStr($indexEvent,'kw',$str); // extracted kwords
		
		
		$str= $_SERVER['HTTP_USER_AGENT'];
		$buff.= $this->statsPhpStr($indexEvent,'agent',$str); // 
		
		
		$str= $_SERVER['REQUEST_URI'];
		
		$dwFN= $this->extractDownloadedFN($str);
		$buff.= $this->statsPhpStr($indexEvent,'download',$dwFN); // store the downloaded file name
		
		
		$buff.= $this->statsPhpStr($indexEvent,'url',$str); // own complete UrL
		return $buff;
	} // buildStatsEventStr
		

	/**
		Returns true if the string $s (small) is a prefix of the string $b (big).
	*/
	function isPfxB($s,&$b){
		return stripos($b,$s)===0;
	} // isPfxB
	
	/**
		Looks for the the 1st string in $v starting with $pfx.
	*/
	function firstPfxMatchInV(&$v,$pfx){
		reset($v);
		foreach ($v as $key=>$val){
			if ($this->isPfxB($pfx,$val)){
				return $val;
			}
		}
	} // firstPfxMatchInV
	

	/**
	*/
	function extractUrlKeyW(&$url,$prfx='q='){
		$q= parse_url($url,PHP_URL_QUERY);
		$qV= explode('&',$q);
		$kwString= $this->firstPfxMatchInV($qV,$prfx);
		$szPrfx= strlen($prfx);
		$kwString= substr($kwString, $szPrfx);
		return $kwString;
	} // extractUrlKeyW
	

	/**
		Extract the dowloded file name from the specific download page based paradigm.
	*/
	function extractDownloadedFN(&$url){
		return $this->extractUrlKeyW($url,'ddurl=');
	} // extractDownloadedFN


	
	/**
	*/
	function statsDfN($yearZ,$dayZ,$ext='.txt'){
		return  _c_curStatsPath.$yearZ.'-'.$dayZ.$ext;
	} // statsDfN
	

	/**
	*/
	function currentStatsDfN($ext='.txt'){
		$curYearZ= date('Y');
		$curDayZ= date('z');
		return $this->statsDfN($curYearZ,$curDayZ);
	} // currentStatsDfN

	
	/**
		_c_curStatsPath. currentDay(1- 365) . '.txt'
		... when creating a new file because new day, got to transform the previous file into a (.php)
	*/
	function incrementStatsFN($statsEventStr, $event='st'){
		$curDfn= $this->currentStatsDfN();
		if ( file_exists($curDfn) ){
			// Keep adding
			switch ($event){
				case 'st':
					$this->strToFile($curDfn, _c_R._c_R.$statsEventStr);
				break;
				
				case 'notBot':
					$this->strToFile($curDfn, $statsEventStr);
				break;
				
				default: print('Error in statsWriter::incrementStatsFN');
			} // switch
		}else{
			// Start a new file
			$this->strToFile($curDfn, '<?php'._c_R._c_R.$statsEventStr, 'w');
			
			// Php-ise the previous file if it finds it!
			$xTimeLastDay= time() - (3600*23+3540); // time probe on the previous file etc... + so almost 100% sure to be inside last day (This technique takes leap years in account automatically.)
			$oneDay= 3600*24;
			$gotPrevFile= false;
			$i= 10;
			do {
				$concernedYearZ= date('Y',$xTimeLastDay);
				$prevDayZ= date('z',$xTimeLastDay);
				
				$prevDfn= $this->statsDfN($concernedYearZ,$prevDayZ);
				if (file_exists($prevDfn)){
					$gotPrevFile= true;
					
					// Php activate the previous one so it can be read by a simple inclusion
					$this->strToFile($prevDfn, _c_R._c_R.'?>');
					$prevPhpDfn= $this->statsDfN($concernedYearZ,$prevDayZ,'.php');
					
					rename($prevDfn,$prevPhpDfn);
					break;
				} // may not exists because we just started the stats
				
				
				$xTimeLastDay= $xTimeLastDay-$oneDay;
				$i--;
			} while (!$gotPrevFile && $i>0);
			
			
		}
	} // incrementStatsFN

 
	/**
		Generates a consonances string with numbers! $sz
	*/
	function consonStr($sz){
		$hash='';
		for($i=0;$i<$sz;$i++){
			$idx= rand(0,$this->zLnArrCons-1);
			$hash.= $this->zArrCons[$idx];
		}
		return $hash;
	} // consonStr
	
	/**
		Generates a PURE consonances string.
	*/
	function consStr($sz){
		$hash='';
		for($i=0;$i<$sz;$i++){
			$idx= rand(0,$this->zLnArrC-1);
			$hash.= $this->zArrC[$idx];
		}
		return $hash;
	} // consStr
	

	/**
		Write a new stats event from the current http call. Called from starter.php
	*/
	function writeNewStatsEvent(){
		if ( $this->privateActivities() ){
			return;
		}
		
		// Extract the index and increment it for the file
		$indexEvent= $this->idxNextEvent;
		$this->idxNextEvent++;
		$phpNextStr= $this->phpFileCaps('$this->idxNextEvent='.$this->idxNextEvent.';');
		unlink(_c_statsEventNextIdxDfn);
		$this->strToFile(_c_statsEventNextIdxDfn,$phpNextStr,'w');
				
		/**
			Append the current event to the file. Warning the index latency was solved by the random addition 
			and later removing of the random differenciation also gave correct concurrent results! Problem
			may appear again, so rand() can be activated by changing _c_rndIndexStats constant.
		*/
		if (defined('_c_rndIndexStats') && _c_rndIndexStats===_c_ok){
			$curIndx= $indexEvent.$this->consStr(1).$this->consonStr(3);
			$statsEventStr= $this->buildStatsEventStr($curIndx);
		}else{
			$curIndx= $indexEvent;
			$statsEventStr= $this->buildStatsEventStr($curIndx);
		}
		
		$this->curStatIndx= $curIndx; 
		$this->incrementStatsFN($statsEventStr);
	} // writeNewStatsEvent
	

	/**
		Url to access the Ajax checker. Need to be built with main constants because 
		tools::urlGivenPage() is not accessible while in a cache mode.
	*/
	function ajaxCheckerUrl($idx){
		$pg= '_ajaxChecker';
		return _c_IndexFN._c_Interog.'stAjxEventIdx='.$idx;
	} // ajaxCheckerUrl
	

	/**
		Stats Ajax supplement.	statsWriter::statsAjaxSup()
		The JS function ajaxChecker() is there to send an Ajax page request to have one more way to figure out
		if a client is a bot or a person for statistics purpose on each pages (cookies are too easy to short circuit
		and captchas are too time consuming for every page).
	*/
	function statsAjaxSup(){
		$checkerUrl= $this->ajaxCheckerUrl($this->curStatIndx);
		$checkerUrl= $this->tinyCrypt($checkerUrl);
		
		// Prepare and decypher the given address
		$prep='
			var eUrl= "'.$checkerUrl.'";
			var rUrl= kk.tinyCrypt(eUrl);
		';
		
		// $b : is the simple possibility without using a scare loop
		$b='<script>
function ajaxChecker() {
	'.$prep.'
	var xmlhttp = new XMLHttpRequest();
	xmlhttp.open("GET", rUrl, true);
	xmlhttp.send();
}
</script>';

		// $c : got a huge facultative loop to scare bots off trying JS execution (does the loop twice so automatic simplification unlikely to apply)
		$c='<script>
function ajaxChecker() {
	for (i=0; i<Number.MAX_SAFE_INTEGER; i++){ // :-)
		if (i>0){
			'.$prep.'
			var xmlhttp = new XMLHttpRequest();
			xmlhttp.open("GET", rUrl, true);
			xmlhttp.send();
			break;   
		}
	}
}
</script>';
		
		if (_c_jsBotsScary===_c_ok){
			return $c;
		}else{
			return $b;
		}
		
	} // statsAjaxSup
	
	
	/**
		Tiny crypt / also works to decypher itself
	*/
	function tinyCrypt($mbStr){
		$mbV= $this->mbStrToArr($mbStr);
		$this->swapingPairs($mbV);
		return $this->arrayToStr($mbV);
	} // tinyCrypt
	

	/**
		A naïve encryption (pair swaping) to confuse naïve bots
	*/
	function swapingPairs(&$v){
		reset($v);
		$even= true; // matches indexes 0, 2, 4, ...
		foreach ($v as $i=>$val){
			if (!$even){
				// swaps the pair of elements
				$buff= $v[$i-1];
				$v[$i-1]= $v[$i];
				$v[$i]= $buff;
			}
			
			$even= !$even;
		}
	} // swapingPairs
	
	
	/**
		Convert a UFT8 multi-byte string into an array of single characters, 
		only for small strings! <500 chars {twin of tools::slowMbStrToA()}
	*/
	function mbStrToArr(&$str){
		$sz= mb_strlen($str,'UTF-8');
		$vect= array();
		for ($i=0; $i<$sz; $i++){
			$vect[$i]= mb_substr($str,$i,1,'UTF-8');
		}
		return $vect;
	} // mbStrToArr
	
	

	/**
	*/
	function arrayToStr(&$v){
		$s= '';
		reset($v);
		foreach ($v as $key=>$val){
			$s.= $val;
		}
		return $s;
	} // arrayToStr
	

	
	/**
	*/
	function writeNotBotEvent($ajaxId){
		$rawSz= strlen($ajaxId);
		if ($rawSz<20){	// open for about 10^15 stats event ... should be enough!!
			$ajaxId= strip_tags($ajaxId);
			$stripSz= strlen($ajaxId);
			if ($rawSz===$stripSz){
				$ajaxId= $this->backSlashC($ajaxId);
				$ajaxEventStr= $this->notBotsPhpStr($ajaxId);
				$this->incrementStatsFN($ajaxEventStr,'notBot');
			}
		}
	} // writeNotBotEvent	
	
		
} // statsWriter
?>