<?php
/**
	File name : tagsParser.php
	Author of this file : Oscar Fawer until 2019-08-15
	Contact : www.ataox.org 
	Web sites : ataox.org / ataox.com / ataox.info
	Copyrights open source GPL, GNU General Public License V3 : see ./aMainDr/code/LICENSE.txt


	To parse the eXtandable Mark-up square code (xm2)
*/
class tagsParser extends oTop{
	

	function __construct(&$tagsFu){
		parent::__construct();
		$this->tagsFu= &$tagsFu;
		
		$this->glob= &$tagsFu->glob;
		$glob= &$this->glob;
		
		$this->tools= &$tagsFu->tools;	// hetereogenous tools		
		$this->acc= &$tagsFu->acc;
		
		$this->dtb= &$tagsFu->dtb;
		
		$this->caps= &$tagsFu->caps;
		$this->countParsings= 0;
		
		$this->rMg= &$tagsFu->rMg;			// General rights especially loggin
		$this->wmR= &$tagsFu->wmRights;		// Logged webmaster or host specific rights
		

		/* A flexible global variable to also use ## instead of "" as an easier syntax 
		to include other "", usefull in the builder. */
		$glob->doubleQuote= _c_dblQ;

		/**
			Different States of the parsing process : the main question is to have enough precision within the state 
			possible values to know where we are to load the associated arrays capable to feed the connected functions.
				
			// outsite [tag paramy="yyy" paramz="zzz"]the body[/tag] :
			// outsite     [  tag       paramy  =    "   yyy  " paramz="zzz"  ]   the body   [    /   tag    ]
			//   (1)      (2) (3)  (4)   (5)   (6)  (7)  (8) (9)            (10)   ( 11)   (12) (13) ( 14) (15)                          
			
			Simpler forms :
				[tag param1="hello1"  /   	  ]
									(16)	(17)
				[tag/]
				[/]
				
			Sjortcut :
				[zm apple.jpg golden /]
				
			multi levels notation :
				out [head]  body  [tail]	out
				
			Theory :
				* Simpler forms are well manageable with the syntax defined for the full form.
				* White separators state (4) needs only one definition (easy to manage and would be too many to be efficient
				  for coding). Out of double quotes. If many whites are together it just stays in the white mode and does not
				  add characters in the current buffer.

			_c_sOut equivalent of _c_sBody linked with treatment of unitags, see manHdRgtBrk()
				Originally out of tag state, but in fact the whole $scanStr can be considered as the body 
				of a non existing tag, difference (because we don't want to call a non existing function matching a non existing tag)
				will be done with the level of recursion. In fact it matches also the pseudo tag managing an eventual upward connexion.
		*/
		if (!defined ('_c_sHdTag')){
			// define('_c_sOut', 1);			// kept here for historical reasons!
			define('_c_sHdLftBrk', 2);		// Head Left Bracket
			define('_c_sHdTag', 3);			// Head tag		
			define('_c_sSpace', 4);			// White separator state (anywhere except between "")
			define('_c_sHdParVar', 5);		// Head parameter variable name
			define('_c_sHdParEqu', 6);		// Head parameter equal sign
			define('_c_sHdParLftQ', 7);		// Head parameter left doublequote
			define('_c_sHdParVal', 8);		// Head parameter value
			define('_c_sHdParRgtQ', 9);		// Head parameter right doublequote
			define('_c_sBody', 11);			// Body
			define('_c_sTlLftBrk', 12);		// Tail left bracket
			define('_c_sTlSlash', 13);		// Tail slash
			define('_c_sTlTag', 14);		// Tail Tag
			
			/**
				Useless states, because easier to assume the next state _c_sBody, this avoid by the way digging in past 
				characters to form the body. Other states don't need to recover a past char because once recognise (for
				the action matching the state) it does not need memorisation of the char.
					define('_c_sHdRgtBrk', 10);		// Head right bracket
					define('_c_sTlRgtBrk', 15);		// Tail right bracket
			*/ 
			
			define('_c_sHeadSlash', 16);
			define('_c_sHeadToSolve', 17);
			define('_c_sError', 18);
			
			/**
				Necessary because _c_sBody and checking $scanStr->enoughSpace() is not enough in some specific cases.
				This problem comes because we scan with one char in advance and got to add it at the end except if
				it's part of a xml end tag. See stateEndTagUsefullHere to see where _c_sEnd is usefull.
			*/
			define('_c_sEnd', 100);	
		}
		
		
		
		$this->buffPgInclusion= '';
		$this->buffMnInclusion= '';	
		$this->fuCallingOn= true;		// Function (callFu()) calling If false --> a way to fill $this->globMetTags[$tag] without calling any of the matching fu (used in menus to comptabilise tags and see if the menu is a leaf or a branch)
		$this->imposeScanDefLevel();	// Default scanning initialisation
		
		
		$this->curScanStrId= false;	// just an init to detect eventual bugs.
		
		$this->globMetTags= array();	// the ephemere kind with reinitialisation under addrParseStdStr()
		$this->durableGlbMetTags= array();	// the durable kind.
		
		$this->acceptInvisInfInTgZ= true;	// also see acceptInvInferInTgZ()		
	} // construct


	/**
	*/
	function stopParser(){
		$this->keepParsing=false;
	} // stopParser
	

	/**
		Css injection proof : removes double quotes
		Input :
			$vPV : vector of in "" parameters.
		Output :
			the same vector cleaned from any "
	*/
	function removesDQ(&$vPV){
		$vPV= str_replace ('"','',$vPV);	// just by itself (direct on the array
	} // removesDQ
	
	
	
	/**
		Imposes the default scanning level. Rem : could use a default level defined insisde the database, 
		exmplp : 100% [] site !!!!!!!!!!!!!!!!
	*/
	
	function imposeScanDefLevel(){
		$this->parserScanLevelOne= true;
	} // imposeScanDefLevel

	function ignoresScanLvlOne(){
		$this->parserScanLevelOne= false;
	} // ignoresScanLvlOne
	
	function imposesScanLvlOne(){
		$this->parserScanLevelOne= true;
	} // imposesScanLvlOne
	
	// Return true if we got to scan (keep the characters of) the level one.
	function scanLvlOne(){
		return $this->parserScanLevelOne;
	} // scanLvlOne


	/**
		Forces setAiiiTgZtoFalse to true
		Most useful fuction of the pair, because the ither one is the default state, and this on allows independant
		zoomed tags directly followed by comments.
	*/
	function setAiiiTgZtoFalse(){
		$this->acceptInvisInfInTgZ= false;
	} // setAiiiTgZtoFalse
	
	function setAiiiTgZtoTrue(){
		$this->acceptInvisInfInTgZ= true;
	} // setAiiiTgZtoTrue
											
	/**
		If this function is true, it means that the parser accept the inference of invisible characters after
		a [z][/z] tag.
	*/
	function acceptInvInferInTgZ(){
		return $this->acceptInvisInfInTgZ===true;
	} // acceptInvInferInTgZ
											

	/**
		Parses the main string ($scanStr) and return the expanded (html etc. included) string. Note that full parsing is using
		more functions :
			$this->parsePages( ... ) to include going up possibilities implemented with the tag _c_tgUp / up
			$this->parseMen( ... ) to include	menus scaning.		
		
		Algorithm remarks :		
			Characters : $cp : past (now-1), $cn : now (current character), $cf : future (now+1).
						Just copies, the tempo being set inside the scanStr object					
			Tag for local information :	$vectParVar[$iPar], $vectParVal[$iPar], $currentBody								
			In the main Switch calls the state-functions matching the $cn position if $cn determine a state (may also jump to the next state if faster/possible, 
				because a few states do nothing (lazy states) except leading to the next state (lazy states were defined for clarity and completness of the algorithm)).
		 
		 Parameters :
		 	Input : 	
		 		$scanStr : the "string" to scan. In fact an instance of the class scanStr.
		 		$ldVectParVal : loaded vect par val = a COPY of the vector of xml parameters
		 		$parentTag : the parent tag related to the calling of coreParsing()
		 		$singleTag : and content, uses only this specified kind of tag and ignore everythings else.
		 			Algo : when $singleTag is active, keep the strong parsing mode and avoids storing external characters
		 			by filtering at the string concatenation level.
		 		
		 		$level : recursion deepness.
		 		$xStP : past state
		 		$stN : now (present) state
		 				 		
		 	$this->fuCallingOn	 		
		 		
		 	Output :
		 		$ldVectParVal : to conserve the value when leaving the method to return in the above layer!
		 			Important to note that $ldVectParVal is only used inside parse(...), so we don't
		 			need to bother about it's value at top level.
	*/
	function coreParsing(&$scanStr,&$ldVectParVal,$parentTag,$singleTag,$level,$xStP,$stN){
		$tagsFu= &$this->tagsFu;
		$glob= &$this->glob;
		$tools= &$this->tools;
		
				
		if ($this->countParsings <= _c_parserMaxTimes){
			$this->countParsings++;
		}else{
			$tools->tooManyTagsMess($this->caps);
		}
	
		$level++;
		
		if ($level>_c_maxRecursDeep){
			$this->topErr($glob->sl('Too deep in recursion! limit','kmeo').' : _c_maxRecursDeep ='._c_maxRecursDeep);
		}
		
		$vectParVal= NULL;
		$vectParVar= NULL;
		$ldVectParVal= NULL;
		
		$p_xStN= $xStP;
		$xStN= $stN;
	
		$strongParsing= true;
		
		$takeInvibleChars= true;
		
		$parsedString='';
						
		$gotIt= false;
		$gotShortCut= false;
			
		$currentBody= '';
		$bodyBuff= '';	// current body buffer.
		
		$iPar= 0; // to index parameters of the current tag
		$parVar='';
		$parVal='';
		$parValLast='';
		$iVar= '';
		
		$snglTgResetBodyPrfx= false;
		if ($singleTag!==false){
			$snglTgResetBodyPrfx= true;
		}
		
		$this->keepParsing= true;
		while ($scanStr->enoughSpace() && $this->keepParsing){
		
			$scanStr->updateFirstTracker();		// Update the 1st tracker character
			$cn= $scanStr->cn;
			$cf= $scanStr->cf;
			
									
			if ($stN!==_c_sSpace){
				$p_xStN= $xStN;	// previous no space StN loaded with no space StN (x ---> no Space)
				$xStN= $stN; 	// A copy to detect if a change happend
			}
			
			/**
				State analysis : need to build the data corresponding to the parsing, so the tag name with its parameters inside an assocoative
				array, so we can call the corresponding function to deal with it!
			*/
			switch ($stN){
				// Body
				case _c_sBody:
					// ---------------------------------------------------------
					if ($strongParsing){
					
						switch ($cn){
							case _c_lBrk:							
								if ($cf===_c_slash){
									$stN=  _c_sTlLftBrk; 	// tag end, so end of the body
								}else{
									$tag= $this->standardTag($scanStr->currentWord(_c_szLngstTag));
									if ($this->gotValidChars($tag)){
									
										if (!$glob->tagRestriction() || $glob->tagAllowed($tag)){
										
											$cmplxCatAllowed= $singleTag===false || $singleTag===$tag;
										
											if (array_key_exists($tag,$this->durableGlbMetTags)){
												$virginTag= false;
											}else{
												// The current tag is used for the 1st time during the life of this class instance
												$virginTag= true;
												$this->durableGlbMetTags[$tag]= _c_ok;
											}
									
											$this->globMetTags[$tag]= _c_ok;
											$takeInvibleChars= true;
											
											// ##### --------------------------------------------------------------------------------------------------------
											// Note : this recursive call allows to encompass simple body tags and body tags opening another record to parse.
											// $singleTag is forced to false because tags part of the $singleTag body must be parsed
											$tagBody= $this->coreParsing($scanStr,$ldVectParVal,$tag,false,$level,_c_sBody,_c_sHdLftBrk); // RECURSIVE CALL
											
											if ( $tag==='z' && $this->acceptInvInferInTgZ() ){
												// just finished a zoom tag --> do not want any invisible chars before starting another tag
												$takeInvibleChars= false;
											}
											
										
											/**
												Global variables added for automated menu builder, so inside tag can check outside 
												tags (useful for site building from xml menu definition, see _c_tgMenGen)
											*/
											$this->globVectParVal[$level]= $vectParVal; // not $ldVectParVal !!! because initialised!!									
											if ($glob->tagRestriction()){
												// remove " inside quoted params to avoid CSS injection
												$this->removesDQ($ldVectParVal);
											}
												
											if ($this->fuCallingOn){
												$exeCallFu= true;
												if ($tagsFu->forbTagAttribNmB){
													
													if ( $ldVectParVal!==null && array_key_exists ('name',$ldVectParVal)){
														$curTgNm= $ldVectParVal['name'];
														
														if (in_array($curTgNm,$tagsFu->forbTagAttribNmV)){
															$exeCallFu= false;
														}
													}
												}
											
												// Execute the function/method defined by the tag!
												$this->curScanStrId= $scanStr->cellId();
												if ($exeCallFu){
													// ##### ----------------------------------------------------------------------------------------
													$glob->prsVirginTag= $virginTag;	// set the global parser virgin tag value (not used for now)
													$ldVectParVal['_level']= $level;
													$curTgBody= $tagsFu->callFu($tag,$ldVectParVal,$tagBody,$scanStr);
													
													if ($cmplxCatAllowed){
														$currentBody.= $curTgBody;
													}
													$glob->prsVirginTag= false;
												}
											}
											
											$ldVectParVal= NULL;										
																					
											if ($scanStr->passedLastChar()){
												$stN= _c_sEnd; // stateEndTagUsefullHere (to follow).
											}
											
												
										}else{
											if ($singleTag===false){
												$currentBody.= $glob->sl('Sorry, the tag ','kmep').$tag.$glob->sl(' is not allowed for restricted surf.','kmeq');
											}
										}
									}else{
										if ($singleTag===false){
											$currentBody.= $cn;	// the tag does not exists, so build the current level body with the current character.
										}
									}
									
								}
							break; // _c_lBrk
							
							default:
								/*
									Outside tags, so build the current level body. This is the bottom level or most inside
									tag level, so if a tag need to act at the bottom level of its content without interfering
									with other tags, this is the perfect place to transform the string.
								*/
								if ($singleTag===false || $singleTag===$parentTag){
									if ($takeInvibleChars){
										$substCn= $cn;
									}else{
										$substCn= trim($cn);	// for auto compaction of zoom tags [z .../]
									}
									$currentBody.= $substCn;
								}
								
								
						} // switch ($cn)
					}else{ //	(!$strongParsing)
						/**
							Weak parsing mode
						*/
						if ( $cn===_c_lBrk && $cf===_c_slash){
							$stdT= $this->standardTag($scanStr->currentWord(_c_szLngstTag,1));
							if ($stdT===_c_tgPrsCode){
								// End of the weak parsing mode because got the final _c_tgPrsCode tag								
								$strongParsing= true;	// so will call tagsFu::tgCode(), that just needs to do nothing.
								$stN=  _c_sTlLftBrk;		// tag end.
							}else{
								$currentBody.= $cn;	// build the current level weak parsing body
							}
						}else{
							$currentBody.= $cn;	// build the current level weak parsing body
						}
						
						
					}									
				break; // _c_sBody				
				
				// Head Left Bracket
				case _c_sHdLftBrk:
					$stN= _c_sHdTag;
				break;
				
				// Head tag
				case _c_sHdTag: // _c_tgOz
					$stN= $this->manHdTag($tag,$gotIt,$scanStr);	
					if ($gotIt){				
						if ($tag===_c_tgPrsCode){
							$strongParsing= false; // switch to weak parsing mode
						}
					}
				break;
				
				// White separator state (anywhere except between "")
				case _c_sSpace:
					$stN= $this->manSpace($gotShortCut,$xStN,$scanStr);
					if ($gotShortCut){
						$glob->userError( $glob->el('for the specification (','katp') .$parValLast.' || '. $parVar.$glob->el(') inside the tag ','katq'). '"'.$tag.'".');
						$gotShortCut= false; // back to false, was just there to detect a deformed parameter
					}
				break;
				
				// Head parameter variable name
				case _c_sHdParVar:
					$stN= $this->manHdParVar($parVar,$gotIt,$gotShortCut,$scanStr);
					if ($gotIt){
						if ($gotShortCut){
							$glob->userError( $glob->el('for the specification(s) (','katr'). $parValLast.' || '.$parVar.$glob->el(') inside the tag ','kats').'"'.$tag.'".');
						}
					
						$parVar= strtolower($parVar);
						$vectParVar[$iPar]= $parVar;
						$gotIt= false;
						$parVar='';
					}
					
					$gotShortCut= false; // back to false, was just there to detect a deformed parameter
				break;
				
				// Head parameter equal sign
				case _c_sHdParEqu:
					$stN= $this->manHdParEqu($scanStr,$tag,$iVar,$parValLast);
				break;
				
				// Head parameter left doublequote
				case _c_sHdParLftQ:
					$stN= $this->manHdParLftQ($scanStr);
					
					if ($stN===_c_sHdParRgtQ){
						// got an empty value nd going to jump _c_sHdParVal, so need to treat it here
						$iVar= $vectParVar[$iPar];
						$vectParVal[$iVar]= '';
						$iPar++;						
					}
				break;
				
				// Head parameter value
				case _c_sHdParVal:
					$stN= $this->manHdParVal($parVal,$gotIt,$scanStr);					
					if ($gotIt){
						$iVar= $vectParVar[$iPar];
						$vectParVal[$iVar]= $parVal;
						$parValLast= $parVal;
						$iPar++;						
						$parVal='';
						$gotIt= false;
					}					
				break;
				
				// Head parameter right doublequote
				case _c_sHdParRgtQ:
					$stN= $this->manHdParRgtQ($scanStr);
				break;
								
				// Tail left bracket
				case _c_sTlLftBrk:
					$stN= _c_sTlSlash;
				break;
				
				// Tail slash
				case _c_sTlSlash:
					$stN= _c_sTlTag;
				break;
				
				// End of a single tag (only one head, no tail)
				case _c_sHeadSlash:
				case _c_sHeadToSolve:
					if ($cn!==_c_slash){
						// A bit empirical here (catching up a shift), in fact the real going out state match the ], so waits one turn when on the /
						if (isset($vectParVal)){
							$ldVectParVal= $vectParVal;							
						}
						
						return $currentBody; // also returns to the outside state											
					}else{
						$stN= _c_sHeadToSolve;
					}
				break;				
								
				// Tail Tag
				case _c_sTlTag:
					if ($cn===_c_rBrk){
						if (isset($vectParVal)){
							$ldVectParVal= $vectParVal;
						}
						return $currentBody; // also returns to the outside state
					}else{
						$stN= _c_sTlTag;
					}
				break;
								
				default:
					// $this->topErr('Unknowed default state '.$stN.' in tagsParser::coreParsing() '._c_);
			} // switch($state)
		
			if ($stN!==_c_sSpace){
				if ($p_xStN!==$xStN){
					// a change happend insite the (no space) states
					$xStP= $p_xStN; // memorize the previous value
				}
			}			
			
			$scanStr->next();
		} // while
		
		/**
			Get there only once when the scanning of the current string is finished.
		*/
		if (isset($vectParVal)){
			$ldVectParVal= $vectParVal;
		}

		if ($stN===_c_sBody && $strongParsing){
			if ($this->keepParsing){
				return $currentBody.$scanStr->cn; // don't use $cn because not updated when on the last char of $scanStr
			}else{
				return $currentBody; // got a [urlStopParser /] signal or equivalent
			}
		}else{
			return $currentBody;
		}
	} // coreParsing
	
	
	/**
		A simple call when nothing is up. Standard parsing without incorporation of (_c_tgUp _c_tgPlug) tags, 
		As above but got rid of $this->globCurParsedPg
	*/
	function callParser(&$scanStr,$singleTag=false){
		$ldVectParVal = array();
		$this->globCurParsedPg= $scanStr->cellId;		// Currently parsed page.
		$this->singleTagMd= $singleTag;	// global mode to help actualisation of tags internals to the given $singleTag
		return $this->coreParsing($scanStr,$ldVectParVal,false,$singleTag,0,_c_sBody,_c_sBody);
	} // callParser
	
		
	/**
		Parse a standard string
	*/
	function parseStdStr($str,$visLoc=''){
		return $this->addrParseStdStr($str,$visLoc);
	} // parseStdStr
	
	/**
		Parse a standard string :
		$str : the string to parse.
		$visLoc : a string to visualize the localisation of the record for debugging etc.
	*/
	function addrParseStdStr(&$str,$visLoc='',$kind='std'){
		$tagsFu= &$this->tagsFu;
		if (is_string($str)){
			$this->globMetTags= array();	// initialised
			$sz= strlen($str);
			$scanStr= new scanStr($tagsFu,$str,$sz,0,false,$visLoc);
			return $this->callParser($scanStr);
		}else{
			// Rem : tend to appear in the non SQL version !!
			// print('non string in addrParseStdStr'._c_); return '';
		}
	} // addrParseStdStr
		

	/**
		Page parsing, with possibility to call an upper structure, current string
		embeded in an upper string etc. $this->plugRecordName is the name of the 
		record holding a plug to insert the just processed string.

		Input : 
			$scanStr the 1st string where the scan starts
						
		Globally defined variables for the communication between this method and a few tags :
			$this->buffPgInclusion, $this->gotOneUpTag, $this->plugRecordName
	*/
	function parsePages(&$scanStr,&$count,$imposeSz=false){
		$glob= &$this->glob;
		$tools= &$this->tools;
		$caps= &$this->caps;
		$dtb= &$this->dtb;
		$tagsFu= &$this->tagsFu;
		$rMg= &$this->rMg;
		$wmR= &$this->wmR;
		
		$this->gotOneUpTag= false;
		$count++;
		
		$curPg= $scanStr->cellId();		// Currently parsed page
		$this->globCurParsedPg= $curPg;
		if ( $wmR!==null && $wmR->forbiddenPage($curPg) ){
			$userId= $rMg->realTimeUserId();
			return _c_.$caps->span($glob->el('Sorry the','kbmc').' '.$userId.' '.$glob->el('user is not allowed to access this page!','kbmd'),'ln');
		}
		
		$this->buffPgInclusion= $this->callParser($scanStr);	// Parsing !
							
		if ($this->gotOneUpTag===true){
			$this->gotOneUpTag= false; // initialises it for the next run.
			$strToParse= $dtb->getCellStr($sz,$this->plugRecordName,_c_PolyCellPg,_c_pcMainStr);
			
			if ($imposeSz!==false){
				$sz= $imposeSz;
			}
			
			$scanStr= new scanStr($tagsFu,$strToParse,$sz,0,false,$this->plugRecordName);	// the main string for the current record
			$this->buffPgInclusion= $this->parsePages($scanStr,$count); // RECURSIVE CALL
		}
		
		return $this->buffPgInclusion;		
	} // parsePages
	
	
	/**
		Return true if the given $tag has valid characters
	*/
	function gotValidChars($tag){
		return isset($tag[0]) && $this->acc->allowedChar($tag[0]);		// just test valid 1st char to avoid bug withz the [*] lists bullet
	} // gotValidChars
	
	
	/**
		Standardize the user given tag
	*/
	function standardTag($tag){
		return trim(strtolower($tag));
	} // standardTag
	
	
	/**
		Manages the Head Tag.
	*/	
	function manHdTag(&$tag,&$gotIt,&$scanStr){
		switch ($scanStr->cn){
			case _c_rT:
			case _c_carRet:
			case _c_nL:
			case _c_space:
				if (trim($tag)!==''){
					$gotIt= true;
					
					if ($scanStr->cf===_c_slash){
						$state= _c_sHeadSlash;
					}else{
						$state= _c_sSpace;
					}
				}else{
					$gotIt= false;	// the tag is still not built
					$state= _c_sHdTag;
				}
			break;
			
			case _c_rBrk:
				$gotIt= true;
				$state= _c_sBody;
			break;
			
			case _c_slash:
				$gotIt= true;
				$state= _c_sHeadToSolve;
			break;
			
			default:
				$gotIt= false;	// the tag is still no built
				$tag.= $scanStr->cn;	// build the current tag buffer.
				$state= _c_sHdTag; // keeps the same state.
		} // switch
		
		if ($gotIt){
			$tag= $this->standardTag($tag);
		}
		return $state;
	} // manHdTag
	
	/**
		Manages space related state in the header or the tail.
	*/	
	function manSpace(&$gotShortCut,$stP,&$scanStr){
		$glob= &$this->glob;
		
		$cp= $scanStr->cp;
		$cn= $scanStr->cn;
		$cf= $scanStr->cf;
		switch ($cn){
			case _c_space: 
			case _c_rT: 
			case _c_carRet: 
			case _c_nL: 
				return _c_sSpace;	// another "space", jump it.
			break;
			
			case _c_slash:
				if ($cf===_c_rBrk){
					// end of lonely xml head, just need to call the corresponding tag function, call it from here
					return _c_sHeadSlash;
				}else{
					return _c_sBody; // safest assunption
				}
			break;
			
		
			case _c_rBrk:
				if ($cp===_c_slash){
					return _c_sHeadToSolve;
				}else{
					return _c_sBody;
				}
			break;

			
			default:
				/**
					Not a space anymore --> need to analyse Previous state to know where we are, and adjust
					the present (now) state.
				*/
				switch ($stP){
					case _c_sHdLftBrk: 
						return _c_sHdTag;
					break;
						
					case _c_sHdTag: 
						return _c_sHdParVar;
					break;
						
					case _c_sHdParVar:
						if ($cn!==_c_equ){
							// Considers it as a Syntax pseudo Shortcut, but could be an error : missing the equal.
							$gotShortCut= true;
							return _c_sHdParVar; // SyntaxShortcut start a new parameter.
						}else{
							return _c_sHdParEqu;
						}
					break;
								
					case _c_sHdParEqu:
						if ($cn!==$glob->doubleQuote){
							$this->devErr($this->mMS().$glob->el('a header double quote is missing.','kmer'));
						}else{
							return _c_sHdParLftQ;
						}
					break;
													
					case _c_sHdParLftQ:
						return _c_sHdParVal;
					break;
								
					case _c_sHdParRgtQ:
						if ($cp===_c_slash){
							if ($cn===_c_rBrk){
								return _c_sHeadToSolve;
							}else{
								return _c_sTlSlash;		// end of the tag but deal with next call.
							}
							
						}else if($cn===_c_rBrk){
							return _c_sBody;
						}else{
							return _c_sHdParVar;	// another parameter variable.
						}
					break;
								
								
					case _c_sTlLftBrk:
						if ($cn===_c_slash){
							return _c_sTlSlash;		// terminal slash with a redondant preceding white.
						}else{
							$this->devErr($this->mMS().$glob->el('Tail left bracket without slash','kmes'));
						}
					break;
								
								
					default:
						$this->devErr($this->mMS().$glob->el('the state ','kmet').$stP.$glob->el(' does not exist!','kmeu'));
						return _c_sError;
				} // switch ($stP)

		} // switch ($cn)
	} // manSpace

	/**
		Error message prefix for manSpace.
	*/
	function mMS(){
		return 'tagsParser::manSpace xPetal Code : ';
	} // mMS
					
	
		
	/**
		Manages the Head parameter variable (so to get the name of the variable)
	*/	
	function manHdParVar(&$parVar,&$gotIt,&$gotShortCut,&$scanStr){
		$cn= $scanStr->cn;
		$cf= $scanStr->cf;
		$cp2= $scanStr->cp2;
		
		if ($cp2===_c_space || $cp2===_c_carRet  || $cp2===_c_nL  || $cp2===_c_rT ){
			// $this->cn already one step ahead because of white jumping managment until other chars, then advancing one more char, so need to catch up on the start.
			$parVar= $scanStr->cp;
		}
		
		switch ($cn){
			case _c_space:
			case _c_slash:
			case _c_rBrk:
				$gotIt= true;
				if ($cf!==_c_equ && $cf!==_c_space){
					$gotShortCut= true;
				}
				
				if ($cn===_c_space || $cn===_c_slash){		
					return _c_sSpace;
				}else{
					return _c_sBody;
				}
			break;
			
			case _c_equ:
				$gotIt= true;
				return _c_sHdParEqu;
			break;
						
			default:
				$gotIt= false;
				$parVar.= $cn; // build the parameter variable
				return _c_sHdParVar;
		} // switch
	} // manHdParVar
	
			
	/**
		$vectParVal[$iVar]
	*/	
	function manHdParEqu(&$scanStr,$tag,$iVar,$parValLast){
		$glob= &$this->glob;
		
		switch ($scanStr->cn){
			case _c_space:
				return _c_sSpace;
			break;
			
			case $glob->doubleQuote:
				return _c_sHdParLftQ;
			break;
			
			default:
				$glob->userError($glob->el('the tag ','katt').$tag.$glob->el(' after ','katv').$iVar.' -- '.$parValLast. $glob->el(' missing double quote at the start of a value.','katu'));
				return _c_sError;
		} // switch
	} // manHdParEqu
	
	
	/**
	*/	
	function manHdParLftQ(&$scanStr){
		$glob= &$this->glob;

		$cn= $scanStr->cn;
		if ($cn===$glob->doubleQuote){
			return _c_sHdParRgtQ;
		}else{
			return _c_sHdParVal;
		}
	} // manHdParLftQ
	
	
	/**
	*/	
	function manHdParVal(&$parVal,&$gotIt,&$scanStr){	
		$glob= &$this->glob;
		
		$cp2= $scanStr->cp2;
		$cp= $scanStr->cp;
		$cn= $scanStr->cn;
		
		if ($parVal==='' && $cp2===$glob->doubleQuote && $cp===$glob->doubleQuote){ // case of empty value
			$gotIt= true;	// Got empty value
			if ($cn===_c_rBrk){
				return _c_sHeadToSolve;
			}else{
				return _c_sHdParRgtQ;
			}
		}
		
		switch ($cn){
			case $glob->doubleQuote:
				if ($cp!==_c_bSh){
					$parVal.= $cp;
					$gotIt= true;
					return _c_sHdParRgtQ;
				}else{
					return _c_sHdParVal;
				}
			break;
			
			default:
				$parVal.= $cp; // build the parameter variable
				$gotIt= false;
				return _c_sHdParVal;
		} // switch
	} // manHdParVal
	
	
	
	/**
	*/	
	function manHdParRgtQ(&$scanStr){
		switch ($scanStr->cn){
			case _c_carRet:
			case _c_nL:
			case _c_rT:
			case _c_space:
				return _c_sSpace;
			break;
			
			case _c_rBrk:
				if($scanStr->cp===_c_slash){
					return _c_sHeadToSolve;
				}else{
					return _c_sBody;
				}
			break;
			
			case _c_slash:
				return _c_sHeadSlash;
			break;
						
			default:
				return _c_sHdParVar;
		} // switch
	} // manHdParRgtQ
	
		

} // tagsParser
?>
