<?php
/**
	File name : topHash.php
	Author of this file : Oscar Fawer until 2019-08-15
	Contact : www.ataox.org 
	Web sites : ataox.org / ataox.com / ataox.info
	Copyrights open source GPL, GNU General Public License V3 : see ./aMainDr/code/LICENSE.txt


	Top (used for global edition rights) circular crypt hash to be used for the first hash storing 
	process when the would be webmaster is still not detected as authorised. The circularity increment 
	happens when a new user gets in. Once the webmaster is allowed we shift the hash control directly 
	on the user and topHashTbl table (multi-users allowed with the same Id could be implemented with another hash table 
	as a prolongation for the user record).				
*/
class topHash extends oTop{
	

	function __construct(&$asa){
		parent::__construct();
		$this->glob= &$asa->glob;
		$this->acc= &$asa->acc;
		$this->caps= &$asa->caps;
		$this->tools= &$asa->tools;
		$this->hPwC= &$asa->hPwC;
		$this->aSql= &$asa->aSql;
		$this->dtb= &$asa->dtb;
			
	} // construct
	
	
	/**
		Creates a new top hash record
		Input:
			$ordId : ordinal is determined (outside this method) between 2 general sql variables (topHashMinId,topHashMaxId) and once chosen for a browser it stay on it (cookie).
			$hashId : real hash 20 chars Id.
			$userId : user id for eye ball check.
			$primH : primary hash.
			$ppH:: double primary hash for 1st inset.
	*/
	function newHashRecord($ordId,$hashId,$userId,$primH,$ppH){
		$dtb= &$this->dtb;
		
		$rec= array($ordId,time(),$hashId,$userId,'ko','',$ppH,'',$primH,'');
		$ee= $dtb->elementExists(_c_topHashOrdId,$ordId, _c_topHashTbl);
		if ($ee){
			$logged= $dtb->getElem(_c_topHashTbl, _c_topHashOrdId,$ordId, _c_topHashLog); // ok --> belonged to a logged user
		
			if ($logged===_c_ko){
				$dtb->deleteTableRecord(_c_topHashTbl,_c_topHashOrdId,$ordId);
			}else{
				return false;	// need to try the next id
			}
		}
		
		$flag= $dtb->newRecord(_c_topHashTbl,$rec);
		return true;
	} // newHashRecord
	
	
	/**
		Update the existing record, matching $hashId.
		Input:
			$ordId : ordinal is determined (outside this method) between 2 general sql variables (topHashMinId,topHashMaxId) and once chosen for a browser it stay on it (cookie).
			$hashId : real hash 20 chars Id.
			$userId : user id for eye ball check.
			$primH : primary hash.
			$ppH:: double primary hash for 1st inset.
	*/
	function updateHashRecord($ordId,$hashId,$userId,$primH,$ppH){
		$glob= &$this->glob;
		$dtb= &$this->dtb;
		
		$glob->flagUpdateHashRecord= true;	// just to have access to a better display when debugging
		
		// Update and shifts the hashes
		$cond= $dtb->condStr(_c_topHashOrdId,$ordId);
		$dtb->setElem(_c_topHashTbl,  _c_topHashUserId,$userId,  $cond);	// adapt the user Id each time, in case it changes
		
		
		// Shift actual storage into old
		$oldPH= $dtb->getElem(_c_topHashTbl, $cond,false, _c_topHashPH);
		$oldPPH= $dtb->getElem(_c_topHashTbl, $cond,false, _c_topHashPPH);
		
		$dtb->setElem(_c_topHashTbl,  _c_topHashOldPH,$oldPH,  $cond);
		$dtb->setElem(_c_topHashTbl,  _c_topHashOldPPH,$oldPPH,  $cond);
					
		
		// Update new storage
		$dtb->setElem(_c_topHashTbl,  _c_topHashPH,$primH,  $cond);
		$dtb->setElem(_c_topHashTbl,  _c_topHashPPH,$ppH,  $cond);
		
		$glob->flagUpdateHashRecord= false;		
	} // updateHashRecord	
	

	/**
		Increases the free top hash ordinal Id
	*/
	function incrFreeOrdId(){
		$dtb= &$this->dtb;
		
		$freeId= $dtb->getGenVar('topHashFreeId');
		$maxId= $dtb->getGenVar('topHashMaxId');
		
		if ($freeId>=$maxId){
			$freeId= $dtb->getGenVar('topHashMinId');
		}else{
			$freeId++;
		}
		$dtb->setGenVar('topHashFreeId',$freeId);	
	} // incrFreeOrdId
	

	/**
		Returns the free ordinal hash Id, with a loop test to avoid ecrasing
		still logged users.
	*/
	function freeOrdID(){
		$dtb= &$this->dtb;
		
		$freeId= $dtb->getGenVar('topHashFreeId');		
		$rec= $dtb->getElem(_c_topHashTbl, _c_topHashOrdId,$freeId);
		$i=0;
		while ( ($rec!==false && $rec[_c_topHashLog]==='ok')  && $i<10){
			$this->incrFreeOrdId();
			$freeId= $dtb->getGenVar('topHashFreeId');
			$rec= $dtb->getElem(_c_topHashTbl, _c_topHashOrdId,$freeId);
			$i++;
		}		
		return $freeId;
	} // freeOrdID
	
	
	/**
	*/
	function flagOkStatus($topHshId){
		$dtb= &$this->dtb;
		$dtb->setElem(_c_topHashTbl, _c_topHashLog,_c_ok, _c_topHashIdH,$topHshId);
	} // flagOkStatus
	
	
	/**
		Gets the ordinal Id from the hash, if the hash record does not exists  (erased in between etc) just creates it.
		gocOrdIdFromHash
	*/
	function getOrdIdFromHash($hash){
		$dtb= &$this->dtb;
		return $dtb->getElem(_c_topHashTbl, _c_topHashIdH,$hash, _c_topHashOrdId);
	} // getOrdIdFromHash
	

	/**
	*/
	function getSessionKey($hash){
		$dtb= &$this->dtb;
		return $dtb->getElem(_c_topHashTbl, _c_topHashIdH,$hash, _c_topHashC2Pw);		
	} // getSessionKey
	
	/**
		Directly from the cookie
	*/
	function getDfcSessionKey(){
		$tools= &$this->tools;
		$dtb= &$this->dtb;
		$hash= $tools->spcCookVal('ckTopHashId');	// real hash 20 chars
		return $dtb->getElem(_c_topHashTbl, _c_topHashIdH,$hash, _c_topHashC2Pw);		
	} // getDfcSessionKey
	
	
	/**
		Set the session key
	*/
	function setSessionKey($hash,$val){
		$dtb= &$this->dtb;
		return $dtb->setElemNoMlCd(_c_topHashTbl, _c_topHashC2Pw,$val, _c_topHashIdH,$hash);
	} // setSessionKey
	
		
	/**
		Removes the given record.
	*/
	function remove($ordId){
		$dtb= &$this->dtb;
		$dtb->deleteTableRecord(_c_topHashTbl, _c_topHashOrdId,$ordId);
	} // remove
	
	
	/**
		topHash::eliminatesOneDayOld()
	*/
	function eliminatesOneDayOld(){
		$aSql= &$this->aSql;
		
		$limit= time()-24*3600;
		$cond= $aSql->lessThan(_c_topHashDate,$limit);
		if ($cond!==''){
			// The test is an anti SQL injection scheme
			$aSql->deleteSqlTableRec(_c_topHashTbl,$cond);
		}
	} // eliminatesOneDayOld
	

} // topHash
?>
