<?php
/**
	File name : accents.php
	Author of this file : Oscar Fawer until 2019-08-15
	Contact : www.ataox.org 
	Web sites : ataox.org / ataox.com / ataox.info
	Copyrights open source GPL, GNU General Public License V3 : see ./aMainDr/code/LICENSE.txt


	Accents and other transformnation for different sort of storage or display compatibility.
	Remark :
		To save time hmtl code is stored directly inside the database.
		In many cases table convertion are prefered to fu such as strtolower() because accentuated characters are transformed into their non accentueated equivalent, preserving much of the readability.
		
		accents::backSlashC()		
*/
class accents extends oTop{
	

	function __construct(&$tools){
		parent::__construct();
		$this->tools= &$tools;

		$this->usesHtmlCode= false;

		/* 
			Special chars to eventually be simplified into their non accentuated equivalents 
			for resiatant (file, page names etc.).
		*/
		$this->allAccentsLow= array(
		'à','á','â','ä',
		'è','é','ê','ë',
		'ì','í','î','ï',
		'ò','ó','ô','ö',
		'ù','ú','û','ü');
			
		$this->allAccentsUp= array(
		'À','Á','Â','Ä',
		'È','É','Ê','Ë',
		'Ì','Í','Î','Ï',
		'Ò','Ó','Ô','Ö',
		'Ù','Ú','Û','Ü');
		
		$cedillaArray= array('ç','°','ß');	
		$this->upperAccents= array_combine ($this->allAccentsLow,$this->allAccentsUp);	// To Gives upper accents from lower accents
		$this->allAccents= array_merge($this->allAccentsLow,$this->allAccentsUp,$cedillaArray);
		$this->forSmplFnSymb= array('°','%','&','@','?','!');	// For simpler file names symbols
		$this->special= array_merge($this->allAccents,$this->forSmplFnSymb);

		// simple chars ...
		$this->noAccents= array(
		'a','a','a','a',
		'e','e','e','e',
		'i','i','i','i',
		'o','o','o','o',
		'u','u','u','u',

		'A','A','A','A',
		'E','E','E','E',
		'I','I','I','I',
		'O','O','O','O',
		'U','U','U','U','c','o','ss');
		
		$this->forSmplFnChars= array('o','pc','and','at','','');
		$this->simple= array_merge($this->noAccents,$this->forSmplFnChars);
		
		
		
		$this->html1= array(
		'&agrave;','&aacute;','&acirc;','&auml;',
		'&egrave;','&eacute;','&ecirc;','&euml;',
		'&igrave;','&iacute;','&icirc;','&iuml;',
		'&ograve;','&oacute;','&ocirc;','&ouml;',
		'&ugrave;','&uacute;','&ucirc;','&uuml;');
		
		$this->html2= array(
		'&Agrave;','&Aacute;','&Acirc;','&Auml;',
		'&Egrave;','&Eacute;','&Ecirc;','&Euml;',
		'&Igrave;','&Iacute;','&Icirc;','&Iuml;',
		'&Ograve;','&Oacute;','&Ocirc;','&Ouml;',
		'&Ugrave;','&Uacute;','&Ucirc;','&Uuml;',
		'&ccedil;','&deg;','&szlig;');
		
		$this->html= array_merge($this->html1,$this->html2);
	} // construct
	
	/**
	*/
	function addsFls(&$fls){
		$this->fls= &$fls;
	} // addsFls
	
	/**
	*/
	function setUsesHtmlCode($a){
		$this->usesHtmlCode= $a;
	} // setUsesHtmlCode
		
	/**
	*/
	function normStr(&$s){
		$s= str_replace('<br>',_c_Br,$s);	// for the niceEdit bug
		return $this->toHtmlCode($s);	// to store only the html format! So no accents	
	} // normStr
	
	/* ---------------------------------------------------------------------------------
		Dealing with quotes
	*/	
	
	/**
		Backslash the given char, so it can be capsulated between two of these chars.
		Especially used with : $this-> backSlashC($val,_c_dblQ).
		
			Input :
				$s : string to backslash.
				$c : backslashed character.
			
		Note : a very general way of solving the problem. \" between "" and then capsulated into
		'' will appear as \\" which looks weird but is correct, the \\ between '' having just to
		be interpreted as a standard char. Also \ are added explicitelly when included into "" ou ''
		for sql or php processing, they are removed atomatically with the sql or php interpretation.
		
		Remark : return addslashes($s); // Brute force on all is a catastrophy !!!
		
		// ------------------------------------------------
		$s= str_replace (_c_bkSlash, _c_dblBkSlash, $s);	// too systematic  --> useless doubles
		$s= str_replace ($c, _c_bkSlash.$c, $s);
		
		// -------------------------------------------------------
		// Error $c taken twice --> full crash with \' \" \\" \\''
		$s= str_replace (_c_bkSlash.$c, _c_dblBkSlash._c_bkSlash.$c, $s); 
		$s= str_replace ($c, _c_bkSlash.$c, $s);
	*/
	function backSlashC($s,$c='"'){
		// $s= str_replace (_c_bkSlash, _c_dblBkSlash, $s);	// should be done ONLY ONCE on the same string!!!
		// $s= str_replace ($c, _c_bkSlash.$c, $s);
		
		$s= $this->doubleBSlash($s);
		$s= $this->bSlashChar($s,$c);
		return $s;
	} // backSlashC
	
	/**
	*/
	function doubleBSlash($s){
		// print('doubleBSlash'._c_);
		return str_replace (_c_bkSlash, _c_dblBkSlash, $s);
	} // doubleBSlash
	
	/*
	*/
	function bSlashChar($s,$c='"'){
		return str_replace ($c, _c_bkSlash.$c, $s);
	} // bSlashChar
	
	
	/**
		Backslash, quotes and comma separates and concats all the elements of $v.
		$v : given associative array (vector), got to quote the values, separate them with , and return the concat string.
	*/
	function bSlhQuoteParams($v, $norm=false, $fullBS=true){
		$b= ''; // also foreach ($v as $key => $val)
		if (is_array($v)){
			reset($v);
			foreach ($v  as $val){
				if ($norm){
					$val= $this->normStr($val);
				}
				
				if ($fullBS){
					$b.= '"' . $this->backSlashC($val,_c_dblQ) . '",'; // ful operation
				}else{
					// neglect extra \ for undo / redo processes
					$b.= '"' . $this->bSlashChar($val,_c_dblQ) . '",'; 
				}
			}
			return rtrim($b,', ');	// removes extra comma and,or whites
		}else{
			$this->topErrAndDie('accents::bSlhQuoteParams : '.$v.' is not an array.');
		}
	} // bSlhQuoteParams	
	
	
	/**
	 	Only used inside the recovery.php class.
	 	Should have a thid param to fals only in 100%php mode otherwise \ can crash the cms !!!
	*/
	function bSlhQP($v,$norm=false){
		switch (_c_CurrentDtbAbstract){
			case 'simulSql':
				return $this->bSlhQuoteParams($v, $norm, false);
			break;
			
			case 'aMySql':
				return $this->bSlhQuoteParams($v, $norm, true);
			break;
			
			default: print('Error in accents::bSlhQP()');
		} // switch
		
	} // bSlhQP
	
	
	
	/**
	 	Special recovery
	function bSlhQP($v,$norm=false){
		$b= ''; // also foreach ($v as $key => $val)
		if (is_array($v)){
			reset($v);
			foreach ($v  as $val){
				if ($norm){
					$val= $this->normStr($val);
				}
				
				$b.= '"' . $this->bSlashChar($val,_c_dblQ) . '",';
			}
			return rtrim($b,', ');	// removes extra comma and,or whites
		}else{
			$this->topErrAndDie('accents::bSlhQP : '.$v.' is not an array.');
		}
	} // bSlhQP	
	*/
	
	
	
	
	
	
	/*
	function TESTbackSlashC($s,$c='"'){
		$s= $this->stringRepl(_c_bkSlash, _c_dblBkSlash, $s);
		$s= $this->stringRepl($c, _c_bkSlash.$c, $s);
		return $s;
	} // backSlashC
	*/
	
	/**
		Subset for str_replace($search,$replace,$subject)
		Search a character-byte $c and replace it by a string $r for the given string str

	function stringRepl($c,$r,$str){
		$sz= strlen($str);
		$n= '';	// new string
		for ($i=0; $i<$sz; $i++){
			$d= $str[$i];	// current char
			if ($d===$c){
				$n.= $r;
			}else{
				$n.= $d;
			}
		} // for
		return  $n;      
	} // stringRepl
	*/
		
	
	/**
	*/
	function toNoAccents($s){
		return str_replace($this->allAccents,$this->noAccents,$s);
	} // toSimpleAccents
	
	/**
		Mainly used to generate file,url,record names the most compatible
		with diverse automation algorithm (search engines, dtb record names etc.).
	*/
	function toSimpleChars(&$str,&$sz){
		$tools= &$this->tools;
		$vect= $tools->mbStrToAry($str,$sz);
		return $this->vectReplace($this->special,$this->simple,$vect);
	} // toSimpleChars
	
	
	/**
		Just get rid of appostrophy.
	*/
	function noAppostroph($s){
		return str_replace (_c_snglQ,' ',$s);
	} // codeGreatLess
	
	
	/* ------------------------------------------------------------------------------
	*/	
	function toHtmlCode($s,$usesHtDbQ=false){
		if ($s===false){
			return false;
		}
	
		if ($this->usesHtmlCode){
			$s= str_replace($this->allAccents,$this->html,$s);
		}
		
		if ($usesHtDbQ){
			return str_replace(_c_dblQ,_c_hDbQuote,$s);
		}else{
			return $s;
		}
	} // toHtmlCode

			
	/**
	*/
	function fromHtmlToAccents($s){
		return str_replace($this->html,$this->allAccents,$s);
	} // fromHtmlToAccents
			
	/**
		Warning : must stay totally independant from $this->usesHtmlCode) :
			* Because it's used in specific case of desired conversion
			* Must not be used for the core algorithm etc, Use toHtmlCode() instead!
	*/
	function fromAccentsToHtml($s){
		return str_replace($this->allAccents,$this->html,$s);
	} // fromAccentsToHtml

	
	/**
	*/
	function toSimpleAccents($s){
		return str_replace($this->html,$this->allAccents,$s);
	} // toSimpleAccents


	function addsHtmlBreaks($s){
		return str_replace(_c_nL, _c_nL._c_Br, $s);
	} // addsHtmlBreaks


	/**
		An equivalent of str_replace() for arrays.
		$this->vectReplace($searchV,$replaceV,$vect)
	*/
	function vectReplace(&$searchV,&$replaceV,$vect){
		reset($searchV);
		foreach ($searchV as $key=>$vlSrch){
			$vlRpl= $replaceV[$key];
			$this->vectChrReplace($vlSrch,$vlRpl,$vect);
		}
		return $vect;
	} // vectReplace

	/**
		An equivalent of str_replace() for arrays. (just one couple)
		Input : 
			$srch : search char.
			$rplc : replace char.
		Input / Replace :
			$vect : vector to change.
	*/
	function vectChrReplace($srch,$rplc,&$vect){
		reset($vect);
		foreach ($vect as $key=>$val){
			if ($srch===$val){
				$vect[$key]= $rplc;
			}
		}
	} // vectChrReplace


	/**
		Hyphen capsulate the given vector.
	*/
	function hyphenCaps(&$v){
		reset($v);
		foreach ($v as $key=>$val){
			$v2[$key]= '-'.$val.'-';
		}
				
		return $v2;
	} // hyphenCaps


	/*
		Allowed identificator chars : True if the given char belong to the allowed char set to create a 
		file/record/tag/variable name,and incidentally a good chunk of email characters!
	*/
	function allowedChar($c){
		return $this->idChar($c) || $c==='-';
	} // allowedChar

	/**
		Strict identificator chars.
	*/
	function idChar($c){
		return $this->idCoreChar($c) || $c==='_';
	} // idChar

	function idCoreChar($c){
		return ($c>='a' && $c<='z') || ($c>='A' && $c<='Z')  || ($c>='0' && $c<='9');
	} // idCoreChar


	/**
		Returns true if $s gots only allowed chars.
	*/
	function onlyAllowedChars($s){
		$n= strlen($s);
		$i=0;
		while($i<$n && $this->allowedChar($s[$i])){
			$i++;
		}
		return $i===$n;
	} // onlyAllowedChars


	/**
		Returns true if $s gots only allowed chars.
	*/
	function onlyIdChars($s){
		if (is_string($s)){
			$n= strlen($s);
		}else{
			$n= count($s);
		}
		
		$i=0;
		while($i<$n && $this->idChar($s[$i])){
			$i++;
		}
		
		// Total IF to allow easy warning / print if false !
		if ($i===$n){
			return true;
		}else{
			return false;
		}
	} // onlyIdChars


	/**
		Generates a character safe identifier (file/url/field/variable etc,) string from a given string. Admit "-"
	*/
	function safeIdStr($sOrig,$maxSz=_c_largestIdSz){
		$fls= &$this->fls;
		return $fls->safIdStr($sOrig, false, $maxSz);
	} // safeIdStr2

	
	/**
		Concats the parameters of $v without quotes and without bothering back-slashing things in
	*/
	function catParams($v){
		$b= ''; // also foreach ($v as $key => $val)
		reset($v);
		foreach ($v as $val)
			$b.= $val . ', ';

		return rtrim(trim($b),',');	// removes extra comma
	} // catParams	

			
	/**
		Returns an xml attribute string from a standardized $v
		Input :
			$v : to extract the parameters
			$quote : the sort of quote
			$bSlashExtrn : back slash external double quotes (don't want any inside!!!)
	*/
	function xmlAttributeStr(&$v, $quote=_c_dblQ, $bSlashExtrn=false){
		$buff= ''; // also foreach ($v as $key => $val)
		reset($v);
		foreach ($v as $key=>$val){
			if ($val!==false){
				$b= $key._c_equ;
				if ($bSlashExtrn){
					$b.= '\\'.$quote.$val.'\\'.$quote._c_space;
				}else{
					$b.= $quote . $this->backSlashC($val,$quote) . $quote._c_space;
				}
				
				$buff.= $b;
			}
		}
		
		return rtrim($buff,_c_space);	// removes extra space
	} // xmlAttributeStr
	

	/**
		Removes tabs, linefeeds and carriage returns
	*/
	function removeTabs($str){
		return  str_replace (_c_rT,'',$str);
	} // removeTabs

	/**
		Returns true if $c is an upper case without accent.
	*/
	function simpleUpCase($c){
	 	return ord($c)>=ord('A') && ord($c)<=ord('Z'); // return $c>='A' && $c<='Z';
	} // simpleUpCase
	
	function simpleLowCase($c){
	 	return ord($c)>=ord('a') && ord($c)<=ord('z'); // return $c>='A' && $c<='Z';
	} // simpleLowCase
	
	/**
	*/
	function emailChar($c){
		return $this->allowedChar($c) || $c=='@' || $c=='.';
	} // emailChar
	
	/**
		True if the string $s holds only characters allowed into an email address
	*/
	function emailChars($s){
		$sz= strlen($s);
		$i=0;
		while ($i<$sz && $this->emailChar($s[$i])){
			$i++;
		}
		return $i===$sz;
	} // emailChars
	
	/**
		Change the case of the beginning of words in a string (includiong 2 bytes accents)
	*/		
	function capitWordStart($str){
		$sz= strlen($str);
		$cn= _c_Rs;
		$cf= $str[0];
		$i=1; 
		while ($i<$sz){
			$cp= $cn;	// past byte
			$cn= $cf;	// now/present byte
			$cf= $str[$i];	// futur byte
		
			if ($cp===_c_Rs && $cn!==_c_Rs){
				// at the beginning of a word (tabs, enter etc forbidden)
				
				if ($this->simpleLowCase($cn)){
					$str[$i-1]= strtoupper($cn);
				}else if (in_array($cn.$cf,$this->allAccentsLow)){
					$upAcc= $this->upperAccents[$cn.$cf];
					$str[$i-1]= $upAcc[0];
					$str[$i]= $upAcc[1];
					$i++;
				}
				
			}
			
			$i++;
		} // while
		return $str;
	} // capitWordStart


	/**
		Extract the xml body in simple cases (won't work with multi brackets), when the xml capsule is just
		an annoying artefact.
	*/
	function simpleXmlBody($str){
		$sz= strlen($str);
		$grab= '';
		$startGrab= false;
		$keepGoing= true;
		
		$i=1;
		while ($i<$sz && $keepGoing){
			$c= $str[$i];
			
			if ($c===_c_lBrk){
				$startGrab=false;
				$keepGoing= false;
			}
						
			if ($startGrab){
				$grab.= $c;
			}			
			
			if ($c===_c_rBrk){
				$startGrab=true;
			}
			
			$i++;
		}
		
		return $grab;
	} // simpleXmlBody

} // accents
?>