<?php
/**
	File name : cncatFiles.php
	Author of this file : Oscar Fawer until 2019-08-15
	Contact : www.ataox.org 
	Web sites : ataox.org / ataox.com / ataox.info
	Copyrights open source GPL, GNU General Public License V3 : see ./aMainDr/code/LICENSE.txt


	Strategy : 
		- Considers only files to concat small enough to be read on one move (must fit into the machine VM).
		- Enormous .cat files are read small chunk after small chunk (stops just on file limit).
		- Writes chunk of data when bigger than given size or end is reached, limit always set between files.
		
	See :
		** Main concatenation functions
		function cocatFl($srcPath){
		function concatDirectory($srcPath,$catFilePath)

		** Main extraction functions
		function expandCatFile($catFileName,$whereToStoreIt=''){
		expandDirName
		addsVerbose
		***
		$this->expandedRoot : init by default with '.'
		expandFileData		

	Key words for web page distribution : data, file, string, concatenation, php + (pluriel & french)
*/
class cncatFiles{


	/*
		* Constructor *
		$lengthFileSzNb : standard number of digits==bytes to store the file size.
		$bufferSz : size of the longuest string (mainly used as file buffers), optimum size == 100000
	*/
	function __construct(&$asa, $lengthFileSzNb=10,$bufferSz=100000){
		$this->glob= &$asa->glob;
		$this->fls= &$asa->fls;

		define('_c_okArchOpen',true);		//	True if we want to really write something, false for testing wihtout writing
	
		$this->lastDataBlock= ''; // for debugging purpose
		
		$this->ecrasesF= isset($_POST['brEcrase']) && $_POST['brEcrase']=='ok';

		$this->hdConcatFileSignature= 'www.ziiiiipweb.xyConcatFormat';	// the concatenation signature
		$this->hdConcatFileSignaMask= '11110000000111100111111111111';	// a mask to compare only the 1 positions
		
		/*
			Length of the first number used to express the length of the numbers memorizing the concatenated file size.
			This number never change and means that we got $this->lengthFirstNb (4) digits to set the length of
			$this->lengthFileSzNb	--> the number specified can have up to 9'999 digits implying a
			maximum file size of 10^9'999 (totally out of reach!)
		*/
		$this->lengthFirstNb= 4;	// never change

		/*
			Given size of the string-number defining the length of directory-names and files.
		*/
		if ($lengthFileSzNb<9999)
			// standard file size==10 --> allows intake-file size up to 10 Gigabytes
			$this->lengthFileSzNb= $lengthFileSzNb;
		else
			$this->lengthFileSzNb= 9999;

		/*
			Maximum length in bytes of a given string holding file content == max size if the I/O file buffer
			test it with a small size, so we can test all cases.
			Optimum size, depending on machine (bigger better), for web sites server --> files rarely above 100kb
			--> optimum size == 100kb
		*/
		$this->maxLengthStr= $bufferSz; 
		$this->tstSffx= '';	// just to have an eyeball test
		$this->verbose= '';
		$this->verbOn= true;
	} // cncatFiles	* Constructor *

	/*
	*/
	function resetVerboseBuff(){
		$this->verbose= '';
	} // resetVerboseBuff

	/*
		Adds some verbose in the main verbose buffer ---> Hept as a debugger tool just un case!
	*/
	function addsVerbose($gStr){
		// if ($this->verbOn) $this->verbose.= $gStr._c_;
	} // addsVerbose

	/*
		Concat the given char $n times
	*/
	function concatNChar($gChar,$n){
		$r='';
		for ($i=0;$i<$n;$i++)
			$r.= $gChar;
		return $r;
	} // concatNChar

	/*
		Gives back a constant string length representation of a integer
		eg : 314 --> 0000314
		standard call : $this->integerToCstStrLength(314,$this->lengthFileSzNb)
		$gInt : given integer.
		$gSz : given size $gInt must fit in.
	*/
	function integerToCstStrLength($gInt,$gSz){
		$glob= &$this->glob;
		
		$cStr= (string)$gInt;	// calculated string
		$cSz= strlen($cStr);
		if ($cSz>$gSz){
			$glob->userError( $glob->el('One file is larger than the allowed limit of : ','kbaq'). $gSz . $glob->el(' bytes','kbar')  );
			exit();
		}else{
			$prefix0= $this->concatNChar('0',$gSz-$cSz);
			return $prefix0.$cStr;
		}
	} // integerToCstStrLength

	/*
		calculates the capsule label, allowing any content inside
		$dataType one char : 'D'==Directory, 'F'==File, 'A'==dAta
		$sizeData : the size of the data to be capsulised
	*/
	function calcCapsuleLabel($dataType,$sizeData){
		$cstIntStr= $this->integerToCstStrLength($sizeData,$this->lengthFileSzNb);
		return $dataType.$cstIntStr;
	} // calcCapsuleLabel

	/*
		Writes once the file Head : the signature and the length of the byte counter
	*/
	function writeSignCntHeader(){
		// Header Signature
		$buff= $this->hdConcatFileSignature;

		// Length File Byte Counters
		$buff.=$this->integerToCstStrLength($this->lengthFileSzNb,$this->lengthFirstNb).$this->tstSffx;

		fwrite($this->catFilePt,$buff);
	} // writeSignCntHeader

	/*
		Concat the current directory string
		* fixed length : 'D' . fixedSizeNumber
		* the path itself
	*/
	function concatCurDirStr($pathCurDir){
		$pathCurDir= substr($pathCurDir,$this->leftSubstractFromPath);
		$this->addsVerbose($pathCurDir);
		$buff= $this->calcCapsuleLabel('D',strlen($pathCurDir)).$this->tstSffx;
		$buff.= $pathCurDir.$this->tstSffx;
		fwrite($this->catFilePt,$buff);
	} // concatCurDirStr

	/*
		Concat the current file name
		* fixed length : 'F' . fixedSizeNumber
		* the file name itself
	*/
	function concatCurFileName($fileName){
		$this->addsVerbose('&nbsp;&nbsp;'.$fileName);
		$buff= $this->calcCapsuleLabel('F',strlen($fileName)).$this->tstSffx;
		$buff.= $fileName.$this->tstSffx;
		fwrite($this->catFilePt,$buff);
	} // concatCurFileName

	/*
		Concat the current data
		* fixed length : 'A' . fixedSizeNumber
		* the file data itself
	*/
	function concatCurFileData($filePathName){
		$maxLS= $this->maxLengthStr;
		$curFilesize= filesize($filePathName);

		// Calculates and write the file data capsule
		$buff= $this->calcCapsuleLabel('A',$curFilesize).$this->tstSffx;
		fwrite($this->catFilePt,$buff);

		$curFP= fopen($filePathName,'r');	// # open current file for reading

		$leftOver= $curFilesize;
		do{
			if ($leftOver<=$maxLS){
				$curLength= $leftOver;
				$leftOver=0;
			}else{
				$leftOver-=$maxLS;
				$curLength= $maxLS;
			}

			if ($curLength>0){
				$fileContent= fread($curFP,$curLength).$this->tstSffx;
				fwrite($this->catFilePt,$fileContent);	// # writes current file content
			}
		}while($leftOver>0);

		fclose($curFP);	// # closes current file
	} // concatCurFileData

	/*
	*/
	function concatCurFileNameAndData($fileName, $filePathName){
		$this->concatCurFileName($fileName);
		$this->concatCurFileData($filePathName);
	} // concatCurFileNameAndData

	/*
	*/
	function concatEndFlag(){
		$buff= $this->calcCapsuleLabel('E',strlen('End')).$this->tstSffx;
		$buff.= 'End'.$this->tstSffx;
		fwrite($this->catFilePt,$buff);
	} // concatEndFlag


	/*
		Does the concatenation.

		Directory paths are given without trailing '/'
		$srcPath : current source directory (first call : path of the directory to concat)
		$this->catFilePath : destination path of the concatenation algo : it's a global instance variable to
			not jam the recursive algo.

		All files of a given directory have a guaranty of being processed (--> stored) after their directory (no
		other directory stored between).

		links not processed for now :
		nb : bool is_file (filename), bool is_link (filename) 
	*/
	function cocatFl($srcPath){
		if ($handle= opendir($srcPath)){

			$this->concatCurDirStr($srcPath);

			// Scans all files of the current directory first
			while (false !== ($file = readdir($handle))){
				if ($file != '.' && $file != '..'){
					$srcPF= $srcPath.'/'.$file;
					if (!is_dir($srcPF)){
						// -- Deals with the current file
						$this->concatCurFileNameAndData($file,$srcPF);
					}
				}
			} // while

			rewinddir($handle);
			// Scans all directories of the current directory
			while (false !== ($file = readdir($handle))){
				if ($file != '.' && $file != '..'){
					$srcPF= $srcPath.'/'.$file;
					if (is_dir($srcPF))
						$this->cocatFl($srcPF);	// -- Go to the next directory
				}
			} // while

			closedir($handle);
		}
	} // cocatFl


	/* ----------------------------------------------------------------
		Concat the given directories and files
	*/
	function concatDirectory(&$dirs, &$files, $motherDir,$catFilePath,$verbOn=true){
		$glob= &$this->glob;
		$fls= &$this->fls;

		$this->resetVerboseBuff();
		$this->verbOn= $verbOn;
		
		$this->catFilePath= $catFilePath;
		if (file_exists($catFilePath)){
			$err= $glob->el('Sorry, the concatenated archive target file :','kbax');
			$err.= ' '.$catFilePath.' ';
			$err.= $glob->el('exists already!','kayy');
			$glob->userError($err);
			return;
		}

		$this->leftSubstractFromPath= strlen($motherDir);	// to not store the path calling part
		$this->catFilePt= fopen($catFilePath,'ab');	// open the output concatenation file
		$this->writeSignCntHeader();	// header

		if ($files!==false){
			reset($files);
			foreach ($files as $key=>$fileDFN){
				// Check that $file is given without the path (will work only for one side of browser)
				$fls->splitOnLastC($fileDFN,$left,$file);
				$this->concatCurFileNameAndData($file, $motherDir.$file);
			}
		}
		
		if ($dirs!==false){
			reset($dirs);
			foreach ($dirs as $key=>$dirFN){
				$fls->splitOnLastC($dirFN,$left,$dir);
				$this->cocatFl($motherDir. $dir);
			}
		}

		$this->concatEndFlag();
		fclose($this->catFilePt); // close the concatenation file
	} // concatDirectory


	/* ---------------------------------------------------------------------------
		Start of the deconcatenation methods.
	--------------------------------------------------------------------------- */

	/*
		Transforms a string with constant length (0 prefixes) into an integer
	*/
	function cstStrLengthToInteger($gStr){
		return (int)$gStr; // get automatically rid of z�ro prefixes  !!
	} // cstStrLengthToInteger


	/* ------------------------------------------------------------------------
		Returns :
			$dataType : the type of data following the capsule header (the capsulised info).
			$dataLength : the number of bytes of the capsulised info.
	*/
	function readCapsuleHeader(&$dataType,&$dataLength){
		$capsuleHeader= fread($this->catFilePt,$this->lengthFileSzNb+1);
		$dataType= $capsuleHeader[0];
		$dataLength= $this->cstStrLengthToInteger(substr($capsuleHeader,1));
	} // readCapsuleHeader


	/**
		Rem : the directory is created only if it does not exist already
		--> if it already exists, the old xrw mode is conserved !
	*/
	function expandDirName($dataLength){
		$fls= &$this->fls;
		
		$dirName= fread($this->catFilePt,$dataLength);

		if ($this->expandedRoot=='.'){
			// . --> default file, could leave it, but that way works even on OS with other convention for local file
			$this->curExpandedDir= $dirName;
		}else{
			$this->curExpandedDir= $this->expandedRoot.'/'.$dirName;
		}

		if (!file_exists($this->curExpandedDir)){
			if (_c_okArchOpen){
				$fls->wMkDir($this->curExpandedDir); // creates the directory with all permissions
			}
			
			$this->addsVerbose($dirName);
		}else{
			$this->addsVerbose($dirName.' already created');
		}
	} // expandDirName


	/*
	*/
	function expandFileName($dataLength){
		$fileName= fread($this->catFilePt,$dataLength);
		$this->curExpandedFN= $this->curExpandedDir.'/'.$fileName;
		$this->addsVerbose('&nbsp;&nbsp;'.$fileName); // if already exists --> Not going to be overwritten
	} // expandFileName
	
	
	/*
	*/
	function expandFileData($dataLength){
		$okToWrite= _c_okArchOpen && (!file_exists($this->curExpandedFN) || $this->ecrasesF);

		$maxLS= $this->maxLengthStr;

		if ($okToWrite){
			$curFPt= fopen($this->curExpandedFN,'wb');	// open the new file to be written
		}

		$leftOver= $dataLength;
		do{
			if ($leftOver<=$maxLS){
				$curLength= $leftOver;
				$leftOver=0;
			}else{
				$leftOver-=$maxLS;
				$curLength= $maxLS;
			}

			if ($curLength>0){
				$fileData= fread($this->catFilePt,$curLength);
				if ($okToWrite){
					fwrite($curFPt,$fileData);
				}
			}else{
				$fileData= '';
			}
		} while($leftOver>0);

		if ($okToWrite) fclose($curFPt);
		
		$this->lastDataBlock= $fileData; // for debugging purpose
	} // expandFileData


	/* -----------------------------------------------------------------------------------------------------
		Reads the archive corresponding to $catFileName and set it in place (eventual file ecrasement).
		Could probably speed up the algo with a big reading buffer (does it only if necessary!)
		$catFileName : the concatenated file itself, as a full filename
		$whereToStoreIt (no trailing /) : where to store the expanded files and directories, if nothing is specified '.',
			stores it on the level of the calling script.
	*/
	function expandCatFile($catFileName,$whereToStoreIt='.',$verbOn=true){
		$glob= &$this->glob;
		
		$this->resetVerboseBuff();
		$this->verbOn= $verbOn;
		$this->expandedRoot= $whereToStoreIt;
		$this->curExpandedDir= $whereToStoreIt;	// default for first files

		$this->catFilePt= fopen($catFileName, 'rb');

		$hdSignature= fread($this->catFilePt,strlen($this->hdConcatFileSignature));
		
		if (!$this->compWithMask($this->hdConcatFileSignature,$hdSignature,$this->hdConcatFileSignaMask)){	
			$glob->userError( $glob->el('The file can\'t be unconcatenated because it does not match the expected format','kbas') );
			fclose($this->catFilePt);
			return false;
		}

		$firstStr= fread($this->catFilePt,$this->lengthFirstNb);
		$this->lengthFileSzNb= $this->cstStrLengthToInteger($firstStr);


		/* ------------------------------
			Loops on capsules
		------------------------------ */
		$kepGoing= true;
		while ($kepGoing && !feof($this->catFilePt)) {
			$this->readCapsuleHeader($dataType,$dataLength);
			switch ($dataType) {
				// Directory
				case 'D':
				$this->expandDirName($dataLength);
				break;

				// File
				case 'F':
				$this->expandFileName($dataLength);
				break;

				// dAta
				case 'A':
				$this->expandFileData($dataLength);
				break;

				case 'E':
				// $kepGoing= false;
				break;


				default:
				print('Unknown data type : '.$dataType._c_);
				break;	
			} // switch
			
			
			if (_c_okArchOpen===false){
				print('$this->curExpandedDir='.$this->curExpandedDir._c_);
				print('$this->curExpandedFN='.$this->curExpandedFN._c_);
				print('$this->lastDataBlock='.$this->lastDataBlock._c_);
			}
				
		} // while
		fclose($this->catFilePt);
	} // expandCatFile


	/* -----------------------------------------------------------------------------------------------------
		Compares 2 strings using a 01 string mask, 1 does the comparison and 0 does not!
	*/
	function compWithMask($s1,$s2,$mask){
		$sz1= strlen($s1);
		$sz2= strlen($s2);
		if ($sz1===$sz2){
			$i=0;
			while($i<$sz1){
				if ($mask[$i]==='1'){
					if ($s1[$i]!==$s2[$i]){
						return false;
					}
				} // else don't want to compare or ok!
				$i++;
			}
			return true;
		}else{
			return false;			
		}
	} // compWithMask
	
	
	// ----------------------------------------------------->
	// Could substitute with standard ataox methods !!!!!!!!!
	
	
	/* ---------------------------------------------------------------------------------------
		Gives the position of the first slash met from the right of $gStr, starting at 1.
		If no slash if found --> returns strlen(original $gStr).
		All that should work with right removing and right taking used in splitFullName() 
	*/
	function rightPosSlash1($gStr){
		$lastI= strlen($gStr)-1;
		for ($i=$lastI;  $i>0 && $gStr[$i]!='/' && $gStr[$i]!='\\';  $i--);

		// right pos numerotation starts at one because of substr() definition
		if ($i==0)
			return strlen($gStr); 	// did not get a '/'
		else
			return $lastI-$i;		// got a '/', the loop so went one step further
	} // rightPosSlash1

	/* ----------------------------------------------------------------
		Splits the last file or dir name from its path.
		$gFullName --> $path . $fileName
		The split is done once the first '/' or '\' is found!
		$gFullName==$path.$fileName 	// $path got the slash
	*/
	function splitFullName(&$path,&$fileName,$gFullName){
		$lstChr= $gFullName[strlen($gFullName)-1];
		if ($lstChr=='/' || $lstChr=='\\')
			$gFullName= substr($gFullName,0,-1);	// removes last char if it's a slash

		$rps= $this->rightPosSlash1($gFullName);
		$path= substr($gFullName, 0, 0-$rps);	// removes $rps char from the right
		$fileName= substr($gFullName, 0-$rps);	// takes the $rps first chars from the right
	} // splitFullName
	
	
	
} // cncatFiles
?>