<?php
/**
	File name : aMySql.php
	Author of this file : Oscar Fawer until 2019-08-15
	Contact : www.ataox.org 
	Web sites : ataox.org / ataox.com / ataox.info
	Copyrights open source GPL, GNU General Public License V3 : see ./aMainDr/code/LICENSE.txt


	Abstract all mySqli functions used in Ataox as to allow :
		- Easy adaptation to switch to another database.
		- A shorter call syntax than the usual wordy database commands.
		- Future general search, see : aMySql::likeStr()		
*/
class aMySql extends oTop{
	
	/**
		$asa : ataox main class
		$dtbCmpl : complementary Dtb if it exists (symetry for simulSql, but not useful here).
	*/
	function __construct(&$asa,&$dtbCmpl,$dtbNme=_c_UsrDbName){
		parent::__construct();
		$this->acc= &$asa->acc;
		$this->glob= &$asa->glob;
		$this->tools= &$asa->tools;
		$this->fls= &$asa->fls;
		
		$this->dtbNme= $dtbNme;
		
		$this->msqlImp= mysqli_init();	// initialisation of the object MySql Improved.
	} // construct

	/**
		To differentiate it easily from its twin.		print( $this->identity() );
	*/
	function identity(){
		return 'aMySql';
	}
	

	/**
		Establish the MySql connection!
		
		Warning --> should :
			(1) establish the connection wihout specifying the database $defDtb=false
			(2) create the database function createDtb()
			(3) do a specific connection with function sqlSelectDb()
		
		Input :
			$server : address of the sql server, can be either a host name or an IP address
			$userName : user name for the sql server (the MySQL user name. )
			$dbPw : password for that username and naturally also to access the DTB.
			$defDtb : database name.or false for trials
		Output :
			Returns TRUE on success or FALSE on failure. 
	*/
	function sqlDbConnect($server, $userName, $dbPw, $defDtb=false){
		$sqli= &$this->msqlImp;
		
		if ($defDtb===false){
			return $sqli->real_connect($server, $userName, $dbPw);	// to set/test the connection
		}else{
			return $sqli->real_connect($server, $userName, $dbPw, $defDtb);
		}
	} // sqlDbConnect
	
	/**
	*/
	function close(){
		$sqli= &$this->msqlImp;
		return $sqli->close();
	} // close
	
	
	/**
		Return a string to describe the last DTB error.
	*/
	function lastDtbError(){
		$sqli= &$this->msqlImp;
		return $sqli->error;
	} // lastDtbError


	/**
		Runs and return the MySql query result for the query string $q.
		with high security for $glob->cndtWrRghts===false that in fact are the guest users.
		Returns FALSE on failure. success returns mysqli_result object
	*/
	function localQry($qStr,$onlyReading=true){
		if ($qStr!=='' && ($onlyReading || $glob->cndtWrRghts===true) ){
			$sqli= &$this->msqlImp;
			$b= $sqli->query($qStr);	// also return error codes
			if ($b===false){
				$this->treatSqlError($qStr);
			}
			return $b;
		}else{
			if ($glob->cndtWrRghts===true){
				$this->treatSqlError($qStr);
			}
			return false;
		}
	} // localQry
	

	/**
		Returns an associative array (a row) matching the query criteria, and position
		the pointor on the following row.
	*/
	function fetchArray(&$query){
		if (is_object($query)){
			$row= $query->fetch_assoc();	// !!!! risk of name confusion !!!!!
			if ($row===NULL){
				return false;
			}else{
				return $row;
			}
		}else{
			return false;
		}
	} // fetchArray
	
	
	/**
	*/
	function sqliError(){
		$sqli= &$this->msqlImp;
		$err= $sqli->error;
		print('mysqli_error() $err='.$err._c_);
	} // sqliError	

	
	/**
		Select the current database.
	*/
	function sqlSelectDb($dbName){
		$sqli= &$this->msqlImp;
		return $sqli->select_db($dbName);
	} // sqlSelectDb	


	/**
		Returns the (equality) condition string.
		condStr / assignStr	--> separeted for theoretical purposes.
	*/
	function basicCond($field,$val,$oper='='){
		$acc= &$this->acc;
		if ($field===''){
			return '';	// --> no conditions so return the whole table!
		}else{
			if (!$this->secureSqlId($field)) return '';
			return $field.$oper.'"'.$acc->backSlashC($val,_c_dblQ).'"';
		}
	} // basicCond
	

	/*
		Full condition for a symetry with the class simulSql
	*/
	function fullCond($field,$val,$oper){
		return $this->basicCond($field,$val,$oper);
	} // fullCond
	
	/**
		Nothing to do, jus keep $condition as it is.
	*/
	function transForMultiplCond($condition){
		return $condition;
	} // transForMultiplCond
	
	

	/**
	*/
	function splitAssignation($set, &$field,&$val){
		$fls= &$this->fls;
		$fls->splitOnFirstC($set,$field,$val);
	} // splitAssignation
	

	/**
	*/
	function valFromLocator($where){
		$fls= &$this->fls;
		$fls->splitOnFirstC($where,$left,$right);
		return $right;
	} // valFromLocator
	
	
	/**
		Adds one more and condition (sql inject secure dealt with at the condStr() level !!)
		The nmethod parameters have been adapted so it's compatible for stacking strings opr vectors (simulSql)
		so returns the full result, and can be repeated if necessary!
	*/
	function andCondition(&$a,$b){
		$a.= ' AND '.$b;
	} // andCondition

	
	/**
		Not used for now. Need top sql inject checks!
	*/
	function vectorAnd(&$vCond){	
		$totCondStr= current($vCond);
		reset($vCond);
		foreach ($vCond as $i=>$curConStr){
			if ($i!==0){
				$totCondStr.= $this->andCondition($curConStr);
			}
		}
		return $totCondStr;
	} // vectorAnd
	

	/**
		Pure set methods defined as set['field']= value
	*/
	function emptyPureSet(){
		return '';
	} // emptyPureSet
	
	function assignSetElement($field,$val=false){
		return $this->assignStr($field,$val);
	} // assignSetElement

	function buildPureSet(&$curSet, $newElem){
		if ($curSet===''){
			$curSet= $newElem;
		}else{
			$curSet.= ', '.$newElem;
		}
	} // buildPureSet
	

	/**
		Normalises the values strings of a set
	*/
	function normSetStr(&$s){
		$s= $this->acc->normStr($s);
	} // normSetStr

	
	/**
	*/
	function treatSqlError($qStr){
		$glob= &$this->glob;
		if ($glob->allowsSqlErrors){
			$glob->userError('aMySql::query='.$qStr);
		}
	} // treatSqlError


	/**
		Return true if the parameter is a broad (definition) identificator --> '-' admited but not contiguous --
		Rem : mainly does the job upstream!
	*/
	function secureSqlId($s){
		$tools= &$this->tools;
		
		if ($s===''){
			return false;
		}
		
		// Total IF to allow easy warning / print if false !
		if ( !$tools->sIsInB('--',$s) && $this->almostSafeSqlChars($s) ){
			$bool= true;
		}else{
			$bool= false;
		}
		
		if (!$bool){
			$fls= &$this->fls;
			$fls->appendOnLogFile($s,'secureSqlId');
		}
		
		return $bool;
	} // secureSqlId
	

	/**
		Strict number string!
	*/
	function strictNb($s){
		$tools= &$this->tools;
		$bool= $tools->onlyDigits($s);
		
		if (!$bool){
			$fls= &$this->fls;
			$fls->appendOnLogFile($s,'strictNb');
		}
		return $bool;
	} // strictNb
	
	
	/**
	*/
	function onlyIdChars($s){
		$acc= &$this->acc;
		$bool= $acc->onlyIdChars($s);
		
		if (!$bool){
			$fls= &$this->fls;
			$fls->appendOnLogFile($s,'onlyIdChars');
		}
		return $bool;
	} // chabl


	/**
		Returns true if $s gots only safe sql chars.
	*/
	function almostSafeSqlChars($s){
		$n= strlen($s);
		$i=0;
		while($i<$n && $this->safeSqlChar($s[$i])){
			$i++;
		}
		return $i===$n;
	} // almostSafeSqlChars
	
	
	/**
		Safe character so can't build any meanningfull sql instruction with that!
		Rem : used at the top for $_GET and $_COOKIE.
		In fact ALMOST safe, becazuse don't want double -
	*/
	function safeSqlChar($c){
		return ($c>='a' && $c<='z') || ($c>='A' && $c<='Z')  || ($c>='0' && $c<='9') || $c==='_' || $c==='-' || $c==='/' || $c==='.';
	} // safeSqlChar
	

	/**
	*/
	function setTagFu(&$tgFu){
		$this->tgFu= &$tgFu;	// don't need it up to now, keep it for compatibility with php dtb
	} // setTagFu

	/**
		Got a symetrical function in simulSql.
		For now just does nothing.
	*/
	function updateData(){}	
	
	
	// Not included in non SQL part !!!!
	function lessThan($field,$val){
		if (!$this->secureSqlId($field)) return '';
		if (!$this->strictNb($val)) return '';
		return $field.' < '.$val;
	} // lessThan
	
	
	/**
		Return a Mysql assignment string.
	*/
	function assignStr($field,$val=false){
		if ($val===false){
			/* to bypass the assignation when the fully assigned string is already available
			   but faster to solve it at the above level. */
			return $field;
		}else{
			$acc= &$this->acc;
			if (!$this->secureSqlId($field)){
				return '';
			}
			return $field.'="'.$acc->backSlashC($val,_c_dblQ).'"';
		}
	} // assignStr
	

	/**
		A method just here for compatibility with simulSql(). Condition Key Val to 
		String. Because all conditions calculated upware are strings, just got
		to return it.
	*/
	function condKyVlToStr(&$str){
		return $str;
	} // condKyVlToStr
	

	/**
		array mysqli_result::fetch_fields ( void )
		Input :
			$q : query result
	*/
	function fetchFields(&$q){
		if ($q===false){
			return false;
		}
	
		$fieldV= array();
		$i= 0;

		$objV= $q->fetch_fields();
		reset($objV);
		foreach ($objV as $key=>$obj){
			$fieldV[$i]= $obj->name;
			$i++;
		}
		
		if ($i>0){
			return $fieldV;
		}else{
			return false;
		}
	} // fetchFields
	
	
	/**
	*/
	function selectResource($table,$condition='',$column='*',$orderColN='',$curOrder=false,$limit=false){		
		$q= $this->selectStr($table,$condition,$column,$orderColN,$curOrder,$limit);
		return $this->localQry($q);
	} // selectResource

		
	/* ---------------------------------------------------------------------------------------------
		Gives the maximum value of $colName for the given table.
	*/
	function maxColValue($tableName,$colName){	
		$acc= &$this->acc;
		
		if (!$acc->onlyIdChars($tableName)) return 0;
		if (!$acc->onlyIdChars($colName)) return 0;
		
		$selector= 'max('.$colName.')';
		$q= 'select '.$selector.' from '.$tableName;
		$qObj= $this->localQry($q);
		$res= $this->fetchArray($qObj);		
		$maxVal= $res[$selector];
		if ($maxVal!==NULL){
			$mv= $maxVal;
		}else{
			$mv= 1000;	// default maximum, in fact first value !!
		}
		return $mv;
	} // maxColValue

	
	/**
		Counts the number of rows where the concerned $fields got the concerned $val.
		Probably not very fast, scan the whole table, so at some stage, may have to store these number
		somewhere and increment them each time a new post is written!
	*/
	function cntNbRows($tblN,$fields,$val){
		$acc= &$this->acc;
		
		if (!$acc->onlyIdChars($tblN)) return 0;
		if (!$acc->onlyIdChars($fields)) return 0;
		if (!$this->secureSqlId($val)) return 0;	// restriction on what sort of values !! ok for now !!
		
		$cntCond= 'COUNT(*)';
		$qStr= 'SELECT '.$fields.', COUNT(*) FROM '.$tblN.' WHERE '.$fields.'="'.$val.'"';
		$qObj= $this->localQry($qStr);
		$arr= $this->fetchArray($qObj);
		return $arr[$cntCond];
	} // cntNbRows	

	/**
		Count the number of rows for the given table name.
	*/
	function countNbRows($tblN){
		$acc= &$this->acc;
		if (!$acc->onlyIdChars($tblN)) return 0;
		
		$cntCond= 'COUNT(*)';
		$str= $this->selectFrom($tblN,$cntCond);		
		$q= $this->localQry($str);
		if ($q!==false){
			$arr= $this->fetchArray($q);
			return $arr[$cntCond];
		}else{
			return 0;
		}
	} // countNbRows
	
	
	/**
	*/
	function queryNbRows(&$q){
		if ($q!==false){
			return $q->num_rows;
		}else{
			return 0;
		}
	} // queryNbRows
	
	

	/**
		SqlAntiInject = OK
	*/
	function selectFrom($table,$x='*'){
		$acc= &$this->acc;
		if (!$acc->onlyIdChars($table)) return '';
		return 'SELECT '.$x.' FROM '.$table;
	} // selectFrom
	
	
	/**
		Returns the query to select From the specified Table. Returns an associative array 
		depending on the conditions imposed on the specified table.
			$table : the table name
			$condition : the condition(s) to select specific rows
			$column : the column(s) we want to extract information from (usually all=*)
			$orderColN : the name of the column to order
			
		Returns the resource matching the select query string
		SqlAntiInject = OK
	*/
	function selectStr($table,$condition='',$column='*',$orderColN='',$curOrder=false,$limit=false){
		$acc= &$this->acc;
		
		$q= $this->selectFrom($table,$column);
		if ($condition!==''){
			// first param hard coded and second param inside quote
			$q.= ' WHERE '.$condition;
		}
					
		if ($orderColN!==''){
			if (!$acc->onlyIdChars($orderColN)) return '';
		
			$q.= ' ORDER BY '.$orderColN;
			
			if ($curOrder){
				$q.= ' ASC';
			}else{
				$q.= ' DESC';
			}
				
			if ($limit!==false){
				if (!$acc->onlyIdChars($limit)) return '';
				$q.= ' LIMIT '.$limit;
			}
		}
		
		return $q;
	} // selectStr
	
	
	/**
		Returns a mysql query object form $start within the $span scope.
			LIMIT : prioritar operator !!!!
		SqlAntiInject = OK
	*/
	function limitedRowSet($table,$start,$span, $condition='',$column='*',$orderColN='',$curOrder=false){
		$acc= &$this->acc;
		$tools= &$this->tools;
		
		$q= $this->selectStr($table,$condition,$column);
		if ($start!=='all'){
			$start= (string)$start;
			$span= (string)$span;
		
			if (!$tools->onlyDigits($start)) return '';
			if (!$tools->onlyDigits($span)) return '';
		
			$q.= ' LIMIT '.$start.', '.$span;
		}
		
		
		if ($orderColN!==''){
			if (!$acc->onlyIdChars($orderColN)) return '';
		
			$q= '('.$q.') ORDER BY '.$orderColN;
			
			if ($curOrder){
				/* 1st choice standard alphab. order is ASCemdind because values keep going higher, even if inside
				   a classical dictionary we DESCend when reading it! */
				$q.= ' ASC';
			}else{
				$q.= ' DESC';
			}
		}
		
		return $this->localQry($q);
	} // limitedRowSet
	

	/**
		Standard Sql Insert String. Outside calls just used for dumping.
		Rem : does not uses the fields specification, so easier to abstract into another database model.
			Input :
				$tblN : name of the table.
				$val : vector of values that need to be concatenated.
				$where : if false, got to backslash the quotes because we are not in a recovery case where bS was done at storage time.
			Output :
				The sql string.
		SqlAntiInject = OK
	*/	
	function insertQueryStr ($tblN,&$val, $where=false){
		$acc= &$this->acc;
		$tools= &$this->tools;
		
		if (!$acc->onlyIdChars($tblN)) return '';
		$vGiv= true;
		if (is_array($val)){
			$val= $acc->bSlhQuoteParams($val);	// buils a string of back slashed quoted parameters separated by ,
		}else{
			// rare : does not happend at standard installation
			$vGiv= false;
			if ($where===false){
				$val= '"'.$acc->backSlashC($val,_c_dblQ).'"';
			}else{
				// recovery case, back slashing was done at the string sql storage time
			}
		}
		
		return 'INSERT INTO '.$tblN.' VALUES ('.$val.')';
	} // insertQueryStr

	
	/**		
		Returns the query to create a new record inside the specified table of the current database :
			$tblN : the table to add a row to.
			$val : values separated by , either associated with respective field or the whole bunch
			$where : supp information only used inside recovery.php
		SqlAntiInject = OK
	*/
	function newSqlRec($tblN,&$valV,$where=false){
		$q= $this->insertQueryStr($tblN,$valV,$where);	
		return $this->localQry($q,false);
	} // newSqlRec
		
			
	/**
		Returns the query to change the specified record at the specified place
			$tbl : the table name to add a row to.
			$set : column names to be changed with assigned value a=2, b=3, ...
			$where : where the change needs to be done.
		SqlAntiInject = OK
	*/
	function chgSqlRec($tbl,$set,$where){
		$acc= &$this->acc;
		
		if (!$acc->onlyIdChars($tbl)) return false;
		$q= 'UPDATE '.$tbl.' SET '.$set.' WHERE '.$where;
		return $this->localQry($q,false);
	} // chgSqlRec
		
	/**
		Returns the query to return the vector of table names
		SqlAntiInject = OK
	*/
	function tblNames($db=_c_UsrDbName){
		$acc= &$this->acc;
		if (!$acc->onlyIdChars($db)) return false;
		$q= 'SHOW TABLES FROM ' . $db;
		return $this->localQry($q);
	} // tblNames
	

	/**
		Returns true if the given table exists inside the current database.
		Warning : time consuming, should not be calld for each calls !!!
	*/
	function tableExists($tblN,$dtbN=_c_UsrDbName){
		$tools= &$this->tools;
		$v= $this->onlyTblNames($dtbN);
		return $tools->aIsInV($tblN,$v);
	} // tableExists
	

	/**
	*/
	function onlyTblNames($dbName=_c_UsrDbName){
		$result= $this->tblNames($dbName);
		
		$v= array();
		$i= 0;
		while ( $row= $this->fetchArray($result) ){
			$v[$i]= current($row); // first element of row
			$i++;
		}
		return $v;
	} // onlyTblNames
	
	
	/**
		Returns the query to return the vector of columns names
		SqlAntiInject = OK
	*/
	private function tblColNames($tbl,$db=_c_UsrDbName){
		$acc= &$this->acc;
		
		if (!$acc->onlyIdChars($tbl)) return false;
		if (!$acc->onlyIdChars($db)) return false;
		$q= 'SHOW COLUMNS FROM ' . $tbl . ' FROM ' . $db;		
		return $this->localQry($q);
	} // tblColNames	

			
	/**
		Returns a vector holding all the column names of the table.
		Can be used to display and edit tables.
		Input :
			$tbl : table name
			$db : database name
			
		Similar to the deprecated mysql_list_fields
	*/	
	function tblColumnNames($tbl,$db=_c_UsrDbName){
		$q= $this->tblColNames($tbl,$db);
		if ($q!==false){
			$i=0;
			while ($row= $this->fetchArray($q)) {
				$names[$i]= $row['Field'];
				$i++;
			}
			return $names;
		}else{
			return false;
		}		
	} // tblColumnNames
	
	
	/**
		Returns true if the field exists inside the given table for the specified or default database
	*/
	function fieldExists($fieldN,$tblN,$dtbN=_c_UsrDbName){
		if ($this->tableExists($tblN,$dtbN)){
			$colNmV= $this->tblColumnNames($tblN,$dtbN);
			if (in_array ($fieldN,$colNmV)){
				return true;
			}else{
				return false;
			}
		}else{
			return false;
		}
	} // fieldExists
	
	
		
	/**
		Print all the table $tbl types informations
	*/	
	function pTblTypes($tbl,$db=_c_UsrDbName){
		print('## Table name = ' . $tbl . _c_);
	
		$q= $this->tblColNames($tbl,$db);
		while ($row= $this->fetchArray($q)) {
			$this->pv($row);
		}
		print(_c_.'#####################'._c_);
	} // pTblTypes	
	
		
	/* ---------------------------------------------------------------------------------------------
		Delete a table record (or many) matching the $colName $val equivalence,
		return True on success and false on error!
			Input :
				$tblName, $colName, $val (coord of the element to delete)
				$loadsRecTbl : true if allowed to load the recovery table, so just there to forbid memorising recovery when dealing with the rec-table
		SqlAntiInject = OK
		
		TRUNCATE TABLE `captEnglish` 
	*/
	function deleteSqlTableRec($tblName,$where){
		$acc= &$this->acc;
		if (!$acc->onlyIdChars($tblName)) return false;
		
		$b= 'DELETE FROM '.$tblName.' WHERE '.$where;
		return $this->localQry($b,false);
	} // deleteSqlTableRec
	

	/**
		SqlAntiInject = OK
	*/
	function truncTbl($tblN){
		$acc= &$this->acc;
		if (!$acc->onlyIdChars($tblN)) return false;
		$str= 'TRUNCATE TABLE '.$tblN;
		return $this->localQry($str,false);
	} // truncTbl
	
	
	/**
		Returns the 1st record matching the condition as an associative array. But ...
			
	*/	
	function selectOneRecord(&$r,$table,$condition='',$column='*',$orderColN='',$descendent=false){
		$r=$this->selectResource($table,$condition,$column,$orderColN,$descendent);
		if ($r!==false){
			return $this->fetchArray($r);	// Advance already from one step !!!
		}else{
			return false;
		}	
	} // selectOneRecord
	
		
	/**
		Return true if a record exists with the given condition(s).
		Input :
			$table : tabla name
			$condition : condition sql string
			$column : column sql string
			
			$givenId : only for case comparison purpose
	*/
	function recordExists($table,$condition='',$column='*', $givenId=false){	
		$glob= &$this->glob;

		$b= $glob->allowsSqlErrors;
		$glob->allowsSqlErrors= false;	// Do not want to display any error because SQL errors are normal with this sort of test
		
		$res= $this->selectOneRecord($r,$table,$condition,$column);
		if ($givenId!==false){
			// Supplementary case test on the Id to short circuit the MySql laxism on cases
			$extractedId= $res[_c_genId];
			$ret= $extractedId===$givenId;	// case dependant
		}else{
			if ( $res===false ){
				$ret= false;
			}else{
				$ret= true;
			}
		}
		
		$glob->allowsSqlErrors= $b;
		return $ret;
	} // recordExists
	
	/**
		... and return the found record with $r
	*/
	function recExistAndRetRec(&$r,$table,$condition='',$column='*'){
		$exist= $this->recordExists($table,$condition,$column);
		if ($exist){
			$r= $this->selectOneRecord($qObj,$table,$condition);
		}else{
			$r= false;
		}
		return $exist;	
	} // recExistAndRetRec
	
	
	
	/**
		For a given Query return a vector holding all the values of the specified field : 
		$field, for the given ressource $resQ.
			Input :
				$resQ : result query
				$field	: concerned field.
			Output : return $vect
	*/
	function queryColVect($resQ,$field){
		$vect= false;
		$r=0;
		while( $row= $this->fetchArray($resQ) ){
			if ($field===false){
				$vect[$r]= $row;
			}else{
				$vect[$r]= $row[$field];
			}
			$r++;
		}
		
		return $vect;
	} // queryColVect
	
	
	/**
	*/
	function colKeyVect($resQ,$keyField,$returnedField=false){
		$vect= false;
		while( $row= $this->fetchArray($resQ) ){
			$r= $row[$keyField];
			if ($returnedField===false){
				$vect[$r]= $row;
			}else{
				$vect[$r]= $row[$returnedField];
			}
		}
		return $vect;
	} // colKeyVect
	
	
	/**
		Get the length of the given field of the given resource.
			Rem : mysql_field_len()  ... give the maximum size of the field!!
			
		Input:
			$rsr : computed sql resource.
			$idx : ordinal (0 start) index of the concerned field.
			$field : not used, just for compatibility with other databases.
	*/
	function sizeElement(&$rsr, $idx,$field=''){
		$v= $rsr->lengths;	// mySqli !!
		return $v[$idx];
	} // sizeElement
	
		
	/**
		Create the specified database with the ecurrent mySql connection 
		Returns FALSE on failure. success returns mysqli_result object
	*/
	function createDtb($name){
		$acc= &$this->acc;
		if (!$acc->onlyIdChars($name)) return false;
		return $this->localQry('CREATE DATABASE IF NOT EXISTS '.$name, false);
	} // createDtb


	/**
		Search enhancement :
	*/

	/**
		Returns a vector of strings inside quotes, the rest being returned as a rest string $restStr
		
		Warning : \" inside the quote is not treated (but ok in the case of local search interogation) !!!!
		
			Input :
				$mainStr : string to analyse
				$trim : if true --> trims every element
			Output :
				$vect : vector holding everythings inside the quotes
				$szVect : size of $vect
				$restStr : everything out of quotes
	*/
	function insideQuotesVect(&$vect,&$szVect,&$restStr,$mainStr,$trim=true){
		$inQ= false;
		$restStr= '';	// string left over (all th eparts out of quotes)
		$curElem= '';	// current element
		$j= 0;	// index of block of quotes "quoted pack or string"
		
		$sz= strlen($mainStr);
		
		// loop on the full string
		for($i=0; $i<$sz; $i++){
			$c= $mainStr[$i];
			if ($c==='"'){
				$inQ= !$inQ;
				
				if ($inQ){
					if ($curElem!=''){
						$vect[$j]= trim($curElem);
						$j++;
						$curElem='';	// so can cope with the string concat process
					}
				}
			}else{
				// not ON a quote, so can sum the strings
				if ($inQ){
					$curElem.= $c;
				}else{
					$restStr.= $c;
				}
			}
		} // for
		
		if ($curElem!=''){
			$vect[$j]= trim($curElem); // trim last element
			$szVect= $j+1;
		}else
			$szVect= $j;
	} // insideQuotesVect


	/**
	*/
	function databasesVect(){
		$tblV= array();
		$qObj= $this->localQry('SHOW DATABASES');
		
		if ($qObj!==false){
			$i= 0;
			while ($row= $qObj->fetch_row()){
				$tblV[$i]= current($row);
				$i++;
			}
		}else{
			print('Error in aMySql::databasesVect');
		}
		
		return $tblV;
	} // databasesVect
	
	
	/**
	*/
	function centPcPhpDtbV(){
		$fls= &$this->fls;
		$fls->firstLevelDir($tblV,_c_ProjectsPath,_c_zCmsDirN.'/zPhpData/phpDtb/cellPge');
		return $tblV;
	} // centPcPhpDtbV

	
	/**
	*/
	function curDtbName(){
		return $this->dtbNme;
	} // curDtbName
	

	/**
		Input : $w , the word (needle) to search
		SqlAntiInject = OK
		
		Need a special sql parameter stabilisation for general search, or could remove
		or backslash internal % :(probably works like "")
	*/
	function likeStr($w){
		if (!$this->secureSqlId($w)){	// Better than back slashing quotes. Just do not accept any !!!
			return '';
		}
		return ' LIKE "%'.$w.'%"';
	} // likeStr
	
	/**
	*/
	function likeAndStr($w){
		return $this->likeStr($w).$this->andCondition();
	} // likeAndStr
	
	
	/** ----------------------------------------------------------------
		Return the query result (arr of arr)  for a given :
			table, field, dataBase always default here 100%() 
			where we have a match with $needel
			
		form : SELECT * FROM genVar WHERE str LIKE "%NeedlE%"
	*/
	function needleMatch($needelStr, $tableNm, $fieldNm, $dataBase='phoney'){
		$queryStr= $this->selectStr($tableNm,$fieldNm).$this->likeStr($needelStr);
		return $this->localQry($queryStr);
	} // needleMatch
	
	
	/**
		Transforms an input string (using intuitive/raw/Google-like search conditions) into a MySql search string.
			Input :
				$colName : name of the concerned column
				$searchStr :
					' ' space = AND
					" ..." = treated in one block
					(no OR for now)
			Output :
				returns the mySql compatible string			
				$vTot : the vector of separate searched elements for eventual highlighting.
			
		When used need an attentive look at backslashing !!
	*/
	function rawToSqlSearchStr($colName,$searchStr,&$vTot){
		$elem= '';
		$this->insideQuotesVect($vTot,$szQ,$restStr,$searchStr); 
		$restStr= trim($restStr);
		
		if ($restStr!=''){
			$vR= explode(' ',$restStr);
			$szR= sizeof($vR);
		}else{
			$szR= 0;
		}
		
		// Adds the rest
		for($i=0; $i<$szR; $i++){
			$vTot[$szQ+$i]= trim($vR[$i]);
		}
		
		// Concat $vTot in AND LIKE elements
		$totStr= '';
		$szT= sizeof($vTot);
		
		for($i=0; $i<$szT-1; $i++){
			$elem= $vTot[$i];
			if ($elem!=''){
				$totStr.= $colName.$this->likeAndStr($elem);
			}
		}
		
		$elem= $vTot[$szT-1];
		if ($elem!=''){
			$totStr.= $colName.$this->likeStr($elem);
		}else
			$totStr= substr($totStr,0,-5); // substract the redondant " AND "
			
		return $totStr;
	} // rawToSqlSearchStr


	/**
		As rawToSqlSearchStr but does not bother with $vTot.
	*/
	function rToSqlSearchStr($colName,$searchStr){
		return $this->rawToSqlSearchStr($colName,$searchStr,$vTot);
	} // rToSqlSearchStr

} // aMySql
?>
