<?php
/**
	File name : simulSql.php
	Author of this file : Oscar Fawer until 2019-08-15
	Contact : www.ataox.org 
	Web sites : ataox.org / ataox.com / ataox.info
	Copyrights open source GPL, GNU General Public License V3 : see ./aMainDr/code/LICENSE.txt


	Uses a 100% Php paradigm with a complete simulation of the aMySql class.
		
	Possibilities to speed it :
		The strategy of internal condition loop of simulSql::deleteSqlTableRec(...) is more efficient than
		the strategy of external condition loop of simulSql::selectResource(...) but it does not matter much 
		because most conditions vectors have only a single element, otherwise it could be an interesting acceleration.
	
	Key functions :	
		simulSql::newSqlRec()
		simulSql::rowToFile()
		simulSql::chgSqlRec()

*/
class simulSql extends oTop{

	
	/**
		$asa : ataox main class
		$dtbCmpl : complementary Dtb if it exists. Rem : if it's given non null,  it's means to be used (see class frameBuilder)
		Input :
			$asa : pointor in the main class
			$dtbCmpl : a real localDB complement to do some reading from a frame builder call
			$dtbNm : a pseudo database bame being the access path to the 100% php database
	*/
	function __construct(&$asa,&$dtbCmpl,$dtbNme=_c_phpDtb){
		parent::__construct();
		$this->dtbCmpl= &$dtbCmpl;
		$this->asa= &$asa;
		$this->acc= &$asa->acc;
		$this->glob= &$asa->glob;
		$this->tools= &$asa->tools;
		$this->fls= &$asa->fls;	
		$fls= &$this->fls;
		
			
		if ( !defined('_c_sqlsColNmPrfx') ){
			define('_c_sqlsColNmPrfx', 'col_');		// sql simulation column name prefix for the php array variable
			define('_c_sqlsArrTblNms', 'tblNms');	// instance names of the tables name vector
		}

		$this->CurrentPhpDtbPath= $dtbNme;
		$curPhpDbInf= $dtbNme.'info/';
		$curSqlsColNmDFN= $curPhpDbInf.'colNames.php';
		$curSqlsTblNmsDFN= $curPhpDbInf.'tblNames.php';

		if ($dtbCmpl===null){
			include $curSqlsTblNmsDFN;	// All available names of the possible tables
		}else{
			$sqlCmpl= &$dtbCmpl->aSql;
			$this->sqlCmpl= &$sqlCmpl;
			$this->tblNms= $sqlCmpl->onlyTblNames(_c_UsrDbName);	// origin Dtb name (not used if $dtbCmpl is 100% php)		
		}
		
		$this->seedsFullTblV();	// Seed the full table arrays with the proper type!
		
		
		if ( $dtbCmpl!==null ){
			$this->allColNamesVect(_c_UsrDbName);	// the model is the actually running data base!
		}else{
			include $curSqlsColNmDFN;	// All column or fields names
		}
		
		$this->fullyActivatedTables= array(); // to monitor when tables are fully activated
	} // construct
	
	
	
	/*
		Basic condition with default = operator
		Input :
			...
			$oper is just there to be compatible with the class aMySql!
	*/
	function basicCond($field,$val,$oper='='){	
		if ($field===''){
			return '';	// --> no conditions so return the whole table!
		}else{
			return array($field => $val);
		}
	} // basicCond

	/*
		Full condition with any operator
	*/
	function fullCond($field,$val,$oper){
		$basicCond= $this->basicCond($field,$val);
		return new fullCnd($basicCond,$oper);
	} // fullCond
	
	/**
		Transforms the condition into an array of arrays.
	*/
	function transForMultiplCond($condition){
		return array(0 => $condition);
	} // transForMultiplCond
	
	
	
	/**
		Pushes the 2nd vector on the first.
		Vector of conditions are considered logical 'AND' by default
	*/
	function andCondition(&$v,$w){
		array_push($v,$w);
	} // andCondition
	

	/**
		To differentiate it easily from its twin
	*/
	function identity(){
		return 'simulSql';
	} // identity
	
	
	/**
		Phoney database.connection.		
	*/
	function sqlDbConnect($server, $userName, $dbPw){
		return true;
	} // sqlDbConnect
			
	/**
		// Select the current database.
		Idem sqlDbConnect(). nothing to do.
	*/
	function sqlSelectDb($dbName){
		return true;
	} // sqlSelectDb
	
	/**
	*/
	function close(){
		return true;
	} // close
	


	/**
		Also see function queryColVect($resQ,$field){
	*/
	function localQry(){
		return $this->compulsorySqlQuery();
	} // localQry

	/**
		Secial method for a query that is not shortcircuitable to some more php like instructions instead of SQL,
		sot not possible to get one level before the call, for example when sql instructions are diretly executed
		from the frameBuilder.
	*/
	function compulsorySqlQuery(){
		$glob= &$this->glob;
		return $glob->userError('simulSql::compulsorySqlQuery : can\'t use a real SQL query in the simulated SQL mode.');
	} // compulsorySqlQuery	

	
	/* ---------------------------------------------------------------------
		Returns the condition string.
	*/	

	/**
	*/
	function splitAssignation($set, &$field,&$val){
		$field= key($set);
		$val= current($set);
	} // splitAssignation
	

	/**
	*/
	function valFromLocator($where){
		return current($where);
	} // valFromLocator
	
	
	/**
		Takes the "and" condition into account from a vectorial form.
			Input:
				$vCond : vector of condition pairs
	*/
	function vectorAnd(&$vCond){
		return $vCond; // just got to return the value, because multiple conditions are automatically considered as an and
	} // vectorAnd

	/**
		Pure set methods defined as set['field']= value
	*/
	function emptyPureSet(){
		return array();
	} // emptyPureSet	
	
	function assignSetElement($field,$val){
		return array($field => $val);
	} // assignSetElement

	function buildPureSet(&$curSet, $newElem){
		$curSet[ key($newElem) ]= current($newElem);
	} // buildPureSet


	/**
		Normalises the values strings of a set
	*/
	function normSetStr(&$set){
		$acc= &$this->acc;
		reset($set);
		foreach ($set as $key=>$val){
			$set[$key]= $acc->normStr($val);
		}
	} // normSetStr

	
	
	
	/**
		Return true if the table was fully activated!
	*/
	function tblFullyActivated($tblN){
		return array_key_exists($tblN,$this->fullyActivatedTables);
	} // tblFullyActivated
	
	
	/**
	*/
	function tableVector(){
		$vN= _c_sqlsArrTblNms;
		return $this->{$vN};
	} // tableVector
	
		
	/**
		Returns the query to return the vector of table names
	*/
	function tblNames($db='phoney'){
		return $this->tableVector();
	} // tblNames
	
	
	/**
		Just there to be a symetric for the aMySql original function
	*/
	function onlyTblNames($dbName='phoney'){
		return $this->tableVector();
	} // onlyTblNames
	

	/**
		Seeds the full table arrays with the proper type!
		--> if the table exist as a full file loads it straight away !!  --> acceleration !!
	*/
	function seedsFullTblV(){
		$fls= &$this->fls;
		reset($this->tblNms);
		foreach ($this->tblNms as $k=>$tblN){
			$this->{$tblN}= array();
		}
	} // seedsFullTblV
	
	
	/**
		Directly loads the colNames vector.
		Transformed from mysqlToPhp::colNamesPhpStr()
	*/
	function colNamesPhpVect($tblName,$dbName){
		$sqlCmpl= &$this->sqlCmpl;
		
		$curVectNm= _c_sqlsColNmPrfx.$tblName;
		$colNmPhpStr= '';
		$totColNames= $sqlCmpl->tblColumnNames($tblName,$dbName);
		if ($totColNames===false){
			return false;
		}
		
		$i=0;
		reset($totColNames);
		foreach ($totColNames as $key=>$value){
			$this->{$curVectNm}[$i]= $value;
			$i++;
		} // foreach
	} // colNamesPhpVect
	
	
	/**
		Transformed from mysqlToPhp::dumpAllColNames()
	*/	
	function allColNamesVect($dbName){
		$sqlCmpl= &$this->sqlCmpl;

		$buff= '';	
		$v= $sqlCmpl->onlyTblNames($dbName);
		reset($v);
		foreach ($v as $key=>$tblName){
			$this->colNamesPhpVect($tblName,$dbName);
		}
	} // allColNamesVect

	
	/**
		Just for compatibility with the other, to test SQL data reliability, so don't care --> always true!
	*/
	function secureSqlId($s){
		return true;
	} // secureSqlId
	
	function secuSqlId($s){
		return true;
	} // secuSqlId
	
	function strictNb($s){
		return true;
	} // strictNb
	
	
	/**
		Set $this->tgFu + instanciates the class sqlToPhp() as a standard explicit class,
		so if we call it with a tag, it's directly available.
	*/
	function setTagFu(&$tgFu){
		$this->tgFu= &$tgFu;
		$tgFu->dtbCmpl= &$this->dtbCmpl;	 // update the complementary Dtb in case it's not null
		
		$tgFu->instExplitCl(_c_plugCntrDN.'mysqlToPhp/','mysqlToPhp');
		$this->sqlToPhp= &$tgFu->explicitClasses['mysqlToPhp'];
	} // setTagFu


	/**
		Returns a Mysql assignment string.
	*/
	function assignStr($field,$val=false){
		if ($val===false){
			// to bypass the assignation when the fully assigned array is already available
			return $field;
		}else{
			return array($field => $val);
		}
	} // assignStr
	

	/**
		Transform conditionaly a condition or assignement string into an array.
		If it's an assignment, some "=" can also be inside quotes, so can't just
		explode and trim.
	*/
	function condStrToVect($condStr){
		$fls= &$this->fls;
		
		if ( is_array($condStr) ){
			return $condStr;
		}else if ( $condStr==='' ){
			return array();		
		}else{
			$fls->splitOnFirstC($condStr,$left,$right,'=',_c_dblQ);
			return array($left=>$right);
		}
	} // condStrToVect

	
	/**
		Condition (the condition is dependant on the class instanciated and supporting
		this method) Key Val to String : used for condition to be stored in the recovery table, so need to be sure this in
		a string form. In fact because all conditions calculated upwhere are vectors, we just
		need to transform the vector key and val into a string.
		Only where string on internally screened names, so don't need to backslash + 
		always initialised.
	*/
	function condKyVlToStr(&$vect){
		if (!is_array($vect)){
			return $vect;
		}
	
		$val= reset($vect);
		$key= key($vect);
		return $key.'="'.$val.'"';
	} // condKyVlToStr
	
	
		/**
		Called inside [dspBuilderr /]  ( frameBuilder::tg_dspbuilderr() )
		To do a global saving of the 100% php database at the end of the installation process!
	*/
	function updateData($secondaryDtb){
		$asa= &$this->asa;
		$tgFu= &$asa->tagsFu;
		$fls= &$asa->fls;
		
		$tgFu->dtbCmpl= &$this->dtbCmpl;	// update it, very likely not to be null, so usefullness to do it after setTagFu()

		$tgFu->instExplitCl(_c_plugCntrDN.'mysqlToPhp/','mysqlToPhp');
		$this->sqlToPhp= &$tgFu->explicitClasses['mysqlToPhp'];
		$sqlToPhp= &$this->sqlToPhp;
	} // updateData

	

	/**
		Transforms a set of comma separated double quoted strings ($set) into its assoc. array match.
		Need it only when retrieving pairs from the recovery tables, because  the $set param 
		of chgSqlRec() is only a binome already on a vectorial form! Of course must refrain 
		from using n-tuples!
		
		Input:
			$setStr : a group of double quoted strings (for the recov. table : matching a whole record).
			$tblN  : the name of the implicated table, so can retrieve the column names as indexes.
	*/
	function setStrToVect(&$setStr,$tblN){
		$glob= &$this->glob;
		$tools= &$this->tools;
				
		$values= $this->newRecStrToVect($setStr, $leftOv);	// values ---> all values between quotes.
		$keys= $this->columnVect($tblN);
		if ($keys!==false){
			return array_combine($keys,$values);
		}else{
			return false;
		}
	} // setStrToVect


	/**
		Previously named currentRowAndNext()
		Give the current row and go to the next element, if there is no more next element return false.
			Input :
				$v : 	in std sql a vector of rows. 
						in simulated sql could be just a simple array
			Output :
				RETURNS THE CURRENT ROW.
	*/
	function fetchArray(&$v){
		$glob= &$this->glob;

		if ($v===false){
			return false;
		}

		$cur= $glob->currentNext($v);
		if ($cur!==false){
			return $cur;	// returns the current row, or value of a given vector element.
		}else{
			return false;
		}
	} // fetchArray
	
	/**
		Return an array holding all the fields of the given query result (array of arrays)
			Input :
				$v : query result
	*/
	function fetchFields(&$v){
		if ($v===false){
			return false;
		}

		$fieldV= array();
		$row= current($v);
		reset($row);
		$i= 0;
		foreach ($row as $key=>$val){
			$fieldV[$i]= $key;
			$i++;
		}
	
		if ($i>0){
			return $fieldV;
		}else{
			return false;
		}
	} // fetchFields
	


	/**
		From a complete $row, returns only the specified columns!
		Remark : initially programmed for all * or a single column (forumm proofing --> need general case)
	*/
	function columnFilter(&$row,$col){
		$glob= &$this->glob;
		$tools= &$this->tools;
		
		if ($col==='*'){
			return $row;
		}else{
			$newRow= array();
			if (is_array($row)){
				if ($tools->sIsInB(',',$col)){
					$elemV= $tools->trimStrToVect(',',$col);

					reset($elemV);
					foreach ($elemV as $key=>$elem){
						if (array_key_exists($elem,$row) ){
							$newRow[$elem]= $row[$elem];
						}
					}
				}else{
					if ( array_key_exists($col,$row) ){
						$newRow[$col]= $row[$col];
					}
				}
			}else{
				return false;
			}
		}
		return $newRow;
	} // columnFilter
	
	
	/**
		Select a specific table row, so in 2nd generation can open the correct file straigh away!
	*/
	function specificTableRow($tablN,$value,$column){
		$fls= &$this->fls;
	
		if ( $this->atomFLoadsGlobalAr($tablN,$value) ){
			return $this->columnFilter($this->{$tablN}[$value],$column);
		}else{
			return false;
		}
	} // specificTableRow
	 				

	/**
		Returns the query result (full fledged array of rows) to select From the specified Table. 
		depending on the conditions imposed on the specified table.
			$tblNmOrResFed : the table name or a processesd array ($step>1)
			$condVect : the vectorial condition or empty array
			$column : the column(s) we want to return, (usually all=*)
			$step : actual step index (0 as an integer)
			$oper : given operator.
					
		Remark : in this php-dtb version, the 2 last param are not used! Ok, because up to
		now not used in the other version too.
	*/
	function selectKernel(&$tblNmOrResFed,$condVect,$column,$step,$oper){
		$glob= &$this->glob;
		$tools= &$this->tools;
		
		$i= 0;
		$condKey= key($condVect);
		$condValue= current($condVect);
		
		if ($condKey===_c_genId && $oper==='L'){
			$switchKey= 'any'; // necessary for a broad search.
		}else{
			$switchKey= $condKey;
		}
		
		// Chooses the selection method along the different conditions :
		switch ($switchKey){			
			case _c_genId:
				// Unique id/index, so only one row! --> Only works for equality conditions !! But ok, because Ataox does not treat inequ on main Id!
				$result[$i]= $this->specificTableRow($tblNmOrResFed,$condValue,$column);
				if ($result[$i]!==false){
					$i++; // so can use it globally
				}
			break;
		
			case null;
			case _c_emptyStr:
				// Unconditional, takes all rows.
				if ($step===0){
					$resultFed= &$this->{$tblNmOrResFed};		// (first call got table name) in original case FULL table vector !!!
				}else{
					$resultFed= $tblNmOrResFed;		// 2nd + call an array of array having passed N+1 filters
				}
				if ($resultFed===null){return false;}
				
				reset($resultFed);
				foreach ($resultFed as $curKey=>$curRow){
					$result[$i]= $this->columnFilter($curRow,$column);
					$i++;
				}
			break;
				
			// Any column !!!					
			case 'any':
			default:
				// Condition on any other column, must filter
				if ($step===0){
					$this->fullTableFromAtomFiles($tblNmOrResFed);
					$resultFed= &$this->{$tblNmOrResFed};		// (first call got table name) in original case FULL table vector !!!
				}else{
					$resultFed= $tblNmOrResFed;		// 2nd + call an array of array having passed N+1 filters
				}
				
				if ($resultFed===null || $resultFed===false){
					return false;
				}
				
				reset($resultFed);
				foreach ($resultFed as $curKey=>$curRow){
					if (array_key_exists($condKey,$curRow)){
						switch ($oper){
							case '=':
								if ($curRow[$condKey]===$condValue){
									$result[$i]= $this->columnFilter($curRow,$column);
									$i++;
								}
							break;
							
							case '>':
								if ($curRow[$condKey]>$condValue){
									$result[$i]= $this->columnFilter($curRow,$column);
									$i++;
								}
							break;
							
							case '<':
								if ($curRow[$condKey]<$condValue){
									$result[$i]= $this->columnFilter($curRow,$column);
									$i++;
								}							
							break;
							
							
							case 'L':
								// rough belonging (sql LIKE)
								$hayStack= $curRow[$condKey];
								$needle= $condValue;
								if ($tools->sIsInIB($needle,$hayStack)){
									$result[$i]= $this->columnFilter($curRow,$column);
									$i++;
								}
							break;
							
							default: print('unknown operator in simulSql::selectKernel');
						} // switch
					}
				}
			break;
		} // switch
		
		if ($i===0){
			return false;
		}else{
			return $result;
		}
	} // selectKernel
	

	/**
		simulSql::selectResource
			Input :
				$tableN : table names
				$cnd : a condition vector OR string with equal in the middle OR an empty string meaning take it all!
				...
			Output :
				$tableN : table name
				$cnd : condition
				Query result array of rows!
	*/	
	function selectResource($tableN,$cnd='',$column='*',   $orderColN='',$descendent=false){
		$glob= &$this->glob;
				
		if ($cnd===''){
			$gotTbl= $this->fullTableFromAtomFiles($tableN);
			if ($gotTbl){
				$result= $this->{$tableN};	// full table
			}else{
				$result= false;	// error wrong table name
			}
		}else{
			$step= 0;
			$resultFed= $tableN;
			if ($glob->vectOfVect($cnd)){
				$result= false;
				
				/**
					With a single condition, the loop content is executed only once !!
				*/
				$sz= count($cnd);
				reset($cnd);
				foreach ($cnd as $key=>$curCond){
					$this->sepCondOper($curCnd,$oper, $curCond);
					$result= $this->selectKernel($resultFed,$curCnd,$column,$step,$oper);	// search on the full table
					$step++;
					if ($step<$sz){		// so don't assign the last one, that would not be used!
						$resultFed= $result;
					}
				}
			}else{
				$this->sepCondOper($curCnd,$oper, $cnd);
				$curCnd= $this->condStrToVect($curCnd);	// could be eliminated, all condition should be array at this stage
				$result= $this->selectKernel($resultFed,$curCnd,$column,$step,$oper);
			}
		}
		
		if ($result!==false && $orderColN!==''){
			// Build the sort associate vector
			reset($result);
			foreach ($result as $rKey=>$row){
				$this->sortAV[$rKey]= substr($row[$orderColN],0,128);
			}
		
			if ($descendent){
				uksort($result, array($this,'alphab'));
			}else{
				uksort($result, array($this,'revAlphab'));
			}
		}
		
		return $result;
	} // selectResource
	
	
	/**
		Partial simulation of the original function.
	*/
	function limitedRowSet($tblNm,$start,$span, $condition='',$column='*',$orderColN='',$descendent=false){
		if (!isset($this->{$tblNm})){
			return false;
		}
		
		$this->sortAV= array();	// sorting aid vector
		
		if ($condition!==''){
			$condTblArr= $this->selectResource($tblNm,$condition);
			if ($condTblArr===false){
				return false;
			}
		}else{
			$condTblArr= &$this->{$tblNm};
		}
		
		$newTbl= false;
		$i= 0;
		$max= $start+$span;
		reset($condTblArr);
		foreach ($condTblArr as $key=>$curV){
			if ($i>=$start){
				if ($i<$max){
					$newTbl[$i]= $curV;
					if ($orderColN!==''){
						// for sorting purpose build a parallel assoc vect holding the sorting related value
						$this->sortAV[$i]= substr($curV[$orderColN],0,128);
					}
				}else{
					break;	// out of the loop
				}
			}
			$i++;
		} // for
		
		if ($orderColN!=='' && $newTbl!==false){
			if ($descendent){
				uksort($newTbl, array($this,'alphab'));
			}else{
				uksort($newTbl, array($this,'revAlphab'));
			}
		}
		
		return $newTbl;
	} // limitedRowSet	
		
	/**
		Input :
			$this->sortAV : parallel associative vector to compare easily the given elements.
	*/
	function alphab($k1,$k2){
		$a= $this->sortAV[$k1];
		$b= $this->sortAV[$k2];
		return strcasecmp($a,$b);
	}
	
	function revAlphab($k1,$k2){
		return -$this->alphab($k1,$k2);
	}
		
	
			
	/**
		Sepaartes the given condition (having implicit or explicit operator) into 
			Inout: $gCond
			Output : 
				$c, array(field => value, see basicCond()
				$o	operator ... a string (for now a single character) to natch the operator.
	*/
	function sepCondOper(&$c,&$o,  &$gCond){
		if (is_object($gCond)){
			$c= $gCond->basicCond;
			$o= $gCond->oper;
		}else{
			$c= $gCond;
			$o= '=';
		}
	} // sepCondOper
	
		
	/**
	*/
	function selectOneRecord(&$r,$tableN,$condition='',$column='*',$orderColN='phoney',$descendent='phoney'){
		$glob= &$this->glob;
	
		$r= $this->selectResource($tableN,$condition,$column);
		if ($r!==false){
			return $this->fetchArray($r);
		}else{
			return false;
		}	
	} // selectOneRecord
	
			
	/**
		Return true if a record exists with the given condition(s). If somethiong must be done
		after with the resource or the record, selectOneRecord() must be used directly.
	*/
	function recordExists($tableN,$condition='',$column='*', $givenId=false){
		// nb : in this case $r is just a phoney variable!
		return $this->recExistAndRetRec($r,$tableN,$condition,$column);
	} // recordExists
	
	/**
		... and return the found record with $r
	*/
	function recExistAndRetRec(&$firstRec,$tableN,$condition='',$column='*'){
		if (!isset($this->{$tableN})){
			return false;
		}	
	
		$firstRec=  $this->selectOneRecord($r,$tableN,$condition,$column);
		if ($firstRec===false ){
			return false;
		}else{
			return $firstRec;
		}
	} // recExistAndRetRec
	
		
	/**
		Standard Sql Insert String. Outside calls just used for dumping.
		Need to be kept for backup specification.
			Input :
				$tblN : name of the table.
				$val : vector of values that need to be concatenated.
				$where : for compatibility with aMySql
	*/	
	function insertQueryStr ($tblN,&$val,$where=false){
		if (is_array($val)){
			$val= $this->acc->bSlhQuoteParams($val);
		}
		return 'INSERT INTO '.$tblN.' VALUES ('.$val.')';
	} // insertQueryStr
		
	/**	
		In the SQL-standard version : calling $this->insertQueryStr (...) --> 
		In this case creates the new record inside the specified table of the current database.
			$tblN : the table to add a row to.
			$valV : Only the whole bunch of values (simpler, but must maintain compatibility for that class to run).
			$where : for compatibility with aMySql
	*/
	function newSqlRec($tblN,&$valV,$where=false){
		$glob= &$this->glob;
		
		$properKeyNames= false;		
		if (is_array($valV)){
			if ( reset($valV) === _c_genId ){
				// takes only the given key names and forget the others.
				$properKeyNames= true;
			}
		}else{
			$valV= $this->setStrToVect($valV,$tblN);
			$properKeyNames= true;	// the proper key names have already been loaded in the assoc array $valV
		}
				
		if ($valV!==false){	
			$id= reset($valV);
			$this->storeNewRecord($tblN,$valV,$properKeyNames);
		}else{
			return false;
		}
	} // newSqlRec
	
	
	/**
		Returns true if the field exists inside the given table for the specified or default database
	*/
	function fieldExists($fieldN,$tblN,$dtbN='phoney'){
		if ($this->tableExists($tblN,$dtbN)){
			$colNmV= $this->tblColumnNames($tblN,$dtbN);
			if ($colNmV!==false && in_array ($fieldN,$colNmV)){
				return true;
			}else{
				return false;
			}
		}else{
			return false;
		}
	} // fieldExists
			
				
	/* ---------------------------------------------------------------------------------------------
		Delete a table record (or many) matching the $colName $val equivalence,
		return True on success and false on error!
			Input :
				$tblN, $colName, $val (coord of the element to delete)
				$condV : condition simple sql string, vector or vector of vector if more than one condition-
	*/
	function deleteSqlTableRec($tblN,$condV){
		$glob= &$this->glob;
		$fls= &$this->fls;
		
		$isCondAr= is_array($condV);	// true if $condV is a conditional array
		
		$vofv= false;
		if ($glob->vectOfVect($condV)){
			$vofv= true;
		}

		if ( is_object($condV) || $isCondAr && is_object(reset($condV)) ){
			// Condition with object, relying on the main select for deletion! Slower but not so much to write!
			$result= $this->selectResource($tblN,$condV,_c_genId);
			while( $row= $this->fetchArray($result) ){
				$this->unsetAndDeleteRow($tblN,$row[_c_genId]);
			}
			return;
		}
		
		$ky= false;
		if (!$vofv && $isCondAr){
			$ky= key($condV);
			$vl= current($condV);
		}else if ($vofv===false ){
			$fls->splitOnFirstC($condV,$ky,$vl,'=','"');
		}
		
		if ($ky===_c_genId){
			// Main Id so only one row concerned, so no loop !
			$this->unsetAndDeleteRow($tblN,$vl);
		}else{
			// Need to find the correct record with a scan
			$i= 0;
			reset($this->{$tblN});
			foreach ($this->{$tblN} as $key=>$row){
				if ($vofv){
					if ($this->condVectOfVect($row,$condV)){
						$rmId[$i]= $row[_c_genId];	// stores the Ids to remove.
						$i++;
					}
				}else{
					if ($this->condVect($row,$condV)){
						$rmId[$i]= $row[_c_genId];	// stores the Ids to remove.
						$i++;
					}
				}
			}
			
			/*
				Remark : would be faster with direct deletion just above, but :
					- Not deleting much.
					- Got similaritie with the above $this->fetchArray() 
			*/
			if ($i>0){
				// Removes all the found ids
				reset($rmId);
				foreach ($rmId as $j=>$id){
					$this->unsetAndDeleteRow($tblN,$id);
				}
			}
		}
	} // deleteSqlTableRec
	

	/**
		Return true if the given $row (an array of arrays) match all the conditions of $condV. !! only AND conditions by default
		Input :
			$row : the current row to analyse
			$condV : a vector of vector is the conditon AND array holding at least on pair $ky / $vl
		Output :
			Return true if the condition(s) are met
	*/
	function condVectOfVect(&$row,&$condV){
		$result= true;
		reset($condV);
		foreach ($condV as $ky=>$cdV){
			$cdKey= key($cdV);
			$cdValue= current($cdV);
		
			if ( array_key_exists($cdKey,$row) && $row[$cdKey]===$cdValue ){
			}else{
				$result= false;
				break;
			}
		}
		return $result;
	} // condVectOfVect


	function condVect(&$row,&$condV){
		$result= true;
		reset($condV);
		foreach ($condV as $ky=>$vl){
			if ( array_key_exists($ky,$row) && $row[$ky]===$vl ){
			}else{
				$result= false;
				break;
			}
		}
		return $result;
	} // condVect
	
	
	/**
	*/
	function truncTbl($tblN){
		$cpyV= array();
		reset($this->{$tblN});
		foreach ($this->{$tblN} as $key=>$val){
			$cpyV[$key]= _c_ok;	// pseudo copy instead of real copy $cpyV[$key]= $val;
		}
		
		// looping on a pseudo copy may be redundant ?
		reset($cpyV);
		foreach ($cpyV as $key=>$val){
			$this->unsetAndDeleteRow($tblN,$key);
		}
		
		return true;	// supposed
	} // truncTbl
	
	
	/**
		Get the length of the given field of the given resource.
			Rem : mysql_field_len()  ... give the maximum size of the field!!
			
		Input:
			$rsr : computed sql resource.
			$idx : ordinal (0 start) index of the concerned field.
	*/
	function sizeElement(&$rsr, $idx,$field){
		$uniqueRow= reset($rsr);			// return 1st element
		return strlen($uniqueRow[$field]);
	} // sizeElement

	
	/**
	*/
	function createDtb($name){
		return true;	// can be created with files actualisation
	} // createDtb
	

	function likeAndStr($w){
		$this->glob->userError('Can\'t use simulSql::likeAndStr() with a non SQL database.');
		return false;
	} // likeAndStr

	function insideQuotesVect(&$vect,&$szVect,&$restStr,$mainStr,$trim=true){
		$this->glob->userError('Can\'t use simulSql::insideQuotesVect() with a non SQL database.');
		return false;
	} // insideQuotesVect

	function rawToSqlSearchStr($colName,$searchStr,&$vTot){
		$this->glob->userError('Can\'t use simulSql::rawToSqlSearchStr() with a non SQL database.');
		return false;
	} // rawToSqlSearchStr

	function rToSqlSearchStr($colName,$searchStr){
		$this->glob->userError('Can\'t use simulSql::rToSqlSearchStr() with a non SQL database.');
		return false;
	} // rToSqlSearchStr




	/**
		Special tools to untangle some of the strings present in standard SQL.
	*/
		
	/**
		Transform a classical new record sql string into a vector.
		Rem : necessary at the raw vectorised-php database level to convert block of sql style strings
		into a storable vector! Even if pseudo-sql instruction stay vectorise to the last level, we still
		have the recovery data stored as a single string, so sqlstring --> vector conmvertion is necessary!
		(2nd) Bottom line sql vectorisation is up to now done only for ::newRecord(), so this method 
		is going to be more used than just for undoing operations.
		++ Need to be added to simulSql(), because only used there.
		
			Input :
				$s : string to analyse.
				$leftOv : to concat the leftover characters, to eventually recover the left members
					of assignation specifications.
				
			Output
				$leftOv : returns the left over chars (diff from = ) concatenated into a string.
				returns : the vector matching elements between "
	*/
	function newRecStrToVect(&$s, &$leftOv){
		$glob= &$this->glob;
		$tools= &$this->tools;
		
		$inQ= false;
		$oddBS= _c_ko;	// an isolated or group of contiguous odd back slash. (odd --> true, even --> false, other char --> 0)
		
		$leftOv= '';	// to concat what's left over.
		$pc= '';
		$bStr= '';
		$e= 0;
		$sz= strlen($s);
		for ($i=0;$i<$sz;$i++){
			$c= $s[$i];
			
			if ($pc===_c_bSh){
				switch ($oddBS){
					case  _c_ko:
						$oddBS= true;	// never met a slash or met a discontinuation.
					break;
					
					case true:			// previously got an odd number of slash.
						$oddBS= false;
					break;
					
					case false:			// previously got an even number of slash.
						$oddBS= true;
					break;
					
					default:
						userError('localDB::newRecStrToVect');
				} // switch
			}else{
				$oddBS= _c_ko;	// normal state or breaks a string of back slashes.
			}
						
			if ($inQ){
				if ($c===_c_dblQ && $this->validTerminalQ($oddBS)){
					// Got terminal quote
					$v[$e]= $tools->bksDblQ($bStr); // necessary because the unquoting job is done explicitely (not left to a sql or php interpretor)
					
					$bStr= '';
					$e++;				
					$inQ= false;
				}else{
					$bStr.= $c;
				}
			}else{
				if ($c===_c_dblQ){
					$inQ= true;	// Got an opening quote					
				}else{
					// whites and commas between strings.+ other char in other contexts!
					if ($c!==','){
						$leftOv.= $c;
					}
				}
			}
		
			$pc= $c;	// load the previous character
		} // for
		
		if (!isset($v)){
			return false;
		}
		
		return $v;
	} // newRecStrToVect


	/**
		Returns true if we got a valid terminating quote.
		Input :
			$oddBS : status of previous char \ to detect the quote validity.
	*/
	function validTerminalQ($oddBS){
		return $oddBS===_c_ko || $oddBS===false; // (a non \ before the ") || (an even number of preceding \)
	} // validTerminalQ
	
	
	/* ---------------------------------------------------------------------------------------------
		Gives the maximum value of $colName for the given table $table / $colName
	*/
	function maxColValue($tblNm,$colName){	
		$glob= &$this->glob;

		if ( !isset($this->{$tblNm}) || isset($this->{$tblNm}) && sizeof($this->{$tblNm})<1 ){
			$max= '1000';
		}else{	
			$max= $glob->currentNext($this->{$tblNm});
			reset($this->{$tblNm});
			foreach ($this->{$tblNm} as $key=>$curV){
				if ($max<$curV[$colName]){
					$max= $curV[$colName];
				}
			}
		}
		return $max;
	} // maxColValue

	
	/**
		Counts the number of rows where the concerned $field got the concerned $val.
	*/
	function cntNbRows($tblN,$field,$val){
		$this->fullTableFromAtomFiles($tblN);
	
		$cnt= 0;
		reset($this->{$tblN});
		foreach ($this->{$tblN} as $kScan=>$cRow){
			if (array_key_exists ($field,$cRow) && $cRow[$field]===$val){
				$cnt++;
			}
		}
		return $cnt;
	} // cntNbRows


	/**
		Count the number of rows for the given table name.
		Remark : instead of counting could update a table specific for the sizes
	*/
	function countNbRows($tblN){
		if ( $this->tblFullyActivated($tblN) ){
			return count($this->{$tblN});
		}else{
			// Does not loads the whole bunch of tables because it may be long and not necessary, so just scan the concerned files
			$dirPath= $this->atomDtbTblPath($tblN,$this->CurrentPhpDtbPath);
			return $this->nbFilesInDir($dirPath);
		}
	} // countNbRows
	
	
	/**
	*/
	function queryNbRows(&$q){
		return count($q);
	} // queryNbRows

	
	/**
		For a given Query return a vector holding all the values of the specified field : $gField
			Input :
				$resQ : result query
				$field	: concerned field.
			Output : return $vect
	*/
	function queryColVect($resQ,$field){
		$glob= &$this->glob;
		$tools= &$this->tools;
		
		$vect= false;
		$r=0;
		if ($glob->vectOfVect($resQ)){
			while( $row= $this->fetchArray($resQ) ){
				if (array_key_exists($field,$row)){
					$vect[$r]= $row[$field];
				}
				
				$r++;
			}
			return $this->sortUnderEnd($vect);
		}else{
			// $resQ already in the correct form, probabbly because standard sql calls insist for complex arrays when php storage is minimalist.
			if ($tools->isBoolean($resQ)){
				return false;
			}else{
				return $this->sortUnderEnd($vect);
			}
		}
	} // queryColVect


	/**
		Sort a vector with _ (undelined) garantie at the end.
		Sort Under End
	*/
	function sortUnderEnd(&$v){
		if ($v===false){
			return false;
		}
		
		reset($v);
		foreach ($v as $key=>$val){
			if ($val!==''){
				if ($val[0]==='_'){
					$vU[$key]= $val;
				}else{
					$vO[$key]= $val;	// Others
				}
			}
		}
		
		// sort($v,SORT_FLAG_CASE | SORT_STRING);	// The flags case unsensitivity do not seem to work !!!!
		if (isset($vU) && is_array($vU)){
			sort($vU);
		}else{
			$vU= array();
		}
		
		if (isset($vO) && is_array($vO)){
			sort($vO);
		}else{
			$vO= array();
		}
		
		return array_merge($vO,$vU);
	} // sortUnderEnd



	/**
	*/
	function colKeyVect($resQ,$keyField,$returnedField=false){
		$vect= false;
		while( $row= $this->fetchArray($resQ) ){
			$r= $row[$keyField];
			if ($returnedField===false){
				$vect[$r]= $row;
			}else{
				$vect[$r]= $row[$returnedField];
			}
		}
		return $vect;
	} // colKeyVect


	/**
		Returns true if the given table exists inside the current database.
	*/
	function tableExists($tblN,$dtbN='phoney'){
		$tools= &$this->tools;
		$v= $this->onlyTblNames();
		return $tools->aIsInV($tblN,$v);
	} // tableExists
	
	
	/**
		Loads a new record into the matching record of the full table (the inout
		comes originally frrom an atom file, but here from an intermediate record!)
	*/
	function loadsRecordInFullTbl($tblN,&$valV){
		$id= reset($valV);
		if ($id!==false){
			$this->{$tblN}[$id]= $valV;	
		}
	} // loadsRecordInFullTbl
	
	
	/**
		Loads the full table from atom files
	*/
	function fullTableFromAtomFiles($tblN){
		$fls= &$this->fls;
		
		if ( $this->tblFullyActivated($tblN) ){
			return true;
		}
	
		$dirPath= $this->atomDtbTblPath($tblN,$this->CurrentPhpDtbPath);
		$gotFl= $fls->firstLevelFiles($flNameV,$dirPath,false);		// remove the file extension
		if (!$gotFl){
			return false;
		}
		
		reset($flNameV);
		foreach ($flNameV as $key=>$fileNn){
			$this->atomFLoadsGlobalAr($tblN,$fileNn);
		}
		
		$this->fullyActivatedTables[$tblN]= true;
		return true;
	} // fullTableFromAtomFiles
	
	
	/**
		Loads the part of the global array that matches the atom file-stored record $r (atom File Loads 
		the matching Global Attay row) Returns true if the Global row exists before the call or after active loading
	*/
	function atomFLoadsGlobalAr($tblN,$value){
		$curDFN= $this->atomDtbTblRowDfn($tblN,$value,$this->CurrentPhpDtbPath);
		if (file_exists($curDFN)){
		
			if (!array_key_exists ( $value, $this->{$tblN} )){
				$r= array();
				include_once $curDFN;		// loads/includes the local array $r locally
				$this->loadsRecordInFullTbl($tblN,$r);
			}
							
			return true;
		}else{
			return false;
		}
	} // atomFLoadsGlobalAr
		

	/**
		Change the specified record at the specified place
			$tbl : the table name to add a row to.
			$set : column names to be changed with assigned value a="xy" (must be minimalist to a single n-tuple)
			$where : where the change needs to be done.
	*/
	function chgSqlRec($tblN,$set,$where){
		$glob= &$this->glob;
		$tools= &$this->tools;
		
		if (!is_array($set)){
			$set= $this->setStrToVect($set,$tblN);
		}
		
		if ($set===false){
			return false;
		}
		
		$set= $this->condStrToVect($set);
		$where= $this->condStrToVect($where);
		$key= key($where);
		$val= current($where);	// should use reset instead of current!!!!!!!
		
		$setIdKey= key($set);
		$setIdVal= reset($set);
						
		if ($key===_c_genId){
			if ($setIdKey===_c_genId && $val!==$setIdVal){
				// $setIdVal is the final value : The id where value is different from the Id value of the set, so we have a renaming action
				$idxVal= $setIdVal;
			}else{
				$idxVal= $val;
			}
			
			$atomDfn= $this->atomDtbTblRowDfn($tblN, $idxVal);
			$this->atomFLoadsGlobalAr($tblN,$val);	// Current record full loading with stored version so can complete with new values
			foreach ($set as $sKey=>$sVal){
				$this->{$tblN}[$val][$sKey]= $sVal;
			}
			
			$this->rowToFile($this->{$tblN}[$val],$atomDfn);
		}else{
			// need to find the correct record(s) with a scan
			if (!$this->tblFullyActivated($tblN) ){
				// loads the full table in memory if not already done
				$this->fullTableFromAtomFiles($tblN);
			}
			
			reset($this->{$tblN});
			foreach ($this->{$tblN} as $kScan=>$row){
				if ( is_array($row) && array_key_exists ($key,$row) ){
					if ($row[$key]===$val){
						$concernedRow= $row[_c_genId];
						reset($set);
						foreach ($set as $sKey=>$sVal){
							$this->{$tblN}[$concernedRow][$sKey]= $sVal;
						}
						
						$val= $this->{$tblN}[ $concernedRow ][_c_genId];
						$atomDfn= $this->atomDtbTblRowDfn($tblN,$val);
						break;
					}
				}
			}
			
			$this->rowToFile($this->{$tblN}[$concernedRow],$atomDfn);
		}
	} // chgSqlRec
		
	
	/**
	*/
	function storeNewRecord($tblN,&$valV,$properKeyNames=true){
		// To load a new row with the given values in $valV
		$id= reset($valV);						
		foreach ($valV as $key=>$val){
			if ($properKeyNames){
				$colName= $key;
			}else{
				$colName= $this->vColumnName($tblN,$key);			
			}
				
			$this->{$tblN}[$id][$colName]= $val;	// pure file concerned part after that
		}		
		
		$dfN= $this->atomDtbTblRowDfn($tblN, $id,$this->CurrentPhpDtbPath);
		$this->rowToFile($this->{$tblN}[$id],$dfN);
	} // storeNewRecord	
	
	
	/**
		Specific Dfn for the simul sql 2 table row
		Input :
			$tblN : table name
			$iVal :index string value
			$tblsPath (in reading writing standard mode but in writing at building time, its the target directory, 
			so the parameter is necessary
	*/
	function atomDtbTblRowDfn($tblN, $iVal, $tblsPath=false){
		if ($tblsPath===false){
			$tblsPath= $this->CurrentPhpDtbPath;
		}
		return $this->atomDtbTblPath($tblN, $tblsPath).$iVal._c_phpExt;
	} // atomDtbTblRowDfn


	/**
	*/
	function atomDtbTblPath($tblN, $tblsPath=false){
		if ($tblsPath===false){
			$tblsPath= $this->CurrentPhpDtbPath;
		}
		return $tblsPath.$tblN.'/';
	} // atomDtbTblPath

	
	/**
	*/
	function databasesVect(){
		$fls= &$this->fls;
		$fls->firstLevelDir($tblV,_c_ProjectsPath,_c_zCmsDirN.'/zPhpData/phpDtb/cellPge');
		return $tblV;
	} // databasesVect
	
	
	/**
	*/
	function centPcPhpDtbV(){
		return $this->databasesVect();
	} // centPcPhpDtbV
	
	

	/**
		Warning the 100& Php implementation having no name for its database, the access path becomes a bane substuture
		and must be treated carefully to be compatible with the connected methods.
	*/
	function curDtbName(){
		return $this->CurrentPhpDtbPath;
	} // curDtbName
		

	/** ----------------------------------------------------------------
		Deprecate/disable with a message the few functions below.
	*/
	function likeStr($w){
		$this->glob->userError('Can\'t use simulSql::likeStr() with a non SQL database.');
		return false;
	} // likeStr
	
	
	/** ----------------------------------------------------------------
		Return the query result (arr of arr)  for a given table, field, dataBase always default here 100%() 
		where we have a match with $needel
		Return the array of concerned rows when matches have been found or false if nothing was found.
	*/
	function needleMatch($needelStr, $tableNm, $fieldNm, $dataBase='phoney'){
		$condVect[$fieldNm]= $needelStr;
		return $this->selectKernel($tableNm,$condVect,'*',0,'L');
	} // needleMatch
	
	/**
	*/
	function lessThan($field,$val){
		return $this->fullCond($field,$val,'<');
	} // lessThan
	
	
	
				
	/**
		For a given $tblN, returns the name of the $c /th columns
		File data base Vector Column Name
			Input :
				$tblN : the name of the table
				$c : index of the column
	*/
	function vColumnName($tblN,$c){
		$acc= &$this->acc;
		
		if (gettype($c)==='string' && $acc->simpleLowCase($c[0])){
			return $c;	// !! All column names of the table must start with a lower case !!!!
		}else{
			$v= _c_sqlsColNmPrfx.$tblN;
			return $this->{$v}[$c];
		}
	} // vColumnName
	
	
	/**
		Returns a vector holding all the column names of the table.
		Can be used to display and edit tables.
	*/	
	function tblColumnNames($tblN,$db='phoney'){
		return $this->columnVect($tblN);
	} // tblColumnNames

	/**
	*/
	function columnVect($tblN){
		$v= _c_sqlsColNmPrfx.$tblN;
		if (property_exists($this,$v)){
			return $this->{$v};
		}else{
			return false;	// instead of empty array for compatibility with MySql + easier to avoid displaying whole structure for empty content
		}
	} // columnVect
	
	
	function colVectSize($tblN){
		$v= _c_sqlsColNmPrfx.$tblN;
		return sizeof($this->{$v});
	} // colVectSize

	
	
	
	/**
		Returns the number of files in the given directory. Carved from fileMg::firstLevelFiles()
	*/
	function nbFilesInDir($dir){
		$fls= &$this->fls;

		if ($handle=$fls->locOpDir($dir)){
			$i= 0;
			while ($file = readdir($handle)){
				$dfn= $dir.$file;
				if ($file!='.' && $file!='..' && is_file($dfn)){
					$i++;
				}
			} // while
			closedir($handle);
			return $i;
		}
		return 0;
	} // nbFilesInDir
	
	
	/**
		Returns the vector of all file names matching a given table. Much faster in some
		cases to get all the fields of a table.
		Uses with caution because it short circuit the data base system for speed!
	*/
	function allFnVect($tblN){
		$i= 0;
		$v= array();
		$w= array();
		$rsr= opendir(_c_phpDtb.$tblN);
		while ($fn = readdir($rsr)){
			if ($fn!='.' && $fn!='..'){
				if ($fn[0]==='_'){
					$w[$i]= substr($fn, 0, -4);	// substract the .php extension
				}else{
					$v[$i]= substr($fn, 0, -4);
				}
				$i++;
			}
		} // while
		closedir($rsr);
		sort($v);
		sort($w);
		return array_merge($v,$w);
	} // allFnVect
	

	/**
		Unset the considered table row and delete the atom/row file.
	*/
	function unsetAndDeleteRow($tblN,$id){
		$fls= &$this->fls;
	
		$curDfn= $this->atomDtbTblRowDfn($tblN, $id);
		$fls->secUnlink($curDfn);
		unset($this->{$tblN}[$id]);
	} // unsetAndDeleteRow

		
	/**
		Writes the given row into the concerned file
	*/
	function rowToFile(&$row,$atomDfn){
		$glob= &$this->glob;
		$fls= &$this->fls;
		
		$aDtbStr= '';
		reset($row);
		foreach ($row as $key=>$val){
			$aDtbStr.= $fls->atomDtbCelStr($key,$val);
		}
				
		$aDtbStr= $fls->phpFileCaps($aDtbStr);
		$fls->strongStrToFile($atomDfn,$aDtbStr);
	} // rowToFile

} // simulSql
?>