<?php
/**
	File name : filesMg.php
	Author of this file : Oscar Fawer until 2019-08-15
	Contact : www.ataox.org 
	Web sites : ataox.org / ataox.com / ataox.info
	Copyrights open source GPL, GNU General Public License V3 : see ./aMainDr/code/LICENSE.txt


	General file manager to look at file and directories.
	filesMg::cpyFileWithPossRsz()
	
	filesMg::safIdStr()
	
	Improvements see :
		firstLevelFiles() .. scandir($dir) should be much faster.
		dirIsEmpty($dir) .. scandir($dir) should be much faster.
		Disambiguation : 	function fileExist($fn) / file_exists($fn)
							$fls->fileExist($dfn)
*/
class filesMg extends oTop{
	

	function __construct(&$asa){
		parent::__construct();
		$this->glob= &$asa->glob;
		$this->acc= &$asa->acc;
		$this->caps= &$asa->caps;
		$this->tools= &$asa->tools;
		$tools= &$this->tools;
		
		$this->dtb= &$asa->dtb;
		
		$this->curDumpContent= '';
		if (!defined('_c_autoDirFlRemover')){
			define('_c_autoDirFlRemover','ecrUpload');	// a name for the checkbox holding the general authorization to remove whatever old file or directory that get a new one uploaded, copied etc.
		}
		
		$this->doneUnzippedPaths= array();	// being extra cautious to avoid spinning around on zip files (happended one because of wrond concatenation of dir and file, corrected since!)
		
		$this->alreadySawIt= array();	
		$this->virginLogShot= true;		// true if virgin of any log writing up to now in the current server call.
		
		$this->globMonthVect= false;
		$this->ultraRmOn= false;
		
		$this->archChgPwFiles= false; // default save it all for distribution, so no pw removal!
		
		$this->countOpArc= 0;
	} // construct

	/**
	*/
	function setDtbAccess(&$dtb){
		$this->dtb= &$dtb;
	} // setDtbAccess	
	
	function loadsRMg(&$rMg){$this->rMg= &$rMg;} // loadsRMg
	function hPwC(&$hPwC){$this->hPwC= &$hPwC;}
	function loadsTagsFu(&$tagsFu){$this->tagsFu= &$tagsFu;}
	function loadsWmRights(&$wmRights){$this->wmRights= &$wmRights;}
	

	/**
		Just to set $this->archChgPwFiles from an outside call
	*/
	function setArchChgPwFiles(){
		$dtb= &$this->dtb;
		
		$this->archChgPwFiles= !$dtb->gotOkGenVar('archCriticalCol');
		$this->plUsersTblIds= array();	// the list of PowerLess (or almost powerless) user Ids for who changing the password at backup time is useless
		if ($this->archChgPwFiles){
			// --> got to change the archive fields holding webmaster passwords to avoid risk of spying
			$this->plUsersTblIds['identification']= _c_ok;
			$this->plUsersTblIds['idMark']= _c_ok;
			$this->plUsersTblIds['guest']= _c_ok;
			$this->plUsersTblIds['ersatzid']= _c_ok;
		}
	} // setArchChgPwFiles
	
	
	
	/**
		Remark the genVar variable :: forcesHttps could be used  to force the Https protocol when needed, but up to know,
		it's up to the user to chose it if it's available on the server.
	*/
	function httpKindStr(){
		if (array_key_exists('HTTPS',$_SERVER) && $_SERVER['HTTPS']!==''){
			return _c_slashHttp; // return _c_slashHttps;	// Not because it's activated on the php server that the site use it and the certificate valid!
		}else{
			return _c_slashHttp;
		}
	} // httpKindStr
	
	/**
		Remark : wortk in most cases even if the DTB is still not connected!
	*/
	function domainName(){
		if (array_key_exists('HTTP_HOST',$_SERVER)){
			$sName= $_SERVER['HTTP_HOST'];
		}else if (array_key_exists('SERVER_NAME',$_SERVER)){
			$sName= $_SERVER['SERVER_NAME'];
		}else{
			$dtb= &$this->dtb;
			$sName= $dtb->getGenVar('domainName');
		}
		return $sName;
	} // domainName
	
	/**
	*/
	function httpDomCombi(){
		return $this->httpKindStr().$this->domainName().'/';
	} // httpDomCombi
	

	/**
		!! 8 bits chars only
	*/
	function removeLastChar($str){
		return substr($str,0,-1);
	} // removeLastChar
		
		
	/**
		Give back the last char (to detect 8 bits chars only)
	*/
	function lastChar($str){
		return substr($str,-1);
	} // lastChar
		

	/**
		Secure un link / delete. Could add a security parameter as being inside the data directory, otherwise check ecr !!
	*/
	function secUnlink($pF){
		if ($this->fileExist($pF)){
			$this->unLnk($pF);
		}
	} // secUnlink

	
	/**
		Local secured fwrite
	*/
	function flWrite($hdl, $s, $forceIt= false){
		$glob= &$this->glob;

		if ($glob->cndtWrRghts===true || $forceIt){
			return fwrite($hdl, $s);
		}else{
			$this->appendOnLogFile($s,'flWrite');
			return false;
		}
	} // flWrite


	/* -------------------------------------------------
		SPECIAL STRING MANIPULATIONS FOR FILENAMES :
	*/
	
	
	/**
		Generates a character safe identifier (file/url/field/variable etc,) string from a given string. kWords : safIdStr --> secureId
		Input :
			$sOrig : original string
			$maxSz : maximum size of the target string
			$admitHyph : true if we admit hyphen.
			$fu :param function to restrict the sort of accepted characters.
	*/
	function safIdStr($sOrig, $acceptHash=false, $maxSz=_c_largestIdSz, $admitHyph=true, $fu='idChar', $shrink=true){
		$glob= &$this->glob;
		
		$newS= $this->safeIdString($sOrig,$acceptHash,$admitHyph,$fu);
		
		if ($shrink===true){
			$lastStr= $this->smartShrink($newS,$maxSz);		// Makes sure it's of the proper size :
			$sz1= strlen($newS);
			$sz2= strlen($lastStr);
			if ($sz1!==$sz2){
				$this->oWarning($glob->el('Identifier : ','kayq').$sOrig.$glob->el(' was trunkated to ','kayr').$lastStr._c_);
			}
			return $lastStr;
		}else{
			return $newS;
		}
	} // safIdStr
	
	
	/**
		Generates a character safe identifier without bothering about size!.
	*/
	function safeIdString($sOrig, $acceptHash=false, $admitHyph=true, $fu='idChar'){
		$acc= &$this->acc;
		$tools= &$this->tools;
	
		// Makes sure we have the proper set of characters : Strict identificator fitness
		$vect= $acc->toSimpleChars($sOrig,$n); // larger effect than the method accents::toNoAccents()
		$lastAddedChar='';
		$lastForbidChar='';
		$i=0;
		$newS= '';
		while($i<$n){
			$c= $vect[$i];
			if ( ($acc->$fu($c) || $acceptHash && $c==='#' && $i===0) ){
				if($lastForbidChar==='\''){
					// special patch, mainly for French, so eg "collection d'articles" becomes "collection articles" 
					$newS= rtrim($newS,_c_snglQ);
				}
			
				$newS.= $c;	// build the new string
				$lastAddedChar= $c;
			}else{
				$lastForbidChar= $c;
			
				// substitute space or spaces packet with - (intermediate tabs, cr/ln etc. are just jump)
				if ( $admitHyph && ($c===_c_Rs || $c==='-' || $c===_c_snglQ) && $i+1<$n  && $vect[$i+1]!==_c_Rs && $lastAddedChar!=='-'){
					if ($newS!==''){
						// to avoid leading -
						$newS.= '-';
						$lastAddedChar= '-';
					}
				}
			}
			$i++;
		} // while
		
		if ($lastAddedChar==='-'){
			// Needs to remove it !
			$tools->rmLastByte($newS);
		}
		return $newS;
	} // safeIdString		
	
	
	/**
		Smart size reduction of identificators so keep the string as readable as possible and as different from
		another Id capable of colision. --->
			Input :
				$str : given string.
				$maxSz : maximum size.
	*/
	function smartShrink($str,$maxSz){
		$origSz= strlen($str);
		if ($origSz <= $maxSz){
			return $str;
		}else{
			// Need to Shrink
			
			// step 1 removes all vowel
			$consStr= $this->rmVowelSmallSet($str);		// --> consonant characters
			$consStr= $this->rmHyphens($consStr);
			$consSz= strlen($consStr);
			if ($consSz <= $maxSz){
				return $consStr;
			}else{
				// step 2 remmove every 2nd chars except extremities and redo the process if not small enough
				do{
					$consStr= $this->binShievedStr($consStr,$retSz);
				}while( $retSz > $maxSz );
				return $consStr;
			}
		}		
	} // smartShrink

	
	/**
		Remmove every 2nd chars except extremities and return the resulting string
	*/
	function binShievedStr($str, &$retSz){
		$sz= strlen($str);
		$newStr= '';
		$i=0; 
		$retSz=0; 
		while ($i<$sz-1){
			$newStr.= $str[$i];
			$i=$i+2;
			$retSz++;
		}
	
		return $newStr;
	} // binShievedStr

	
	/**
		Removes all vowel from the given string
	*/
	function rmVowel($s){
		$vowels = array('a', 'e', 'i', 'o', 'u', 'y', 'A', 'E', 'I', 'O', 'U', 'Y');
		return str_replace($vowels, '', $s);			
	} // rmVowel
	
	function rmVowelSmallSet($s){
		$vowels = array('a', 'e', 'i', 'o', 'u', 'A', 'E', 'I', 'O', 'U');
		return str_replace($vowels, '', $s);			
	} // rmVowelSmallSet
	
	function rmHyphens($s){
		return str_replace('-', '', $s);			
	} // rmHyphens
	
	
	
	/**
		Returns the extension (without the dot) if the string got an extension and false otherwise. The 1st
		dot found from the right is considered as an extension dot.
	*/
	function extension($str,$sep='.'){
		$pos= strrpos($str,$sep,-1);
		if ($pos===false){
			return false;	
		}else{
			return substr($str,$pos+1);	// returns the extension
		}
	} // extension
		
	/**
		Returns true if the given extension is from a php file
	*/
	function isPhp($ext){
		return $ext==='php';
	} // isPhp
		
	/**
		Return true if a string got a php extension.
	*/
	function gotPhpExt($str){
		$ext= $this->extension($str);
		if ($ext===false) return false;
		return $this->isPhp($ext);
	} // gotPhpExt
	

	/**
		Separate a full file name into $fn & $ext.
		From a given file name with or without path, Outputs :
			$fn : file name = path and filename without $ext
			$ext : the full extension exmpl : '.jpg'
	*/
	function sepFullFN($fullFN,&$fn,&$ext,$sep='.'){
		$ext= strrchr($fullFN,$sep);
		$sz= strlen($ext);
		$fn= substr($fullFN, 0, -$sz);
	} // sepFullFN
	
	
	/**
		Split $str on the last apparition of $c. ($c is non inclusive.)
		Exmpl : $this->splitOnLastC($str,$left,$right,'.');
	*/
	function splitOnLastC($str,&$left,&$right,$c='/',$plusR=false){
		$sz= strlen($str);
	
		$pos= strrpos($str,$c);
		if ($pos===false){
			$left= $str;
			if ($plusR){
				$right= $str;
			}else{
				$right= '';
			}
			return false;	// did not find $c
		}		
		
		$left= substr($str,0,$pos);
		if ($pos<$sz-1){
			$right= substr($str,$pos+1);
		}else{
			$right= '';
		}
		
		return true;	// got $c
	} // splitOnLastC
	
	/**
		More versatile than splitOnLastC()
	*/
	function splitOnLstC($str,&$left,&$right,$c='/'){
		return $this->splitOnLastC($str,$left,$right,$c,true);
	} // splitOnLastC
	
	
	/**
		remove the given file name extension.	// $caps->fNmeNoExt($fn)
	*/
	function fNmeNoExt($fn){
		$r= $this->splitOnLastC($fn,$bd,$ext,'.');
		if ($r){
			return $bd;
		}else{
			return $fn;
		}
	} // fNmeNoExt
	
	
	/**
		Split on the BEFORE last apparition of $c. ($c is non inclusive.)
	*/
	function splitOnSecondLastC($str,&$left,&$right,$c='-'){
		$ret= $this->splitOnLastC($str,$left,$right,$c);
		if ($ret){
			$ret= $this->splitOnLastC($left,$secL,$secR,$c);
			if ($ret){
				$left= $secL;
				$right= $secR.$c.$right;	// the last $c is re-introduced but the split char is not.
				return true;
			}
		}
		return false;
	} // splitOnSecondLastC
	
	
	/**
		Split on the 1st apparition of $c.
		$c is non inclusive. Uses trim because the equal is likely to be espaced.
	*/
	function splitOnFirstC($str,&$left,&$right,$c='=',$charlist=''){
		$sz= strlen($str);
		$pos= strpos($str,$c);
		if ($pos===false){
			$left= $str;
			$right= '';
			return false;	// did not find $c
		}		
		
		$left= trim(substr($str,0,$pos));
		if ($pos<$sz-1){
			$right= trim(substr($str,$pos+1),$charlist);
		}else{
			$right= '';
		}
	} // splitOnFirstC
	
	
	/**
		Current Url ...
	*/
	function urlWithoutSignet(){
		error_reporting(NULL);
		$url= $_SERVER['PATH_INFO'];
		error_reporting(E_ALL);
		$this->splitOnLastC($url,$left,$right,'#');
		return $left;
	} // urlWithoutSignet
	

	/**
		Trunk and split on any $c and return the vector of components
	*/
	function trunkSplit($str,$c='.',$trunk= false){
		$tools= &$this->tools;
		
		if ($trunk!==false){
			$str= substr($str,0,$trunk);
		}
		
		return $tools->trimStrToVect($c,$str);
	} // trunkSplit


	/**
	*/
	function delFirstChar($str){
		return substr($str,1);
	} // delFirstChar

	/**
	*/
	function removeExternChar($str){
		$str= substr($str,0,-1);
		$str= substr($str,1);
		return $str;
	} // removeExternChar
	

	/**
		Makes sure file names to display are breakable. So adds a white
		every $width chars.
	*/
	function breakableFN($str,$width=25){
		$j=0;
		$buff= '';
		$sz= strlen($str);
		if ($sz<=$width){
			return $str;	// nothing to break
		}
		
		for($i=0; $i<$sz; $i++){
			$c= $str[$i];
			
			if ($j<$width){
				$buff.= $c;
			}else{
				$buff.= ' '.$c;
				$j=0;
			}
		
			$j++;
		}
		
		return $buff;
	} // breakableFN


	/**
		Return the common prefix belonging to the 2 given strings. Remark : for paths got
		to make sure each path is terminated with a slasch, so comparison is even up to the 
		last slash.
			Input :
				$s,$t : the 2 strings to extrtact the common prefix from.
			Output : return the common prefix.
	*/
	function commonPrfx($s,$t){
		$sz= strlen($s);
		$tz= strlen($t);
	
		$b= ''; $i= 0;
		while($i<$sz && $i<$tz && $s[$i]===$t[$i]){
			$b.= $s[$i];
			$i++;
		}
		return $b;
	} // commonPrfx


	/**
		Gives the specified Editor Image. Abstracted because could wish to check the user directory
		before for a personalized image.
		Input : image name.
		Return the full path.
	*/
	function editorImage($im){
		return _c_EditorImgPath.$im;
	} // editorImage

	
	/**
		Editor linked image.
	*/
	function editorLinkImg($url,$imgFN,$alt,$style='lz',$id='',$js=''){
		$caps= &$this->caps;
	
		$src= $this->editorImage($imgFN);
		$img= $caps->image($src,$alt,'lnImg',$id,$js);
		return $caps->linkTC($url,$img,$alt,$style);
	} // editorLinkImg


	/*
		END SPECIAL STRING MANIPULATIONS FOR FILENAMES :
	-------------------------------------------------- */

	

	/**
		Loads an associative array $v holding the name of all the 1st level sub directories of $dir.
			Input :
				$dir : directory to analyse with a  / suffix.
			Output :
				$v : the associative array holding al the 1st level sub directories of $dir
				
		Second call to preserve the original way (simpler call) without the special directory.
	*/
	function firstLevelDir(&$v,$dir,$compulSubDir=false){
		$special= false;
		$rawFlDrNb= 0; // does not use it above
		return $this->frstLvlDir($v,$special,$rawFlDrNb,$dir,$compulSubDir);
	} // firstLevelDir


	/**
		Loads an associative array $v holding the name of all the 1st level sub directories of $dir,
		except if $special is met, in that case, $special (except if set to false before the call),
		$special will keep its value, if not met, will be set to false.
			Input :
				$dir : directory to analyse, given with a  / suffix.
				$special : special dirName that needs special treatment.
				$mmColIdx : index of the concerned column.
			Output :
				$v : the associative array holding all the 1st level sub directories of $dir
				$special : keeps its value if met and false otherwise.
				$rawFlDrNb : number of files and directories at the current level
	*/
	function mmFrstLvlDir(&$v,&$special,&$rawFlDrNb,$dir,$mmColIdx){
		return $this->frstLvlDir($v,$special,$rawFlDrNb,$dir,false,$mmColIdx);
	} // mmFrstLvlDir
	
	function frstLvlDir(&$v,&$special,&$rawFlDrNb,$dir,$compulSubDir=false,$mmColIdx=false){
		$glob= &$this->glob;
		$tools= &$this->tools;
		$dtb= &$this->dtb;
	
		if (!is_readable($dir)){
			$glob->userError( $glob->el('The directory ','kays').$dir.$glob->el(' is not readable!','kayt') );
			return false;
		}
		
		$uvChunk= 'mmDFChunkPos'.$mmColIdx;
		$chunkStart= $tools->binLoads( $tools->getVal($uvChunk),0 );
		$szChunk= $dtb->getGenVar('mmSizeDirFlsChunk','4000');
		$chunkEnd= $chunkStart+$szChunk;
	
		$rawFlDrNb= 0;
		$gotSpec= false;
		$v = array();	// initialization		
		if ( $handle = $this->locOpDir($dir) ){
			$i= 0;	// index of the directories real order on the media
			$j= 0;	// index matching the exact number of directories
			while (($file= readdir($handle)) !== false){
				// scanning file and directories, so the global chunk numbers of file and dir is matched, 
				if ( $mmColIdx===false || ($i>=$chunkStart && $i<$chunkEnd) ){	
					$dfn= $dir.$file;
					if ($file!=$special){
						if ($file!='.' && $file!='..' && is_dir($dfn)){
							if ($compulSubDir===false){
								// Standard case : got a directory to load (just loads the dir name == $file, not the path).
								$v[$j]= $file;
								$j++;
							}else{
								// $compulSubDir ---> going inside the current directory.
								$subDirDfn= $dir.$file.'/'.$compulSubDir;
								if (file_exists($subDirDfn)){
									$v[$j]= $file;
									$j++;
								}
							}
						}
					}else{
						$gotSpec= true;
					}
				}
				$i++;
			} // while
			
			$rawFlDrNb= $i;
			closedir($handle);
		}
		
		if (!$gotSpec){
			$special= false;
		}else{
			return true; // even if $v is empty, so carefull at interpretation
		}
		
		return isset($v[0]); // return true if got at least one directory
	} // frstLvlDir


	/**
		Idem firstLevelDir() but for files.
			Output :
				$v : the associative array holding all the 1st level files of $dir
			Input :
				$dir : directory path to analyse with a  / suffix.
				$keepExt : true to keep the file extension.
				$mmColIdx : in the case of the file / media manager 1 or 2 otherwise false!
				$specType : loads only a specific type of files
				
		Remark ; based on scandir($dir) could be much faster.
	*/
	function firstLevelFiles(&$v, $dir, $keepExt=true, $mmColIdx=false, $specType=false){
		$glob= &$this->glob;
		$tools= &$this->tools;
		$dtb= &$this->dtb;
		
		$keStr= $tools->boolToOk($keepExt);
		$v = array();	// initialization		
		if (!is_readable($dir)){
			$glob->userError( $glob->el('The directory ','kays').$dir.$glob->el(' is not readable!','kayt') );
			return false;
		}
			
		if ( $handle = $this->locOpDir($dir) ){
			$uvChunk= 'mmDFChunkPos'.$mmColIdx;
			$chunkStart= $tools->binLoads( $tools->getVal($uvChunk),0 );
			$szChunk= $dtb->getGenVar('mmSizeDirFlsChunk','4000');
			$chunkEnd= $chunkStart+$szChunk;
									
			$i= 0;
			$j= 0; $k=0;
			while (($file= readdir($handle)) !== false){
				if ( $mmColIdx===false || ($i>=$chunkStart && $i<$chunkEnd) ){	
					$dfn= $dir.$file;
					if ($file!='.' && $file!='..' && is_file($dfn)){
						if ($keepExt){
							if ($specType!==false){								
								switch ($specType){
									case 'image':
										if (exif_imagetype($dfn)!==false){
											$v[$k]= $file;
											$k++;
										} // else returns predef constants related to image type : http://php.net/manual/en/image.constants.php
									break;
									
									default:
										// apply the single specific extension
										$this->splitOnLastC($file,$flBdy,$ext,'.');
										if ($specType===$ext){
											$v[$k]= $file;
											$k++;
										}
									break;
								} // switch
							}else{
								$v[$j]= $file;	// got a file to load (just loads the dir name == $file, not the path)
							}
						}else{
							$this->splitOnLastC($file,$flBdy,$ext,'.');
							$v[$j]= $flBdy;
						}
					}
					$j++;
				}
				$i++;
			} // while
			closedir($handle);
		}
		return $i>0; // return true if got at least one file
	} // firstLevelFiles

	
	/**
		Return true if the given directory $dir is empty or hold only the specified file.
	*/
	function emptyOrUnique($dir,$file){
		return $this->dirGotOneFile($dir);
	} // emptyOrUnique
	
	/**
		Returns true if the fiven directory got only one relevant file 
		(a file different from . or ..) and practically the file to extract an archive from!
	*/
	function dirGotOneFile($dir){
		$dirs= scandir($dir);
		return count($dirs)===3;	// 3 for '.' and '..' and one "real" file
	} // dirGotOneFile
	

	/**
		Return true if the given directory $dir is empty.
		Remark ; based on scandir($dir) could be much faster.
	*/
	function dirIsEmpty($dir){
		$dirs= scandir($dir);
		return count($dirs)===2;	// '.' and '..'
	} // dirIsEmpty


	/**
		Returns true if the vector holds only the given element
	*/
	function onlyElem(&$v,$elem){
		return is_array($v) && sizeof($v)===1 && $v[0]===$elem;
	} // onlyElem
	

	/**	
		Make a directory from a non-slashed path with a few tests! kw: newDir, makeDir, etc.
	*/
	function wMkDir($pathFN,$mode=_c_stdFoldersRights){
		$glob= &$this->glob;	
		
		$r= false;
		if (!$this->fileExist($pathFN)){
			$this->splitOnLastC($pathFN,$left,$right);
		
			if ($this->fileExist($left)){
				$r= mkdir($pathFN,$mode);
				
				/*
					Really got to call chmod() below to get a real change of mode, because
					the mode imposed by mkdir() does not seem to change anything. or may be 
					is dependant of umask() when chmod() is not !!!
				*/
				$this->chgMode($pathFN);
			}else{
				$glob->userError( $glob->el('The directory','kbbm').$left.$glob->el(' does not exists!','kayu') );
			}
		}
		return $r;
	} // wMkDir

	/**
		Simpler make dir than wMkDir() when we now the environment better.
	*/
	function makeDir($pathFN,$mode=_c_stdFoldersRights){
		$r= mkdir($pathFN,$mode);
		$this->chgMode($pathFN);
		return $r;
	} // makeDir

	/**
		versatileDotSplit
	*/
	function versatileDotSplit($str,&$left,&$right){
		$tools= &$this->tools;
		if ($tools->sIsInB('.',$str)){
			$this->splitOnLastC($str,$left,$right,'.');
		}else{
			// no dot --> just uses the whole string
			$left= $str;
			$right= '';
		}
	} // versatileDotSplit

	/**
		Try to find a new file name $target, if the given target exists up to $limit trials. If 
		the 1st proposed target does not work, try other [.1,.2 etc] till $limit,
			Input : 
				$target
			Output : 
				$target
				return true or false depending on objectif realisation.
	*/
	function okOrNewName(&$target, $start=2,$limit=100,$retIndex=false){
		$this->versatileDotSplit($target,$pathF,$ext);
		
		$i=$start;
		while ($this->fileExist($target) and $i<$limit){
			if ($ext!==''){
				$target= $pathF.$i.'.'.$ext;
			}else{
				$target= $pathF.$i;
			}
			
			$i++;
		}
		
		if ($retIndex){
			if ($i===$start){
				return false;
			}else{
				return $i-1;
			}
		}else{
			if ($i===$limit){
				return false;
			}else{
				return true;
			}
		}
	} // okOrNewName	


	/**
	*/	
	function okOrNewDirName(&$target,$start=2,$limit=100){
		$newTarget= $target;
		$i=$start;
		while ($this->fileExist($newTarget) and $i<$limit){
			$newTarget= $target.$i;
			$i++;
		}
		
		$target= $newTarget;
		if ($i===$limit){
			return false;
		}else{
			return true;
		}	
	} // okOrNewDirName	

	/**
		Input :
			$dfn : directory and file name.
			$gStr : given string.
			$mode : to open the writing mode
			$forceIt : to bypass the parameter $glob->cndtWrRghts===true !!
		Output :
			writes the file
			returns the number of byte written or false.
	*/
	function strToFile($dfn,$gStr,$mode="w", $forceIt= false){
		if ($this->okToCreate($dfn)){
			$fd= fopen($dfn,$mode);
			$res= $this->flWrite($fd,$gStr, $forceIt);
			fclose($fd);
			$this->chgMode($dfn,_c_stdFilesRights);
			return $res; // for an eventual checking
		}else{
			return false;
		}
	} // strToFile
	
	
	/**
		Delete the targeted file directly if it exists with automatic buffer creation.
	*/
	function strongStrToFile($dfn,$gStr, $bufferize=false){
		if ($bufferize){
			// Creates a back up buffer
			$this->splitOnLastC($dfn, $leftDfn,$ext,'.');
			$buffDfn= $leftDfn.'Buffer.'.$ext;
			$bkpStr= $this->fileToStr($dfn);
			$this->strongStrToFile($buffDfn,$bkpStr);
		}
	
		if ( file_exists($dfn) ){
			$this->unLnk($dfn);
		}
		
		return $this->strToFile($dfn,$gStr);
	} // strongStrToFile
	
	/**
		Quicker and rougher that the above strongStrToFile()
	*/
	function strngStrToFile($dfn,$gStr,$mode="w", $forceIt= false){
		if ( file_exists($dfn) ){
			$this->unLnk($dfn);
		}
		$this->strToFile($dfn,$gStr,$mode,$forceIt);
	} // strngStrToFile
	
	

	/**
		To test on parent directory of a file to create or overwrite : 
			everyBodyWritingRights || isOwner($fn) && ownerWritingRights.
		Input :
			$dfn : the file or directory to create.
	*/
	function okToCreate($dfn){
		if ( file_exists($dfn) ){
			// the file exists so it depends on the file itself and the directory
			$fOk= $this->isWritable($dfn);
			$dOk= $this->holdDirOkToChg($dfn);
			return $fOk && $dOk;
		}else{
			// the file does not exists so it depends only on the surrounding directory
			return $this->holdDirOkToChg($dfn);
		}		
	} // okToCreate
	
	/**
		Returns true if the holding directory can be written in!
	*/
	function holdDirOkToChg($dfn){
		$this->splitOnLastC($dfn,$dirDfn,$right);
		return $this->isWritable($dirDfn);
	} // holdDirOkToChg
	

	/**
		To read small/average file easily memorized into a string.
			Input : $fn : file path and name.
			Output : returns the file content as a string.
			
		Rem : probably need to interpret better the case of a false return!
	*/
	function fileToStr($fn,$errDspl=true){
		$glob= &$this->glob;	
		if ($this->fileExist($fn)){
			if ($this->isReadable($fn)){
				return file_get_contents($fn);
			}else{
				$glob->userError( $glob->el('* The file ','kayv').$fn.$glob->el(' is not readable!','kbao') );
			}
		}else{
			if ($errDspl){
				$glob->userError( $glob->el('* The file ','kayv').$fn.$glob->el(' does not exists!','kayu') );
			}
			return false;
		}
	} // fileToStr

	
	/**
		Checks whether a file or directory exists
	*/
	function fileExist($fn){
		if ($fn===false){
			return false;
		}
			
		return file_exists($fn);
	} // fileExist
	
	
	/**
		Encapsulates the given string into the standard Php tags.
	*/
	function phpFileCaps($s){
		return	'<?php'._c_R.$s._c_R.'?>';
	} // phpFileCaps
	

	/**
	*/
	function clickableDirImages($dir,$css){
		$caps= &$this->caps;
		
		$dir= $dir.'/';
		$this->firstLevelFiles($vImg,_c_UsrImagePath.$dir);
		
		// Build the images structure
		$allClickImg= '';
		reset($vImg);
		foreach ($vImg as $key=>$val){
			$this->splitOnLastC($val,$smlImgName,$ext,'.');
			$allClickImg.= $caps->smallImage($smlImgName, _c_UsrImagePath.$dir.$val);
		}
		return $caps->div( $allClickImg, $css);
	} // clickableDirImages


	/**
		JavaScript clickable images for zoom image substitution on the same page
		!!!!!!!!!!!! could associate it with jsChgParentZoom
	*/
	function jsSamePgImg($dirThumbs,$css){
		$caps= &$this->caps;
		$this->splitOnLastC($dirThumbs,$dirBigs,$thumbsDN,'/');	// $thumbsDN is a phoney variable (just the right part)
		
		$dirThumbs= $dirThumbs.'/';
		$this->firstLevelFiles($vImg,_c_UsrImagePath.$dirThumbs);
		
		// Build the images structure
		$allClickImg= '';
		reset($vImg);
		foreach ($vImg as $key=>$val){
			$this->splitOnLastC($val,$smlImgName,$ext,'.');
			$srcBig= _c_UsrImagePath.$dirBigs.'/'.$smlImgName.'.'.$ext;
			$allClickImg.= $caps->image(_c_UsrImagePath.$dirThumbs.$val,'','linkedImg',
					$smlImgName,' onclick="chgSamePgImg(\'idZoom\', \''.$srcBig.'\');" ', 'cursor:pointer');
		}
		
		return $caps->div( $allClickImg, $css);
	} // jsSamePgImg


	/**
		JavaScript clickable images for zoom image substitution.
		For now specialized for the prjector !! --> either extand the scope of this method or relocate it into projector.php.
	*/
	function jsChgParentZoom($dirThumbs,$css){
		$caps= &$this->caps;
		
		$this->splitOnLastC($dirThumbs,$dirBigs,$thumbsDN,'/');	// $thumbsDN is a phoney variable (just the right part)
		$dirThumbs= $dirThumbs.'/';
		$this->firstLevelFiles($vImg,_c_UsrImagePath.$dirThumbs);
		
		// Build the images structure
		$allClickImg= '';
		reset($vImg);
		foreach ($vImg as $key=>$val){
			$this->splitOnLastC($val,$smlImgName,$ext,'.');
			$fileName= $smlImgName.'.'.$ext;
			$srcBig= _c_UsrImagePath.$dirBigs.'/'.$fileName;
			$allClickImg.= $caps->image(_c_UsrImagePath.$dirThumbs.$val,'','linkedImg',
					$smlImgName,' onclick="window.parent.projector.chgSamePgImg(\'idZoom\', \''.$srcBig.'\');" ', 'cursor:pointer');
		}
		return $caps->div( $allClickImg, $css);
	} // jsChgParentZoom


	/**
		Removes only within the user directory, so compulsory prefix : _c_nsUsrDataPath
	*/
	function secureRemoveDir($srcPath){
		$glob= &$this->glob;
		$tools= &$this->tools;
		$rMg= &$this->rMg;
		
		if ( $rMg->suRights() || $tools->sIsPrefixB(_c_nsUsrDataPath,$srcPath) || $tools->sIsPrefixB(_c_archiveDirPath,$srcPath) ){
			$this->lRmoveDir($srcPath);
		}else{
			$glob->userError($glob->el('Trying to delete ','kayw').$srcPath.$glob->el('Sorry you can\'t use a delete...','kayx')._c_nsUsrDataPath);
		}
	} // secureRemoveDir
	
	/**
		Local remove directory
	*/
	function lRmoveDir($srcPath){
		$glob= &$this->glob;
		if ($glob->cndtWrRghts===true){
			$this->removeDir($srcPath);
		}else{
			$this->appendOnLogFile($srcPath,'lRmoveDir');
			return false;
		}
	} // lRmoveDir
	
	/* ------------------------------------------------------------------------------------------------
		Total secured directory removal (sub D included - delete) !!!! Can be applied directly to a file too!
		Directory paths given without the trailing '/' !!!  (without, because otherwise probl need to get rid of / for removal)
	*/ 
	function removeDir($srcPath){
		$this->coreRmDirKeepRecurs= true;
		$this->ultraRmOn= false;
		$this->coreRemoveDir($srcPath);
	} // removeDir
	
	function ultraRemoveDir($srcPath){
		$this->coreRmDirKeepRecurs= true;
		$this->ultraRmOn= true;
		$this->coreRemoveDir($srcPath);
	} // ultraRemoveDir
		
		
	/**
	*/	
	function coreRemoveDir($srcPath){
		if ($this->ultraRmOn){
			/**
				For ultra strong removal : it changes the rights when getting in and out of the recursive process,
				this avoids lock down of internal files by externals! Also direct chmod() because fileowner() may fail with invisible files etc.
			*/
			chmod($srcPath,_c_stdFoldersRights);
		}
	
		if (is_dir($srcPath) && $handle = $this->locOpDir($srcPath)){
			while (($file= readdir($handle)) !== false){				
				if ($file !== '.' && $file !== '..'){
					$curPath= $srcPath.'/'.$file;
					if (is_dir($curPath)){
						if ($this->coreRmDirKeepRecurs){
							// Got to try to remove another directory !!
							if ( !$this->coreRemoveDir($curPath) ){
								$this->coreRmDirKeepRecurs= false;
								return false;
							}
						}
					}else{
						// Got to try to remove another file !!
						if ( !$this->unLnk($curPath) ){
							$this->coreRmDirKeepRecurs= false;
							return false;
						}
					}
				}
			} // while
			
			closedir($handle);
		}
		
		if (is_dir($srcPath)){
			return $this->coreRmDir($srcPath);
		}else{
			return $this->unLnk($srcPath);
		}
	} // coreRemoveDir		



	/**
		Local secured unlink ... rights of writing test is done up-stream for performance!
		Note : return value not used in most calls!
	*/
	function unLnk($pF){		
		$glob= &$this->glob;
		if ( $glob->cndtWrRghts===true && ($this->ultraRmOn || $this->isWritable($pF)) ){
			if ($this->ultraRmOn){
				chmod($pF,_c_stdFoldersRights);
			}
			return unlink($pF);
		}else{
			if ($glob->cndtWrRghts===true){
				$glob= &$this->glob;
				$glob->userError('filesMg::unLnk '.$pF.$glob->el(' cannot be deleted!','kayp'));
			}else{
				$this->appendOnLogFile($pF,'unLnk');
			}
			return false;
		}
		
		return true;
	} // unLnk

	/**
		Brutal SIMPLE STRONG unlink()
	*/
	function strongUnlink($pF){		
		chmod($pF,_c_stdFoldersRights);
		return unlink($pF);
	} // strongUnlink

	
	/**
	*/
	function coreRmDir($pF){
		$glob= &$this->glob;
		
		if ( $glob->cndtWrRghts===true && ($this->ultraRmOn || $this->isWritable($pF)) ){
			if ($this->ultraRmOn){
				chmod($pF,_c_stdFoldersRights);
			}
			return rmdir($pF);
		}else{
			if ($glob->cndtWrRghts===true){
				$glob= &$this->glob;
				$glob->userError('filesMg::coreRmDir '.$pF.$glob->el(' cannot be deleted!','kayp'));
			}else{
				$this->appendOnLogFile($pF,'coreRmDir');
			}
			return false;
		}
		
		return true;
	} // coreRmDir
	
	/**
		Returns true if we can change the given pathed file or directory.
		bad spell : writeable
	*/
	function isWritable($dfn){
		$tools= &$this->tools;
		$dfn= $tools->removeLastSlash($dfn); // useful for directories  terminated with a /
		return is_writable($dfn);
	} // isWritable
	
	function isReadable($dfn){
		$tools= &$this->tools;
		$dfn= $tools->removeLastSlash($dfn);
		return is_readable($dfn);
	} // isReadable
	
		
	/**
	*/
	function secureCopy($srcPF,$destPF){
		$glob= &$this->glob;		
	
		if (!$this->fileExist($destPF)){
			return $this->copyFile($srcPF,$destPF);
		}else{
			$glob->userError('fileMg::secureCopy '.$destPF.$glob->el(' exists already!','kayy'));
			return false;			
		}
	} // secureCopy


	/**
		Standard simple file copying with mode adaptation.
		Return :
			true: no obvious problem if the file was copied correctly or if !$strong and the file already exists.
			false if the copy() function failed
	*/
	function copyFile($srcPF,$destPF, $strong=false){
		$fe= file_exists($destPF);
		if ($strong){
			if ($fe){
				unlink($destPF);
			}
		}else if ($fe){
			return false;
		}
		
		$r= copy($srcPF,$destPF);
		if (is_dir($srcPF)){
			$this->chgMode($destPF); 	// === _c_stdFoldersRights
		}else{
			$this->chgMode($destPF,_c_stdFilesRights);
		}
		return $r;
	} // copyFile

	
	/**
		Remove the last character! Shorter nicer than the original php fi.
	*/
	function rmLC($s){
		return substr($s, 0, -1);
	} // rmLC

	
	/* -----------------------------------------------------------------------------------
		The directory paths are given without the trailing '/'.
			Remove last char : substr($theString, 0, -1)
	*/ 
	function copyDirectories($srcPath,$destPath,$strong=false){
		if ($handle = $this->locOpDir($srcPath)){
		
			if (!$this->fileExist($destPath)){
				$this->wMkDir($destPath);
			}
				
			while (($file= readdir($handle)) !== false){
				if ($file !== '.' && $file !== '..'){
					$srcPF= $srcPath.'/'.$file;
					$destPF= $destPath.'/'.$file;
					if (!is_dir($srcPF)){
						$this->copyFile($srcPF,$destPF,$strong);	// function copyFile(..)
					}else{
						$this->copyDirectories($srcPF,$destPF,$strong);	// got to copy another directory !!
					}
				}
			} // while
			closedir($handle);
		}
	} // copyDirectories
	
	
	/**
		SECURE COPIES FOR FILES OR DIRECTORIES. Paths are given without the trailing '/'
	*/
	function copyFileOrDir($srcPath,$destPath){
		$glob= &$this->glob;		
		$tools= &$this->tools;
	
		if ($tools->postVal(_c_autoDirFlRemover)!==_c_ok){
			if ($this->fileExist($destPath)){
				$glob->userError('fileMg::copyFileOrDir '.$destPath.$glob->el(' exists already!','kayy'));
				return false;
			}
		}
		
		$destPathCmp= $destPath.'/';
		if ($tools->sIsPrefixB($srcPath.'/', $destPathCmp)){
			$glob->userError('Trying to copy a directory inside itself...','kayz');
		}else{
			if (is_dir($srcPath)){
				$this->copyDirectories($srcPath,$destPath);
			}else{
				$this->copyFile($srcPath,$destPath);
			}
		}
	} // copyFileOrDir
	
	
	/**
		Copy the full CONTENT (not the directory itself) of the directory $srcPath into $destPath
		Paths are given without the trailing '/'
	*/
	function cpyContentSrcIntoDest($srcPath,$destPath){
		$glob= &$this->glob;	
	
		if (!is_readable($srcPath)){
			$glob->userError( $glob->el('The directory ','kays').$srcPath.$glob->el(' is not readable!','kayt') );
			return false;
		}
	
		if ( $handle = $this->locOpDir($srcPath) ){
			while (($file= readdir($handle)) !== false){
				if ($file!='.' && $file!='..'){
					$this->copyFileOrDir($srcPath.'/'.$file, $destPath.'/'.$file);
				}
			} // while
			closedir($handle);
		}
	} // cpyContentSrcIntoDest
	

	/**
		Move the full CONTENT (not the directory itself) of the directory $srcPath into $destPath
		Paths are given without the trailing '/'
	*/
	function mvContentSrcIntoDest($srcPath,$destPath){
		$glob= &$this->glob;	
	
		if (!is_readable($srcPath)){
			$glob->userError( $glob->el('The directory ','kays').$srcPath.$glob->el(' is not readable!','kayt') );
			return false;
		}
	
		if ( $handle = $this->locOpDir($srcPath) ){
			while (($file= readdir($handle)) !== false){
				if ($file!='.' && $file!='..'){
					$this->secureRename($srcPath.'/',$file, $destPath,'/'.$file);
				}
			} // while
			closedir($handle);
		}
	} // mvContentSrcIntoDest

		
	/* -----------------------------------------------------------------------------------
		Rename, move, displace a file or a direcetory.
		Paths need to be given with terminal slash
	*/
	function secureRename($oldPath,$oldFN, $newPath,$newFN){
		$glob= &$this->glob;		
		$tools= &$this->tools;
		
		$oldDFN= $oldPath.$oldFN;
		$newDFN= $newPath.$newFN;
		if ($this->fileExist($newDFN) && $tools->postVal(_c_autoDirFlRemover)!==_c_ok){
			$glob->userError('fileMg::secureRename '.$newDFN.$glob->el(' exists already!','kayy'));
			return false;					
		}else{
			if ($oldDFN.'/' === $newPath){
				$glob->userError('Trying to move a directory inside itself!','kaza');
			}else{
				$exist= $this->fileExist($oldDFN);
				if ($exist){
					if ($this->isWritable($oldDFN)){
						rename($oldDFN,$newDFN);
					}else{
						$glob->userError($oldDFN.' '.$glob->el("Can't be renamed...",'kblq'));
					}
				} // do not care for now !  Does not exist, no way to rename it + no way to create it because no content is specified (most cases calling redondancy)
			}
		}
		return true;
	} // secureRename
	
	function secureRenameNS($oldPath,$oldFN, $newPath,$newFN){
		return $this->secureRename($oldPath.'/',$oldFN, $newPath.'/',$newFN);
	}
		

	/**
		Insist on secure rename by trying new names
	*/
	function insistSecRename($oldPath,$oldFN, $newPath,$newFN){
		$origNewFN= $newFN;
		$i= 0;
		do{
			$res= $this->secureRename($oldPath,$oldFN, $newPath,$newFN);
			$newFN= $origNewFN.$i;
		} while ($res===false && $i<100);
	
		return $newFN;
	} // insistSecRename
	
	function insistSecRenameNS($oldPath,$oldFN, $newPath,$newFN){
		return $this->insistSecRename($oldPath.'/',$oldFN, $newPath.'/',$newFN);
	}
	
	
	
	/**
		Exchange / Swap the 2 files $f and $g. Also swap names, so names are going to be inappropriated !!!
		So could instead consider 2 different directories and keep original file name when doing the swapping!
		
		So WOULD swap accross directories.
		Also $buffer should be automatically generated
		Warning BUG : if swapping in a thumbs context, the corresponding thumbs need to be updaded, otherwise
			the wrong image are presented with a random mixing effect !!! Browser proxy!
		
		Input :
			- fullpath and name for $f and $g
			- fullpath and name for the buffer
	*/
	function swap($f,$g,$buffer){
		if ( !$this->fileExist($f) || !$this->fileExist($g) ){
			return false;
		}
		
		if ($this->fileExist($buffer)){
			$this->unLnk($buffer);
		}
		rename($f,$buffer);
		rename($g,$f);
		rename($buffer,$g);
		return true;
	} // swap
	
	
	/* ---------------------------------------------------------------------------------------
		The upload method pair, example of calls :
				$destPF= $pathStartxyz.'/'.$this->origNameUpldFile($locBr=_c_locBrowsrDefNm);
				$this->saveUploadedFile($destPF);
	*/

	
	/**
		Returns the original name of the uploaded file.
	*/
	function origNameUpldFile($locBr=_c_locBrowsrDefNm){
		if (isset($_FILES[$locBr]['error']) && $_FILES[$locBr]['error']===UPLOAD_ERR_OK){
			return $_FILES[$locBr]['name'];
		}else{
			return false;
		}
	} // origNameUpldFile


	/**
		Copy a file with possible resizing if it's an image!
			Input :
				$imgP : image processing class..
				$srcPF : source path and file. In case of upload temporary name no extension appear!
				$destPF : destination path and file name.
			Output :
				- the new copy.
				- try to remove tmp file (especially important when $imgP!==false)
				return the result of the copying operation.
	*/
	function cpyFileWithPossRsz(&$imgP, $srcPF,$destPF){	
		if ($imgP===null){
			return $this->copyFile($srcPF,$destPF);
		}else{		
			// See single used of that part at copyAndPossibleImageReduction
			$tX= $imgP->getDesiredWidth();
			$tY= $tX;
			
			$r= $imgP->resizeProc($srcPF,$destPF,$tX,$tY,50,false);
			if ($r!==false){
				if (is_file($srcPF)){
					$this->unLnk($srcPF);
				}
				$this->chgMode($destPF);
			}
			return $r;
		}
	} // cpyFileWithPossRsz

	
	/**
		Saves on the server the locally uploaded file.
		Input :
			$imgP : image processing class.
			$destPF : destination path and file name.
			$locBr : local browser name.
	
		Remark : $locBr must be === to $name of the method capsHtml::localFileBrowser($name, .... )  --->
			---> "Xml" called with [mLocalBrowser name=_c_locBrowsrDefNm size="10" /]
	*/
	function saveUploadedImg(&$imgP, $destPF,$locBr=_c_locBrowsrDefNm){
		$this->coreSaveUploadFile($imgP, $destPF,$locBr);
	} // saveUploadedImg
	
	
	function saveUploadedFile($destPF,$locBr=_c_locBrowsrDefNm){
		// Standard saveUploadedFile (no image resize)
		$imgP= null;
		$this->coreSaveUploadFile($imgP, $destPF,$locBr);
	} // saveUploadedFile
	
	
	function coreSaveUploadFile(&$imgP, $destPF,$locBr=_c_locBrowsrDefNm){
		$glob= &$this->glob;		
		$tools= &$this->tools;

		if (isset($_FILES[$locBr]['error'])){		// if Check if we have a browser action
			$currentError= $_FILES[$locBr]['error'];
			if ($currentError===UPLOAD_ERR_OK){
				$tmpName= $_FILES[$locBr]['tmp_name'];
				$ecrUpload= $tools->postVal(_c_autoDirFlRemover);	// current upload ecrases evnetual preexisting file (define in squares tags file manager)
				if ($ecrUpload===_c_ok){
					if ($this->fileExist($destPF)){
						$this->unLnk($destPF);
					}
					$this->cpyFileWithPossRsz($imgP,$tmpName,$destPF);		// copyAndPossibleImageReduction
				}else{
					if ($this->okOrNewName($destPF)){
						$this->cpyFileWithPossRsz($imgP,$tmpName,$destPF);	// copyAndPossibleImageReduction
					}else{
						$glob->userError('Exhausted new names generation!','kazb');
					}
				}
			}else{
				$this->uploadErrDealer($currentError);
			}
		}
	} // coreSaveUploadFile
	

	/**
		Deal with upload errors add it onto glob user error and return the sentence!!
	*/
	function uploadErrDealer($currentError){
		$glob= &$this->glob;

		switch ($currentError){
			case UPLOAD_ERR_FORM_SIZE:
				$err= $glob->es('... exceeds the MAX_FILE_SIZE ...','uploadErrA');
			break;
			
			case UPLOAD_ERR_INI_SIZE:
				$err= $glob->es('... exceeds the upload_max_filesize ...','uploadErrB');
			break;
			
			case UPLOAD_ERR_PARTIAL:
				$err= $glob->es('The uploaded file was only partially','uploadErrC');
			break;
			
			case UPLOAD_ERR_NO_FILE:
				$err= $glob->es('No file was uploaded','uploadErrD');
			break;
			
			case UPLOAD_ERR_NO_TMP_DIR:
				$err= $glob->es('Missing a temporary folder.','uploadErrE');
			break;
			
			case UPLOAD_ERR_CANT_WRITE:
				$err= $glob->es('Failed to write file to disk.','uploadErrF');
			break;
			
			case UPLOAD_ERR_EXTENSION:
				$err= $glob->es('A PHP extension caused an upload error','uploadErrG');
			break;
			
			default: $err= $glob->es('Unspecified upload error!','uploadErrH'); // no worry
		}
			
		$glob->userError($err);	
		return $err;
	} // uploadErrDealer
	
	
	/**
		Open or close (creation state) the considered archive
			
		Input :
			$state, $destination
		Output :
			$zip : the zip class instanciation.
			
		Rem :sumarizes the 3 main steps.
	*/
	function openCloseArch(&$zip, $state='',$destination=''){
		$glob= &$this->glob;
		
		if ($glob->cndtWrRghts===false){
			return false;
		}
	
		switch ($state){
			case 'open':
				$zip = new ZipArchive();
				$this->globalZip= &$zip;
				$this->zipDestination= $destination;
				$res= $zip->open($destination, ZipArchive::CREATE);
				$this->analyseArchIO($res,'Open', 'Archive creation, path ='.$destination);
				return $res;
			break;

			// case 'add': return $this->addToArchive($file); break; // faster to call it directly.
		
			case 'close':
				$res= $zip->close();
		   		$this->analyseArchIO($res,'Close');
				return $res;
			break;
			
			default:
				$this->oError('filesMg::openCloseArch');
		}
	} // openCloseArch
	
	
	/**
	*/
	function analyseArchIO($code,$contextLbl='Open',$pathArchFN=''){
		if ($code===true){
			return true;	// formal return : no error
		}
		
		if ($code===false){
			return false;	// formal return
		}
		
		$glob= &$this->glob;
		
		$dspl= $contextLbl.' ZipArchive Error : ';
		$codeDspl= $code.' - ';
		switch ($code){
		
			/* ------------------------------------------------------------------
				Constants strictely specified under ZipArchive::open() doc.
			*/
			case ZipArchive::ER_EXISTS:
				$dspl.= $codeDspl.'ER_EXISTS';
			break;
			
			case ZipArchive::ER_INCONS:
				$dspl.= $codeDspl.'ER_INCONS';
			break;
			
			case ZipArchive::ER_INVAL:
				$dspl.= $codeDspl.'ER_INVAL';
			break;
			
			case ZipArchive::ER_MEMORY:
				$dspl.= $codeDspl.'ER_MEMORY';
			break;
			
			case ZipArchive::ER_NOENT:
				$dspl.= $codeDspl.'ER_NOENT';
			break;
			
			case ZipArchive::ER_NOZIP:
				$dspl.= $codeDspl.'ER_NOZIP';
			break;
			
			case ZipArchive::ER_OPEN:
				$dspl.= $codeDspl.'ER_OPEN';
			break;
			
			case ZipArchive::ER_READ:
				$dspl.= $codeDspl.'ER_READ';
			break;
			
			case ZipArchive::ER_SEEK:
				$dspl.= $codeDspl.'ER_SEEK';
			break;
			
			/* ------------------------------------------------------------------
				Supplement values see http://php.net/manual/en/zip.constants.php
				Suspicious about ZipArchive::close() returning non boolean values
			*/
			case ZipArchive::ER_TMPOPEN:
				$dspl.= $codeDspl.'ER_TMPOPEN';
			break;
			
			case ZipArchive::ER_CLOSE:
				$dspl.= $codeDspl.'ER_CLOSE';
			break;
			
			case ZipArchive::ER_ZIPCLOSED:
				$dspl.= $codeDspl.'ER_ZIPCLOSED';
			break;
			
			default:
				$dspl.= $codeDspl.'Unknown Error code!';
			break;
		} // switch
		
		if ($pathArchFN!==''){
			$pathArchFN= ' $pathArchFN='.$pathArchFN;
		}
		
		$dspl.= $pathArchFN . _c_;
		$glob->userError($dspl);
	} // analyseArchIO


	/**
		Extract one archive content.
			Input :
				$destDir : destination directory.
				$pathFN : the archive path.
			Output :
				Inflates the archive content in $destDir
	*/
	function openOneArch($destDir,$pathFN){
		$glob= &$this->glob;
		$tools= &$this->tools;
				
		$this->splitOnLastC($pathFN,$left,$file);
		if (!$this->emptyOrUnique($destDir,$file)){
			$glob->userHelp( $pathFN.' : '.$glob->el('To avoid accidental deletion...','kazg') );
			return false;
		}
			
		$zip = new ZipArchive;	// bid
		$res= $zip->open($pathFN);
		if ($res===true) {
		    $bool= $this->zipExtractTo($zip,$destDir);	// n zipExtractTo(
		    $res= $zip->close();
		    $this->analyseArchIO($res,'Close');
		    return $bool;		// true if successfully extracted
		}else{
			$this->analyseArchIO($res,'Open',$pathFN);
			return false;
		}	
	} // openOneArch
	
	/**
		Only to execute if dirIsEmpty($destDir)
	*/
	function zipExtractTo(&$zip, $destDir){
		$glob= &$this->glob;
		$tagsFu= &$this->tagsFu;
		$wmR= &$tagsFu->wmRights;
	
		if ($wmR->wmTechAcess()){
			$bool= $zip->extractTo($destDir);
			if (!$bool){
				print($glob->el('If a Php error appeared above with a .// indication ,,,','kmgn'));
			}
		}else{
			error_reporting(NULL);
			$bool= $zip->extractTo($destDir);
			error_reporting(E_ALL);
		}
		
		return $bool;
	} // zipExtractTo


	/**
		Abstraction of addFile() to trace it better.
			$file : path and file to add (given file) 
			$localname : local name inside the ZIP archive
	*/
	function addToArchive($file,$localname=NULL){
		$glob= &$this->glob;
		$gZip= &$this->globalZip;
		
		$errZip= false;
		$exists= file_exists($file);
		if ($exists){
			$readable= is_readable($file);
		}else{
			$readable= false;
		}
		
		if ($exists){
			if ($readable){
				$errZip= $gZip->addFile($file,$localname);
			}
		}
		
		if ($errZip!==true){
			if ($errZip===false){
				$errZip= '';
			}else{
				$errZip= ' odd addFile() error, code='.$errZip;	// normally should not happend because addFile() returns only booleans
			}
		
			$glob->userError($glob->el('Can\'t archive : ','kazc').$file.$glob->el(' at ','kazd').$localname.$errZip);
		}
		
		if (!$readable){
			$glob->userError($file.$glob->el(' is not readable!','kbao'));
		}
		
		if (!$exists){
			$glob->userError($file.$glob->el(' does not exists!','kayu'));
		}
		
		return $errZip;
	} // addToArchive


	/**
		Archive a file or a directory-file with the substraction of the given prefix size, if
			$prfxSz==='g' uses $glob->szComPrfxArchP as a constant for substraction.
		
			Input :
				$file : full name.of the file to read and archived
				$prfxSz : number of characters to substract from the left of $file to obtain its archive name.
	*/
	function subAddToArch($file, $prfxSz=0, $prfxLocN='./'){
		$glob= &$this->glob;
							
		if ($prfxSz==='g'){
			$prfxSz= $glob->szComPrfxArchP;
		}
		
		$localname= substr($file,$prfxSz);
		if ($prfxSz>0){
			$localname= $prfxLocN.$localname;
		}
		
		return $this->addToArchive($file,$localname);
	} // subAddToArch


	/**
		Scan the directories to archive the files with the path
			Input
				$srcPath : source path of the directory to archive.
				$prfxSz : size of the global prefix string to substract to avoid deep nested archive
				$maxFlSz : maximum file size to archive if given
				$prfxLocN : prefix local name to substract to avoid huge path inside the archives.
	*/
	function scanDirToArch($srcPath,$prfxSz=0, $maxFlSz=false,$prfxLocN='./'){
		$glob= &$this->glob;
		
		$glob->szComPrfxArchP= $prfxSz;		// so avoid clumsy recursive identical parameter. (historic reasons for glob)
		$this->maxFlSz= $maxFlSz;
		$this->prfxLocN= $prfxLocN;		
		$this->scanDirToArchive($srcPath);
	} // scanDirToArch
	
	
	/**
	*/
	function scanDirToArchive($srcPath){
		$glob= &$this->glob;
		$dtb= &$this->dtb;
		$gZip= &$this->globalZip;
		
		if ( is_dir($srcPath) && $handle = $this->locOpDir($srcPath) ){
			while (($file= readdir($handle)) !== false){				
				if ($file !== '.' && $file !== '..'){
					$curPath= $srcPath.'/'.$file;
					if (is_dir($curPath)){
						$this->scanDirToArchive($curPath);	// got to scan another directory !!
					}else{
						// Adds the current file
						if ($this->maxFlSz===false || filesize($curPath)<=$this->maxFlSz){
						
							if ($this->archChgPwFiles && $srcPath===_c_phpDtb._c_UsersTbl){
								$this->splitOnLastC($file,$bodyFl,$extFl,'.');
								
								if ( !array_key_exists($bodyFl, $this->plUsersTblIds) ){
									// We have a significant user Id
									$activeCPw= $dtb->getElem(_c_UsersTbl, _c_userId,$bodyFl, _c_cPwServer);
									$resCryptCPw= $dtb->getGenVar('reservedCryptedPw');
									
									$dtb->setElem(_c_UsersTbl,  _c_cPwServer,$resCryptCPw,  _c_userId,$bodyFl);
									$this->subAddToArch($curPath,'g',$this->prfxLocN);
									$dtb->setElem(_c_UsersTbl,  _c_cPwServer,$activeCPw,  _c_userId,$bodyFl);
								}
							
							}else{
								$this->subAddToArch($curPath,'g',$this->prfxLocN);
							}
						
						}else{
							$glob->userHelp($curPath.$glob->el(' is too large (above ','kaze').$this->maxFlSz.$glob->el(' bytes) to be archived!','kazf'));
						}
					}
				}
			} // while
			closedir($handle);
		} // if
		
		if (is_dir($srcPath)){
			$srcPath= substr($srcPath, $glob->szComPrfxArchP);	// substract the redondant prefix
			$ret= $gZip->addEmptyDir($srcPath);
		}else{
			if (isset($curPath)){
				$this->subAddToArch($curPath,'g',$this->prfxLocN);
			}else{
				$this->subAddToArch($srcPath,'g',$this->prfxLocN);
			}
		}
	} // scanDirToArchive


	/**
		CREATES A FULL ARCHIVE WITH ONLY ONE FUNCTION.
		
			Input :
				$vectFn : vector (or just one path without end slash) of all files and directories to archive, but as well just a simple string for one file to archive.
				$archDFn : the full archive file name.
	*/
	function smplArchCreation($vectFn, $archDFn, $prfxSz=0, $maxFlSz=false, $prfxLocN='./'){
		$this->openCloseArch($zip,'open',$archDFn);
		if (is_array($vectFn)){
			reset($vectFn);
			foreach ($vectFn as $key=>$srcPath){
				$this->scanDirToArch($srcPath,$prfxSz,$maxFlSz,$prfxLocN);
			}
		}else{
			$res= $this->scanDirToArch($vectFn,$prfxSz,$maxFlSz,$prfxLocN);
		}
						
		$this->openCloseArch($zip,'close');
	} // smplArchCreation


	/**
	*/
	function archSavior($pFn){
		$tools= &$this->tools;
		
		$this->splitOnLastC($pFn,$noExtPfn,$ext,'.');
		$zipTmpDfn= $noExtPfn.'.'._c_extZiptmp;
		if ( file_exists($pFn) ){
			rename($pFn, $zipTmpDfn);	// to avoid trying again ..
		}
	} // archSavior
	
	/* -----------------------------------------------------------------------------------
		Open All Archives !!
		Rem : $pathFN could be a directory, in that case don't unzip
	*/ 
	function opAllArchives($destDir,$pathFN, $zipDeletion= false){
		$tools= &$this->tools;
		
		$this->countOpArc++;
		$destDir= $tools->removeLastSlash($destDir);
		if ( $this->extension($pathFN)==='zip'  ){
			if ( file_exists($pathFN) && $this->dirGotOneFile($destDir) ){				
				$openRes= $this->openOneArch($destDir,$pathFN);	// ###  n openOneArch(
				if ($openRes){
					
					if ($zipDeletion){
						$this->unLnk($pathFN);	// strong case of deletion (not used internally)
					}else{
						// Weak case of deletion
						if ( $this->countOpArc>1 && $tools->postVal('ecrUpload')===_c_ok ){
							$this->unLnk($pathFN);
						}else{
							$this->archSavior($pathFN);
						}
					}
				}else{
					$this->archSavior($pathFN);
				}
			}
		}
		
		if ( is_dir($destDir) ){			
			if ($handle = $this->locOpDir($destDir)){
				while (($file= readdir($handle)) !== false){
					if ($file !== '.' && $file !== '..'){
						$srcPF= $destDir.'/'.$file;
						if (is_dir($srcPF) || $this->extension($file)==='zip'){
							if ( is_dir($srcPF) ){
								$this->opAllArchives($srcPF,$srcPF);
							}else{
								if ( !array_key_exists($destDir,$this->doneUnzippedPaths) ){
									$this->doneUnzippedPaths[$destDir]= _c_ok;
									$this->opAllArchives($destDir,$srcPF);
								}
							}
						}
					}
				} // while
				closedir($handle);
			}
		}
	} // opAllArchives
	
	
	/**
		Rename all the ._c_extZiptmp (1st level) files left inside the destination directory $dir, or 
		delete them if Ecr: check box is checked! Just on TOP level, NOT internally!
	*/
	function renameTmpZip($dir){
		$this->firstLevelFiles($v,$dir);
		reset($v);
		foreach ($v as $key=>$fullFn){
			$this->splitOnLastC($fullFn,$lFn,$ext,'.');
			if ($ext===_c_extZiptmp){
				$origDfn= $dir.$fullFn;
				$newDfn= $dir.$lFn.'.zip';
				rename($origDfn,$newDfn);
			}
		}
	} // renameTmpZip


	/** 
		The CORE function : return true if we run on the local server.+ returns the domain name via $sNam.
		Rem : atHome ---> localhost (allow a few differences on local machines like some  facilities to browse files).
	*/
	function atHome(&$dName){
		$dName= $this->domainName();
		return $dName === _c_localhost;
	} // atHome

	// Return true if we run on the local server.
	function atHme(){
		return $this->atHome($dName);
	} // atHme


	/**
		Action on directories
	*/ 
	function dsplDirTree($srcPath,$depth,$buff){
		$glob= &$this->glob;	// uses $glob for global "constant" or variables that are used by 2 or more classes, otherwise uses an instance of that class	
		$mm= &$glob->mediaMgPt;			
		$caps= &$this->caps;
	
		if (is_dir($srcPath)){
			$buffHz= '';
			$depth++;
			if ($handle = $this->locOpDir($srcPath)){
			
				// Scan the possible directories
				$i=0;
				while (($file= readdir($handle)) !== false){				
					if ($file!=='.' && $file!=='..'){
						$curPath= $srcPath.'/'.$file;
						if (is_dir($curPath)){
							$v[$i]= $curPath;
							$i++;
						}
					}
				} // while
				
				// Sort it before to look for other directories		
				if (isset($v[0])){
					if (!sort($v)){
						$this->topErr('filesMg::dirAction');
					}
										
					// Keep looking within the sorted vector
					$sz= $i;					
					for($i=0;$i<$sz;$i++){
						$curPath= $v[$i];
												
						
						$this->splitOnLastC($curPath,$left,$dsplDirNm);
						$dsplDirNm= $this->breakableFN($dsplDirNm,$glob->dirTreeWidthBrFn);
						$url= $mm->getNewUrl('2',$curPath);		// column 2
						$clickDirNm= $caps->linkTC($url,$dsplDirNm,'','treeFont');
						
						
						/**
							If the recursion is too long (scanDirTreeCnt>scanDirTreeLim), display
							whatever has been calculated from the diretory, mainly the first part with
							a fuzzy cut on the end.
						*/
						if ($glob->scanDirTreeCnt>=$glob->scanDirTreeLim){
							$buffHz.= $caps->div($clickDirNm,$glob->dirTreeItemcss);
							return $buffHz;
						}else{
							$glob->scanDirTreeCnt++;
						}
						
						$buff= $this->dsplDirTree($curPath,$depth,$buff);	// Recursion
						
						$buffHz.= $caps->div($clickDirNm.$buff,$glob->dirTreeItemcss);	// Concatenation
					}
				}
				
				closedir($handle);
			}
			return $buffHz;
		}
	} // dsplDirTree


	/**
	*/
	function locOpDir($d){
		if (is_readable($d)){
			return opendir($d);
		}else{
			return false;
		}
	} // locOpDir

	/**
		Gives a directory/file name with yearMonthDayHourMinuteSeconds (easier for alphab. ordering)
	*/
	function timeStamp($gTime=false, $onlyDays=false){
		$datom= $this->dateAtom($gTime);
		$this->splitOnLastC($datom,$newStamp,$right,'+');
		$newStamp= str_replace(':','-',$newStamp);
		
		if ($onlyDays){
			return substr($newStamp,0,10);	
		}else{
			return str_replace('T','t',$newStamp);
		}
	} // timeStamp
	
	/**
	*/
	function dateAtom($gTime=false){
		if ($gTime===false){
			$gTime= time();
		}
		return date(DATE_ATOM,$gTime);
	} // dateAtom
	
	
	/**
		"Random" time stamp : so a time stamp with a random string.
	*/
	function rndTmStamp($gTime=false){
		$hPwC= &$this->hPwC;
		return $this->timeStamp($gTime=false).$hPwC->consonStr(5);
	} // rndTmStamp
	

	/**
		Generates an unpronouncable random string.
			Input :
				$sz : size of the random string.
			Output :
				returns thes random string.
	*/
	function rndStr($sz){
		$base= "BCDFGIJKLMNPQRSTVWXZbcdfghjklmnpqrstvwxwz0123456789";
		$szM1= strlen($base)-1;
	
		$rndStr='';
		for ($i=0; $i<$sz; $i++){
			$idx= rand(0,$szM1);
			$rndStr.= $base[$idx];
		}
		return $rndStr;
	} // newRndStr
	
	
	/**
		Hybrid chronological string with a random part for obfuscation to eve dropping.
			Input :
				$sz : size of the random part..
			Output :
				returns thes random chrono string.
	*/
	function hybridChronoStr($sz){
		return $this->timeStamp().'-'.$this->rndStr($sz);
	} // hybridChronoStr


	/**
		Saves the given string into the given directory with a calculated hybridChrono name.
			Input :
				$dir : directory to save the file in (given with a terminal slash).
				$str : string to be saved.
				$sz : size of the random suffix.
				$ext : extension of the file name.
	*/
	function saveToHybChron($dir,&$str,$sz=5,$ext='.htm'){
		$fName= $dir.$this->hybridChronoStr($sz).$ext;
		$this->strToFile($fName,$str);
	} // saveToHybChron

	/**
		Input :
			$dir : directory to save the file in (given with a terminal slash).
			$sz : size of the random suffix.
	*/
	function hybridChronoFN($dir,$sz=5,$ext='.htm'){
		return $dir.$this->hybridChronoStr($sz).$ext;
	} // hybridChronoFN


	/**
		Local abstraction for manipulation convenience.
	*/
	function fileSz($fn){
		return filesize($fn);
	} // kbFlSz
	
	/**
	*/
	function kbFlSz($dfn){
		if (file_exists($dfn)){
			$bSz= filesize($dfn);
			return floor($bSz/1000);
		}else{
			return '';
		}
	} // kbFlSz

	/**
		Give the size string of a file in Kb for display
	*/
	function kbFileSz($dfn){
		$bSz= filesize($dfn);
		return $this->fromByteSz($bSz);
	} // kbFileSz
	
	
	function fromByteSzOld($bSz){
		return floor($bSz/1000) ._c_Ns.'Kb'._c_Ns.'+'._c_Ns.($bSz % 1000) . _c_Ns .'bytes';
	} // fromByteSz
	
	
	function fromByteSz($bSz){
		$mega= (int)floor($bSz/1000000);
		
		$roundMBytes= $mega*1000000;
		$kilo= (int)floor(($bSz-$roundMBytes)/1000);
		
		$roundKBytes= $kilo*1000;
		$unit= $bSz - $roundMBytes - $roundKBytes;
		
		$buff= '';
		if ($mega>0){
			$buff.= $mega._c_Ns.'Mb'._c_Ns.'+'._c_Ns;
		}
		
		if ($kilo>0){
			$buff.= $kilo._c_Ns.'Kb'._c_Ns.'+'._c_Ns;
		}
		
		$buff.= $unit._c_Ns.'bytes';
		return $buff;
	} // fromByteSz
	
	

	/**
	*/
	function rightsStr($dfn){
		$glob= &$this->glob;
		return $glob->el('Rights : ','kacn').$this->filepermStr($dfn);
	} // rightsStr
	
	/**
	*/
	function filepermStr($dfn){
		return substr(sprintf('%o', fileperms($dfn)), -4);
	} // filepermStr
	
	
	/**
		Rem : could be accelarated by removing the $recurs param when it's at ok
	*/ 
	function setDirRights($srcPath,$recurs, $dirMode=_c_stdFoldersRights,$flMode=_c_stdFilesRights){
		if ($recurs===_c_ok && is_dir($srcPath) && $handle = $this->locOpDir($srcPath)){
			while (($file= readdir($handle)) !== false){				
				if ($file !== '.' && $file !== '..'){
					$curPath= $srcPath.'/'.$file;
					if (is_dir($curPath)){
						$this->setDirRights($curPath,$recurs, $dirMode,$flMode);	// Set the rights of another directory !!
					}else{
						$this->setFileRights($curPath,$flMode);	
					}
				}
			} // while
			
			closedir($handle);
		}
		
		if (is_dir($srcPath)){
			$this->setFileRights($srcPath,$dirMode);	
		}else{
			$this->setFileRights($srcPath,$flMode);	
		}
	} // setDirRights

	/**
	*/
	function setFileRights($phFile, $flMode){
		if (!$this->chgMode($phFile,$flMode)){
			$glob= &$this->glob;	
			$glob->userError('Can\'t change the rights of '.$phFile.' into '.$flMode);
		}
	} // setFileRights	
	
	
	/**
		Unspecified category writing rights / general writing rights.
	*/
	function genWRights($g){
		return $g=='2' || $g=='3' || $g=='6' || $g=='7';
	} // genWRights
	
	
	/**
	*/
	function vectPermStr(){
		return array(
			'0'=>'---',	// 000
			'1'=>'--x',	// 001
			'2'=>'-w-',	// 010
			'3'=>'-wx',	// 011
			'4'=>'r--',	// 100
			'5'=>'r-x',	// 101
			'6'=>'rw-',	// 110
			'7'=>'rwx'	// 111
			);
	} // vectPermStr


	/**
	*/
	function dateStr($url){
		$glob= &$this->glob;
		return $glob->el('Modified : ','kaco').$this->fileDate($url);
	} // dateStr
	
	
	
	/**
		Abstraction for convenience
	*/
	function fileModTm($dfn){
		return filemtime($dfn);
	} // fileModTm
	


	/**
	*/
	function fileDate($url){
		$glob= &$this->glob;
		$dtb= &$this->dtb;
		
		$fTm= filemtime($url);
		$dateStr= date("F d Y H:i:s.", $fTm);
		
		
		$enMonth= date('F',$fTm);
		$monthIdx= date('n',$fTm)-1;
		
		if ($this->globMonthVect===false){
			$lg= $glob->esLg();
			$this->globMonthVect= $this->monthVector($lg);
		}
		
		$translatMonth= $this->globMonthVect[$monthIdx];
		$dateStr= str_replace($enMonth,$translatMonth,$dateStr);
		return $dateStr;
	} // fileDate


	/**
	*/
	function monthVector($lg){
		$glob= &$this->glob;
		$tools= &$this->tools;
		$monthStr= $glob->st('January, February, March, April, May, June, July, August, September, October, November, December','stMonthString',$lg);
		return $tools->mbExplodeAndTrim($monthStr);
	} // monthVector


	/**
		Input
			$dfn : full file or directory name
	*/
	function genFileInfos($dfn,$prfx=''){
		if ($prfx!==''){
			$s= $prfx.' : ';
		}else{
			$s= '';
		}
		
		if (is_file($dfn)){
			$s.= $this->kbFileSz($dfn).' -- ';
			
			$this->splitOnLastC($dfn,$path,$fn,'/');
			$this->splitOnLastC($fn,$onlyFn,$ext,'.');
			
			if ($this->imgOkToDspl($ext)){
				$s.= $this->whStr($dfn).' -- ';
				
				$thbDfn= $path.'/'._c_thumbsDirN.'/'.$fn;
				if ( file_exists($thbDfn) ){
					$s.= $this->whStr($thbDfn,'tW','tH').' -- ';
				}
			}
		}	// else assume it to be a directory or a link	
		
		$s.= $this->rightsStr($dfn).' -- '.$this->ownerStr($dfn).' -- '.$this->dateStr($dfn);			
		return $s;
	} // genFileInfos

	/**
		Thumbs DFN from original image DFN
			If $path & $fn are given does not calculate them.
	*/
	function thumbFromOrig($dfn,$path='',$fn=''){
		if ($fn===''){
			$this->splitOnLastC($dfn,$path,$fn,'/');
		}
		return $path.'/'._c_thumbsDirN.'/'.$fn;
	} // thumbFromOrig
	
	/**
		Return the current thumbs directory from any local orginale file name
	*/
	function curThumbPath($dfn){
		$this->splitOnLastC($dfn,$path,$fn,'/');
		return $path.'/'._c_thumbsDirN;
	} // curThumbPath
	
	
	/**
		Return a string with the W & H textual information of an image.
	*/
	function whStr($dfn,$remW='W',$remH='H'){
		$tools= &$this->tools;
		$tools->imageDim($w,$h,$dfn);
		return $remW.'='.$w.' '.$remH.'='.$h;
	} // whStr
	

	/**
		Ok for thumbs image extension/type
	*/
	function imgOkToDspl($ext){
		$ext= strtolower($ext);		
		return $ext==='jpg' || $ext==='jpeg' || $ext==='png' || $ext==='gif'
			|| $ext==='ico' || $ext==='bmp' || $ext==='svg' || $ext==='jp2';  
	} // imgOkToDspl
	
	/**
		Returns true if we have a annoying/unsightly jpeg type
	*/
	function unJPEG($ext){
		$extLow= strtolower($ext);
		if ($extLow==='jpg' || $extLow==='jpeg'){
			return $ext!=='jpg';	// // got a jpg ...  unsightly if !== from jpg
		}else{
			return false;	// not a jpg
		}
	} // unJPEG
	
	/**
	*/
	function imposejpg($dfn){
		$this->splitOnLastC($dfn,$extFreeDfn,$ext,'.');
		if ( $this->unJPEG($ext) ){
			$this->splitOnLastC($extFreeDfn,$path,$fn,'/');
			$oldFnExt= $fn.'.'.$ext;
			$newFnExt= $fn.'.jpg';
			$pathSlh= $path.'/';
			$this->secureRename($pathSlh,$oldFnExt, $pathSlh,$newFnExt);
			return $pathSlh.$newFnExt; // new dfn
		}else{
			return $dfn; 	// could not impose anything, return orginal
		}
	} // imposejpg
	
	
	/**
	*/
	function imposeFnjpg($fn){
		$this->splitOnLastC($fn,$fnBdy,$ext,'.');
		if ( $this->unJPEG($ext) ){
			return $fnBdy.'.jpg';
		}else{
			return $fn; 	// could not impose anything, return orginal
		}
	} // imposejpg
	
	

	/**
	*/
	function imageType($fn){
		$this->splitOnLastC($fn,$left,$ext,'.');
		return $this->imgOkToDspl($ext);
	} // imageType
	
	
	/**
	*/
	function fullImgNmOkTDspl($fn){
		$this->splitOnLastC($fn,$left,$ext,'.');
		return $this->mgOkToDspl($ext);
	} // fullImgNmOkTDspl





	/* -----------------------------------------------------------------------------------
		Scan the whole given directory.or file and apply the given method on each files
		directly given or full deepness inside the directories.
		Input :
			$srcPath,&$curClass,$methodName : ... obvious!
			$dealFirstLvlFl : if true, deal directly with top level files!
		
	*/
	function scanWholeDir($srcPath,&$curClass,$methodName,$dealFirstLvlFl=true){
		$this->gCurClass= &$curClass;
		$this->gMethodName= $methodName;
		$this->dealFirstLvlFl= $dealFirstLvlFl;
		$this->scanWhlDir($srcPath);
	} // scanWholeDir

	/**
		Recursive scan of the directory $srcPath (given without slash) and 
		apply the method $curClass->$methodName() to all elements!
	*/ 
	function scanWhlDir($srcPath){
		$curClass= &$this->gCurClass;
		$methodName= &$this->gMethodName;
	
		if (is_dir($srcPath)){
			$curClass->$methodName($srcPath);
		}else{
			if ($this->dealFirstLvlFl){
				$curClass->$methodName($srcPath);
			}
			return;
		}
	
		if ($handle = $this->locOpDir($srcPath)){
			while (($file= readdir($handle)) !== false){
				if ($file !== '.' && $file !== '..'){
					$srcPF= $srcPath.'/'.$file;
					if (is_dir($srcPF)){
						$this->scanWhlDir($srcPF);	// got to scan another directory !!
					}else{
						$curClass->$methodName($srcPF);
					}
				}
			} // while
			closedir($handle);
		}
	} // scanWhlDir       
	
	/**
		Simple scan of the directory $srcPath (given without slash) and 
		apply the method $curClass->$methodName() to all elements 
		or only to files depending on the value of $applyAll
	*/ 
	function scanDir($srcPath, &$curClass,$methodName,$onlyFN=false,$applyAll=false){
		if ($rsrc = $this->locOpDir($srcPath)){
			while (($file= readdir($rsrc)) !== false){
				if ($file !== '.' && $file !== '..'){
					$srcPF= $srcPath.'/'.$file;
					if ($applyAll || !is_dir($srcPF)){
						if ($onlyFN){
							// when we are only interested to operate on the file name (rare)
							$curClass->$methodName($file);
						}else{
							$curClass->$methodName($srcPF);
						}
					}
				}
			} // while
			closedir($rsrc);
		}
	} // scanDir	
	
	
	
	/**
		Manage an easy file output when we can't print on the screen because of cookies interference!
	*/
	function concatDumpStr($addedStr){
		$this->curDumpContent.= $addedStr;
	} // concatDumpStr
	
	function fileDumpStr(){
		if ($this->curDumpContent!==''){
			$curDumpDFN= _c_mainDumpDrPth.$this->timeStamp().'.txt';
			$this->strToFile($curDumpDFN,$this->curDumpContent);
		}
	} // fileDumpStr	


	/**
		Concat information on the main secret log file
		!! Need strict caution to avoid too large files --> don't store strings larger than X, identical ones etc...
		test appartenance to $this->alreadySawIt[]
	*/
	function appendOnLogFile($gStr,$origin){
		$tools= &$this->tools;
		$dtb= &$this->dtb;
		
		if ($dtb->getGenVar('secreLogDumpActive')===_c_ok && !in_array ($gStr,$this->alreadySawIt,true) ){
			$storedStr= _c_R;
			if ($this->virginLogShot){
				$storedStr.= _c_R.'IP= '.$tools->remtAddr()._c_R;
			}
		
			$storedStr.= $origin.' : '.$gStr;
			$dfn= _c_mainDumpDrPth.$dtb->getGenVar('secreLogDumpFN');
			$this->strToFile($dfn,$storedStr,'a',true);
			array_push($this->alreadySawIt,$gStr);		// udem ... $this->alreadySawIt[]= $gStr;
			$this->virginLogShot= false;
		}
	} // appendOnLogFile


	/**
		Return true if the file is an image
		EXIF. Stands for "Exchangeable Image File Format."
		Remark faster to use : 
			return exif_imagetype($dFn)!==false;
		Not safe anyway, to be sure against uppload-hackers should conver the would be image
	*/
	function isImage($dFn){
		$sz= filesize($dFn);
		if ($sz >= 12){
			// Could be an image ... also otherwise can't apply the function without generating an error!
			$type= exif_imagetype($dFn);
			if ($type===false){
				return false;
			}else{
				return true;
			}
		}else{
			return false;
		}
	} // isImage
	

	/**
		Getting there because the php doc indicates :
			Avoid using umask()in multithreaded webservers. It is better to change the 
			file permissions with chmod()
		! chgMode() Should have been named change rightrs!!!
	*/
	function chgMode($fn, $mode= _c_stdFoldersRights){
		$glob= &$this->glob;

		if ( $this->isOwner($fn) && $glob->cndtWrRghts===true){
			// changes the mode only if the script user is the file owner
			return chmod($fn,$mode); 
		}else{
			$this->appendOnLogFile($fn.' - '.$mode, 'chgMode');
			return false;
		}
	} // chgMode
	
	
	/**
	*/
	function ownerStr($dfn){
		$glob= &$this->glob;
		
		$ownerN= $this->fileOwnerNm($dfn);
		return $glob->el('Owner : ','kacm').$ownerN;
	} // ownerStr
	
	
	function fileOwnerNm($dfn){
		$dtb= &$this->dtb;
			
		$ownerId= $this->fileOwnerId($dfn);
		if ($dtb->getGenVar('posixFuAllowed')===_c_ok){
			$ownerV= posix_getpwuid($ownerId);
			return $ownerV['name'];
		}else{
			if ($this->isOwner($dfn)){
				return get_current_user();
			}else{
				// We don't know the string matching the user Id, so just return the Id
				return 'id='.$ownerId;
			}
		}
	} // fileOwnerNm


	/**
		Returns true if the current user is the owner of the file.
	*/
	function isOwner($dfn){
		return $this->scriptUserId()===$this->fileOwnerId($dfn);
	} // isOwner
	
	function fileOwnerId($dfn){
		return fileowner($dfn);
	} // fileOwnerId
	
	function scriptUserId(){
		return getmyuid();
	} // scriptUserId	


	/**
	*/
	function alertSafeMode(&$message){
		$glob= &$this->glob;
		$tagsFu= &$this->tagsFu;
		$wmRights= &$tagsFu->wmRights;
	
		$this->splitOnLastC(_c_asppRtPath,$ataoxPathNS,$emptyStr);
		if ( $this->isWritable($ataoxPathNS) ){
			// We may have a case of "safe" mode, need to try further
			$dN= 'testFM';	// test saFe Mode
			$dfn1= _c_UsrDataPath.$dN;
			
			$dfn2= $dfn1.'/'.$dN;
			$dfn3= $dfn2.'/'.$dN;
			$dfn4= $dfn3.'/text.txt';
			
			$repMin= !$wmRights->erroReportMax();
			if ($repMin){
				error_reporting(NULL);	// don't want an error blur for our new user !!!!
			}			
			
			$this->wMkDir($dfn1);
			$this->wMkDir($dfn2);
			$this->wMkDir($dfn3);
			$this->strToFile($dfn4,'Test!');
			error_reporting(E_ALL);
			
			if ($this->fileExist($dfn4)){
				// The only transélation to do, because other won't generally appear (except if the server get security paranoïd!) becasue the system was installed
				$message= $glob->el('The system has enough rights','kaxv');
				return false; // fine no safe mode alert, nothing to write		
			}else{
				$message= $glob->el('Alert : we could not create and read : ','kaxw').$dfn4.$glob->el(' so you probably have a strong freedom restriction','kaxx');
				return true;
			}
		
		}else{
			// Can't write insiude the main directory
			$message= $glob->el('You do not have writing rights on ','kaxy').$ataoxPathNS.$glob->el(' please allow all writing rights "777" ','kaxz');
			return true;	// we have an alert wrong rights on files, may be safe mode too, but tackled on second round!
		}
	} // detectSafeMode

		
	/**
		Define a specific row element in simul sql two : file atom based dtb --> atomDtb
	*/
	function atomDtbCelStr($key,$value){
		$acc= &$this->acc;

		 // $r[...]= ...
		return _c_sSmTwRowArNm.'['._c_snglQ.
			$acc->backSlashC($key,_c_snglQ).
			_c_snglQ.']='._c_snglQ.
			$acc->backSlashC($value,_c_snglQ).
			_c_snglQ.';'._c_R;
	} // atomDtbCelStr
	
			
	/** -----------------------------------------------------
		Installer part :
	*/

	/**
		Return true if we have an index and main data folder name assigned with user defined strings (polyvalent strings) 
		to fit inside the same directory. Return false otherwise.
		filesMg::polyvalentIndex()
	*/
	function polyvalentIndex(){
		$tools= &$this->tools;
		$dtb= &$this->dtb;
		$wmRights= &$this->wmRights;
		
		if (file_exists('./masterFlag.php')){
			return false;
			/*
				It also means that id a master distribution wants to build its own sub.versions, it needs to remove its master flag
				--> at the start manually, may be after automatically !!
			*/
		}
		
		if ($tools->postVal('sortOfDistrib')!==false){
			return false;
		}
		
		if ($wmRights!==null && $wmRights->buildButtAccept() ){
			return true;
		}else{
			return false;
		}
	} // polyvalentIndex

	
	/**
	*/
	function loadsVersionDtDir(&$version,&$dataDir){
		if ($this->polyvalentIndex()){				
			$version= $this->getVersionIdx();
			$dataDir= $this->postOrDtb('installerMainDirName');
		}else{
			$version= '';
			$dataDir= _c_usrStandardDirN;
		}
	} // loadsVersionDtDir
	
	
	/**
	*/
	function getVersionIdx(){
		$tools= &$this->tools;
		
		$accptIdx= $tools->postVal('mNmAcceptIndx',_c_ko);
		if ($accptIdx===_c_ok){
			$indx= $this->postOrDtb('installerMDirNmIdx');
		}else{
			$indx= '';
		}
		return $indx;
	} // getVersionIdx
		
	
	/**
		Probe the post or the Dtb for a value.
	*/
	function postOrDtb($wdgtNm){
		$tools= &$this->tools;
		$dtb= &$this->dtb;

		$wdgVal= $dtb->getGenVar($wdgtNm,'');
		return $tools->postVal($wdgtNm,$wdgVal);
	} // postOrDtb	

	
	/**
	*/
	function builderDestPath(){
		$this->loadsVersionDtDir($version,$dataDir);
		$dataDir.= $version;
		return _c_ProjectsPath.$dataDir.'/';
	} // builderDestPath
					
} // filesMg
?>
