<?php
/**
	File name : genAssoc.php
	Author of this file : Oscar Fawer until 2019-08-15
	Contact : www.ataox.org 
	Web sites : ataox.org / ataox.com / ataox.info
	Copyrights open source GPL, GNU General Public License V3 : see ./aMainDr/code/LICENSE.txt


	A general class to deal with associative table --> General Associative Table. (gat)
	--> mostly for temprary values !!!
	
	Sort of problems to solve :
		- a user eMail waiting for approval through human email checking! Don't want a simple numerical id, 
		because this numerical id will also need to be stored somewhere --> need "balise" directly from
		the standard available data! -->
			Main Context : concerned table, main entity (exemple : forum user)
			Sub Context : contextual addition to the main entity (concerned email address)
			
			So storage / retriving process --> using the stored values in _c_gatContext and _c_gatSubCtxt!
			--> _c_gatId is just there to have a unique id for each possible row, but is not used to access
			the data!
			
	Advantages :
		- Don't need to concat names to have unique long string Ids.
		- "?" Shows that yes there is some associatuive table activity in this CMS
	Inconvenient :
		- Slow retrieval (so mainly for temporary data --> keeps the table small)
*/
class genAssoc extends oTop{
	

	function __construct(&$tagsFu){
		parent::__construct();

		$this->glob= &$tagsFu->glob;
		$this->acc= &$tagsFu->acc;
		$this->caps= &$tagsFu->caps;
		$this->fls= &$tagsFu->fls;
		$this->tools= &$tagsFu->tools;
		$this->dtb= &$tagsFu->dtb;	
		$this->aSql= &$tagsFu->aSql;	
		
		
		define('_c_gatName','genAssoc');
			define('_c_gatId','id');	// numerical string ids	(str 20 pos)
			define('_c_gatContext','context');	// main context
			define('_c_gatSubCtxt','subCtxt');	// sub context
			
			// Rescue context, in case of too small storage place in the one above (string to store any possible search)
			define('_c_gatRescueCtxt','rescueCtxt');
			define('_c_gatTime','time');
			define('_c_gatInfo','info');		// long string, just in case
			define('_c_gatInfSup','infSup');	// just a string (not implemented now)
		
	} // construct


	/**
		Standard get, will do the _c_gatRescueCtxt only if needed.
		
		Could transfer these low level into the specific database package !
		------> For now count as an abstract layer by iself, can't iincrease the size of bottom classes for ever!!!
	*/
	function get($ctx,$subCtx=false){
		$dtb= &$this->dtb;
						
		$qObj= $this->getQuerryObj($ctx,$subCtx);
		if ($qObj===false) return false;
				
		$curAry= $dtb->fetchAry($qObj);
		return $curAry[_c_gatInfo];
	} // get


	/**
	*/
	function getQuerryObj($ctx,$subCtx=false){
		$glob= &$this->glob;
		$aSql= &$this->aSql;
		$dtb= &$this->dtb;
	
		if ($subCtx===false){
			$cd1= $aSql->basicCond(_c_gatContext,$ctx);
		}else{
			/*
				simulSql : Transforms $cd1 into an array of arrays !!!!!!!! Need local sql fu for compatibility 2 DTB kinds!!!!!!!!!!
				aMySql : nothing to do	
			*/
			// $cd1= array(0 => $aSql->basicCond(_c_gatContext,$ctx));	// so array of arrays matching what's done with ->andCondition()
			$cd1= $aSql->transForMultiplCond( $aSql->basicCond(_c_gatContext,$ctx) );
			$cd2= $aSql->basicCond(_c_gatSubCtxt,$subCtx);
			$aSql->andCondition($cd1,$cd2);		// $cd1 get loaded with the final AND condition.
		}
		
		return $dtb->slctResource(_c_gatName,$cd1);	// see $aSql->selectResource() !!!!!!!!
	} // getQuerryObj
	
		
	/**
	*/
	function store($ctx,$subCtx,$info,$infSup=''){
		$dtb= &$this->dtb;
		
		$this->deleteAll($ctx,$subCtx);
		$max= $dtb->maxColValue(_c_gatName,_c_gatId);
		$max= (int)$max + 1;
		$id= (string)$max;
		$infV= array($id,$ctx,$subCtx,'',time(),$info,$infSup);
		$dtb->newRecord(_c_gatName,$infV);
		return $id;
	} // store


	/**
		Delete all fields with the given conditions
		$condition
	*/
	function deleteAll($ctx,$subCtx=false){
		$aSql= &$this->aSql;
	
		$condition= $aSql->basicCond(_c_gatContext,$ctx);
		if ($condition!==''){
			if ($subCtx!==false){
				/*
					simulSql : Transforms $condition into an array of arrays !!!!!!!! Need local sql fu for compatibility 2 DTB kinds!!!!!!!!!!
					aMySql : just returns $condition	
				*/
				// $condition= array(0 => $condition);
				$condition= $aSql->transForMultiplCond($condition);
				$aSql->andCondition($condition, $aSql->basicCond(_c_gatSubCtxt,$subCtx) );
			}
			
			return $aSql->deleteSqlTableRec(_c_gatName,$condition);
		}
	} // deleteAll
	

	/**
		Delete a specific Id row
	*/
	function delSpecicRow($idx){
		$dtb= &$this->dtb;
		$dtb->deleteTableRecord(_c_gatName,_c_gatId,$idx);
	} // delSpecicRow
	

	/**
		Get the 2 info fields!
	*/
	function getInfos(&$strA,&$strB,$id){
		$dtb= &$this->dtb;
	
		$rec= $dtb->getElem(_c_gatName, _c_gatId,$id);
		$strA= $rec[_c_gatInfo];
		$strB= $rec[_c_gatInfSup];
	} // getInfos
	

} // genAssoc
?>
