<?php
/**
	File name : geomArray.php
	Author of this file : Oscar Fawer until 2019-08-15
	Contact : www.ataox.org 
	Web sites : ataox.org / ataox.com / ataox.info
	Copyrights open source GPL, GNU General Public License V3 : see ./aMainDr/code/LICENSE.txt


    A class to define a pseudo-geometrical array that is practical to scan quickly a greater contiguous array without
    having to display all occurrences of the original array! It's a pseudo geometrical progression because the neighbors
    of the activated pages are coerced into contiguity so we can easily reach them.
    	
	Improvment : could try to have a constent length! of the bar! The simplest way is probably  :
		- Filling holes when too short!.
		- Increasing multiplying last factor when too long! Then filling holes if too short!
*/
class geomArray extends oTop{
	
	

	function __construct(&$tagsFu){
		parent::__construct();
		
		$this->glob= &$tagsFu->glob;
		$glob= &$this->glob;
		
		$this->caps= &$tagsFu->caps;	// access many html methods
		$this->tools= &$tagsFu->tools;
		$this->dtb= &$tagsFu->dtb;
		$dtb= &$this->dtb;
		
		
		$glob->addCssFile(_c_LibPath.'geomArray.css');
				
		if (!defined ('_c_geoTblChunkId')){
			define('_c_geoTblChunkId','chunkId');
		}
		
		// $this->curChunkLength= $dtb->getGenVar('tedTblScanSz');
		$this->setChunkLength($dtb->getGenVar('tedTblScanSz'));
	} // construct

	

	/**
	*/
	function setChunkLength($sz){
		$this->curChunkLength= $sz;
	} // setChunkLength

	/**
		Returns a geometrical array from a defined original contiguous space in Z.
			Input :
				$oStrt : origin start.
				$oEnd : origin end.
				$curPos : current position.
				$progres : sort of geometric progression.
	*/
	function contigToGeom($oStrt,$oEnd,$origCurPos,$progres='short'){
		$firstVect= array();
		$realStartV= array();
		if ($origCurPos!=='all'){
			if ($origCurPos>$oEnd){
				$origCurPos= $oEnd;
			}
		
			// FIRST ARRAY
			$curStep= 1;
			$dist= 1;
			$curPos= $origCurPos-1;
			while($curPos>=$oStrt){
				$curStep= $this->nextArithmIncr($dist,$curStep,$progres);
				$firstVect[$dist]= $curPos;
				$lastFVIndx= $dist;
				
				$curPos= $curPos - $curStep;
				$dist++;
			} // while
			
			$soFstV= sizeof($firstVect);
			if ($soFstV>0){
				if ((int)$firstVect[$lastFVIndx]!==(int)$oStrt){
					$realStartV[0]= $oStrt;
				}
			}
			$firstVect= array_reverse($firstVect);
		}
	
		// MIDDLE ARRAY (ONLY ONE ELEMENT)
		$midVect= array();
		if ($origCurPos!=='all'){
			$midVect[0]= $origCurPos;
		}
		
		
		if ($origCurPos==='all'){
			$origCurPos= 0;
		}
		
		// THIRD ARRAY
		$thirdVect= array();
		$curStep= 1;
		$dist= 1;
		$curPos=$origCurPos+1;
		while($curPos<=$oEnd){
		
			$curStep= $this->nextArithmIncr($dist,$curStep,$progres);
			$thirdVect[$dist]= $curPos;
			$lastSVIndx= $dist;
			
			$curPos= $curPos + $curStep;
			$dist++;
		} // while
	
		$realEndV= array();
		if (sizeof($thirdVect)>0){
			if ((int)$thirdVect[$lastSVIndx]!==(int)$oEnd){
				$realEndV[0]= $oEnd;
			}
		}
		
		$totArray= array_merge($realStartV,$firstVect,$midVect,$thirdVect,$realEndV);		
		return $totArray;
	} // contigToGeom
	

	/**
		Gives the next arithmetical increment,along a geometrical progression
			Input :
				$dist : actual distance from the activated button
				$curStep : actual step (previously calculated)
				
			Output :
				returns the space (next $step) to add to the original set.
	*/
	
	/**
	*/
	function nextArithmIncr($dist,$curStep,$progres){
		switch ($progres){
			case 'short':
				return $this->shortGeomProgr($dist,$curStep);
			break;
			
			
			case 'long':
				return $this->longGeomProgr($dist,$curStep);
			break;
			
			
			default: print('error in nextArithmIncr');
		} // switch
	} // chabl
	
	
	/**
	*/
	function shortGeomProgr($dist,$curStep){
		switch ($dist){
			case 1:
			case 2:
			case 3:
				return 1;
			break;

			case 4:
			case 5:
				return 3;
			break;
			
			default:
				return 2*$curStep;
		} // switch
	} // shortGeomProgr
	
	
	/**
	*/
	function longGeomProgr($dist,$curStep){
		switch ($dist){
			case 1:
			case 2:
			case 3:
			case 4:
			case 5:
			case 6:
				return 1;
			break;
			

	
			case 7:
			case 8:
			case 9:
			case 10:
				return 3;
			break;
			
			
			
			case 11:
			case 12:
			case 13:
				return 6;
			break;
			
			
			case 14:
			case 15:
				return 12;
			break;
			
			
			default:
				return 2*$curStep;
			
		} // switch
	} // longGeomProgr
	
	
	/**
		Display a given vector of numbers using a parameter function to get the right URL
		Input:
			$v : given vector
			$givCls : goven class
			$givUrlF : goven url FUNCTION (for specific clicks on numbers)
			$urlIdxVar : current index variable.
	*/
	function dsplNumericMenu($v, $givCls,$givUrlF,$urlIdxVar,$neutralElem='#',$sep=' - '){
		$caps= &$this->caps;
		$tools= &$this->tools;
	
		$idIn= $tools->getVal($urlIdxVar,1);
		$soV= sizeof($v);
		if ($soV<=0){
			return '';
		}		
		
		$oEnd= (int)$v[$soV-1];
		if ($idIn>$oEnd){
			$idIn= $oEnd;
		}
		
		$prevVal= false;
		if ($neutralElem===false){
			$buff= '';
		}else{
			$url= $givCls->$givUrlF('all');	// ask for a neutral/all element
			$buff= $caps->linkC($url,$neutralElem, $this->neutralElemFont($idIn) ).$caps->span($sep,'geArStd');
		}
		
		reset($v);
		foreach ($v as $key=>$val){
			if ($prevVal!==false){
				$url= $givCls->$givUrlF($prevVal);
				$buff.= $caps->linkC($url,$prevVal,$this->nbMenFont($prevVal,$idIn)).$caps->span($sep,'geArStd');
			}
			$prevVal= $val;
		}
		
		$url= $givCls->$givUrlF($val);		
		$buff.= $caps->linkC($url,$val,$this->nbMenFont($val,$idIn));
		return $buff;
	} // dsplNumericMenu
	
	/**
		$this->nbMenFont($val,$idIn)
	*/
	function nbMenFont($i,$idIn){
		if ((int)$i===(int)$idIn){
			return 'geArStdIn';
		}else{
			return 'geArStd';
		}
	} // nbMenFont
	
	
	/**
		$this->neutralElemFont($idIn)
	*/
	function neutralElemFont($idIn){
		if ($idIn==='all'){
			return 'geArStdIn';
		}else{
			return 'geArStd';
		}
	} // neutralElemFont


	/**
		Does the full work, excpept that a function calculating the url for each 
		element of the numerical scale must be given.
		
			Input :
				$curTblN : current table name.
				$userId
				$givCls : given class.
				$givUrlF : given url function.
				
		Call example :
			$geom->geomArrDisplay(_c_forumPostsTbl, $this, 'postChunkUrl');
	*/
	function geomArrDisplay($curTblN,$userId, $givCls,$givUrlF){
		$caps= &$this->caps;
		$tools= &$this->tools;
		$dtb= &$this->dtb;
		
		$tblChunkId= $tools->getVal(_c_geoTblChunkId,1);
		
		
		// $chunkLength= $dtb->getGenVar('tedTblScanSz');
		$chunkLength= $this->curChunkLength;
		
		
		$totRowNb= $dtb->cntNbRows($curTblN,_c_fumFUserId,$userId);
		
		$totNbChunks= ceil($totRowNb/$chunkLength);
		
		if ($tblChunkId>$totNbChunks){
			$tblChunkId= $totNbChunks;
		}
		
		if ($tblChunkId===false){
			$adaptedTCId= 1;	// not indicated --> number 1
		}else{
			$adaptedTCId= $tblChunkId;
		}
		
		$gArr= $this->contigToGeom(1,$totNbChunks, $adaptedTCId);
		$numericMenu= $this->dsplNumericMenu($gArr, $givCls,$givUrlF,_c_geoTblChunkId);
		return $numericMenu;
	} // geomArrDisplay
	

	/**
	*/
	function chabl(){
	} // chabl
	

} // geomArray
?>
