<?php
/**
	File name : imgProc.php
	Author of this file : Oscar Fawer until 2019-08-15
	Contact : www.ataox.org 
	Web sites : ataox.org / ataox.com / ataox.info
	Copyrights open source GPL, GNU General Public License V3 : see ./aMainDr/code/LICENSE.txt


	Image Processing
	
	Future improvments for image manipulation GD library :
		Logo multipasting :
			imagecopy(...); imagecopymerge(...); imagealphablending(...)
		imageflip()
			
*/
class imgProc extends oTop{
	

	function __construct(&$tagsFu){
		parent::__construct();
		$this->tools= &$tagsFu->tools;
		$this->glob= &$tagsFu->glob;
		$this->fls= &$tagsFu->fls;
		$this->dtb= &$tagsFu->dtb;
		$this->hPwC= &$tagsFu->hPwC;
		$this->rMg= &$tagsFu->rMg;
		
		
		define('_c_rndImgDefAction','clMix');
	} // construct


	/**
		Stores (point on) a given scaling vector
	*/
	function storesScalingVect(&$sv){
		$this->scaleV= &$sv;
	} // storesScalingVect

	function getScalingVect(){
		return $this->scaleV;
	} // getScalingVect
	
	
	/**
		Fixed width
	*/
	function storesDesiredWidth($w){
		$this->desiredWidth= $w;
	} // storesDesiredWidth

	function getDesiredWidth(){
		return $this->desiredWidth;
	} // getDesiredWidth
	

	/**
		Ok for thumbs image extension/type
	*/
	function imgOkToDspl($ext){
		$fls= &$this->fls;
		return $fls->imgOkToDspl($ext);
	} // imgOkToDspl


	/*
		Scales down within the given linear limits : $limWidth, $limHeight.
		
		Imposes the height first, then giving a height greater than normal allow a large tolerance with the same height.
			Input:
				$limWidth,$limHeight : given linear limits.
				$srcW, $srcH : with and height of the image source.
			Output :
				$dstW, $dstH : new dimentions of the thumb/image
	*/
	function scaleDownLinear(&$dstW,&$dstH,$srcW,$srcH,$limWidth,$limHeight){
		if ($srcW==$limWidth && $srcH==$limHeight){
			// Don't want to waste time on same dimension + could be bad when adjusting galleries.
			$dstW= $srcW;
			$dstH= $srcH;
			return false;
		}else{
			if ($srcH<1 || $srcW<1){
				return false;
			}
			
			// Also accept scaling up
			$dstH= $limHeight;
			$dstW= (int)($dstH*$srcW/$srcH);
			if ($dstW > $limWidth){
				// $dstW is too big --> imposes the other side
				$dstW= $limWidth;
				$dstH= (int)($dstW*$srcH/$srcW);
			}
			return true;
		}
	} // scaleDownLinear
	

	/**
		Resize processing : From a given image (fullPath).
			Input :
				$imSrPFN : image path & file name to resize.
				$imDstPFN : image destination (path and file name) for the resized image.
				$limWidth, $limHeight : won't go above these limits when resizing.
				$extOnSrc : given extension when it does not appear in the original file (temporary upload)
			Output :
				the resized image file.
	*/
	function resizeProc($imSrPFN,$imDstPFN,$limWidth,$limHeight,$quality=50,$extOnSrc=true){
		return $this->adaptProc($imSrPFN,$imDstPFN,$limWidth,$limHeight,$quality,$extOnSrc);
	} // resizeProc
	
	
	/**
		Input :
			$rSrc	// resource Source
			...			
		Output :
				the resized image file.
	*/
	function resizeFromResource($rSrc,$srcW,$srcH, $dstW,$dstH,$imDstPFN, $ext='jpg',$quality=50){
		return $this->adaptFromResource($rSrc,$srcW,$srcH,$dstW,$dstH,$imDstPFN,$ext,$quality);
	} // resizeFromResource
	
	
	/**
		Rotation from the image file.
	*/
	function rotLeftProc($imSrPFN,$quality,$angle){
		$imSzV= getimagesize($imSrPFN);	
		$srcW= $imSzV[0];
		$srcH= $imSzV[1];
		return $this->adaptProc($imSrPFN,$imSrPFN,$srcW,$srcH,$quality,true,'rot',$angle);
	} // rotLeftProc
	
	/**
		Rotation directly from a previous resource to avoid lost of information..
	*/
	function rotLeftPrc($rSrc,$imDstPFN,$quality,$angle){
		$fls= &$this->fls;
		
		$fls->splitOnLastC($imDstPFN,$left,$ext,'.');
		$ext= strtolower($ext);	
		return $this->adaptFromRsrc($rSrc,$imDstPFN,$ext,$quality, 'rot',$angle)	;
	} // rotLeftPrc
	
	function cropImage($imSrPFN,$quality,$rect){
		return $this->adaptProc($imSrPFN,$imSrPFN,'','',$quality,true,'crop',$rect);
	} // cropImage

	
	/**
		Adaptation (sz, (rot, crop) ) processing : From a given image (fullPath).
			Input :
				$imSrPFN : image path & file name to resize.
				$imDstPFN : image destination (path and file name) for the resized image.
				
				Parameters  1st within the $adapt==='sz' ---> size only paradigme
					$limWidth, $limHeight : won't go above these limits when resizing.
					
				$extOnSrc : given extension, when it does not appear in the original file (temporary upload)
				$adapt : the sort of operation we ask sz, rot
				$freeParam : supplementary parameter un case of need
			Output :
				the resized image file.
	*/
	function adaptProc($imSrPFN,$imDstPFN,$limWidth,$limHeight,$quality=50,$extOnSrc=true, $adapt='sz',$freeParam=false){
		$glob= &$this->glob;
		$tools= &$this->tools;
		$rMg= &$this->rMg;
		$fls= &$this->fls;
		$dtb= &$this->dtb;
		
		if ($glob->cndtWrRghts===false){
			return false;
		}		
	
		$imSrPFN= urldecode($imSrPFN);	// because getimagesize does not cope with url encoded strings !!
		$fls->splitOnLastC($imSrPFN,$path,$imgFN,'/');
		$imSzV= getimagesize($imSrPFN);		// ( $imSzV[0], $imSzV[1] ) -- (width, height)
		
	
		switch ($adapt){
			case 'sz':
				if (!$this->scaleDownLinear($dstW,$dstH,$imSzV[0],$imSzV[1],$limWidth,$limHeight) && $extOnSrc){
					// The image is small enough and sure (got extension) --->  just copy the image to the destination dir
					$fls->copyFile($imSrPFN,$imDstPFN);
					return true;
				}			
			break;
			
			// The following indications are just informative to have a correct values inside $dstW, $dstH
			case 'rot':
				$dstW= $limWidth;
				$dstH= $limHeight;
			break;
			
			// The following indications are just informative to have a correct values inisde $dstW, $dstH
			case 'crop':
				$dstW= $freeParam['width'];
				$dstH= $freeParam['height'];
			break;
			
			default:
				$this->topErr('1st Switch, imgProc::adaptFromResource');
				return false;
			break;
		} // switch
		
		if ($extOnSrc){
			$fls->splitOnLastC($imgFN,$left,$ext,'.');
		}else{
			// uses the destination string to extract the extension (so eventual file type conversion)
			$fls->splitOnLastC($imDstPFN,$left,$ext,'.');
		}
		$ext= strtolower($ext);	

		switch ($adapt){
			case 'sz':
			case 'rot':
			case 'crop':
				switch ($ext){		
					case 'jpg':
					case 'jpeg':
						$rSrc= imagecreatefromjpeg($imSrPFN); // resource Source
					break;
					
					case 'png':
						$rSrc= imagecreatefrompng($imSrPFN);
						if ($adapt==='sz'){
							$contrastStr= $dtb->getGenVar('pngShrinkerContrast','-22');
							if ($contrastStr!==_c_ko){
								$contrast= (int)$contrastStr;
								// imagefilter($rSrc, IMG_FILTER_BRIGHTNESS, 10);							
								imagefilter($rSrc, IMG_FILTER_CONTRAST, $contrast);
							}
						}
					break;
					
					case 'gif':
						$rSrc= imagecreatefromgif($imSrPFN);
					break;
					
					default:
						$this->topErr('1, imgProc::resises');
						return false;
				} // switch
			break;
						
			default:
				$rSrc= imagecreatetruecolor ($limWidth,$limHeight);
			break;
		} // switch ($adapt)
		
		
		
		$srcW= $imSzV[0];
		$srcH= $imSzV[1];
		if ( !$rMg->quickWMRight() && $adapt==='sz' && ($srcW<=$dstW || $srcH<=$dstH) ){
			$glob->pubImUpsuplErrMess.= $glob->sl('Sorry the algorithm dealing with uploaded images must shrink them ...','kmdh');
			return false;
		}
		
		if ( $adapt==='sz' && ($srcW<=$dstW || $srcH<=$dstH) ){ // #ShouldAllowSizeIncreaseWithSupMenuOptions
			// keep the source size ... don't shrink but still crunch for compression!
			return $this->adaptFromResource($rSrc,  $srcW,$srcH, $srcW,$srcH,   $imDstPFN, $ext,$quality, $adapt,$freeParam);
		}else{
			return $this->adaptFromResource($rSrc, $srcW,$srcH, $dstW,$dstH, $imDstPFN, $ext,$quality, $adapt,$freeParam);
		}
	} // adaptProc
	
	
	/**
		A smaller version of adaptFromResource() working for everythin except for $adapt==='sz'
	*/
	function adaptFromRsrc(&$rSrc,$imDstPFN, $ext='jpg',$quality=50, $adapt='sz',$freeParam=false){
		$srcW= false;
		$srcH= false;
		$dstW= false;
		$dstH= false;
		return $this->adaptFromResource($rSrc,$srcW,$srcH, $dstW,$dstH,$imDstPFN, $ext,$quality, $adapt,$freeParam);
	} // adaptFromRsrc
	
	
	/**
		Input :
			$rSrc	// resource Source
			...			
		Output :
				the resized image file.
	*/
	function adaptFromResource(&$rSrc, $srcW, $srcH, $dstW, $dstH, $imDstPFN, $ext='jpg',$quality=50, $adapt='sz',$freeParam=false){
		$glob= &$this->glob;
		$fls= &$this->fls;
		$dtb= &$this->dtb;
		
		
		if ($glob->cndtWrRghts===false){
			return false;
		}
		
		if ($adapt==='sz'){
			if ($dstW<1){
				$dstW= 1;
			}
			
			if ($dstH<1){
				$dstH= 1;
			}
		}
		
		$ext= strtolower($ext);
		
		// if ($adapt==='sz' && $srcW===$dstW && $srcH===$dstH){
		if (false){
			// same dim so just adapt the quality (degradation!)
			$rDst= $rSrc;
			$adaptOk= true;
		}else{
			switch ($adapt){
				case 'sz':
					if ($ext==='png'){
						$rDst= imagecreatetruecolor($dstW, $dstH);
						imagealphablending($rDst, false);
						imagesavealpha($rDst,true);
						$transparent= imagecolorallocatealpha($rDst, 255, 255, 255, 127);					
						imagefilledrectangle($rDst, 0, 0, $dstW, $dstH, $transparent);
					}else{
						$rDst= imagecreatetruecolor($dstW,$dstH);	// image resource destination
					}
					
					$adaptOk= imagecopyresampled($rDst,$rSrc, 0,0,0,0, $dstW,$dstH, $srcW,$srcH);
				break;
				
				case 'rot':
					$rot= (float)$freeParam;					
					$rDst= imagerotate($rSrc,$rot,0);
					if ($rDst===false){
						$adaptOk= false;
					}else{
						$adaptOk= true;
					}
				break;
				
				case 'crop':
					$rect= $freeParam;
					
					$sz= count($rect);
					print('$sz='.$sz._c_);
					$this->pv($rect);
					
					$rDst= imagecrop($rSrc,$rect);
					if ($rDst===false){
						$adaptOk= false;
					}else{
						$adaptOk= true;
					}
				break;
				
				default:
					$this->topErr('1, imgProc::adaptFromResource');
					return false;
				break;
			} // switch
		}
				
		if ($adaptOk){
			$ok= true;
			// Rem image***() erase automatically the original file !!
			switch ($ext){
				case 'jpg':
				case 'jpeg':
					$ok= imagejpeg($rDst,$imDstPFN,$quality);	// + saves it as well
				break;
				
				case 'png':
					$pngCpr= (int)$dtb->getGenVar('genPngCompr','9');
					$ok= imagepng($rDst,$imDstPFN,$pngCpr);	// + saves it as well
				break;
				
				case 'gif':
					$ok= imagegif($rDst,$imDstPFN);		// + saves it as well
				break;
				
				default:
					$this->topErr('2, imgProc::adaptFromResource');
					return false;
			} // switch
			
			if ($ok===false){
				$this->topErr('3, imgProc::adaptFromResource');
				return false;
			}else{
				return $rDst;
			}	
		}else{
			$this->topErr('4, imgProc::adaptFromResource');
			return false;
		}
		
		return true;
	} // adaptFromResource		
	
	
	/**
		Creates one random w h colord grid along different criterion.
			Input :
				$w, $h : width and height of the grid in pixels.
				
				$imDstPFN : image destination (path and file name) for the resized image.
				$dstW,$dstH
				$monoNb : nono color squares number.
				$monoColor : used mono color
			Output :
				the resized image file.
				true if successful
				
		Param avatar :
			createRandomGridImg(3,3, $imDstPFN,40,40);
	*/
	function createRandomGridImg($w,$h, $imDstPFN,$dstW,$dstH, $monoNb='0',$monoColor='FFFFFF'){
		$glob= &$this->glob;

		if ($glob->cndtWrRghts===false){
			return false;
		}		

		$firstImgRsr= imagecreatetruecolor($w,$h);	// first image resource
		for ($j=0; $j<$h; $j++){
			for ($i=0; $i<$w; $i++){
				$red= rand(0,255);
				$green= rand(0,255);
				$blue= rand(0,255);
				$color= imagecolorallocate($firstImgRsr, $red,$green,$blue);
				imagesetpixel($firstImgRsr, $i,$j, $color);
			} // for
		} // for		
		
		$ext= $this->lowerCaseExt($imDstPFN);
		
		if ($monoNb!=='0'){
			$mColorV= $this->rgbHexaToDecVect($monoColor);	// mono color
			$color= imagecolorallocate ($firstImgRsr, $mColorV['r'],$mColorV['g'],$mColorV['b']);
			
			for ($i=0; $i<$monoNb; $i++){			
				$curW= rand (0,$w-1);
				$curH= rand (0,$h-1);				
				imagesetpixel($firstImgRsr, $curW,$curH, $color);
			} // for
		}
		
		return $this->resizeFromResource($firstImgRsr,$w,$h, $dstW,$dstH,$imDstPFN,$ext);
	} // createRandomGridImg


	/**
	*/
	function randomCmplxImage($pushedPath, $dimV){
		$tools= &$this->tools;
		$hPwC= &$this->hPwC;
	
		$imgAct= $tools->postVal('rndImgAction',_c_rndImgDefAction);
		switch ($imgAct){
			case 'ovals':
			case 'stars3':
			case 'stars4':
			case 'stars5':
			case 'stars6':
			case 'stars7':
			case 'starsF':
			case 'starsM':
			
			case 'oRect':	// orthogonal rectangles
			case 'aRect':	// all sort of rectangles
				$destDfn= $pushedPath.$hPwC->consonStr(10).'.jpg';
				$this->createCmplxRandomShapes($dimV[0],$dimV[1], $destDfn,$imgAct);
			break;
			
			case 'text':
				$destDfn= $pushedPath.$hPwC->consonStr(10).'.png';
				$this->createCmplxRandomShapes($dimV[0],$dimV[1], $destDfn,$imgAct);
			break;

			case 'clHdTl':
			case 'clStarC':
			case 'clStar':
			case 'clMix':
				$destDfn= $pushedPath.$hPwC->consonStr(10).'.jpg';
				$this->contiguousLines($dimV[0],$dimV[1], $destDfn,$imgAct);
			break;
			
			default: print('Error in imgProc::randomCmplxImage');
		} // switch
		
	} // randomCmplxImage
	
	
	/**
	*/
	function createCmplxRandomShapes($dstW,$dstH, $imDstPFN,$imgAct, $N=20, $M=18,$xMin=0,$yMin=0){
		$glob= &$this->glob;
		$tools= &$this->tools;

		if ($glob->cndtWrRghts===false){
			return false;
		}
		
		if ($imgAct==='text'){
			$str= $tools->postVal('imgManipStrTBN');
			if ($str===''){
				$str= 'Ataox';
			}
			
			$imgIter= $tools->postVal('imgManipIterTBN','');
			if ($imgIter!==''){
				$N= (int)$imgIter;
			}else{
				$strSz= strlen($str);
				do {
					if ($strSz<=2){ $N=16; break;}
					if ($strSz<=4){ $N=10; break;}
					if ($strSz<=8){ $N=7; break;}
					if ($strSz<=12){ $N=3; break;}
					if ($strSz<=16){ $N=3; break;}
					$N=2;
				} while (false);
			}
			
		}
		
		$ext= $this->lowerCaseExt($imDstPFN);
		$im= imagecreatetruecolor($dstW,$dstH);		// first image resource
				
		$xMax= $dstW;
		$yMax= $dstH;
		
		$shapeW= $dstW;
		$shapeH= $dstH;
		
		$shiftX= round($dstW / $M );
		$shiftY= round($dstH / $M );
		
		$xTendency= rand(-$shiftX,$shiftX);
		$yTendency= rand(-$shiftY,$shiftY);
		
		$factDownW= 0.85;
		$factDownH= 0.85;
		$fam= rand(1,5);
		for ($j=0; $j<$M; $j++){
			if ($imgAct!=='text'){
				$N+= 1;
			}
			for ($i=0; $i<$N; $i++){
				$color= $this->alocInFamilyRndColors($im,$fam);
				$xC= rand($xMin,$xMax);
				$yC= rand($yMin,$yMax);
				$width= rand(3,$shapeW);
				$height= rand(3,$shapeH);
						
				switch ($imgAct){
					case 'ovals':
						imagefilledellipse($im,$xC,$yC,$width,$height,$color);
					break;
					
					case 'stars3':
					case 'stars4':
					case 'stars5':
					case 'stars6':
					case 'stars7':
					case 'starsF':
					case 'starsM':
						$this->fillStar($im,$imgAct,$xC,$yC,$width,$height,$color);
					break;
					
					case 'oRect':
						$this->fillRect($im,$xC,$yC,$width,$height,$color);
					break;
					
					case 'aRect':
						$this->randomThickLine($im,$xC,$yC,$width,$height,$color);
					break;
					
					case 'text':
						$fontDfn = _c_mainFontDfn.'courbd.ttf';
						$set= rand(0,1);
						if ($set===0){
							$angle= rand(0,45);
						}else{
							$angle= rand(316,359);
						}
						
						imagettftext($im, $height, $angle, $xC,$yC, $color, $fontDfn, $str);
					break;
					
					default: print('Error in imgProc::createCmplxRandomShapes');
				} // switch
			}
			
			$shapeW= $shapeW * $factDownW;
			$shapeH= $shapeH * $factDownH;
			
			
			if ($imgAct!=='text'){
				$xMin+= rand(0,$shiftX)+$xTendency;
				$yMin+= rand(0,$shiftY)+$yTendency;
				
				$xMax= $xMax-rand(0,$shiftX)-$xTendency;
				$yMax= $yMax-rand(0,$shiftY)-$yTendency;
			}
			
		}
		
		// In this case resizing is useless, just use it for the file saving of the image file
		if ($imgAct==='text'){
			// n resizeFromResource
			return $this->resizeFromResource($im,$dstW,$dstH, $dstW,$dstH,$imDstPFN,'png',0);
		}else{
			return $this->resizeFromResource($im,$dstW,$dstH, $dstW,$dstH,$imDstPFN,$ext);
		}
	} // createCmplxRandomShapes
		
	
	/**
	*/
	function contiguousLines($dstW,$dstH,$imDstPFN,$kind='clMix',$N=1000){
		$glob= &$this->glob;

		if ($glob->cndtWrRghts===false){
			return false;
		}
		
		if ($N<=100){
			$factL= 0.98;
			$factE= 0.97;
		}else{
			$factL= 0.998;
			$factE= 0.9972;
		}
	
		$ext= $this->lowerCaseExt($imDstPFN);
		$im= imagecreatetruecolor($dstW,$dstH);		// first image resource
	
		$fam= rand(1,5);
		
		if ($kind==='clStarC'){
			$xStart= $dstW/2;
			$yStart= $dstH/2;
		}else{
			$xStart= rand(0,$dstW);
			$yStart= rand(0,$dstH);
		}
		
		// screen center
		$cX= $dstW/2;
		$cY= $dstH/2;
	
		$lReductor= 1;
		$hicUpFrequency=150; // to have a chance to displace the aglomeration
		$hicUp= 0;	// hic uo countor
		$curE= 50;	// thickness
		$drawStar= 100;
		for ($i=0; $i<$N; $i++){
			$color= $this->alocInFamilyRndColors($im,$fam);
			
			// The deltas from the start of the segment to the end
			if ($kind==='clMix'){
				if ($hicUp===0){
					$deltaX= rand(-$dstW,$dstW);
					$deltaY= rand(-$dstH,$dstH);
				}else{
					$deltaX= rand(-$dstW,$dstW)*$lReductor;
					$deltaY= rand(-$dstH,$dstH)*$lReductor;
					$lReductor= $lReductor*$factL;
				}
			}else{
				$deltaX= rand(-$dstW,$dstW)*$lReductor;
				$deltaY= rand(-$dstH,$dstH)*$lReductor;
				$lReductor= $lReductor*$factL;
			}
			
			$xEnd= $this->nextProjectionVal($xStart,$cX,$deltaX,$dstW,$curE,$drawStar);
			$yEnd= $this->nextProjectionVal($yStart,$cY,$deltaY,$dstH,$curE,$drawStar,false);
			$this->traceThickLine($im,$color,$xStart,$yStart,$xEnd,$yEnd,$curE);
			
			if ($kind==='clMix'){
				if ($drawStar>0){
					$drawStar--;
				}else{
					$xStart= $xEnd;
					$yStart= $yEnd;
				}
				
				if ($hicUp===0){
					$drawStar= 100;
				}
				
				$hicUp++;
				if ($hicUp>$hicUpFrequency){
					$hicUp= 0;
				}
				
			}
			
			if ($kind==='clHdTl'){
				$xStart= $xEnd;
				$yStart= $yEnd;
			}
			
			$curE= $curE*$factE;
		} // for
		return $this->resizeFromResource($im,$dstW,$dstH, $dstW,$dstH,$imDstPFN,$ext);		
	} // contiguousLines
	
			
	/**
		Input : $xStart,$cX,$deltaX
			--> also valid for coord Y
				$dstW,$curE
	*/
	function nextProjectionVal($xStart,$cX,$deltaX,$dstW,$curE,$drawStar,$realX=true){
		$bump= false;
		$xEnd= $xStart+$deltaX;
		
		if ($realX){
			if ($xEnd<$curE){
				$xEnd= $curE;
				$bump= true;
			}
		}else{
			// y
			if ($xEnd<0){
				$xEnd= $curE;
				$bump= true;
			}
		}
		
		if ($xEnd > ($dstW-$curE) ){
			$xEnd= $dstW-$curE;
			$bump= true;
		}
		
		
		if ($bump){
			$r= rand(0,2);
			if ( $drawStar===0 && ($r===0 || $r===1) ){
				$xEnd= $dstW / 2;	// insist sometimes toward the center
			}
		}
		return $xEnd;
	} // nextProjectionVal
	
	
	
	/**
	*/
	function traceThickLine($im, $color, $p1x,$p1y,$p2x,$p2y,$e){
		$deltaX= $p2x-$p1x;
		$deltaY= $p2y-$p1y;
		
		if ($deltaX==0 || $deltaY==0){
			return;	// to avoid a crash when second point is chosen COMPLETLY randomly
		}
	
		$tgAlpha= $deltaY/$deltaX;
		$alpha= atan($tgAlpha);
		
		$h= $e*cos($alpha);
		$l= $e*sin($alpha);
		
		$q1x= $p1x - $l;
		$q1y= $p1y + $h;
		
		$q2x= $p2x - $l;
		$q2y= $p2y + $h;
		
		$points= array($p1x,$p1y, $p2x,$p2y, $q2x,$q2y, $q1x,$q1y);
		imagefilledpolygon($im,$points,4,$color);
	} // traceThickLine
			

	/**
	*/
	function randomThickLine($im,$xC,$yC,$width,$height,$color){
		$p2x= rand(5,8*$width);
		$p2y= rand(5,8*$height);
		$e= rand(3,100);
		$this->traceThickLine($im, $color, $xC,$yC,$p2x,$p2y,$e);
	} // randomThickLine
	
	
	/**
	*/
	function fillStar(&$im,$imgAct,$xC,$yC,$width,$height,$color){
		$tools= &$this->tools;
		
		$p= array();
	
		$r= ($width+$height)/2;
		$delta= $tools->smallest($width,$height)/2.5;
	
		// --> number of star peaks
		switch ($imgAct){
			case 'stars3':
				$peaksNb= 3;
			break;
			
			case 'stars4':
				$peaksNb= 4;
			break;
			
			case 'stars5':
				$peaksNb= 5;
			break;
			
			case 'stars6':
				$peaksNb= 6;
			break;
			
			case 'stars7':
				$peaksNb= 7;
			break;
			
			case 'starsM':
				$peaksNb= rand(3,9);
			break;
			
			
			case 'starsF':
				$peaksNb= $tools->postVal('imgManipIterTBN');
				if ($peaksNb<3 || $peaksNb>500){
					$peaksNb= rand(3,rand(6,12));
				}
			break;
			
			
			default: print('Error in imgProc::fillStar');
		} // switch
	
		$nbDots= $peaksNb*2;
		$angleIncr= M_PI/$peaksNb;
		$angle= 2*M_PI/$peaksNb  *  rand(0,100)/100;
		
		for ($i=0; $i<$peaksNb; $i++){
			// out			
			$p[$i*4]= $xC + ($r+$delta)*cos($angle);		// x
			$p[$i*4+1]= $yC + ($r+$delta)*sin($angle);		// y
			
			$angle+= $angleIncr;
			
			// in
			$p[$i*4+2]= $xC + ($r-$delta)*cos($angle);		// x
			$p[$i*4+3]= $yC + ($r-$delta)*sin($angle);		// y

			$angle+= $angleIncr;
		} // for
	
		$points= array();
		imagefilledpolygon($im,$p,$nbDots,$color);
	} // fillStar
	
	
	/**
	*/
	function fillRect($im,$xC,$yC,$width,$height,$color){
		$halfW= $width/2;
		$halfH= $height/2;
		
		$x1= $xC - $halfW;
		$y1= $yC - $halfH;
		
		$x2= $xC + $halfW;
		$y2= $yC + $halfH;
		imagefilledrectangle ($im,$x1,$y1,$x2,$y2,$color);
	} // fillRect
	

	/**
		Allocates inside a given family of random colors.
	*/
	function alocInFamilyRndColors(&$im,$fam){
		$st= 0;	// small Span Start   rand($st,255)
		$c= 0;	// stable color
		
		switch ($fam){
			case 1:
			case 2:
			case 3:
				// multicolor
				$color= imagecolorallocate($im, rand(0,210), rand(0,210), rand(0,210));
			break;
			
			case 4:
				$color= imagecolorallocate($im, rand($st,255), rand(0,255), $c);	// not the max 255 to avoid too white!
			break;
						
			case 5:
				$color= imagecolorallocate($im, $c, rand(0,255), rand($st,255));	// not the max 255 to avoid too white!
			break;
									
			case 6:
				//  grey
				$same= rand(0,255);
				$color= imagecolorallocate($im, $same, $same, $same);	// not the max 255 to avoid too white!
			break;
			
			default: print('Error in alocInFamilyRndColors'._c_);
		} // switch
		return $color;
	} // alocInFamilyRndColors


	/**
	*/
	function randomImage($pushedPath, $v){
		$hPwC= &$this->hPwC;
	
		$destDfn= $pushedPath.$hPwC->consonStr(10).'.jpg';
		$this->createRandomElipse($v[0],$v[1], $destDfn);
	} // randomImage

	
	/**
	*/
	function createRandomElipse($dstW,$dstH, $imDstPFN, $N=20,$xMin=0,$yMin=0){
		$glob= &$this->glob;

		if ($glob->cndtWrRghts===false){
			return false;
		}
		
		$xMax= $dstW;
		$yMax= $dstH;

		$ext= $this->lowerCaseExt($imDstPFN);
		$im= imagecreatetruecolor($dstW,$dstH);		// first image resource
		for ($i=0; $i<$N; $i++){
			$color= imagecolorallocate($im, rand(0,210), rand(0,210), rand(0,210));	// not the max 255 to avoid too white!
			$xC= rand($xMin,$xMax);
			$yC= rand($yMin,$yMax);
			$width= rand($xMin,$xMax);
			$height= rand($yMin,$yMax);
			imagefilledellipse($im,$xC,$yC,$width,$height,$color);
		}
		
		// In this case resizing is useless, just use it for the file saving of the image file
		return $this->resizeFromResource($im,$dstW,$dstH, $dstW,$dstH,$imDstPFN,$ext);
	} // createRandomElipse


	/**
		RGB hexadecimal color notation to a decimal vector :
			FF0000 --> 256,0,0
	*/
	function rgbHexaToDecVect($h){
		$r= hexdec($h[0].$h[1]);
		$g= hexdec($h[2].$h[3]);
		$b= hexdec($h[4].$h[5]);
	
		$v['r']= $r;
		$v['g']= $g;
		$v['b']= $b;
		return $v;
	} // rgbHexaToDecVect


	/**
	*/
	function lowerCaseExt($imDstPFN){
		$fls= &$this->fls;
		
		$fls->splitOnLastC($imDstPFN,$left,$ext,'.');		
		return strtolower($ext);		
	} // lowerCaseExt


	/**
		Changes the file type of a given image, keep size and stores the result inside the given directory.
		Manual input : included dirName ; fileType (no dot)
		
		Input :
			$imgOrigPFn : full path and file name of the image to convert.
			$includedDirNm : directory name included inside $imgOrigPFn. (passed with terminal /)
			$newExt : new file extension.
			$quality : quality of th enew file.
			
		Remark ; ICO convertion not implemented (direct rial with bmp or png don't work! Could try :
			http://www.phpclasses.org/package/2369-PHP-Extract-graphics-from-ico-files-into-PNG-images.html
			But could not log at that time (probably old browseer).
	*/
	function changeFileType($imgOrigPFn, $includedDirNm,$newExt, $quality=50){
		$glob= &$this->glob;
		$fls= &$this->fls;
		$dtb= &$this->dtb;
		
		if ($glob->cndtWrRghts===false){
			return false;
		}		

		$newExt= strtolower($newExt);
		$fls->splitOnLastC($imgOrigPFn,$mainPath,$origFullFn,'/');
		$fls->splitOnLastC($origFullFn,$curFn,$curExt,'.');
		
		$curExt= strtolower($curExt);
		switch ($curExt){		
			case 'jpg':
			case 'jpeg':
				$imgRsrc= imagecreatefromjpeg($imgOrigPFn);
			break;
			
			case 'png':
				$imgRsrc= imagecreatefrompng($imgOrigPFn);
			break;
			
			case 'gif':
				$imgRsrc= imagecreatefromgif($imgOrigPFn);
			break;
			
			case 'bmp':
				$imgRsrc= imagecreatefromwbmp($imgOrigPFn);
			break;
			
			default:
				$this->topErr('1, imgProc::changeFileType');
				return false;
		} // switch
		
				
		if ($imgRsrc!==false){
			$newDestPFN= $mainPath.'/'.$includedDirNm.$curFn.'.'.$newExt;
			switch ($newExt){		
				case 'jpg':
				case 'jpeg':
					$result= imagejpeg($imgRsrc, $newDestPFN, $quality);
				break;
				
				case 'png':				
					$pngCpr= (int)$dtb->getGenVar('genPngCompr','9');
					// $pngCpr= (int)$pngCpr;
					$result= imagepng($imgRsrc, $newDestPFN, $pngCpr);
				break;
				
				case 'gif':
					$result= imagegif($imgRsrc, $newDestPFN);
				break;
				
				case 'bmp':
					$result= imagewbmp($imgRsrc, $newDestPFN);
				break;
				
				default:
					$this->topErr('2, imgProc::changeFileType');
					return false;
			} // switch
			
			return $result;
		}else{
			$this->topErr('2, imgProc::changeFileType, '.$glob->sl('could not convert the ','kmdi').$curExt.$glob->sl(' type!','kmdj'));
			return false;
		}
	} // changeFileType

} // imgProc
?>
