/**
	File name : kk.js
	Author of this file : Oscar Fawer until 2019-08-15
	Contact : www.ataox.org 
	Web sites : ataox.org / ataox.com / ataox.info
	Copyrights open source GPL, GNU General Public License V3 : see ./aMainDr/code/LICENSE.txt


*/

/*
	A collection of globally useful constants, stored as object instance variable! 
	General coding remarks :
		- Up to now (as far as I know) there is not a js "constant" k-word, or a direct "include" instruction.
		- Looks like the easier way to share identical values between many js files and objects.
		- The all upper case style is not chosen, because it looks (personal opinion) overwhelming in the code, 
		  especially compared to algorithmically more meaningful instructions (loops, tests etc.).
		  
	Debug tools :
		writeLn('**'+arguments.callee.caller.toString() );	// displays the calling method!
		  
*/
function kk(){
		
	/* ---------------------------------------------------------
		Public Variables
	*/	
	this.crLn= "\r\n";
	this.nbsp= '&nbsp;';
	this.splitLn= "\n";
	
		
	/* ---------------------------------------------------------
		Public Methods .. obvious fu that are not in JS
	*/	
	this.guestVarAccess= guestVarAccess;
	this.adaptCombo= adaptCombo;
	this.trim= trim;
	this.gotChar= gotChar;
	this.bStrToVect= bStrToVect;
	this.vectToStr= vectToStr;
	this.winOpen= winOpen;
	this.isInvisible= isInvisible;
	this.isTabable= isTabable;
	this.cloneArray= cloneArray;
	this.initCurrentDisplay= initCurrentDisplay;
	this.initDebugDisplay= initDebugDisplay;
	this.currentDisplayPt= currentDisplayPt;
	this.getString= getString;	
	this.write= write;
	this.writeLn= writeLn;
	this.clearWrite= clearWrite;
	this.writeBefore= writeBefore;
	this.writeNl= writeNl;		// as writeLn(), but writes ahead.
	this.getSelVal= getSelVal;
	this.setSelVal= setSelVal;
	this.strReplace= strReplace;
	this.setInnerHtml= setInnerHtml;
	this.setInnHtml= setInnHtml;
	this.replaceCodedChar= replaceCodedChar;
	
	this.strLeftOfC= strLeftOfC;
	this.strRightOfC= strRightOfC;
	this.toTitleCase= toTitleCase;
	this.tinyCrypt= tinyCrypt;

	this.isUndef= isUndef;
	this.isNotUndef= isNotUndef;
	
	
	this.stringBase= stringBase;
	
	
	


	/* ---------------------------------------------------------
		Private Variables
	*/	
	var crLn= "\r\n";		// carriage return
	
	
	/*
		Deal with simple undefined variables.
			Note object properties that do not exist will confuse this function and should be dealt with 
			like that : key in obj && obj.key===undefined; But these cases do not appear often!
			Not advised : // return typeof x === 'undefined'; https://stackoverflow.com/questions/27509/detecting-an-undefined-object-property
	*/
	function isUndef(x){
	    return x === undefined;
	} // isUndef
	
	/**
		kk.isNotUndef()
	*/
	function isNotUndef(x){
	    return !isUndef(x);
	} // isNotUndef
	
	
	/**
		Concats the given string s n times.
	*/
	function concatN(s,n){
		b='';
		for (var i=0;i<n;i++){
			b+=s;
		}
		return b;
	} // getNumbers


	function trim(s){
		var sz= s.length;
		var i= 0;
		while(i<sz && isInvisible(s.charAt(i))){i++;}
		var j= sz-1;
		while(i>0 && isInvisible(s.charAt(j))){j--;}
		return s.substr(i, j-i+1);
	} // trim


	/**
	*/
	function isInvisible(c){
		switch (c){
			case ' ':
			case "\n":
			case "\r":
			case "\t":
				return true;
			break;
						
			default:
				return false;
		} // switch
	} // isInvisible

	
	/*
		Return true if the examined character can be used as a tab, so tabs and spaces.
	*/
	function isTabable(c){
		switch (c){
			case ' ':
			case "\t":
				return true;
			break;
			
			default:
				return false;
		} // switch
	} // isTabable

	
	/**
		Returns true if the specified string got the specified char at least once.
	*/
	function gotChar(s,c){
		var sz= s.length;
		var i=0;
		while(i<sz && s.charAt(i)!==c){
			i++;
		}
		return i!==sz;
	} // gotChar

	/**
		Only does it at the byte level.
	*/
	function bStrToVect(s){
		var v= new Array();
		var sz= s.length;
		for (var i=0; i<sz; i++){
			v[i]= s.charAt(i);
		}
		return v;
	} // bStrToVect
	
	
	
	function vectToStr(v){
		var s= '';
		var sz= v.length;
		for (var i=0; i<sz; i++){
			s+= v[i];
		}		
		return s;
	} // vectToStr

	/**
	*/
	function winOpen(url){
		var win=window.open(url,'','width=700,height=500,scrollbars=yes,toolbar=yes,menubar=yes');
		win.focus();
	}; // winOpen
	

	/* ---------------------------------------------
		Local stores the Id of the current display.
	*/
	function initCurrentDisplay(displayId){
		localStorage.currentDisplay= displayId;
	} // initCurrentDisplay
	
	function initDebugDisplay(){
		localStorage.currentDisplay= 'debugDisplay';
	} // initDebugDisplay
	
	
	
	/*
		Returns the pointer of the side extension screen.
	*/
	function currentDisplayPt(){
		return document.getElementById(localStorage.currentDisplay);
	} // currentDisplayPt
	
	
	/* ---------------------------------------------
		Get the current string.
	*/
	function getString(s){
		var d= currentDisplayPt();
		return d.value;
	} // getString
	
	

	/* ---------------------------------------------
		Writing methods o have something more user-friendly than
		the classic 
			alert('Hello World!');
		method!
	*/
	

	/* ---------------------------------------------
		Writing methods 1st level.
	*/
	function write(s){
		var d= currentDisplayPt();
		if (d!==null){
			d.value+= s;
		}
	} // write
	
	function clearWrite(s){
		var d= currentDisplayPt();
		if (d!==null){
			d.value= s;
		}
	} // clearWrite
	
	function writeBefore(s){
		var d= currentDisplayPt();
		d.value= s + d.value;
	} // writeBefore


	/* ---------------------------------------------
		Writing methods 2nd level.
	*/
	function writeLn(s){
		if (isUndef(s)){
			write(crLn);
		}else{
			write(s+crLn);
		}
	} // writeLn

	function writeNl(s){
		writeBefore(s+crLn);
	} // writeNl
	

	/**
		Clone a single level array, for multi levels, could have a recursive function.
		
		for (var i in ray) !!!
	*/
	function cloneArray(a){
		var b= new Array();
		var sz= a.length;
		for (var i=0; i<sz; i++){
			b[i]= a[i];
		}
		return b;
	}; // cloneArray


	/**
		Getthe selected value of the select selName
	*/
	function getSelVal(selName){
		list= document.getElementById(selName);
		if (list!==null){
			return list.options[list.selectedIndex].value;
		}else{
			return '';
		}
	}  // getSelVal
	
	
	function setSelVal(selName,val){
		var pt= document.getElementById(selName)
		if (pt!==null){
			pt.value= val;
		}
	}  // setSelVal


	/**
		Replaces all occurence of srch but don't avoid regular expression looking traps.
			srch : the string to search for.		(needle)
			rplc : the string to substitute.		(replacement)
			sbjct : the subject string to work on 	(hay)
	*/
	function strReplace(srch,rplc,sbjct){
		return sbjct.replace(new RegExp(srch,'g'), rplc);
	}; // strReplace
	
	
	/*
		Set the general Html object tagName with the given content and get rid of the pseudo code!
		Input:
			content : the subject string to strReplace and store
			tagName : the name of the tag to change its content.
			infx : for substitution in complex strings that can't be inserted directly.
	*/
	function setInnerHtml(content,tagName,infx){
		if (isNotUndef(infx)){
			content= replaceCodedChar(content,infx);
		}
		document.getElementById(tagName).innerHTML= content;
	}; // setInnerHtml
	
	/**
		Idem above but faster.
	*/
	function setInnHtml(content,tagName){
		document.getElementById(tagName).innerHTML= content;
	}; // setInnHtml
	
	
	
	/*
		Also see symertic : tools::jsStorageStr()
		Input param :
			content : content to change
			infx : prfx ... infx ... sffx
	*/
	function replaceCodedChar(content,infx){
		if (isUndef(infx)){
			infx= '';
		}
		
		content= strReplace('bS'+infx+'SQ', '\'', content);
		content= strReplace('bS'+infx+'dG', '\"', content);
		content= strReplace('bS'+infx+'dG', '\\"', content);
		content= strReplace('dB'+infx+'lG', '"', content);
		content= strReplace('Cr'+infx+'Ln', crLn, content);
		return content;
	}; // replaceCodedChar
	
	
	/*
	*/
	function guestVarAccess(pw,nm){
		localStorage.serverGuestPw= pw;	
		localStorage.serverGuestName= nm;	
	}; // guestVarAccess


	/* --------------------------------------
	*/
	function adaptCombo(comboId,val){
		var list= document.getElementById(comboId);
		list.value = val;
	}; // adaptCombo
	
	
	/*
		Returns the value of the string on the left of the character c
	*/
	function strLeftOfC(str,c){
		if (str===''){
			return '';
		}
	
		var sz= str.length;
		var curC= str.charAt(0);
		var curStr= '';
		var i=0;
		while (i<sz && curC!==c) {
			curStr+= curC;
			i++;
			curC= str.charAt(i);
		};
		return curStr;
	}; // strLeftOfC
	
	
	/*
		Returns the value of the string on the right of the character c
	*/
	function strRightOfC(str,c){
		if (str===''){
			return '';
		}
		
		var sz= str.length;
		var curC= str.charAt(0);
		
		var i=0;
		while (i<sz && curC!==c) {
			i++;
			curC= str.charAt(i);
		};
		
		var curStr= '';
		if (curC===c){
			i++;	// jumps c
			while (i<sz) {
				curC= str.charAt(i);
				curStr+= curC;
				i++;
			};
		}
		return curStr;
	}; // strRightOfC
	
	
	/*
		Capitalize 1st char of each str words
		From : http://stackoverflow.com/questions/4878756/javascript-how-to-capitalize-first-letter-of-each-word-like-a-2-word-city
	*/
	function toTitleCase(str){
	    return str.replace(/\w\S*/g, function(txt){return txt.charAt(0).toUpperCase() + txt.substr(1).toLowerCase();});
	} // toTitleCase


	/**
		Used to decypher a naïve pair to pair encryption, but could also be used to encrypt.
	*/
	function tinyCrypt(str){
		var sz= str.length;
		var curC= str.charAt(0);
		var bStr= '';
		
		var even= true;
		for (var i=0; i<sz; i++){
			if (!even){
				bStr= bStr + str.charAt(i) + str.charAt(i-1);
			}
			even= !even;
		}
		
		if (!even){
			// catches the misssing tip
			bStr= bStr + str.charAt(i-1);
		}
		
		return bStr;
	}; // tinyCrypt
	
	
	/**
		Removes carefully a known string suffix and return the string base.
	*/
	function stringBase(fullString, sffx){
		var fsL= fullString.length;

		if (this.isUndef(sffx)){
			sffx='px';
		}

		var sxL= sffx.length;	
		var extStr= '';


		if (fsL>=sxL){
			extStr= fullString.slice(fsL-sxL, fsL);
		}
		
		if (extStr==sffx){
			return fullString.slice(0, fsL-sxL); // suffix removed
		}else{
			return fullString;	// suppose whole string originally without suffix is ok!
		}
	}; // stringBase
	
	
	
} // kk