<?php
/**
	File name : mbCrypt.php
	Author of this file : Oscar Fawer until 2019-08-15
	Contact : www.ataox.org 
	Web sites : ataox.org / ataox.com / ataox.info
	Copyrights open source GPL, GNU General Public License V3 : see ./aMainDr/code/LICENSE.txt


	Multi-byte encryption with symetrical javascript.
*/
class mbCrypt extends oTop{
	

	function __construct(&$asa){
		parent::__construct();
		
		$this->glob= &$asa->glob;
		$this->tools= &$asa->tools;
		$this->caps= &$asa->caps;
		$this->pRnd= &$asa->pRnd;
		
		$this->buffCHToH= '';
	} // construct



	/**
		Multi-byte encryption.
		Input :
			$mbStr : a multi byte string to crypt, in fact any string protocol that can be directly split into bytes
			$crptK : the given crypting key, (format : as above any string that can be split into bytes) with substr() or $crptK[index]
		Output :
			return an hexadecimal crypted string.
	*/
	function mbCrpt($mbStr,$crptK,$sortOfInt='def'){
		$hxInfo= $this->utfStrToHexStr($mbStr);
		$hxKey= $this->utfStrToHexStr($crptK);
		return $this->cryptHxToHx($hxInfo,$hxKey, $sortOfInt);
	} // mbCrpt	
	

	/**
		Multi-byte decyphering.:
		Input :
			$mbStr : a hexadecimal string to decipher (ascii and parent or utf-8) eg : 'af12dce45', internally transformed
				into a real hexadecimal string.
			$crptK : the given crypting key, (format : utf-8 etc]
		Output :
			return a utf-8 readable string..
	*/
	function mbUnCrpt($hxInfo,$crptK,$sortOfInt='def'){
		$hxKey= $this->utfStrToHexStr($crptK);
	    $hxResult= $this->cryptHxToHx($hxInfo,$hxKey, $sortOfInt);
	    return $this->hexStrToUtfStr($hxResult);
	} // mbUnCrpt
	
	
	
	
	
	function hexaCrpt($hxInfo,$hxKey,$sortOfInt='def'){
		return $this->cryptHxToHx($hxInfo,$hxKey, $sortOfInt);
	} // hexaCrpt

	// Idem hexaCrpt
	function hexaUnCrpt($hxInfo,$hxKey,$sortOfInt='def'){
		return $this->cryptHxToHx($hxInfo,$hxKey, $sortOfInt);
	} // hexaUnCrpt
	

	/**
		Input :
			$hxInfo : the hexadecimal string to crypt.or decyphered
			$hxKey : the hexadecimal un / crypting key.
		Output :
			return the hexadecimal crypted or decyphered string
			
		$this->buffCHToH= '';
	*/
	function cryptHxToHx($hxInfo,$hxKey, $sortOfInt='def'){
		$pRnd= &$this->pRnd;

		$sumKey= $pRnd->sumHexStr($hxKey, $sortOfInt);
		$lnInfoStr= strlen($hxInfo);
		
		$rAr= $pRnd->serieMCG($sumKey,$lnInfoStr, $sortOfInt);	// loads the random vector, values 0 - 16
		$rndV= &$rAr['v'];
		
		$buffHxK= '';
						
	    $resultHxStr='';
		for ($i=0; $i<$lnInfoStr; $i++){
			$hxInfoUnit= $hxInfo[$i];	
			
			$intK= (int)$rndV[$i];
			$hxKey= dechex($intK);
			$buffHxK.= $hxKey;

			$resultHxStr.= $this->charHexaXor($hxInfoUnit,$hxKey);
		} // for
		
		$this->buffCHToH.= '## function cryptHxToHx :'._c_;
		$this->buffCHToH.= 'hxInfo= '.$hxInfo._c_;
		$this->buffCHToH.= 'Pseudo random hexa Crypt key= '.$buffHxK._c_;
		$this->buffCHToH.= 'resultHxStr= '.$resultHxStr._c_;
		
		return $resultHxStr;	
	} // cryptHxToHx
	
	/**
		Deprecated cryptHToH() Version machine register XOR etc
		Reasonment : hexadecimal string are a very practical common ground for the communication between javascript utf-16
		and php utf-8 !! Or if want to use the fast machine routine like the xor "^" we have to recompose the input into bytes,
		and not only 4 bits hexadecimal glyph! So it would be still faster, but is not worth the work for now! Especially because
		php encryption is aimed at small strings like passwords processed a small number of times. The JS encryption is faster
		because of machine routines and the both ways encryption in utf-16 can be used to crypt large text locally and it's quite
		secure because only crypted version travel on the Internet.
		Also see : http://stackoverflow.com/questions/14338640/xor-two-hex-strings
	*/
	function cryptHToH($hxInfo,$hxKey, $sortOfInt){}
	
		
	/**
		Simulation of two hex2bin() not available before (PHP >= 5.4.0)
	*/
	function hxToBin($hx){
		$packedStr= pack('H',$hx);	// $hxInt= ord(chr($hxInt));	// useless at the reverse of ord() in mbCrpt() ?
		return $packedStr;
	} // hxToBin

		
	/**
		Covert 2 contiguous hexadecimal digits (a byte) into a single integer
	*/
	function hexByteToInt($hexByte){
		$pRnd= &$this->pRnd;
		
		$str= (string)$hexByte;
		$c0= $str[0];
		$c1= $str[1];
		return 16*$pRnd->hexToInt($c0) + $pRnd->hexToInt($c1);
	} // hexByteToInt
	
		
	/**
		Simulation of two hex2bin() not available before (PHP >= 5.4.0)
	*/
	function hexToBin($h1,$h2){
		$pRnd= &$this->pRnd;
		$byteInt= 16*$pRnd->hexToInt($h1) + $pRnd->hexToInt($h2);
		$packedStr= pack('C',$byteInt);
		return $packedStr;
	} // hexToBin
	
		
	/**
	*/
	function doubleHexToByte($h1,$h2){
		$pRnd= &$this->pRnd;
		$byteInt= 16*$pRnd->hexToInt($h1) + $pRnd->hexToInt($h2);
		$packedStr= pack('C',$byteInt);
		return $packedStr;
	} // doubleHexToByte
	

	/**
		Transform a php utf8 string into an JS uncompatible hexadecimal string.
		Warning : not symetrical with JS because ord() give the ASCII order !! Still works for coding
		when staying in Php (usage), because bytes are bytes, but the resulting hex code is not an hexadecimal UTF8
	*/
	function utfStrToHexStr($str){
		$buff= '';
		$sz= strlen($str);
		for ($i=0; $i<$sz; $i++){
			$c= $str[$i];
			
			// $iC= ord($c); $buff.= dechex($iC); // previously used ---> not safe if the charset changes
			$buff.= bin2hex($c);
		} // for
	
		return $buff;
	} // utfStrToHexStr
	
	// hxStr
	function hexStrToUtfStr($hxStr){
		$buff= '';
		$even= true;
		$sz= strlen($hxStr);
		for ($i=0; $i<$sz; $i++){
		
			if ($even){
				$h1= $hxStr[$i];
			}else{
				$h2= $hxStr[$i];
				$buff.= $this->doubleHexToByte($h1,$h2);
			}
			
			$even= !$even;
		} // for
	
		return $buff;
	} // hexStrToUtfStr	
	
	/**
		Convert a utf8 string into a utf16 string, then into its hexadecimal representation, 
		for Javascript compatibility.
		Warning!! Only Php7 has been tested as fully competent (Php 5.? failed on accentuated chars),
		the probable culprit is mb_convert_encoding(), unlikely bin2hex().
	*/
	function utf8to16toHx($mb8){
		$mb16= mb_convert_encoding($mb8,'UTF-16');
		return bin2hex($mb16);
	} // utf8to16toHx
		



	/* -----------------------------------------------------------------------------------------------------
		Xor working on hexa character at string / vector level for a better control of the whole process
	*/
	


	/**
		Operate a Xor on 2 hexa strings of identical length.
	*/
	function stringHexaXor($hxA,$hxB){
	
		$buff= '';
		$sz= strlen($hxA);
		for ($i=0; $i<$sz; $i++){
			$hexaChar= $this->charHexaXor($hxA[$i],$hxB[$i]);
			$buff.= $hexaChar;
		} // for
		
		return $buff;
	} // stringHexaXor


	/**
		Takes 2 hexadecimal characters and computes the XOR character.
	*/
	function charHexaXor($hxA,$hxB){
	
		$binStrA= $this->hxChrToBinStr($hxA);
		$binStrB= $this->hxChrToBinStr($hxB);
		
		$xBitStr= $this->xorBitStr($binStrA,$binStrB);
	
		return $this->binStrToHxChr($xBitStr);
	} // charHexaXor
	
	
	/**
		Transform an Hexadecimal charater into a binary string.
	*/
	function hxChrToBinStr($hChr){
		$pRnd= &$this->pRnd;
	
		$hInt= $pRnd->hexToInt($hChr);
		$bStr= '';
		$i= 0;
		do {
			$Q= (int)floor($hInt / 2);
			$R= $hInt % 2;
			$hInt= $Q;
			
			if ($R===0){
				$bStr= '0'.$bStr;
			}else{
				$bStr= '1'.$bStr;
			}
			$i++;
		} while($Q > 0);
		
		if ($i<4){
			$bStr= $this->zerosStr(4-$i).$bStr;
		}
		
		return $bStr;
	} // hxChrToBinStr
	
	
	/**
		Transform a 4 "bits" bit string into an hexadecimal characters.string.
	*/
	function binStrToHxChr($bStr){
		$intSum= 0;
		$power= 1;	// 2^0
		for ($i=3; $i>=0; $i--){
			$bit= (int)$bStr[$i];
			$intSum= $intSum + $bit * $power;
			$power= 2*$power;
		} // for
		
		return $this->intToHex($intSum);
	} // binStrToHxChr


	/**
		Xor the 2 given bit string.
	*/
	function xorBitStr($s,$t){
		$buff= '';
		for ($i=0; $i<4; $i++){
			$buff.= $this->xorBitChar( $s[$i], $t[$i] );
		} // for
		return $buff;
	} // xorBitStr


	/**
		Xor the 2 given '0' or '1'
	*/
	function xorBitChar($c1,$c2){
		$a= $c1==='1';
		$b= $c2==='1';
		$bXor= ($a && !$b) || (!$a && $b);
		if ($bXor){
			return '1';
		}else{
			return '0';
		}
	} // xorBitChar

	
	/**
	*/
	function zerosStr($sz){
		$str= '';	
		for ($i=0; $i<$sz; $i++){
			$str.= '0';
		} // for
		return $str;
	} // zerosStr
	

	/**
		Alias of dechex().. Also see pseudoRnd::hexToInt()
	*/
	function intToHex($n){
		return dechex($n);
	} // intToHex


	/**
	*/
	function chabl(){
	} // chabl

} // mbCrypt
?>
