<?php
/**
	File name : primJsObfusc.php
	Author of this file : Oscar Fawer until 2019-08-15
	Contact : www.ataox.org 
	Web sites : ataox.org / ataox.com / ataox.info
	Copyrights open source GPL, GNU General Public License V3 : see ./aMainDr/code/LICENSE.txt


	Primitive JS email obfuscation to hide them from lazy bots.
	
	Algorithm :
		- The "@" is almost always coded differently because it's coded as a function of the address itself 
		(eg : String.fromCharCode(46+18) or String.fromCharCode(50+14) etc.).
		- The displayed string is obfuscated too.
*/
class primJsObfusc extends oTop{
	
	/*	---------------
		- CONSTRUCTOR -
		--------------- */
	function __construct(&$tagsFu){
		parent::__construct();	
		$this->tagsFu= &$tagsFu;
		$this->caps= &$tagsFu->caps;
	} // construct


	/* ----------------------------------------------------------------------------------------

		----------------------------------------
		- The only method to call from outside -
		----------------------------------------

		Parameters :

			$eMail : complete e-mail address xyz@somewhere.com
			$displTxt : displayed text (this is coded as well!)
			$usedCSSFont : the CSS font used to define the look of $displTxt

	*/
	function fullJsObfuscation($eMail,$displTxt,$usedCSSFont){
		$hrefMailStr= 'mailto:'.$eMail;
		$hrefMailStr= $this->jsStrObfuscation($hrefMailStr,3);
		$displTxt= $this->jsStrObfuscation($displTxt,1); // obfuscation of the displayed text
		$hrefMailStr= $this->jsFuHref($hrefMailStr,$displTxt,$usedCSSFont);
		
		return $this->jsWriteString($hrefMailStr);
	} // fullJsObfuscation

	
	/*
		obfuscates $nb1
	*/
	function numberObfuscation($nb1,$cyphNb){
		if ($nb1>$cyphNb)
			$res= $nb1 % $cyphNb;
		else
			$res= $cyphNb % $nb1;
		if ($res==0) 
			$res= $nb1/2;
		$res= (int)$res;
		$leftNb= $nb1-$res;
		return $leftNb.'+'.$res;
	} // numberObfuscation

	/*
		$gHref is formated as a js string, so surrounded with '
		the other param are not betwen quotes.
	*/ 
	function jsFuHref($gHref, $gDspl, $gStyle='lnLinks'){
		if ($gStyle=='')
			$sStyl= '';
		else
			$sStyl= ' class="' . $gStyle . '"';

		return '\'<a href="\'' .'+'. $gHref .'+'. '\'"' . $sStyl . '>\'' .'+ '. $gDspl .'+'. '\'<\/a>\'';
	} // jsFuHref

	/*
		Obfuscation $gStr.
	*/
	function jsStrObfuscation($gStr,$step=1){
		$szStr= strlen($gStr);
		if ($step >= $szStr)
			return $gStr; // no obfuscation

		$pickLetter= $szStr * 0.75;
		$pickLetter= (int)$pickLetter;
		$obfNb= $this->numberObfuscation( ord('@'),ord($gStr[$pickLetter]) );

		$obfStr= '\''.substr($gStr,0,$step).'\''; // starter string, so can add JS + on next string
		$stepStr='';
		$szStepS= 0;
		for ($i=$step; $i<$szStr; $i++){
			$cChar= $gStr[$i];
			$gotAt= $cChar==='@';
			if (!$gotAt){
				$stepStr.= $cChar;
				$szStepS++;
			}
			if ($szStepS===$step || $gotAt){
				$obfStr.= '+\'' . $stepStr . '\'';
				$stepStr='';
				$szStepS=0;
				if ($gotAt) $obfStr.= '+' . 'String.fromCharCode('.$obfNb.')';
			}
		} // for
		
		if ($szStepS!=0){
			$obfStr.= '+\'' . $stepStr . '\'';
		}
			
		return $obfStr;
	} // jsStrObfuscation

	/*
		Just writes the given string through javaScript means
	*/
	function jsWriteString($gStr){
		$caps= &$this->caps;	// just interested in the caps part of tagFu
	
		$b= _c_R.'var str=' . $gStr . ';';
		$b.= _c_R.'document.write(str);';
		$b.= _c_R;
		
		return $caps->jsTag($b);
	} // jsWriteString	

} // primJsObfusc
?>
