<?php
/**
	File name : tools.php
	Author of this file : Oscar Fawer until 2019-08-15
	Contact : www.ataox.org 
	Web sites : ataox.org / ataox.com / ataox.info
	Copyrights open source GPL, GNU General Public License V3 : see ./aMainDr/code/LICENSE.txt


	General php tools to complement different existing set of functions
	
	Most important functins :
		n urlGivenPage()
		n urlCurPg()
		n curPageName()
		addUrlVar()
		phpGotLargeInt()
		mbStrToAry()
		n getVal($key,$val=false)
		n postVal($key,$val=false)
		n gostVal($key,$val=false, $cryptUrl=false)

		
	Rem :
		mbTrim() : multi-bytes triming is done inside a few multibytes methods, directly tacking at array elements,
			but a direct multi bytes trimming could be useful in some functions considering that invisible bytes coulld
			start a complex UTF8 character ?
*/
class tools extends oTop{
	

	function __construct(&$asa){
		parent::__construct();
		$this->asa= &$asa;
		$this->glob= &$asa->glob;
		$this->acc= &$asa->acc;
		$this->rMg= &$asa->rMg;
		
		$this->shortCircuitUPN= false;
		$this->uniqEdtStr= false;	// only used inside this class
		$this->idxCookV= 0;
		$this->preCalculatedCurLg= false;
		$this->phpMemoGotLargeInt= false;
		$this->lastCreatedPgId= false;
		$this->theCurrentPgNm= false;
		
		define('_c_mbStrAtom',100);	// _c_mbString "Atom" size --> if bigger got splitted in 2 parts

		/*
			A quick shared spot to define a few constant for imgTags.php & tablesEditor.php
		*/
		define('_c_tedCurDtbTblNm','curTblToDspl');
		define('_c_tedGvFilter','ted_filter');	// Table Editor gen var filter
		
		
		// !!! Could put that table related code somewhere else or define a specific class
		define('_c_tedCurDtbRowId','rowId');	// concerned row of the database table, used for formular identification and url.
		define('_c_tedTblChunkId','tblChunkId');
		
		
		define('_c_snglEdtFieldName','snglEdtFieldName');
		
		define('_c_zHidOrigPgWdgtNm','zoomOrigPage');	// Widget name for origin page.
		
		$this->spcSetCookOnce= array();
		
		$this->isHzSPhoneState= _c_unspecified;
		
		
		// print('End tools constructor'._c_);
	} // construct
	
	
	function hPwC(&$hPwC){$this->hPwC= &$hPwC;}
	function rMg(&$rMg){$this->rMg= &$rMg;}
	
	function setAsqlAccess(&$aSql){$this->aSql= &$aSql;}
	function setDtbAccess(&$dtb){$this->dtb= &$dtb;}
	
	function setAccAccess(&$acc){$this->acc= &$acc;}
	function setCaps(&$caps){$this->caps= &$caps;}
	function addsFls(&$fls){$this->fls= &$fls;}
	function setTagsFu(&$tagsFu){$this->tagsFu= &$tagsFu;}	
	function setMbCrypt(&$mbCrypt){$this->mbCrypt= &$mbCrypt;}

	function setGenAssoc(&$assoc){
		$this->assoc= &$assoc;
	} // setRecov
	
		
	/**
		Backslash simple quotes. (substitute,change) $subject= str_replace($search,$replace,$subject);
	*/
	function bsOneQ($s){
		return str_replace('\'', '\\\'', $s);	
	} // bsOneQ
	
	/**
		Replace \" with ". Have a look at accents::backSlashC() for SQL anti injection.
	*/
	function bksDblQ($s){
		return str_replace(_c_bSh._c_dblQ,  _c_dblQ,  $s);	
	} // bksDblQ
	
	
	
	/**
		Makes sure $s holds only non breakable spaces.
	*/
	function nonBrkSpace($s){
		return str_replace(_c_Rs,_c_Ns,$s);	
	} // nonBrkSpace
		
	
	
	/**
		Returns the specified posted value.
			Could have tested the NULL value directly in many cases, 
			but this function gives other possibilities, is more explicit on the NULL value therefore more secure.
	*/
	function postVal($key,$val=false){
		if (isset( $_POST[$key] ))
			return $_POST[$key];
		else
			return $val;
	} // postVal
	
	function trimPostVal($key,$val=false){
		$res= $this->postVal($key,$val);
		if ($res!==false){
			return trim($res);
		}else{
			return false;
		}
	} // trimPostVal
	
	/**
		Returns a preferential trimed posted value from key 1 or 2 :
			If both valid (not empty not false) return $k1
			If only one valid, returns it
			Otherwise return false.
	*/
	function prefPostVal($k1,$k2,$d=false){
		$a= $this->trimPostVal($k1,$d);
		if ($a!==false && $a!==''){
			return $a;
		}else{
			$b= $this->trimPostVal($k2,$d);
			if ($b!==false && $b!==''){
				return $b;
			}else{
				return false;
			}
		}
	} // prefPostVal	
	
		
	/**
		Adds a 2nd and Nth url variable. THE PAGE NAME MUST ALWAYS BE BEFORE !!
		Input :
			$var : variable name
			$val : value. Must be given as url compatible.
	*/
	function addUrlVar($var,$val){
		if ($val===false){
			return '';
		}else{
			return _c_rawAmp.$var.'='.$val;
		}
	} // addUrlVar
	
	function addFirstUrlVar($var,$val){
		return _c_Interog.$var.'='.$val;
	} // addFirstUrlVar
	
	function varEquVal($var,$val){
		return $var.'='.$val;
	} // varEquVal

	/**
		As reprodUrlVar but with an integer incrementation !!!
	*/
	function incRepeatUrlVar($var='', $add=false){
		$val= $this->gostVal($var);
		if ($val!==false){
			if ($add!==false){
				$val= $val + $add;
			}
			return $this->addUrlVar($var,$val);
		}else{
			return '';
		}
	} // incRepeatUrlVar
	
	
	/**
		Reproduce a url or posted parameter within the URL if it exists. ! Also work for posts.
	*/
	function reprodUrlVar($var, $cryptUrl=false){
		$val= $this->gostVal($var,false,$cryptUrl);
		if ($val!==false && $val!=='' ){
			return _c_rawAmp.$var.'='.$val;
		}else{
			return '';
		}
	} // reprodUrlVar
	
	/**
		Reproduce a url with a priority to the post side..
	*/
	function reprodUrlPet($var, $cryptUrl=false){
		$val= $this->petVal($var,false,$cryptUrl);
		if ($val!==false){
			return _c_rawAmp.$var.'='.$val;
		}else{
			return '';
		}
	} // reprodUrlPet
	

	/**
	*/
	function gostVal($key,$val=false, $cryptUrl=false){
		$u= $this->getVal($key);
		
		if ($u===false){
			$u= $this->postVal($key);
			if ($u!==false && $cryptUrl){
				// A clear post result need to be crypted, (often to be added to a url)
				$mbCrypt= &$this->mbCrypt;
				$u= $mbCrypt->utfStrToHexStr($u);
			}
		}
		
		if ($u===false){
			$u= $val;
		}
		return $u;
	} // gostVal
	

	/**
	*/
	function petVal($key,$val=false, $cryptedUrl=false){
		$u= $this->postVal($key);
		if ($u===false){
			$u= $this->getVal($key);
			if ($u!==false && $cryptedUrl){
				$mbCrypt= &$this->mbCrypt;
				$u= $mbCrypt->hexStrToUtfStr($u);
			}
		}
		if ($u===false){
			$u= $val;
		}
		return $u;
	} // petVal
	
	
	/**
	*/
	function getVal($key,$val=false){
		if (array_key_exists($key,$_GET) ){		// or issert($_GET[$key])  fastest ??
			return $_GET[$key];
		}else{
			return $val;
		}
	} // getVal
	
	
	/**
		Input :
			$forbVrV : forbidden variable vector.
	*/
	function urlFromGet($forbVrV){
		$buff= _c_IndexFN;		
		$i= 0;
		reset($_GET);
		foreach ($_GET as $key=>$val){
			if (!in_array($key,$forbVrV)){
				if ($i===0){
					$buff.= _c_Interog.$key.'='.$val;
					$i++;
				}else{
					$buff.= _c_rawAmp.$key.'='.$val;
				}
			}
		}	
		return $buff;
	} // urlFromGet	
	
	
	/**
	*/
	function gotVal($key,$gVal){
		$val= $this->getVal($key,false);
		return $val===$gVal;
	} // gotVal
	
	
	/**
		From a general url string, extract the matching url variable array.
	*/
	function urlVarArray($url, $full=false){
		$url= str_replace(_c_realHtmlAmp,_c_rawAmp,$url);
		
		/* Rem : would be faster to explode with 3rd param $concat=false, then would be already exploded for
		   the following steps, then would need another method to replace the internal actual >mbExplode().
		*/
		$v= $this->mbExplode($url,'&');	
		
		$newV= array();	// the new url vaeriable vector
		$i= 0;
		reset($v);
		foreach ($v as $key=>$val){
			if ($i===0){
				if ($full){
					$stV= $this->mbExplode($val,'?');
					
					$rightStr= $stV[1];	// first variable name = sign and value
					$interV= $this->mbExplode($rightStr,'=');
					
					$newV[$i]= $interV[0];
				}else{
					$newV[$i]= '';	// jump the # prefix
				}
			}else{
				$iV= $this->mbExplode($val,'=');
				$newV[$i]= $iV[0];
			}
			$i++;
		}
		return $newV;
	} // urlVarArray
	
	
	/**
	*/
	function trimGetVal($key,$val=false){
		$res= $this->getVal($key,$val);
		if ($res!==false){
			return trim($res);
		}else{
			return false;
		}
	} // trimGetVal
	
	
	/**
		Return true if the $a is $_GET[] or $_POST[] defined
	*/
	function isGostDefined($a){
		return $this->gostVal($a)!==false;
	} // isGostDefined
	

	/**
		Deal with a bi mode button state dependant for its start state on a dtn genVar.
		Input : 
			$butName : the button name as cookie canonical variable and a general variable,
	*/
	function biModeButState($butName){
		$dtb= &$this->dtb;
		$state= $this->spcUTrigCook($butName,$butName);
		if ($state===false){
			$state= $dtb->getGenVar($butName,_c_ok);
			$this->spcSetCook($butName,$state);
		}
		return $state===_c_ok;
	} // biModeButState
	
	
	/**
		Return the global tacked state of the menu.
	*/
	function tacked(){
		$tcked= $this->biModeButState('tackMenu');
		$editorPos= $this->gostVal('editorPos');
		return $tcked || $editorPos!=='left';
	} // tacked
		
	/**
	*/
	function wrapped(){
		$dtb= &$this->dtb;
	
		$wrapVal= $this->spcUTrigCook('wrap','wrap');
		if ($wrapVal===false){
			// nothing is set, so need to use the database value
			$wrapVal= $dtb->getGenVar('textEditWrap',_c_ok);
			$this->spcSetCook('wrap',_c_ok);
		}
		return $wrapVal;
	} // wrapped
			
			
	/**
	*/
	function invertVal($val,$a,$b){
		if ($val===$a){
			return $b;
		}else{
			return $a;
		}
	} // invertVal
	
		
	/**
		invert ok
	*/
	function invertCok($val){	
		return $this->invertVal($val,_c_ok,_c_ko);
	} // invertCok


	// Alias !! Current language.
	function currentLg($gPg=false){
		$dtb= &$this->dtb;
		return $dtb->currentLg($gPg);
	} // currentLg


	// Give the current language.of the current page with a single "calculation".
	function curLgOnce(){
		if ($this->preCalculatedCurLg===false){
			$dtb= &$this->dtb;
			$this->preCalculatedCurLg= $dtb->currentLg();
		}
		return $this->preCalculatedCurLg;
	} // curLgOnce
	

	/**
		Input val from any relevant sources, priority to the forms (even if we should not 
		have conflicting situations).
		
		Warning : Access to getVal() results possible only if $valP is set to false! 
	*/
	function inputVal($key,$valP=false,$valG=false){
		$pv= $this->postVal($key,$valP);		
		if ($pv!==false && $pv!==''){
			return $pv;
		}else{
			return $this->getVal($key,$valG);
		}
	} // inputVal
	
	
	
	function inptVal($key,$val){
		return $this->inputVal($key,$val,$val);
	} // inptVal
	
	
	function linkCurrentPage($lbl,$cls,$urlPlus){
		$caps= &$this->caps;
		return $caps->linkC($this->urlCurPg().$urlPlus, $lbl,$cls);
	} // linkCurrentPage	
	
	
	/**
		Return the home page url (edition home page that can be different from the public home page)
	*/
	function urlHomeEdit(){
		$dtb= &$this->dtb;
		$pg= $dtb->getGenVar('mainHomePage');
		return $this->urlGivenPage($pg);
	} // urlHomeEdit
	
	/**
		Return the general url default page url
	*/
	function urlDefaultPg(){
		return $this->urlGivenPage(_c_permanDefPage);
	} // urlDefaultPg
	
	
	
	/**
		Returns the url of the current page!
	*/
	function urlCurPage(){
		$ar['justsrf']= _c_ok;	// without the unique prefix
		return $this->urlGivenPage($this->curPageName(),$ar);
	} // urlCurPage	


	/**
		A general version, free to pass whatever restricting arrea !!
	*/
	function genUrlCurPage($arCmd=false){
		return $this->urlGivenPage($this->curPageName(),$arCmd);
	} // genUrlCurPage	
	
			
	/**
		Returns true if we are in smartphone/mobileMode mode and false otherwise
	*/
	function isInSmartPM(){
		$dtb= &$this->dtb;

		$smVal= $this->getVal(_c_smartPhoneMode);
		if ($smVal===false){
			$retVal= $dtb->defltSmartPhoneMode===_c_ok;
		}else{
			$retVal= $smVal===_c_ok;	// Explicitelly defined in url, so comply
		}
		
		return $retVal;
	} // isInSmartPM
	
		
	/**
		Return true if we are in a horizontal smartphone state
		$tools->hzSmartPhoneState()
	*/
	function hzSmartPhoneState(){
		$dtb= &$this->dtb;

		if ($this->isHzSPhoneState!==_c_unspecified){
			return $this->isHzSPhoneState;
		}
		
		$retVal= $this->isInSmartPM() && $dtb->getGenVar('hzSmartPhoneMen')===_c_ok;
		$this->isHzSPhoneState= $retVal;
		return $retVal;		
	} // hzSmartPhoneState
	
	
	
	function isInCatWalk(){
		return $this->getVal(_c_catWalkMode)=== _c_ok;
	} // isInCatWalk
	
	
			
	function sPhoneKnock(){
		$sPhVal= $this->getVal(_c_smartPhoneMode);
		return $this->invertCok($sPhVal);
	} // sPhoneKnock
			

	/** -----------------------------------------
		Get the current page name/id
	*/
	function curPageName(){
		$glob= &$this->glob;
		$rMg= &$this->rMg;
		$dtb= &$this->dtb;
		
		if ($this->theCurrentPgNm!==false){
			return $this->theCurrentPgNm;
		}
		
		$zmKey= $this->getVal(_c_idTextsZm);
		$gotZoomKey= $zmKey!==false;
		if ($gotZoomKey){
			// #gotZoomKey : test the existance of the gallery Id
			$gotZoomKey= $dtb->elementExists(_c_idTextKey,$zmKey, _c_idTextsTbl, $zmKey); // checks if the key exists inside the table
		}
		
		if ( $gotZoomKey ){
			// Special zoom, the page name does not appear directly
			$pg= _c_bigZoomPg;
		}else{
			$pg= $this->getVal(_c_urlPageVar,_c_permanDefPage);
			if ($pg==='' || $pg===false){
				return _c_permanDefPage;
			}
						
			if ( $this->sIsPrefixB('_random',$pg) ){
				$pg= '_random';
			}else{
				// standard case
				if (_c_acceptUtfPgN){
					// if a $pg matching utf8 is defined inside the table consider  that the input is indeed a utf8 and return the ascii page Id
					$pgCandidate= $dtb->getElem(_c_ascUtfTN, _c_auiUtfId,$pg, _c_auiAscId);
					if ($pgCandidate!==false && $pgCandidate!==''){
						$pg= $pgCandidate;
					}
				}
			}
		}
		
		$this->assignCurrentPg($pg);
		return $pg;
	} // curPageName
	

	/**
		Assign the current page shortcut
	*/
	function assignCurrentPg($pg){
		$this->theCurrentPgNm= $pg;
	} // assignCurrentPg
	


	/**
	*/
	function curMothrPg(){
		return $this->getVal(_c_mothrPage);
	} // curMothrPg


	
	
	/**
		English Ascii page name to Utf8 page name if the gemeral langiage flag exist and the field is filled!
		Input :
			$pg : the ascii page name
		Ouput :
			return utfPg (the utf8 page name), or pg again if it's not accessible
	*/
	function enAsciiPgToUtfPg($pg){
		$dtb= &$this->dtb;
		$utfPg= $dtb->getElem(_c_ascUtfTN, _c_auiAscId,$pg, _c_auiUtfId);
		if ($utfPg===false || $utfPg===''){
			return $pg;
		}else{
			return $utfPg;
		}
	} // enAsciiPgToUtfPg
	
	
	/**
		Get the current page name/id. Also work when sending a form because the page name is again in the url
		of the form results.
	*/
	function getUrlPageName(){
		if ($this->shortCircuitUPN){
			return $this->shortCircUpnVal;
		}else{
			return $this->curPageName();
		}
	} // getUrlPageName
			
	/**
		Reproduce a url param that is not forbidden externally ( forbidden state known through $this->aay() )
		Long explanation :
			Reproduce a url or posted parameter within the URL : 
				if it is not explicitelly forbidden externally (so dealt with externally)
				if it exists {tested inside tools::reprodUrlVar() }
				
		Input/OutPut : $urlBuff : the full url buffer to eventually increase.
		Input : 
			$uVarN : the name of the concerned url variable.
			$forbidVal : a forbiden value if different from false
	*/
	function noTwinReprodUVar(&$urlBuff, $uVarN, $forbidVal=false){
		if ($forbidVal!==false){
			$curVal= $this->gostVal($uVarN);
			if ($curVal===$forbidVal){
				return;
			}
		}
		
		$doneOutside= $this->aay($uVarN);
		if ($doneOutside!==true){
			$urlBuff.= $this->reprodUrlVar($uVarN);
		}
	} // noTwinReprodUVar
	
		
	/**
		Returns 
			true if the _c_okDecom url passed value belongs to allowed values
			false if the value is not accepted ... anymore or ever been!
	*/
	function validOkDecomUrlVal(){
		$dtb= &$this->dtb;
		$okDecomVStr= $dtb->getGenVar(_c_okDecom);
		if ($okDecomVStr===false || $okDecomVStr===''){
			// The webmaster did not bother about these advert global concepts so adverts depend uniquely on their specific autorization, see : wrb2NarrowVrtAdOk, wrb1ThinHzAdOk
			return false;
		}
		
		
		
		if ($okDecomVStr===_c_ok){
			// initial state  "ok" avoid adverts everytime
			return true;
		}
		
		
		
		if ($okDecomVStr===_c_ko){
			// initial state  "ko" no way to avoid the adverts except for the webmaster in edition or THE valid webmaster value
			$strV= array();
		}else{
			$strV= $this->trimStrToV($okDecomVStr);
		}

		
		// To complete the main array with the webmaster string
		$mastStr= $dtb->getGenVar(_c_deComMasterVal);
		if ($mastStr!==false && $mastStr!==_c_ko){
			array_push($strV,$mastStr);
		}
		
		$needle= $this->gostVal(_c_okDecom);
		return in_array($needle,$strV,true);
	} // validOkDecomUrlVal

	
	/**
		function urlGivenPage() ... Returns the url for the given page $pg.
		
		To search : n aay(
		
		Input :
			$pg : imposed page.
			$arCmd : array commands, to pass differents complementary values with an associated array 
			(the idea is to avoid parameters cloging of this function). ---> $this->aay() mainly used to
			control other parameters.
				
		Used array index strings :
			supUrl : supplementary url.
			forceIndex : to force an index (1st php file name) different from _c_IndexFN.
			forceUnqStr : a unique string to differentiate the cookie name of different user using the same browser!! mainly usefull to test multi-user capabilities with the same browser.
			
			/----------------------------------------------------------
			shortCircAtt : to short circuit one specified attribute :
				$arCmd['shortCircAtt']= _c_surfEdt;
			Rem : shortCircAtt can be practical when there is a single usrl variable that need redoncy avoidance
			... but as the cms become larger many avoidance can interact together implying that a specific
			index most be added nullifying the need for the general one along the was.
	*/
	
	
		/**
			Returns the url of the current page. $tools->urlCurPg()
			!! When it's not false, passes a small array in parameter by values !!
		*/
		function urlCurPg($ar=false){
			$pg= $this->curPageName();
			return $this->urlGivenPage($pg,$ar);
		} // urlCurPg
	
	
	
	function urlGivenPage($pg, $arCmd=false, $dotSlash=true){
		$glob= &$this->glob;
		$dtb= &$this->dtb;
		$rMg= &$this->rMg;
		
		// Sets a new $this->globArCmd each times!
		if ($arCmd===false){
			$this->globArCmd= array();
		}else{
			$this->globArCmd= &$arCmd;
		}
		
		
		$pwCheckT= $rMg->pwCheck3();
			
		if ($pg==='#'){
			$buff= '#';	// stay on the same page
		}else{
			if (!$pwCheckT){
				// uses redirection from _c_pgRedirectTbl only when not logged, because the webmaster need to access all pages
				$redirPg= $dtb->getElem(_c_pgRedirectTbl, _c_redirectPgId,$pg,_c_redirectPgTrgt);
				if ($redirPg!==false){
					$pg= $redirPg;
				}
			}
			
			if (_c_acceptUtfPgN){
				$pg= $this->enAsciiPgToUtfPg($pg);	
			}
			
			if ($dotSlash){
				$buff= _c_flTchHomeSlsh;
			}else{
				$buff= '';
			}
				
			$forceIndex= $this->aay('forceIndex');
			if ($forceIndex===false){
				$buff.= _c_IndexFN._c_Interog._c_urlPageVar.'='.$pg;
			}else{
				$buff.= $forceIndex._c_Interog._c_urlPageVar.'='.$pg;
			}
			
			if ($this->aay('rawUrl')===_c_ok){
				return $buff;
			}
			
			$justsrf= $this->aay('justsrf')===_c_ok;
			$avoidState= $this->getVal(_c_statsAvoidUVar)===_c_ok;
			if ($justsrf || $avoidState){
				// so avoid adding stats when the surfing start from a webmaster editing environment
				if ($this->aay(_c_statsAvoidUVar)!==_c_ko){	// avoids redondancy
					$buff.= $this->addUrlVar(_c_statsAvoidUVar,_c_ok);
				}
			}
			
			
			if ( ($this->aay('forceUnqStr') || $pwCheckT) && !$justsrf ){
				$buff.= $this->globCondUniqStrUV();
			}
			
			/**
				(*1) Simple url variables : to have them accessible to external plugins 
				---> uses a dedicated associative array.
			*/
			$uSP= $this->getVal('uStopParser');
			if ($uSP!==false){
				$buff.= $this->addUrlVar('uStopParser',$uSP);
			}
			
			
			$this->noTwinReprodUVar($buff,_c_catWalkMode,_c_ko);
			$this->noTwinReprodUVar($buff,_c_sortOfZoom);
			$this->noTwinReprodUVar($buff,_c_smartPhoneMode);
			if ($this->validOkDecomUrlVal()){
				$this->noTwinReprodUVar($buff,_c_okDecom);
			}
			
			
			$buff.= $this->simpleUvStr();
			if ($pwCheckT){
				// Got full editing rights!
				$sredit= $this->aay(_c_surfEdt);
				if ($sredit!==true){
					$buff.= $this->stdUrlStringAdd(_c_surfEdt);
				}
				
				$buff.= $this->stdUrlStringAdd(_c_editorPos);
				$buff.= $this->stdUrlStringAdd(_c_hideTopEditBar);
								
				$eLg= $this->getVal(_c_edtLang);	
				if ($eLg!== false && $this->aay('shortCircAtt')!==_c_edtLang){
					$this->noTwinReprodUVar($buff,_c_edtLang);
				}
				
				if (_c_allowNoPwMode===_c_ok){
					// Unique couple var/val to surf discretly in a no-password mode for a short time.
					$buff.= $this->addUrlVar(_c_nPwModeUVar,_c_nPwModeUVal);
				}
				
				if ($this->aay('shortCircAtt')!==_c_tableUrlVar){
					$buff.= $this->reprodUrlVar(_c_tableUrlVar);
				}
				
				if ( $this->aay('resetEdtMultiDtb')!==_c_ok && $this->gostVal('tedExoDtbImplem')!==false ){
					// for exo-database edition
					$buff.= $this->reprodUrlVar('tedExoDtbImplem');
					$buff.= $this->reprodUrlVar('tedExoDtbUsrId');
					$buff.= $this->reprodUrlPet('exoDtbNmOrDirNm');
					$buff.= $this->reprodUrlVar('tedExoDtbPath');
				}
				
				if ( $this->aay('resetEdtMultiDtb')!==_c_ok && $this->aay('tedSearchBut')!==_c_ok ){
					$buff.= $this->reprodUrlVar('tedSearch');
					$buff.= $this->reprodUrlVar('sqlStringTxBx',true);	// the sql string itself !!
					$buff.= $this->reprodUrlVar('gSearchWrdTBx1');
					$buff.= $this->reprodUrlVar('curTblFieldsCombo');
				}
				
				if ($this->aay(_c_tedCurDtbTblNm)!==_c_ko){
					$buff.= $this->reprodUrlVar(_c_tedCurDtbTblNm);
					$buff.= $this->reprodUrlVar(_c_tedCurDtbRowId);
				}
				
				$uvChunk1= 'mmDFChunkPos1';
				if ($this->aay($uvChunk1)!==_c_ko){
					$buff.= $this->reprodUrlVar($uvChunk1);
				}
				
				$uvChunk2= 'mmDFChunkPos2';
				if ($this->aay($uvChunk2)!==_c_ko){
					$buff.= $this->reprodUrlVar($uvChunk2);
				}
			} // if ($pwCheckT)
			
			// Defining a specific key to avoid url addition so can have many specific forbidding in // !!
			if ($this->aay(_c_mothrPage)!==_c_ko){
				$buff.= $this->reprodUrlVar(_c_mothrPage);	// mother page for forums
			}
			
			if ($this->aay('shortCircAtt')!==_c_loginTrialVar){
				if (!$this->sIsInB('loginNb=0',$buff)){
					$buff.= $this->incRepeatUrlVar(_c_loginTrialVar,1);
				}
			}
			
			/**
				Unusual use of $this->aay() that is normally used to restrict, 
				here it's used to add whatever was defined at the given index.
			*/
			$buff.= $this->aay('supUrl','');
		}	
		return $buff;
	} // urlGivenPage	
	

	/**
		Returns the value of a specific associative array row, with existence test and specified default value.
		--> A compact way to pass parameters to $this->urlGivenPage() when they are treated internally explicitelly
		Input:
			$entry : index of the $this->globArCmd array
			$defVal : default value if $entry is not defined.
			
		They are 3 different ways of using this function depending how we assign the main array and how 
		the matching variables are treated in tools::urlGivenPage(...) :	
		
		(A) Best practice, values that are tested explicitelly inside tools::urlGivenPage(...) :
			---> just got to assign TRUE to the concerned index of the array, like with 
				 $arCmd[_c_smartPhoneMode]= true; // localise the control.
			Fully advantageous on (B)
		    
		(B) 'shortCircAtt' using this index to short circuit (in that case, not add automatically) the matching
			value (significative strings not boolean as above) on the list of parameters.
			Not very practical, and can short circuit only one url variable using that convention inside  tools::urlGivenPage(...)
			But ... was implemented before (A)
			
		(C) A combination of both ways as done in _c_surfEdt
		
		
	*/
	function aay($entry,$defVal=false){	
		$arCmd= &$this->globArCmd;
				
		if ($arCmd!==false && $arCmd!==null && array_key_exists($entry,$arCmd)){
			return $arCmd[$entry]; // If returns ko --> we do not stack this variable on Urls
		}else{
			return $defVal;
		}	
	} // aay
	
	
	
	/**
		Calculates the surf/edit url or another url string (standard Url string) addition.
		!!! Works only if the same parameter is not used somewhere else in between!!!
	*/
	function stdUrlStringAdd($urlVar){
		if ($this->aay('shortCircAtt')!==$urlVar){
			$sdit= $this->gostVal($urlVar);
			if ($sdit!==false){		// so don't specify it all the time
				return $this->addUrlVar($urlVar,$sdit);
			}
		}
		
		return '';
	} // stdUrlStringAdd
	


	/**
		Return the edition url language addition, if the given one is valid
	*/	
	function urLangAddition($eLg){
		$glob= &$this->glob;
		if ($glob->isaDefinedLg($eLg)){
			return $this->addUrlVar(_c_edtLang,$eLg);
		}else{
			return '';
		}
	} // urLangAddition	
	
	
	/**
		Treat all the simple url variable (*1) scanning associative array and returns the full string.
		The concerned url var/val couple is detected and if active used again for all urls.
		
		In other words : 
			(A simpler) if a key of $this->arrSUV is already in $_GET (as a url variable) and is not forbidden 
			(a match between the value of $this->arrSUV and the value of $_GET) then bufferise the url current var/val pair!
			(B even simpler) $this->arrSUV is loaded with keys we want to repeat in the url generation 
			except if they have the matching value in $this->arrSUV
	*/
	function simpleUvStr(){
		$tagsFu= &$this->tagsFu;
		
		$gotValAr= isset($this->arrSUV) && is_array($this->arrSUV);
		if (!$gotValAr){
			return '';
		}
	
		$buff= '';
		reset($this->arrSUV);
		foreach ($this->arrSUV as $varName=>$forbidVal){
				$varVal= $this->getVal($varName);
				if ($varVal!==false && $varVal!==$forbidVal){
					$buff.= $this->addUrlVar($varName,$varVal);
				}
		}
	
		return $buff;
	} // simpleUvStr
	
	
	/**
		Just to add a variable to repeat inside the url.
		Not really a push but could be seen as one because element are stack in first in order
	*/
	function suvPush($varName,$forbidVal=_c_ko){
		$this->arrSUV[$varName]= $forbidVal;
	} // suvPush
	
	/**
		Called after a call to suvPush() eg : to make sure a variable with a spec val is not repeated!
	*/
	function suvModif($varName,$forbidVal){
		$this->arrSUV[$varName]= $forbidVal;
	} // suvModif
	

	/**
		Global condition on the Unique string url variable
	*/
	function globCondUniqStrUV(){
		$glob= &$this->glob;
		
		if ($glob->inhibUniqEdtPfx){
			return '';
		}else{
			return $this->urlVarUniqueEdtPrfx();
		}		
	} // globCondUniqStrUV
	
	/**
	*/
	function urlVarUniqueEdtPrfx(){
		return $this->addUrlVar( _c_urlUniqEdtPfx, $this->uniqueEdtStr() );
	} // urlVarUniqueEdtPrfx
	
	function uniqEdtPrfxVarEqVal(){
		return $this->varEquVal( _c_urlUniqEdtPfx, $this->uniqueEdtStr() ); // n varEquVal
	} // uniqEdtPrfxVarEqVal
	
	
	/**
		Return a unique Edt string url  to differentiates the "cookies" of different users
		on the same site/browser!
			1st generate
			2nd take it from the url
	*/
	function uniqueEdtStr(){
		$hPwC= &$this->hPwC;
		
		$pfx= $this->gostVal(_c_urlUniqEdtPfx);
		if ($this->uniqEdtStr!==false){
			$pfx= $this->uniqEdtStr;
		}
		
		if ($pfx===false){
			// generates a new digit-consonance random string
			$pfx= $this->uniqIndepEdtStr();	// used only once for each page calculation
			$this->uniqEdtStr= $pfx;
		}
		
		return $pfx;
	} // uniqueEdtStr
	
	/**
		Unique and independant editor string
	*/
	function uniqIndepEdtStr(){
		$hPwC= &$this->hPwC;
		return $hPwC->consonStr(_c_urlUniqEdtSz);
	} // uniqIndepEdtStr
	
	
	/**
		Basic url for the forum! Why here? Because we want a single tagsFu::tg_jumptobm
		tag definition, so the forum book mark definition is also at bottom level.
	*/
	function urlForumPost(){
		$url= $this->urlCurPg();
		if ($this->getVal(_c_fxCodeIndx)!==false){
			$url.= $this->reprodUrlVar(_c_fxCodeIndx);		// single thread forum
		}
		
		if ($this->getVal(_c_rfxPage)!==false){
			$url.= $this->reprodUrlVar(_c_rfxPage);		// multi thread forum
		}
		
		return $url;
	} // urlForumPost


	/**
		For the "instenciation" of the mark, see capsHtml::bookMark($id)
	*/
	function signet($id){
		$s= '';
		if ($id!==false){
			$s= '#'.$id;
		}
		return $s;
	} // signet

	
	/**
		Gives the last created id/page name
	*/
	function lastCreatedId(){
		if ($this->lastCreatedPgId!==false){
			return $this->lastCreatedPgId;
		}else{
			return false;
		}
	} // lastCreatedId


	/* ---------------------------------------------------------------------------------------------
		Local abstraction level 2 of the cookie methods. Similar named cookies on different database 
		posed a problem (cookies confusion), so it becomes database specific because of being prefixed 
		with _c_ckUniquePrfx. Database Specific. Cookies (spc prefix).
		Remark : it was optimised to manage private loggin access for webmaster, so the _c_ckUniqPrfx
		prefix changes on each page reload ---> simple cookie programming should manage the complete name
		using $this->ckVal() and $this->setCk() directly.
	*/

	// Dtb specific : Returns the cookie value.
	function spcCookVal($name,$val=false){
		return $this->ckVal($this->spc($name),$val);
	} // spcCookVal

	
	// Dtb specific : Sets the cookie value.
	function spcSetCook($name,$value,$expire=false){
		return $this->setCk($this->spc($name),$value,$expire);
	} // spcSetCook
	
	// Idem as above but if used exclusively,does it only once
	function spcSetCookOnce($name,$value,$expire=false){	
		if ( !array_key_exists($name,$this->spcSetCookOnce) ){
			$this->spcSetCookOnce[$name]= _c_ok;
			return $this->setCk($this->spc($name),$value,$expire);
		}else{
			return true;	// Already set and returns true by symetry to spcSetCook() because $this->setCk() always returns true!
		}
	} // spcSetCookOnce
	
	

	// Delete the cookie
	function spcDeleteCook($name,$path='/'){
		return $this->deleteCk($this->spc($name),$path);
	} // spcDeleteCook

	// Instant state of a cookie triggered by a url Variable.
	function spcUTrigCook($urlVar,$name,$defVal=false){
		return $this->uTrigCk($urlVar,$this->spc($name),$defVal);
	} // spcUTrigCook

	// specific cookie name. $this->spc($name)
	function spc($name){
		return _c_ckUniqPrfx.$name;
	} // spc

	/*
		End local abstraction level 2 of the cookie methods.
	--------------------------------------------------------------------------------------------- */



	/* ---------------------------------------------------------------------------------------------
		Local abstraction level 1 of the cookie methods.
	*/
	function ckVal($name,$val=false){
		$aSql= &$this->aSql;
		if ( isset($_COOKIE[$name]) && $aSql->secureSqlId($_COOKIE[$name])){
			return $_COOKIE[$name];
		}else{
			return $val;
		}
	} // ckVal
	
		
	/**
		Old abstraction of setcookie(), so now uses the javascript function createCookie() that avoids anoying server side effects of cookies creation, like ouput restriction.
		So NO MORE SERVER SIDE COOKIE SETTINGS (even so, cookies are quite messy and should not be used too often).  Remark read cookies with n ckVal(...) / $tools->ckVal(...)
		
		Input :
			$name : name of the cookie.
			$value : value of the cookie
			$expire : life timem of the cookie, default= one day otherwise time given in days!
			
		$tools->setCk($name,$value,$expire)
	*/
	function setCk($name,$value,$expire=false){
		$glob= &$this->glob;
		$caps= &$this->caps;
			
		if ($expire===false){
			$expire= 1; // one day
		}	
		
		$jsAddition= $caps->jsSimpleFu('genAsa.createCookie',3,0,$name,$value,$expire).';'._c_R;		
		$glob->setGlobJsCode($jsAddition);
		return true;
	} // setCk
	
	/**
	*/	
	function deleteCk($name){
		return $this->setCk($name,'',time()-3600);
	} // deleteCk
		
	
	/**
		Url Triggered Cookie Value : returns the instant state of a cookie triggered by a url Variable on a common value.
			Useful when url dependant cookies are used before they are set! And setting them before is not practicle
			for reasons like code destructuration (practicle to have same cookies access localised) ... could
			also be practical to have all url cookies assignation done at the strict beginning (no special test)
			but more dispersion of cookies identificators! ... May use both! ... If a cookie value is going
			to be used often, it's better to set it asap, but it it's used for sure only once, local function
			calls are probably better.
			
			Ideal call, with only one parameter : $this->uTrigCk($urlVar);
	*/
	function uTrigCk($urlVar,$cookName,$defVal=false){
		$urlVal= $this->getVal($urlVar);
		if ($urlVal===false){
			return $this->ckVal($cookName,$defVal);
		}else{
			return $urlVal;
		}
	} // uTrigCk
	
	function uTrigSpcCk($urlVar,$cookName,$defVal=false){
		$urlVal= $this->getVal($urlVar);
		if ($urlVal===false){
			return $this->spcCookVal($cookName,$defVal);
		}else{
			return $urlVal;
		}
	} // uTrigSpcCk
	
	
	
	
	/*
		End local abstraction level 1 of the cookie methods.
	--------------------------------------------------------------------------------------------- */
		
	/**
		List of parameters comPacted to an arRay
	*/
	function pactToRay($nbParam,$a0='',$a1='',$a2='',$a3='',$a4='',$a5='',$a6='',$a7='',$a8='',$a9='',$a10='',$a11=''){
		if ($nbParam===0){
			return '';
		}
		return array($a0,$a1,$a2,$a3,$a4,$a5,$a6,$a7,$a8,$a9,$a10,$a11);
	} // pactToRay
	
			
	/**
		To display the value of a specific cookie
	*/
	function viewCookie($key){
		print(_c_.'Cookie '.$key.'='.$this->spcCookVal($key,'undeclaredCookie'));
	} // viewCookie
	
	function niceCookieStr($key=''){
		if ($key!==''){
			$bf= _c_.'Cookie '.$key.'='.$this->spcCookVal($key,'undeclaredCookie')._c_;
			print($bf);
		}else{
			return '';
		}
	} // niceCookieStr
	
	
	/**
	*/
	function cookieWatcher($a,$b='',$c='',$d='',$e='',$f='',$g='',$h=''){
		$vect= $this->pactToRay(8,$a,$b,$c,$d,$e,$f,$g,$h);
		$sz= sizeof($vect);
		$bf= '';
		for($i=0;$i<$sz;$i++){
			$this->niceCookieStr($vect[$i]);
		}
	} // cookieWatcher
		
	/* 
		End cookie abstraction methods.
	--------------------------------------------------------------------------------------------- */
		
	
	/**
		Very close to binLoads($a,$b) !!!!!!
	*/
	function mostValid($a,$b){
		if ($a!==false){
			return $a;
		}
		return $b;
	} // mostValid
	
	
	/**
		Binary (boolean, true/false) loads : get the 1st param if not false, otherwise take the second. Especially usefull
		for the standard vector of variable functions.
	*/
	function binLoads($a,$b='',$k=false){
		if ($a!==$k){
			return $a;
		}else{
			return $b;
		}
	} // binLoads
		
	/**
		$tools->falseOrKo($a)
	*/
	function falseOrKo($a){
		return $a===false || $a===_c_ko;
	} // falseOrKo
	

	/**
		Translate an ok/ko state into a boolean equivalent.
	*/
	function okToBool($a){
		switch ($a){
			case _c_ok:
				return true;
			break;
			
			case _c_ko:
				return false;
			break;
			
			default:
				return $a; // consider that a boolean ws given, so no translation
		} // switch
	} // okToBool
	
	function boolToOk($k){
		if ($k===true){
			return _c_ok;
		}else{
			return _c_ko;
		}
	} // boolToOk
	

	
	/**
		Return true if $b is a boolean
	*/
	function isBoolean($b){
		return $b===true || $b===false;
	} // isBoolean
	
		
	/**
		When we get an empty string that we don't want to display, or a false than means more directly
		that there is nothing to display, it's practicle to test both cases at once!
	*/
	function noDisplay($a){
		return $a===false || $a==='';
	} // noDisplay
	
	
	/**
		Print a message only once.
		Input :
			$id : unique id.
			$mess : message.
	*/	
	function printOnce($id,$mess){
		if (!isset($this->printOnceV[$id])){
			$this->printOnceV[$id]= true;		
			print($mess);
		}
	} // printOnce	

	/**
	*/	
	function tooManyTagsMess(){
		$caps= &$this->caps;
		
		// Also the constant name in clear to find it easily
		$this->printOnce('cnt', $caps->eFont('Nb Tags > _c_parserMaxTimes ('._c_parserMaxTimes.')')._c_); 
	} // tooManyTagsMess	

	
	/**
		Return true if we are editing a post (single situation where the Nect field of the main
		table must appear).  _c_postT
	*/
	function editingAPost($id,$tbl){
		$dtb= &$this->dtb;
		return $dtb->getType($id,$tbl)===_c_postT;
	} // editingAPost
		
	/**
		Short circuit the url page name (rarely used : deletion and page rename)
	*/
	function setShortCircUPN($set,$newVal){
		$this->shortCircuitUPN= $set;
		$this->shortCircUpnVal= $newVal;
	} // setShortCircUPN
	
	/**
	*/
	function isUrlPg($pg){
		return $this->getUrlPageName()===$pg;
	} // isUrlPg
	
		
	/**
		Uses instead of the above methods.
	*/
	function urlMethodDef($methloc,$methparam){
		return _c_IndexFN._c_Interog.$this->addUrlVar(_c_urlMethLoc,$methloc)
			.$this->addUrlVar(_c_urlMethPar,$methparam);
	} // urlMethodDef
	
			
	/*-------------------------------------------------------------------------------
		StringMethodsStart : Also see "SPECIAL STRING MANIPULATIONS FOR FILENAMES"
			In filesMg.php
	*/	
	
	function isSimpleNumber($c){
		return ($c>='0' && $c<='9');
	} // isSimpleNumber

	function onlyDigits($s){
		$sz= strlen($s);		
		$i=0;
		while ($i<$sz && $this->isSimpleNumber($s[$i])){
			$i++;
		}	
		return $i===$sz;
	} // onlyDigits


	/**
		Returns true if the string $s (small) is a part of (belongs, is inside, exists etc.) 
		inside the string $b (big) & false otherwise. strpos($haystack, $needle)
	*/
	function sIsInB($s,&$b){
		if ($s===''){
			return true;	// convention : an empty string is a part of any string
		}
		
		if ($b===''){
			return false;	// obviously false except if ($s==='') treated above
		}
		
		return !(strpos($b,$s)===false);
	} // sIsInB
	
	/**
		As sIsInB() but case insensitive.
	*/
	function sIsInIB($s,&$b){
		if ($s===''){
			return true;	// convention : an empty string is a part of any string
		}
		
		if ($b===''){
			return false;	// obviously false except if ($s==='') treated above
		}
		
		return !(stripos($b,$s)===false);
	} // sIsInIB
	
	/**
		Return true if all elements of $v are inside $b
	*/
	function sVectIsInB($v,&$b){
		$buff= true;
		reset($v);
		foreach ($v as $key=>$val){
			$buff= $buff && $this->sIsInIB($val,$b);
			if (!$buff){
				break;
			}
		}
		return $buff;
	} // sVectIsInB
	
	/**
		Return true if at least ONE elements of $v is inside $b
	*/
	function vectElemIsInB($v,&$b){
		reset($v);
		foreach ($v as $key=>$val){
			if ($this->sIsInIB($val,$b)){
				return true;
			}
		}
		return false;
	} // vectElemIsInB
		
	/**
		Returns true if the string $s (small) is a prefix of the string $b (big) & false otherwise.
	*/
	function sIsPrefixB($s,&$b){
		return stripos($b,$s)===0;
	} // sIsPrefixB	
	function isPfxB($s,$b){return stripos($b,$s)===0;}	// shorter and by values
	
	
	/**
		Returns true if the string $s (small) is a suffix of the string $b (big) & false otherwise.
	*/
	function sIsSuffixB($s,&$b){
		$szS= strlen($s);
		$couldBeSffx= substr($b,-$szS);
		return $couldBeSffx===$s;
	} // sIsSuffixB	
	

	/**
		Return the size of the longuest string of the given array (should really have been included
		in the php functions), and an easy copy/paste location to get the array walk syntax.
	*/
	function szMaxStr(&$v){
		$max= 0;
		reset($v);
		foreach ($v as $key=>$val){
			$curLen= strlen($val);
			if ($curLen > $max)
				$max= $curLen;
		}

		return $max;
	} // szMaxStr


	/**
		Adapt the given string for the Vertical Display, so just adds spaces between characters,
		so the string can flow vertically.
	*/
	function whiteInflate($str){	
		$sz= strlen($str);		
		$b='';
		for ($i=0;$i<$sz;$i++){
			$b.= $str[$i]._c_;
		}		
		return trim($b);
	} // whiteInflate	
	
	
	/**
		Supresses invisible chars in the middle of a string and arround it
		Used mainly to format a string safe for file names!
		Inout : s : string to trim
	*/	
	function fullTrim($s){
		$acc= &$this->acc;
	
		$s= trim($s);
		$sz= strlen($s);
		$gotSpace= false;
		$buff= '';
		for($i=0; $i<$sz; $i++){
			$c= $s[$i];
			if ($acc->allowedChar($c) || $c===' '){
				if (!$this->invisCharOrHyphen($c)){
					if ($gotSpace){
						$c= strtoupper($c);
					}			
				
					$buff.= $c;
					$gotSpace= false;
				}else if ($c===' '){
					$gotSpace= true;
				}
			}
		} // for
		return $buff;
	} // fullTrim
		
	/**
		Trim the external parts and adds the given separator instead of spaces.
	*/
	function fullTrimAndSep($s,$sep=','){
		$s= trim($s);
		$sz= strlen($s);
		$gotSpace= false;	// space as invisible char
		$buff= '';
		for($i=0; $i<$sz; $i++){
			$c= $s[$i];
			if ($this->invisChar($c)){
				$gotSpace= true;
			}else{
				if ($gotSpace){
					$c= $sep.$c;						
				}			
				$buff.= $c;
				$gotSpace= false;
			}
		} // for
		return $buff;
	} // fullTrimAndSep
		

	/**
		Returns true if $c is an invisible char.
	*/
	function invisChar($c){
		return $c===_c_Rs || $c===_c_rT || $c===_c_nL || $c===_c_carRet; 
	} // invisChar	
	
	/**
		Returns true if got only invisible chars in $s
	*/
	function onlyInvsChar(&$s){
		$sz= strlen($s);
		for($i=0; $i<$sz; $i++){
			if ( !$this->invisChar($s[$i]) ){
				return false;
			}
		} // for
		return true;
	} // onlyInvsChar
	
	

	/**
		Returns true if $c is an invisible / unvisible char.
		invisCharOrHyphen
	*/
	function invisCharOrHyphen($c){
		return $this->invisChar($c) || $c==="-"; 
	} // invisCharOrHyphen	
	
	
	/**
		Supresses invisible chars outside the string and cariage returns in the middle.
	*/
	function jsSafeString($s){
		$acc= &$this->acc;
	
		$s= trim($s);
		$sz= strlen($s);
		$buff= '';
		for($i=0; $i<$sz; $i++){
			$c= $s[$i];
			if (!$this->jsStrUncomp($c)){
				$buff.= $c;
			}
		} // for
		return $buff;
	} // jsSafeString
	
	
	/**
		Returns true if $c is a character uncompatible with javascript strings.
	*/
	function jsStrUncomp($c){
		return $c==="\n" || $c==="\r"; 
	} // jsStrUncomp
	
	/*
		STRING METHODS END
	-------------------------------------------------------------- */	


	/**
		Return a visualizable string from a vector.
	*/
	function vecToStr(&$v){
		$s= '';
		reset($v);
		foreach ($v as $k=>$a){
			$s.='['.$k.']'.$a._c_;
		}
		return $s;
	} // vecToStr
	


	/*--------------------------------------------------------------
		VECTOR METHODS START
	*/
	
	/**
	*/
	function headV($v){
		reset($v);
		return current($v);
	} // headV
	
	/**
		Returns an array between start and end, separated by step
	*/
	function customNumVect($start,$end,$step=1,$onlyString=false){
		$a= array();
		for ($i=$start; $i<$end; $i+=$step){
			if ($onlyString){
				$strI= (string)$i;
				$a[$strI]= $strI;
			}else{
				$a[$i]= $i;
			}
		} // for
		return $a;
	} // customNumVect
	
	
	/**
		Returns true if $a belongs to the array $v.
	*/
	function aIsInV($a,&$v){
		if ($v===false){
			return false;
		}else{
			return array_search($a,$v)!==false;
		}
	} // aIsInV
	
	// kex exist ...
	function aIsInKv($k,$v){
		if ($v===false){
			return false;
		}else{
			return array_key_exists($k,$v);
		}
	} // aIsInKv
		
	
	/**
		Vectorise the list parameters
	*/
	function vectListParam(&$bd){
		$bd= ltrim($bd);
		
		// Substract the 1st bullet if there is one
		$sz= strlen($bd);
		if ($sz>2){
			$start= $bd[0].$bd[1].$bd[2];
			if ($start===_c_listBull){
				$bd= substr ($bd,3,$sz-3); 
			}
		}
		
		return explode(_c_listBull,$bd);
	} // vectListParam

	
	/**
		From String to vector (explode & trim) to have a better access to separated entities.
			Input :
				$sep separator
				$str string
			kw : split
		Remark : may secure case of contigus separators later.
		
		Do not mistake it with strToVect() that convert a string (multibyte as well) to a vector.
	*/
	function trimStrToVect($sep,&$str){
		return $this->trimStrToV($str,$sep);
	} // trimStrToVect
	
	function trimStoV($str,$sep=','){
		return $this->trimStrToV($str,$sep);
	} // trimStoV	
	
	/**
		Returns true if $needle is inside $str ($str is considered as a vector expressed as a string ..
		do not want vector-cell-parts considered as positive).
	*/
	function inVString($str,$needle,$sep=','){
		$v= $this->trimStrToV($str,$sep);
		return in_array($needle,$v,true); // strict comparaison
	} // inVString
	
	function trimStrToV(&$str,$sep=','){
		if (trim($str)==='' || $str===false){
			return false;
		}
	
		$v2= array();
		$v= explode($sep,$str);
		reset($v);
		foreach ($v as $key=>$val){
			$val= trim($val);
			if ($val!==_c_emptyStr){
				$v2[$key]= $val;
			}
		}
		return $v2;
	} // trimStrToV
	
	
		
	/**
	*/
	function trimFullVect(&$v){
		reset($v);
		foreach ($v as $key=>$val){
			$v[$key]= trim($val);
		}
	} // trimFullVect
	
		
	/**
		Trim the full vector of strings and collect only the Non Empty Strings!
		And remove empty strings !!
	*/
	function trimFVNES(&$v){
		$w= array();
		reset($v);
		$i= 0;
		foreach ($v as $key=>$val){
			$curS= trim($val);
			if ($curS!==''){
				$w[$i]= $curS;
				$i++;
			}
		}
		return $w;
	} // trimFVNES
	

	/**
		From 2 arrays, return one array holding all elements (values) with no duplicata.
		An empty associative array is === to false
	*/
	function arrayUnion(&$a1,&$a2){
		// Solves the trivial unions
		if ($a1===false && $a2===false){
			return false;
		}else{
			if ($a1===false){
				return $a2;
			}else{
				// $this->  pv($a1);
			}
			
			if ($a2===false){
				return $a1;
			}else{
				// $this->  pv($a2);
			}
		}
	
		$sz2= sizeof($a2);
		$j=0;
		// Scans $a2
		for($i=0;$i<$sz2;$i++){
			if (!in_array($a2[$i],$a1)){
				$diff[$j]= $a2[$i];
				$j++;
			}
		}
	
		if ($j===0){
			return $a1;
		}else{
			return array_merge($a1,$diff);
		}
	} // arrayUnion


	/**
		Remove the 1st element (plouf === inversed onomtopea of pop ?). Not very efficient.
		(Plouf : onomeatopea -- falling from the bottom of a column)
	*/
	function arrayPlouf($v){
		$first= true;
		reset($v);
		foreach ($v as $key=>$val){
			if ($first){
				$first= false;
			}else{
				$n[$key]= $val;
			}
		}
		return $n;
	} // arrayPlouf


	/**
		Add the 1st element at the beginning of the array. Not very efficient.
		(Foulp is the inverse of Plouf, sticking at the bottom)
	*/
	function arrayFoulp($v,$k1,$elem){
		$first= true;
		reset($v);
		foreach ($v as $key=>$val){
			if ($first){
				$first= false;
				$n[$k1]= $elem;
			}
			
			$n[$key]= $val;
		}
		return $n;
	} // arrayFoulp

	/**
		Replaces the litteral index of an array by numerical indexes.
			Input :
				$v : given vector.
				$cmboLblExept : combo box labelling exception flag applied on keys
	*/
	function numIndxVect($v,$cmboLblExept=false){
		$caps= &$this->caps;

		$i=0;
		reset($v);
		foreach ($v as $key=>$val){
			if ($cmboLblExept){
				if ($caps->isOGSE($key)){
					$a[$key]= $val;
				}else{
					$a[$i]= $val;
					$i++;
				}
			}else{
				$a[$i]= $val;
				$i++;
			}
		}
		return $a;
	} // numIndxVect
	
	
	/**
		Takes the keys of an array for the values of the new numerical (0 -- N-1) array
	*/
	function onlyKeys($v){
		$i=0;
		reset($v);
		foreach ($v as $key=>$val){
			$a[$i]= $key;
			$i++;
		}
		return $a;	
	} // onlyKeys
	
	
	/**
		Gives a string numerical vector from a given number
	*/
	function numStrVect($firstNb,$sz){
		for ($i=0; $i<$sz; $i++){
			$v[$i]= (string)($firstNb + $i);
		}
	
		return $v;
	} // numStrVect

	/**
		Adds a separator ($sep) between each element of an array ($v) and returns the total ($tot) string.
	*/
	function addSepar(&$v,$sep){
		$sz= sizeof($v);
	
		$tot= $v[0];
		for($i=1;$i<$sz;$i++){
			$tot.= $sep.$v[$i];
		}
		return $tot;
	} // addSepar

	/**
		Returns an array with the elements from $iStart to $iEnd.
		$iStart and $iEnd baing ABSTRACT indexes, so even if actual indexes are not numerical it is still working.
	*/
	function subArray(&$v,$iStart,$iEnd){	
		$j=0; $i=0;
		reset($v);
		foreach ($v as $key=>$val){
			if ($i>=$iStart && $i<=$iEnd){
				$newVect[$j]= $val;
				$j++;
			}
			$i++;
			
			if ($i>=$iEnd){
				break;
			}
		}
		
		return $newVect;
	} // subArray


	/**
	*/
	function reverseVect(&$A){
		$i= 0;
		reset($A);
		foreach ($A as $key=>$val){
			$k[$i]= $key;
			$v[$i]= $val;
			$i++;
		}
		
		for($j=$i-1; $j>=0; $j--){
			$key= $k[$j];
			$val= $v[$j];
			$B[$key]= $val;
		}
		
		return $B;
	} // reverseVect
	
	
	/**
		Don't have to worry about multi-bytes characters here because they are stored in correct lumps.
	*/
	function arrayToStr(&$v,$sep=''){
		$s= '';
		
		if ($sep!==''){
			end($v);
			$endKey= key($v);
			reset($v);
			foreach ($v as $key=>$val){
				if ($key!==$endKey){
					$s.= $val.$sep;
				}else{
					$s.= $val;
				}
			}
		}else{
			reset($v);
			foreach ($v as $key=>$val){
				$s.= $val;
			}
		}
		
		return $s;
	} // arrayToStr
			
	
	/**
		Standard (bytewise separated) string to array!  Faster than  mbStrToAry. So advised for strict one byte characters!
		Warning : when parsing and looking for byte tags UTF-8 could contain tags code in complex characters, so avoid if
		the whole set of characters is not full kmnowed!
	*/
	function stdStrToAry(&$s,&$sz){
	    $sz= strlen($s);
	    return str_split($s);	// return $s;// also works !!
	} // stdStrToAry


	/**
		Convert a UFT8 multi-byte string into an array.of single characters of any byte size!
			Input :
				$str
			Output :
				$sz : nb of elements of the returned array.
				Return the array itself.
				
		Advantage of using mbStrToAry() instead of mbStrToA() : deal explicitely with empty strings + may count() faster at some stage!
		
		Warning : can't use only successive mb_substr() because completion time is in K " $sz²2  (too long for large page)
		So we have the mbStrToA($str) recursive procedure that split the load to avoid the squarring effect!
	*/
	function mbStrToAry(&$str,&$sz){
		$dtb= &$this->dtb;
		if ($str==='' || $str===null){
			$sz= 0;
			return array();
		}
		
		if ( $dtb->getGenVar('fastMbStrToArray')!==_c_ok ){
			$vect= $this->slowMbStrToA($str);
		}else{
			$vect= $this->mbStrToA($str);	// fastMbStrToArray === ok
		}

		$sz= count($vect);
		return $vect;
	} // mbStrToAry
	
	
	/**
		Slow conversion but uses less memory
	*/
	function slowMbStrToA(&$str){
		$sz= mb_strlen($str,'UTF-8');
		$vect= array();
		for ($i=0; $i<$sz; $i++){
			$vect[$i]= mb_substr($str,$i,1,'UTF-8');
		}
		return $vect;
	} // slowMbStrToA
	
	
	/**
		Convert a UFT8 multi-byte string into an array.of single characters of any byte size!
			Input :
				$str
			Output :
				Return the array itself.
		
		Recursive version to avoid bottle neck effect of successive mb_substr(() to split a large strings!
		The optimu size of the smallest chunk to split is arround _c_mbStrAtom= 100  quite a large number considering its 
		squarre, but it happens because the compiled code of mb_substr() is much faster than the code caling ir!!		
		Improvment : mb_split() is probably faster, but regular expression tested to select each char did not work (may be a local implementation bug)
	*/
	function mbStrToA($str){
		$sz= mb_strlen($str,'UTF-8');
		if ($sz>_c_mbStrAtom){
			$sz1= round($sz/2);
			$sz2= $sz-$sz1;
			
			$str1= mb_substr($str,0,$sz1,'UTF-8');
			$str2= mb_substr($str,$sz1,$sz2,'UTF-8');
			
			$vect1= $this->mbStrToA($str1);
			$vect2= $this->mbStrToA($str2);
			
			$vect= array_merge($vect1,$vect2);
		}else{
			$vect= array();
			for ($i=0; $i<$sz; $i++){
				$vect[$i]= mb_substr($str,$i,1,'UTF-8');
			} // for
		}
		return $vect;
	} // mbStrToA


	/**
		To detect if we got the separator $sepV starting at the position $k in $mbV
	
		$mbV : the main multy byte vector
		$sepV : the used separator multibyte vector
		$k : the current key of $mbV certified as being an integer because integer indexes --> array_merge --> unteger keys
	*/
	function gotSep(&$mbV, &$sepV, $k){
		reset($sepV);				
		foreach ($sepV as $sepK=>$curChar)
			if ($mbV[$k+$sepK]!==$curChar)
				return false;
			
		return true;
	} // gotSep


	/**
		Explode a multi-byte string.
		Input :
			$str : given multi byte string.
			$sep : given separator
			$concat : 	true --> concat the distinct blocs into new mb strings
						false --> returns the blocs as array of chars
						
		Output :
			returns an array of mb-strings or an array of arrays
	*/
	function mbExplo($str,$sep=',',$concat=true){
		return $this->mbExplode($str,$sep,$concat);
	}
	
	function mbExplode(&$str,$sep=',',$concat=true){
		$finalV= array();

		$mbV= $this->mbStrToAry($str,$sz);
		
		$jumpNb= 0;	// when > 0 it indicates that the number of separator characters the main loop has to jump
		$sepV= $this->mbStrToAry($sep,$sepSz);
		
		$buff= $this->initVectOrStr($concat);
		$i= 0;
		$j= 0;
		reset($mbV);
		foreach ($mbV as $k=>$char){
			if ($jumpNb===0){
				if ($this->gotSep($mbV,$sepV,$k)){
					$finalV[$j]= $buff;
					$i= 0;
					$buff= $this->initVectOrStr($concat);
					$j++;
					
					if ($sepSz>1){
						$jumpNb= $sepSz-1;
					}
				}else{
					// Concat or build sub arrays
					if ($concat){
						$buff.= $char;
					}else{
						$buff[$i]= $char;
					}
					
					$i++;	// also general char count
				}
			}else{
				$jumpNb--;
			}
		} // foreach
		
		
		if ($i>0){
			$finalV[$j]= $buff;
		}
		
		return $finalV;
	} // mbExplode
	
	
	/**
		Explode Trim and substitute ($sbstOldCh <--- $sbstNewCh) if asked for
	*/
	function mbExplodeAndTrim(&$str,$sep=',',$concat=true, $sbstOldCh=false,$sbstNewCh=false){
		$mainV= $this->mbExplode($str,$sep,false);	// false --> always treat it first as a full vector of vectors
		
		$newMainV= array();
		$j= 0;
		
		reset($mainV);
		foreach ($mainV as $key=>$subV){
			// Left trim $subV -----
			$gotFirst= false;	// got first visible char
			$i= 0;	
			$newSubV= array();
			reset($subV);
			foreach ($subV as $k=>$val){
				if (!$gotFirst && !$this->invisChar($val)){
					$gotFirst= true;
				}
				
				if ($gotFirst){
					$newSubV[$i]= $val;
					$i++;
				}
			}
						
			// Right trim $newSubV -----
			$gotFirst= false;
			$nSubV= array();
			$val= end($newSubV);
			do{
				$k= key($newSubV);
				if (!$gotFirst && !$this->invisChar($val)){
					$gotFirst= true;
				}
				
				if ($gotFirst){
					if ($val===$sbstOldCh){
						$val= $sbstNewCh;	// does the substitution if a relevant char was found
					}
					$nSubV[$k]= $val;
				}
			
				$val= prev($newSubV);
			} while ($val!==false);
			
			
			// indexes are direct but the php order of the array is reverse (numeric index do not dominate they order
			$nSubV= array_reverse($nSubV);	// could avoid thatwith a recursive call ionstead of the loop to reverse scan

			if ($concat){
				$subStr= '';
				reset($nSubV);
				foreach ($nSubV as $k=>$c){
					$subStr.= $c;
				}
				$newMainV[$j]= $subStr;
			}else{
				$newMainV[$j]= $nSubV;
			}
			
			$j++;
		}
		return $newMainV;
	} // mbExplodeAndTrim
	

	/**
	*/
	function initVectOrStr($concat){
		if ($concat){
			$buff= '';
		}else{
			$buff= array();	// sub array buffer
		}
		
		return $buff;
	} // initVectOrStr
	
	
	/**
		Input:
			$c : current single or multiple byte char to test on $set.
			$set : the set of allowed characters.
	*/
	function inSet($c,&$set){
		return in_array($c,$set);
	} // inSet
	
	
	/**
		Deprecated. "Clone" Array --> Copy the given array / vector
	*/
	function cloneArray(&$v){
		return $this->cpyArray($v);
	} // cloneArray
	
	/**
		Copy the given array / vector
	*/
	function cpyArray(&$v){
		reset($v);
		$w= array();
		foreach ($v as $i=>$val){
			$w[$i]= $v[$i];
		} // for
		return $w;
	} // cpyArray
	
		
	/**
		Give back the nth element starting at 0 to size-1
		! Only useful if the array is not accessible directly with integer indexes.
	*/
	function nthElement(&$v,$nth){
		$i= 0;
		reset($v);
		foreach ($v as $key=>$val){
			if ($i===$nth){
				return $val;
			}
			$i++;
		}
		return false;
	} // nthElement
	

	/**
		Horitontaly merges the strings of two identical length array.
	*/
	function horizStrMerge($p,$q){
		reset($p);
		foreach ($p as $key=>$val){
			$r[$key]= $val.$q[$key];
		}
		return $r;
	} // horizStrMerge
	
		
	/**
		Unset the given array.  Non recurssive!
		$this->unsetArray($v);
		$tools->unsetArray($v);
	*/
	function unsetArray(&$v){
		if (is_array($v)){
			reset($v);
			foreach ($v as $key=>$val){
				unset($v[$key]);
			}
		}
		$v= array();
	} // unsetArray
	
	
	/**
		Easy print.
		$tools->easyP($v);
	*/
	function easyPrint(&$v){
		if (is_array($v)){
			reset($v);
			$i= 0;
			foreach ($v as $key=>$val){
				$i++;
				print('key='.$key._c_);
				print('val='.$val._c_);
			}
			if ($i===0){
				print('Empty array'._c_);
			}
		}else{
			print('Not an array'._c_);
		}
	} // easyPrint
	
	
	/*
		VECTOR METHODS END
	-------------------------------------------------------------- */	
	
	
	/**
		Return the largest / maximum element or the common value if they are egual.
	*/
	function largest($a,$b){
		if ($a>$b){
			return $a;
		}else{
			return $b;
		}
	} // largest
	
	
	/**
		Catches most wrong email addresses.
		$regRes= preg_match( "^[a-z0-9._%+-]+@[a-z0-9.-]+\.[a-z]{2,6}$",$eMail);  // stall ?
	*/
	function isValidEmail($eMail){
		$acc= &$this->acc;
		
		if (!$acc->emailChars($eMail)){
			return false;
		}
	
		$gotAt= false;
		$got2ndPartDot= false;
		$k=0;
		$validEmail= false;
		$szStr= strlen($eMail);
		for($i=0; $i<$szStr; $i++){
			if ($gotAt && $eMail[$i]==='@'){
				break; // because got a second @
			}
		
			if (!$gotAt && $i>0 && $eMail[$i]==='@'){
				$gotAt= true;
			}

			if ($gotAt && $eMail[$i]==='.'){
				$got2ndPartDot= true;
			}

			if ($got2ndPartDot){
				$k++;
			}
			
			if ($i<$szStr-1 && $eMail[$i]==='.' && $eMail[$i+1]==='.'){
				break; // // because consequtive .
			}

			if ($k>2)
				$validEmail= true;
		}
		return $validEmail;
	} // isValidEmail	
	

	/**
		Return true if we have an empty string or $s===false (general (large definition) empty string)
	*/
	function genEmptyStr($s){
		return $s==='' || $s===false || $s===null;
	} // genEmptyStr
	
	/**
		Return true if any of the 2 parameters are a general empty string.
	*/
	function anyGenEmptyStr($a,$b){
		return $this->genEmptyStr($a) || $this->genEmptyStr($b);
	} // anyGenEmptyStr

	
	/**
		returns true if $a <= $x <= $b 
	*/
	function betwOrEqual($a,$x,$b){
		return ($a<=$x) && ($x<=$b);
	} // betwOrEqual


	/**
		Build squared tag (the symetric of the JS edit-version), from the tag name, a normalised vectors and the body if given.
		
		Input :
			$tgName : the given tag name
			$nVect : normalised associative array ($v)
			$body : the body for bodied tags

		Duplicata in : tagsFu::rebuildTagStr()		
	*/
	function buildSqrTag($tgName,$nVect,$body=false){
		// Build parameters list
		$pmStr=''; // parameters string to load
		if ($nVect!==false){
			reset($nVect);
			foreach ($nVect as $key=>$val){
				if ($val!==false){
					$pmStr.= _c_space.$key._c_equ._c_dblQ.$val._c_dblQ;
				}
			}
		}
		
		if ($body===false || $body===''){
			return _c_lBrk.$tgName.$pmStr._c_space._c_slash._c_rBrk;
		}else{
			if ($body===' ') $body='';
			return _c_lBrk.$tgName.$pmStr._c_rBrk. $body ._c_lBrk._c_slash.$tgName._c_rBrk;
		}
	} // buildSqrTag

	/**
		Idem buildSqrTag(), just add spaces arround consistent $body.
	*/	
	function spacedSqrTag($tgName,$nVect,$body=false){
		if ($body===false || $body===''){
			return $this->buildSqrTag($tgName, $nVect,$body);
		}else{
			return $this->buildSqrTag($tgName, $nVect, _c_space.$body._c_space);
		}
	} // spacedSqrTag
	
	/**
		Idem spacedSqrTag() without the spaces, so idem buildSqrTag()!! Just easier to call and remember.
	*/		
	function sqrTag($tgName,$nVect,$body=false){
		return $this->buildSqrTag($tgName, $nVect,$body);
	} // sqrTag

	
	/**
		Prepare a string for JS storage (could do it only once but don't worry now)
		Input :
			$s : concerned string
			$sffx : facultative suffix
			
		Based on split : _c_fDblQuotSubst, _c_fBsDblQtSubst, _c_fCrLnSubst
	*/
	function jsStorageStr($s,$sffx=''){
		$s= str_replace('\'','bS'.$sffx.'SQ',$s);		
		$s= str_replace('\"','bS'.$sffx.'dG',$s);		
		$s= str_replace('"','dB'.$sffx.'lG', $s);		// encode the double quotes
		
		// New lines etc.
		$s= str_replace(_c_carRet, 'Cr'.$sffx.'Ln', $s);	// encode the _c_R
		$s= str_replace(_c_nL, 'Cr'.$sffx.'Ln', $s);	// encode the _c_R
		return $s;
	} // jsStorageStr

	
	/**
		Loads image dimensions
	*/
	function imageDim(&$width,&$height,$imgDFN){
		$fls= &$this->fls;
		if ($fls->isImage($imgDFN)){
			$iSzV= getimagesize($imgDFN);	// not an image --> false
			$width= $iSzV[0];	// not an image --> null
			$height= $iSzV[1];
		}else{
			$width= 0;
			$height= 0;
		}
	} // imageDim
	
	
	function imageWidth($imgDFN){
		$v= getimagesize($imgDFN);
		return $v[0];
	} // imageWidth
	
	function imageHeight($imgDFN){
		$v= getimagesize($imgDFN);
		return $v[1];
	} // imageHeight
	
	
	/**
	*/
	function corePhpGotLrgeInt(){
		$kSmallInMax= 1000*2147483647;	// k === 1000
		if (PHP_INT_MAX > $kSmallInMax){
			 $ret= _c_ok;
		}else{
			$ret= _c_ko;
		}		
		return $ret;
	} // corePhpGotLrgeInt
	
	
	/**
	*/
	function toEditSpecificTblElem($curTbl,$id,$colNm,$urlSup='',$target='_top'){
		$tagsFu= &$this->tagsFu;
		$glob= &$this->glob;
		$caps= &$this->caps;
		$rMg= &$this->rMg;
		
		if ($rMg->pwCheck3()){
			$wmR= &$tagsFu->wmRights;
			if ( $this->curPageName()===_c_bigZoomPg && !$wmR->wmTechAcess() ){
				// can only access the big zoom pencil with tech access (to avoid lambda user mistakes by overwrting)
				return '';
			}
			
			$imgDFn= _c_EditorImgPath.'pencil.png';
			$alt= $glob->el('Edit only this field','kazw');
			
			$arCmd[_c_tedCurDtbTblNm]= _c_ko;
			$url= $this->urlGivenPage('_dtb_edit_line',$arCmd);
			$url.= $this->addUrlVar(_c_tedCurDtbTblNm,$curTbl);
			$url.= $this->addUrlVar(_c_tedCurDtbRowId,$id);
			$url.= $this->addUrlVar(_c_snglEdtFieldName, $colNm);
			$url.= $this->reprodUrlVar('htmlentities');
			
			$url.= $urlSup;
			return $caps->linkImg($imgDFn,$alt,$url,'sImg',$target);
		}else{
			return '';
		}
	} // toEditSpecificTblElem


	/* -----------------------------------------------------------------------------
		Immigrated from the file manager to be also used inside inserTags.php
	*/

	/**
	*/
	function urlPathVar($id){
		return _c_mFilePathPrfx.$id;
	} // urlPathVar
	
	/**
	*/
	function fullPathStr($id){
		$tagsFu= &$this->tagsFu;
		$glob= &$this->glob;
		$dtb= &$this->dtb;
		$wmR= &$tagsFu->wmRights;

		
		$path= $this->getVal($this->urlPathVar($id));
		if ($path===false || $path===''){
			return _c_nsUsrDataPath;	// the user home directory
		}
		
		if ($wmR->globallyForbidPath($path)){
			return _c_nsUsrDataPath;	// the user home directory
		}
		
		return $this->removeLastSlash($path);
	} // fullPathStr
	
	
	/**
	*/
	function removeLastSlash($path){
		if ($path===''){
			return '';
		}
	
		$sz= strlen($path);
		$lastChar= $path[$sz-1];
		if ($lastChar==='/'){
			$path= substr($path,0,-1);
		}
		return $path;
	} // removeLastSlash
	
	/**
		Remove the last byte!
	*/
	function rmLastByte(&$s,$n=-1){
		$s= substr($s,0,$n);
	} // rmLastByte
	
	function rmLstByte($s,$n=-1){
		return substr($s,0,$n);
	} // rmLstByte
	
	
	
	/**
		Return the last byte!
	*/
	function lastByte(&$s){
		$sz= strlen($s);
		return $s[$sz-1];
	} // lastByte
	
	/**
		Return the bte before the last byte!
	*/
	function bLstByte(&$s){
		$sz= strlen($s);
		return $s[$sz-2];
	} // bLstByte
	
	
	/**
		Idem above but no invertion.
	*/	
	function sameUrl(){
		$uCPa= $this->urlCurPg();
		$upv1= $this->urlPathVar(1);	
		
		$pathOne= $this->getVal($upv1);
		if ($pathOne===false){
			// file manager not detected!
			return $uCPa;
		}else{
			$upv2= $this->urlPathVar(2);
			$cstPath1= $this->fullPathStr(1);
			$cstPath2= $this->fullPathStr(2);
			return $uCPa.$this->addUrlVar($upv1,$cstPath1).$this->addUrlVar($upv2,$cstPath2).$this->urlEnd();
		}
	} // sameUrl
	
	
	/**
		Url End to avoid a "browser" trim of the full url leading to the lost of eventual usefull ending spaces
		--> only programmed to allow file surfing of very bad named files and directories
	*/
	function urlEnd(){
		return $this->addUrlVar('url','End'); 
	} // urlEnd
	
	
	/**
	*/
	function remtAddr(){
		return $_SERVER['REMOTE_ADDR'];
	} // remtAddr
		
	
	/**
		Compute the name of real utf8 acceptation in Url for page names in genVar!
	*/
	function realUtfUrlPnModeNm($lg){
		return 'realUtf8Url_'.$lg;
	} // realUtfUrlPnModeNm
	
	
	/**
		Remove the first byte!
	*/
	function rmFirstByte(&$s){
		$s= substr($s,1);
	} // rmFirstByte
	
	
	/**
		A locally written version php::array_combine($keys,$values) because the original works strangely in some cases
		does not work!  Not really equivalent :
			reset($keys); reset($values); return array_combine($keys,$values);	// ??
	*/
	function arrayCombine(&$keys,&$values){
		$ac= array();	// array combined (formal!)
		$this->arrayComb($ac,$keys,$values);
		return $ac;
	} // arrayCombine

	function arrayComb(&$ac, &$keys,&$values){
		$ac= array();	// array combined
		$acVal= reset($values);
		
		reset($keys);
		foreach ($keys as $key=>$val){
			$ac[$key]= $acVal;
			$acVal= next($values);
		}
	} // arrayCombine
	
	
	/**
		Returm the smallest of 2 pos numbers
	*/
	function smallest($a,$b){
		if ($a>$b){
			return $a;
		}else{
			return $b;
		}
	} // smallest
	

	/**
		Build a filling string of $n $c characters
	*/
	function fillingStr($c,$n){
		$b= '';
		for ($i=0; $i<$n; $i++){
			$b.= $c;
		} // for
		return $b;
	} // fillingStr
		
} // tools
?>