/**
	File name : jsEncryption.js
	Author of this file : Oscar Fawer until 2019-08-15
	Contact : www.ataox.org 
	Web sites : ataox.org / ataox.com / ataox.info
	Copyrights open source GPL, GNU General Public License V3 : see ./aMainDr/code/LICENSE.txt


*/

/**
	Mainly inline javascript encryption independant from hardware.
*/
function jsEncryption(){

	/* ---------------------------------------------------------------
		Global variable for the "class"
	*/
	
	var crLn= kk.crLn;	
	var brTag= '<br class="flux" />';	
		
	var globFreqTbl= new Array();
	var oGlobFreq= new Object();
		
	var binMapVect = new Array();
	var fCharSet= stringMath.baseFrStr;	// var fCharSet= stringMath.getBaseFr();
	
	
	var rndVectLenghtCst= 20;	// should be at least 20
	
	var minPasswdSz= 8;		// minimum number for password characters
	var optimPasswdSz= 19;	// optimum number for password characters
	
	var rStorageBase= stringMath.getStorageBase();
	
	var oBinStor= stringMath.buildBinConVect(rStorageBase);	// object binary storage translation.
	
	
	
	var binCharSet= '01';	// so can pass it directly to buildRandomStr()
	
	var fChrStSz= fCharSet.length;				// Char Set Size
	var tableSideSz= 8;	// for estethical and fast colorstests
	var binWordSz= stringMath.binMapStringSz(fCharSet);	// Minimal width of the binary representation

	var globCurrentInlineIdx= '0'; 	// so false!
	var globCurrentInlineNm= '';
	var globCurLinkCateg= '';
	
	var globLnTagHeader= '';
	var globLnTagEnd= '';
	var globLongBlinker= true;


	/* -----------------------------------------------------------------
		Global storage variables for comment and encrypted strings. 
			Remark : globCryptVar[...] is always crypted also because 1st php onload (or after password deletion) 
			implies that the password is not available. Problem : wasting time when clicking on it because it needs to be
			decyphered to be displayed (can't take it from the <span innerHTML because innerHTML (in some browsers) 
			change some datas eg : <br/> --> <br> old style compression!). So to save useless crypting actions, could have 
			a third Array to store only non crypted strings (if can't decypher would be empty).
	*/
	var largestCryptedIndex= 0;
	
	var globLineComment= new Array();	// global line comment
	var globCryptVar= new Array();		// global crypted variable 
	var gbRdblCrpVar= new Array();		// global readable crypted variable 



	/* ----------------------------------------------------------------------------------
		ACCELERATIONS :
			Other accel : use a fast navigator! Huge differences!
	*/

	/**
		Avoid recalculation of the inflated binary passwords.
	*/
	var globBinShufflPw= null;
	var globBinXorPw= null;
	var globCryptPw= '';		// 

	/**
		Avoid recalculation of pseudo random series is more complicated because series are adapted to the size
		of the strings to crypt.
			* Can't apply directly because not same length! --->
				* Could buil a dynamic buffer directly inside the xorShift!
				* Couls accelarate pure binary vectors by taking directly the bit serie of the xor-shift.			
			
		Name ideas :
			var globRndShflObj= null;	// global pseudo random object used for shuffling
			var globRndXorVect= null;	// global pseudo random binary array used for xoring
	*/

	
	// --- old style
	// The 2 conversion vectors.
	var chrToBin= new Array();
	var binToChr= new Array();	
	buildConVect();	


	/* ---------------------------------------------------------------
		Methods interface.
	*/	
	this.clearAll= clearAll;
	this.fullCryptUncryptTest= fullCryptUncryptTest;
	this.countChars= countChars;
	this.countCharsInBaseBoxOne= countCharsInBaseBoxOne;
	this.cntChrInGetBaseFr= cntChrInGetBaseFr;
	this.cmpTwoBaseFr= cmpTwoBaseFr;
	this.strgAdd= strgAdd;
	this.strgOperBsN= strgOperBsN;	
	this.strgTimes= strgTimes;
	this.strgSub= strgSub;
	this.strgDivis= strgDivis;	
	this.baseXToBaseY= baseXToBaseY;
	this.pwWatch= pwWatch;
	this.pleaseWYP= pleaseWYP;
	this.plsNoBlinkWYP= plsNoBlinkWYP;
	this.getJsPassword= getJsPassword;
	this.pwDelete= pwDelete;

	/**
		Random numbers methods :
	*/
	this.crptArea= crptArea;
	this.processSmallArea= processSmallArea;
	
	
	this.boxCryptArea= boxCryptArea;
	this.uncryptArea= uncryptArea;
	
	
	
	this.manUncrptArea= manUncrptArea;
	this.pwDelete= pwDelete;
	this.storesPw= storesPw;
	this.pwIndications= pwIndications;
		
	
	this.setBaseDsplOne= setBaseDsplOne;
	this.setBaseFRDsplX= setBaseFRDsplX;
	this.setBaseDsplTwo= setBaseDsplTwo;
	
	this.smllLinkAct= smllLinkAct;	
	this.unCryptSmallLn= unCryptSmallLn;
	
	
	this.hideAll= hideAll;
	this.uncryptAll= uncryptAll;
	this.newCouple= newCouple;
	this.saveAll= saveAll;
	this.winOpen= winOpen;
	
	
	this.gloLnCom= gloLnCom;
	this.initLrgstCrptIndx= initLrgstCrptIndx;
	
	this.actualiseNewPw= actualiseNewPw;
	
	
		
	this.write= write;
	this.writeLn= writeLn;
	this.clearWrite= clearWrite;


	/* -----------------------------------------------------------------------
		Methods definitions
	*/	
	

	/**
		Build the 2 conversion vectors.
	*/
	function buildConVect(){
		var N= fChrStSz;
		var bVect= stringMath.binMapStringVect(N,binWordSz);
		
		chrToBin= stringMath.buildCharToBin(fCharSet,bVect);
		binToChr= stringMath.buildBinToChar(bVect,fCharSet);
	} // buildConVect


	/**
		Transforms a characters string into a binary string.
	*/
	function cStrToBinStr(str){
		var binStr= '';
		var sz= str.length;		
		for(var i=0;i<sz;i++){
			var c= str[i];
			binStr+= chrToBin[c];
		}
		return binStr;
	} // cStrToBinStr


	/**
		Transforms a binary string into a characters string.
	*/
	function bStrToCharStr(str){
		var charStr= '';
		var bWord= '';	// current binary word to build.
		var szBW= 0;		// current size of word.		
		
		var i=0;
		var sz= str.length;
		while(i<sz){
			bWord+= str[i];
			szBW++;
		
			if (szBW===binWordSz){
				// do not care if we miss some filling trailing bits.
			
				var c= binToChr[bWord];
				if (kk.isUndef(c)){
					// Whatever, normally it concerns just the filling trailing part that are chosen directly
					// in binary even if there is no match in the real character set. 
					c= '?';	
				}
							
				charStr+= c;
				bWord= '';
				szBW= 0;
			}				
			i++;
		} // while	
		return charStr;
	} // bStrToCharStr
	
	
	/**
		For an already readable string, remove capsulating information and returns the original string.
			Capsulated string = sizeHeaderNumberChar + headerContentSizeXNumber + content + fillings.
			
		Input :
			strX : readable string in the natural language x. That just got to remove a header and a tail.
		Output :
			Returns the original readable string.
	*/
	function capsStrToOrigStr(strX,charSet){	
		var numPos= vectNumPos(charSet);
		var charNbLength= numPos[strX[0]];
		
		var nbInX= '';
		for (var i=0;i<charNbLength;i++){
			nbInX+= strX[i+1];
		}
		
		if (nbInX.length>4){
			// 4 --> maximum definition of crypting size of (getBaseFr().length)^4 === 128^4 --> much too large, to be ok for smaller bases.
			writeLn('jsEncryption::capsStrToOrigStr Warning : (probably because the password is not given) the size indication of the string to uncrypt is corrupted or greater than 4 characters.');			
			return '';
		}

		var origXSz= baseXAluTen(nbInX,fCharSet);
		if (origXSz>100000){
			writeLn('jsEncryption::capsStrToOrigStr Error : corrupted or too large string, it\'s not possible to crypt/uncrypt a string larger than 100000 characters.');
			return '';
		}		
		
		origXSz=  origXSz / binWordSz;
		var origStr= '';
		var start= charNbLength+1;	// +1 To include the numeric descriptor
		
		var lastOrigIdx= start+origXSz;
		for (var j=start; j<lastOrigIdx; j++){
			origStr+= strX[j];
		}	
		return origStr;
	} // capsStrToOrigStr
		

	/**
		full Crypt Uncrypt Test
	*/
	function fullCryptUncryptTest(){		
		var infoIn= getInputOne();
		var pw= getJsPassword();
		if (pw!==false){		
			var storCode= readableStrToStoredStr(infoIn,fCharSet,pw);		// CRYPT IT
			// writeLn('* storCode='+storCode);	
			var rdblOutStr= storedStrToReadableStr(storCode,fCharSet,pw);	// UNCRYPT IT
			// writeLn('* rdblOutStr='+rdblOutStr);	
		}else{
			pleaseWYP();
		}
	} // fullCryptUncryptTest

	/**
		Get the source string and return the store code.
	*/
	function readableStrToStoredStr(sourceStr,curCharSet,pw){
		var binV= binCrypt(sourceStr,curCharSet,pw);
		return binCodeToStorCode(binV);
	}; // readableStrToStoredStr


	/**
	*/
	function storedStrToReadableStr(storCode,curCharSet,pw){
		var binVTwo= storeCodeToBinCode(storCode);
		return binUncrypt(binVTwo,curCharSet,pw);
	}; // storedStrToReadableStr


	/**
		Computes the random sequence : it holds pseudo random pairs of values indicating respectively lines and columns (x,y)		
		Input :
			pw : binary 128 bits vector password.
		Output : returns an object holding the pairs and the vectors size (o.x[i], o.y[i], o.sz).
	*/
	function buildPairsRndVect(pw,side){
		var sz= rndVectLenghtCst * side;		// Size of the random pair vector !
		strXorShift.iniwBigBin(pw,'shf',sz);	// Initialise the Xor Shift with the 128 bits password derived vector
		
		// remark : can't use a global buffer here because the sizes change!;
		return strXorShift.buildPairsVect(sz,side);	
	}; // buildPairsRndVect
		
	/**
		Input :
			infoIn : a multi char string.
			charSet : the used char set.
			pw : binary 128 bits vector password.
		Output :
			returns the crypted binary string.
	*/
	function binCrypt(infoIn,charSet,pw){
		var switKey= getCryptingMode();
		switch (switKey){
			case 'shffl':
				var binShPw= inflPwToBits(pw,'ko');
				return shuffleCrypt(infoIn,charSet,binShPw);
			break;
						
			case 'xor':
				var binStr= cStrToBinStr(infoIn);	// Change the main info in binary
				var binV=stringMath.strToVect(binStr);
				var binXrPw= inflPwToBits(pw,'ok');
				return xorCrypt(binV,binXrPw);
			break;			
			
			case 'sxor':
				var binShPw= inflPwToBits(pw,'ko');
				var shfldBinV= shuffleCrypt(infoIn,charSet,binShPw);
				
				var binXrPw= inflPwToBits(pw,'ok');
				return xorCrypt(shfldBinV,binXrPw);
			break;			
			
			default:
				writeLn('Error in jsEncryption::binCrypt');
		} // switch
	} // binCrypt		

	/**
	*/
	function getCryptingMode(){
		if (localStorage.externalXorMode==='ok'){
			return 'xor';
		}
		
		return document.getElementById('inlCryptAlgoKind').value;
	}; // getCryptingMode

	/**
	*/
	function xorCrypt(shfldBinV,pw){
		var sz= shfldBinV.length;
		strXorShift.iniwBigBin(pw,'xor',sz);
		var rndBinV= strXorShift.binaryVect(sz);
		return strXorShift.xor(shfldBinV,rndBinV);
	}; // xorCrypt

	/**
	*/
	function shuffleCrypt(infoIn,charSet,pw){
		var binStr= cStrToBinStr(infoIn);	// Change the main info in binary
		var oMatrix= strToObjMatrix(binStr,charSet);
		var oRndV= buildPairsRndVect(pw,oMatrix.side);	// Build the random exchange object vector
		shuffleMx(oMatrix,oRndV); 						// Shuffle the matrix				
		return matrixToVect(oMatrix);			// Returns the shuffled binary vector			
	}; // shuffleCrypt
	
	/**
		Input:
			Mx : the matrix (double array) to shuffle.
			oRnd : object random vector. (oRnd.x, oRnd.y, oRnd.sz)
		Output:
			Mx directly changed
			
		Rem : see jsEncryption::buildPairsRndVect()
	*/
	function shuffleMx(objM,oRnd){
		var i=0;
		while (i<oRnd.sz-3){
			exchangesLines(objM, oRnd.x[i],oRnd.y[i]);
			i++;
			
			exchangesCol(objM, oRnd.x[i],oRnd.y[i]);
			i++;	
			
			exchangesDots(objM, oRnd.x[i],oRnd.y[i], oRnd.x[i+1],oRnd.y[i+1]);		
			i++;
			i++;
		}
	}; // shuffleMx
	
	/**
	*/
	function exchangesDots(obj,a,b,c,d){
		var cell= obj.Mx[a][b];
		obj.Mx[a][b]= obj.Mx[c][d];
		obj.Mx[c][d]= cell;
	}; // exchangesDots


	/**
		Input :
			binV : a binary vector.
			charSet : the used char set.
		Output :
			returns the readable string.
	*/
	function binUncrypt(binV,charSet,pw){
		var switKey= getCryptingMode();
		switch (switKey){
			case 'shffl':
				var binShPw= inflPwToBits(pw,'ko');
				return unshufflCrypt(binV,charSet,binShPw);
			break;
						
			case 'xor':
				var binXrPw= inflPwToBits(pw,'ok');
				var binV= xorCrypt(binV,binXrPw);
				var strV= stringMath.vectToStr(binV);
				return bStrToCharStr(strV);	
			break;
			
			case 'sxor':
				var binXrPw= inflPwToBits(pw,'ok');			
				var xoredV= xorCrypt(binV,binXrPw);	
				
				var binShPw= inflPwToBits(pw,'ko');				
				return unshufflCrypt(xoredV,charSet,binShPw);
			break;			
			
			default:
				writeLn('Error in jsEncryption::binUncrypt');
		} // switch
	} // binUncrypt


	/**
		Just calls binUncrypt(binV,charSet,pw) in a cleaner/shorter way.		
			var globBinShufflPw= null;
			var globBinXorPw= null;
	*/
	function inflPwToBits(pw,boolOk){
		if (boolOk==='ok'){	// xor
			if (globBinXorPw===null){
				var buff= stringMath.inflateStrToBits(pw,19,boolOk);
			}else{
				var buff= globBinXorPw;
			}
			
			// alert('buff='.buff);
			return buff;
			
		}else{	// shuffle
			if (globBinShufflPw===null){
				return stringMath.inflateStrToBits(pw,19,boolOk);
			}else{
				return globBinShufflPw;
			}
		}
	}; // inflPwToBits


	/**
	*/
	function unshufflCrypt(xoredV,charSet,pw){
		var sideMx= mxSideSqrblStr(xoredV);			// matrix side number
		var oMatrix= sqrblStrToMx(xoredV,sideMx);	// Build the binary matrix
				
		var oRndV= buildPairsRndVect(pw,sideMx);	// ready to un-shuffle
		unShufflMx(oMatrix,oRndV);
	
		var binStrTwo= matrixToStr(oMatrix);
		var charStr= bStrToCharStr(binStrTwo);
		return capsStrToOrigStr(charStr,charSet);
	}; // unshufflCrypt
	
	
	/**
		Input:
			Mx : the matrix (double array) to shuffle.
			oRnd : object random vector. (oRnd.x, oRnd.y, oRnd.sz)
		Output:
			Mx directly changed
	*/
	function unShufflMx(objM,oRnd){
		var i=oRnd.sz-1;
		while (i>2){
			exchangesDots(objM, oRnd.x[i-1],oRnd.y[i-1], oRnd.x[i],oRnd.y[i]);
			i--;
			i--;

			exchangesCol(objM, oRnd.x[i],oRnd.y[i]);
			i--;
			
			exchangesLines(objM, oRnd.x[i],oRnd.y[i]);
			i--;
		}
	}; // unShufflMx
	
	/**
	*/
	function exchangesCol(objM,c1,c2){
		for (var l=0; l<objM.side; l++){
			exchangesDots(objM,  l,c1,  l,c2);
		}
	}; // exchangesCol

	/**
	*/
	function exchangesLines(objM,l1,l2){
		for (var c=0; c<objM.side; c++){
			exchangesDots(objM,  l1,c,  l2,c);
		}
	}; // exchangesLines

	
	/**
		Add a length header and square the string to return a square matrix.
		To avoid detection the length header is expressed in base fChrStSz and the 
		first character indicates the length of the numerical field, then the whole
		numerical field is binarised and added as a string header.
		
		Input :
			binStr : original info as a binary string.
			charSet : the used char set.		
		Output :
			return the matrix object
		
	*/
	function strToObjMatrix(binStr,charSet){
		var origBinSz= binStr.length;
		var sizeInBaseX= baseTenAluX(origBinSz,charSet);
		var charNbLength=  sizeInBaseX.length;		// size in base X !!
		var sizeIndic= charSet[charNbLength];
		
		var header= sizeIndic + sizeInBaseX;	// header		
		
		var binHeader= cStrToBinStr(header);
		var wBSqrSz= binHeader.length + origBinSz;			// would Be round Square size
		
		var side= Math.ceil(Math.sqrt(wBSqrSz)) +1;		// decimal side of the full binary matrix
		
		var newSquareSz= side*side;
		var deltaSz= newSquareSz - wBSqrSz;	// the number of random characters or bits we need to add
		var deltaBinStr= buildRandomStr(deltaSz,binCharSet);
		var totBinStr= binHeader+binStr+deltaBinStr;
		
		return sqrblStrToMx(totBinStr,side);
	} // strToObjMatrix
	
	/**
		Squarable string to matrix : build a square matrix from a binary string that is already squarable.
	*/
	function sqrblStrToMx(binStr,side){
		var obj= new Object();
		
		obj.Mx= new Array();	// squared array
		var k= 0;
		for(var i=0; i<side; i++){
			obj.Mx[i]= new Array();
			for(var j=0; j<side; j++){
				obj.Mx[i][j]= binStr[k];
				k++;
			}
		}
		
		obj.side= side;
		return obj;
	}; // sqrblStrToMx


	/**
		Matrix side of a squarable string
	*/
	function mxSideSqrblStr(str){
		return Math.round(Math.sqrt(str.length));
	}; // sideSqrblMx

	
	/**
	*/
	function matrixToStr(objM){
		var sideMx= objM.side;
		var totBinStr= '';
		for(var i=0; i<sideMx; i++){
			for(var j=0; j<sideMx; j++){
				totBinStr+= objM.Mx[i][j];
			}
		}
				
		return totBinStr;	
	} // matrixToStr
	
	
	/**
	*/
	function matrixToVect(objM){
		var sideMx= objM.side;
		var totBinV= new Array();
		var k=0;
		for(var i=0; i<sideMx; i++){
			for(var j=0; j<sideMx; j++){
				totBinV[k]= objM.Mx[i][j];
				k++;
			}
		}
				
		return totBinV;	
	} // matrixToVect
	

	/**
		Build a real random string.
		Input :
			strSz : size of the string we have to build.
			charSet : used characrer set.
	*/
	function buildRandomStr(strSz,charSet){
		var bStr= '';	// buffer string
		for(var i=0;i<strSz;i++){
			var c= randomChar(charSet);
			bStr+= c;
		}		
		return bStr;
	} // buildRandomStr
	
	/**
		Returns a real random character.
		Input :
			strSz : size of the string we have to build.
			charSet : used characrer set.
	*/
	function randomChar(charSet){
		var setSz= charSet.length;
		var rndIndx= calcRndLimits(0,setSz-1);		
		return charSet[rndIndx];
	} // randomChar


	/**
		Return an associative array giving the numerical position of each character of charSet
	*/
	function vectNumPos(charSet){
		return stringMath.vectNumPos(charSet);
	} // vectNumPos


	/**
		Take a string in base 10 and return the equivalent in base X.
		Input :
			nbTen : number to transform given in base ten
			x : decimal numerical span of the new base (binary=2, hexa=16 etc..)
			charSet : char set of the new base.
		-------> To ---> Alu (arith logic unit), meanning that calcultion is done with Alu
		math routines and not stringMath routines!
	*/
	function baseTenAluX(nbTen, charSet){
		var x= charSet.length;
		var buffStr= '';
		do{
			var rem= nbTen % x; 		// remainder
			var nbTen= Math.floor(nbTen/x);	// whole quotient
			buffStr= charSet[rem] + buffStr;
		}while (nbTen!==0);
		return buffStr;
	} // baseTenAluX
	

	/**
	*/
	function zeroPad(sz){
		var b= '';
		for (var i=0; i<sz; i++){
			b+= '0';
		}
		return b;
	}; // zeroPad


	/**
		Take a string in base X and returns the equivalent in base 10.
		Input :
			nbInX : number (a string) to transform given in base X
			charSet : char set of the new base.
	*/
	function baseXAluTen(nbInX, charSet){
		var x= charSet.length;	// decimal numerical value of the new base.
		var buffStr= '';
		var numPos= vectNumPos(charSet);	// leave it local for the sake of generalisation
		
		var nbSz= nbInX.length;
		var p= 0;	// power
		var i= nbSz-p-1;
		
		var powerX= 1;
		var totNb= 0;		
		while (p<nbSz){
			var c= nbInX[i];	// current character from right to left;
			var np= numPos[c];
			
			if (kk.isNotUndef(np)){
				totNb+= powerX*np;
			}else{
				writeLn('Warning:"'+c+'" is not in the set of characters for the base X');
			}
			
			powerX= powerX*x;			
			i--;
			p++;
		}		
		return totNb;
	} // baseXAluTen


	/**
	*/
	function ptInput(){
		return document.getElementById('inputTwo');
	}; // ptInput
	
	function clrInput(){
		var pt= ptInput();
		pt.value= '';
		
		document.getElementById('inputOne').value='';
	}; // clrInput


	/**
	*/
	function clearAll(){
		clear();
		clrInput();
		
		document.getElementById('substDiv').innerHTML= '';
		document.getElementById('substInfDiv').innerHTML= '';
	} // clearAll


	/**
		Already defined in calculator.js, ... need a basic math class for common & usefull methods)
		Return a random number between a and b
		Input :
			a : smaller limit included.
			b : larger limit included.
	*/
	function calcRndLimits(a,b){
		var f= b+1;	// +1 because of the floor
		var delta= f-a;
		return  a + Math.floor(delta*Math.random());
	} // calcRndLimits
	

	/**
	*/
	function countChars(){
		var str= getInputOne();	
		writeLn('Got '+str.length+' characters');
	}; // countChars
	

	/**
	*/
	function countCharsInBaseBoxOne(){
		// var str= kk.trim(getBaseOne());	
		var str= getBaseOne();	
		writeLn('BaseBoxOne got '+str.length+' characters');
	}; // countCharsInBaseBoxOne
	
	/**
	*/
	function cntChrInGetBaseFr(){
		var str= stringMath.getBaseFr();	
		writeLn('GetBaseFr returns '+str.length+' characters');
	}; // cntChrInGetBaseFr
		
	/**
		Compare getBaseOne() and stringMath.getBaseFr() char by char
	*/
	function cmpTwoBaseFr(){
		var areaStr= getBaseOne();	
		var jsStr= stringMath.getBaseFr();	

		var o= stringMath.canonicOperands(areaStr,jsStr);
		
		writeLn('areaStr.length='+areaStr.length + ' jsStr.length='+jsStr.length);
		
		var sz= o.sSz;
		for (var i=0; i<sz; i++){
			if (areaStr[i]!==jsStr[i]){
				writeLn('Diff : i='+i+' areaStr[i]='+areaStr[i]+' jsStr[i]='+jsStr[i])
			}
		}
	}; // cmpTwoBaseFr


	/**
		Test of the JS inbuilt Math.random() --> Build the random frequency table
		Input :
			freqTbl : a 0 initialised frequency table.
			szHalfRndVect : so 2*szHalfRndVect (k & l) --> full size
			sideTbl : the random limit, size of the matrix
		Output :
			returns freqTbl : the frequency table.
	*/
	function inbuiltRndFreqTbl(freqTbl,szHalfRndVect,sideTbl){
		for (var i=0; i<szHalfRndVect; i++){
			var k=jsRnd(sideTbl);
			var l=jsRnd(sideTbl);			
			freqTbl[k][l]+= 1;	
		}
		return freqTbl;
	}; // inbuiltRndFreqTbl


	/**
	*/
	function rndI(){
		return Math.floor(1000000000*Math.random());
	}; // rndInt

	/**
	*/
	function jsRnd(NbElem){
		return Math.floor( Math.random() * (NbElem-1) );
	}; // jsRnd

	
	/**
		Spread random vector pairs into a frenquency table.
	*/
	function rndVectToFrenquencyTbl(rndVect,freqTbl){
		var loadPair= false;
		for (var i in rndVect) {
		
			if (loadPair){
				var right= rndVect[i];
				freqTbl[left][right]+= 1;
			}else{
				var left= rndVect[i];
			}
					
			loadPair= !loadPair;
		}
		return freqTbl;
	}; // rndVectToFrenquencyTbl
	

	/**
		Returns the maximum frenquency
	*/
	function maxFrequency(freqTbl){
		var maxF= 0;
		var minF= 999999999;
		for (var i in freqTbl){
			var curLine=freqTbl[i];			
			for (var j in curLine) {
				var curF= curLine[j];
				if (curF>maxF){
					maxF= curF;
				}
				if (curF<minF){
					minF= curF;
				}				
			}
		}
		
		oGlobFreq.minF= minF;
		oGlobFreq.maxF= maxF;		
		return oGlobFreq;
	}; // maxFrequency
	

	function strgAdd(){
		var a= getInputOne();
		var b= getInputTwo();	
		clear();
		var tot= stringMath.sAdd(a,b);
		writeLn(tot);
	}; // strgAdd
	
	
	function strgOperBsN(op){
		var base= getBaseOne();	
		var a= getInputOne();
		var b= getInputTwo();	
		clear();
							
		var result= stringMath.baseConvAndSolve(a,b,base,op);
		
		if (op==='/'){
			writeLn('Q='+result.quotient);
			writeLn('R='+result.residue);
		}else{
			writeLn(result);
		}				
	}; // strgOperBsN
	
	
	/**
	*/
	function baseXToBaseY(){
		var nbX= getInputOne();
		var baseX= getBaseOne();	
		var baseY= getBaseTwo();	
		clear();

		var nbY= stringMath.baseXToY(nbX,baseX,baseY);
		writeLn(stringMath.vectToStr(nbY));
	}; // baseXToBaseY


	/**
		string.substring(from, to)	*/
	
	function getBaseOne(){
		// return document.getElementById('baseOneDspl').value;
		return getBaseN('baseOneDspl');
	}; // getBaseOne

	function getBaseTwo(){
		// return document.getElementById('baseTwoDspl').value;
		return getBaseN('baseTwoDspl');
	}; // getBaseTwo

	/**
		Get the base N with input area test to avoid eventual html area
		transformations / differences on \? chars (can't risk to have different bases),
		so bases using special char need a special prefix (in trhat case (fr) ) to take only one source
		
		cmpTwoBaseFr
	*/
	function getBaseN(id){
		var str= document.getElementById(id).value;	
		var sz= str.length;
		if (sz>6){
			// 6 as a min would imply a french char set in base 2
			var prfx= str.substring(0,4);
			writeLn('prfx='+prfx);
			
			if (prfx==='(fr)'){
				writeLn('Got the (fr) prefix!');
				return stringMath.getBaseFr()
			}
		}	
		return str;
	}; // getBaseN
	
	
	function getInputOne(){
		return kk.trim(document.getElementById('inputOne').value);
		// return document.getElementById('inputOne').value;
	}; // getInputOne

	function getInputTwo(){
		return kk.trim(document.getElementById('inputTwo').value);
		// return document.getElementById('inputTwo').value;
	}; // getInputTwo
	

	/**
	*/
	function setBaseDsplOne(s){
		document.getElementById('baseOneDspl').value= s;
	}; // setBaseDsplOne

	/**
	*/
	function setBaseDsplTwo(s){
		document.getElementById('baseTwoDspl').value= s;
	}; // setBaseDsplTwo


	function setBaseFRDsplX(x){
		document.getElementById(x).value= '(fr)'+stringMath.getBaseFr();
	}; // setBaseFRDspl



	function strgTimes(){
		var a= getInputOne();
		var b= getInputTwo();
		clear();
		var tms= stringMath.sTimes(a,b);
		writeLn(tms);
	}; // strgTimes


	function strgSub(){
		var a= getInputOne();
		var b= getInputTwo();
		clear();
		var subRes= stringMath.sSub(a,b);
		writeLn(subRes);
	}; // strgSub


	function strgDivis(){
		var a= getInputOne();
		var b= getInputTwo();
		clear();
		var x= stringMath.sDivis(a,b);
		writeLn('quotient='+x.quotient);
		writeLn('residue='+x.residue);
	}; // strgDivis


	/* --------------------------------------------------------------------------
		Tag Php calls :
	*/

	/**
		Called by php jsEncryption::tg_jscrypt
		Crypt the concerned area and load its content inside the main edition areaBox
		
		Input :
			srcAreaNm : source (none crypted string) Area Name.
			mainAreaNm : main area as usual.
			tagHeader, tagEnd : to pinpoint the tag limits.
	*/
	function crptArea(srcAreaNm, mainAreaNm,tagHeader,tagEnd){	
		var pw= getJsPassword();
		if (pw!==false){
			tagHeader= tagHeader.replace('dq','"');	
			tagHeader= tagHeader.replace('dq','"');	// 2nd times because replace does it only once !!
			
			var srcVal= document.getElementById(srcAreaNm).value;
			var storedCode= readableStrToStoredStr(srcVal,fCharSet,pw);
			
			var ptMArea= document.getElementById(mainAreaNm);
			var valMArea= ptMArea.value;
			
			var iTagHd= valMArea.indexOf(tagHeader);	// index of the tag Header [jsCrypt i="1"] given with or withiut the i
					
			var headStr= valMArea.substr(0, iTagHd+tagHeader.length);		
			var iTagNd= valMArea.indexOf(tagEnd);		// index of the end-tag  [/jsCrypt]
			var tailStr= valMArea.substr(iTagNd);
					
			ptMArea.value= headStr + storedCode + tailStr;
		}else{
			pleaseWYP();
		}
	} // crptArea
	
	/**
		Enter button --> Deal with a crypted sentence. (Computes the small editing panel and update all 
		the necessary variables.).
				
		Input :
			srcVal : the new string to incorporate in the structure (crypt it, display etc.)
			mainAreaNm : the name of the main [] area! Define in php as _c_taMainStNm
			tgHeader : the xml[] square head tag with its full index attribute.
			tgEnd : the xml[] square end tag with its concatanated index attribute (special ending case to have unique end)
			curNm : the inline html tag name with its concatanated index.
			
			newPw facultative new password.
		Output :
			Adjust the left and right areas.
	*/
	function lineCryptArea(srcVal,mainAreaNm, tgHeader,tgEnd, curNm, newPw){
		if (kk.isNotUndef(newPw)){
			var pw= getJsPassword(newPw);
		}else{
			var pw= getJsPassword();
		}
		
		if (pw!==false){
			var storedCode= readableStrToStoredStr(srcVal,fCharSet,pw);
			
			var ptMArea= document.getElementById(mainAreaNm);
			var valMArea= ptMArea.value;
			
			var iTagHd= valMArea.indexOf(tgHeader);	// index of the tag Header [jsCrypt i="1"] given with or withiut the i
			var headStr= valMArea.substr(0, iTagHd+tgHeader.length);
			
			var iTagNd= valMArea.indexOf(tgEnd);		// index position of the end-tag  [/jsCrypt]
			var tailStr= valMArea.substr(iTagNd);
			ptMArea.value= headStr + storedCode + tailStr;
						
			// change the left display
			var ptSpan= document.getElementById(curNm);
			ptSpan.innerHTML= srcVal;
			
			globCryptVar[curNm]= storedCode;
			gbRdblCrpVar[curNm]= srcVal;			
		}else{
			pleaseWYP();
		}
	} // lineCryptArea
	
	
	/**
		Enter button --> Deal with a comment.
		
		Input :
			srcVal : the new string to incorporate in the structure (crypt it, display etc.)
			mainAreaNm : the name of the main [] area! Define in php as _c_taMainStNm
			tgHeader : the xml[] square head tag with its full index attribute.
			tgEnd : the xml[] square end tag with its concatanated index attribute (special ending case to have unique end)
			curNm : the inline html tag name with its concatanated index.
		Output :
			Adjust the left and right areas.
	*/
	function lineChangeComment(srcVal,mainAreaNm, tgHeader,tgEnd, curNm){	
		var pw= getJsPassword();	
		if (pw!==false){
			var ptMArea= document.getElementById(mainAreaNm);
			var valMArea= ptMArea.value;
			
			var iTagHd= valMArea.indexOf(tgHeader);	// index of the tag Header [jsCrypt i="1"] given with or withiut the i
			var headStr= valMArea.substr(0, iTagHd+tgHeader.length);
			
			var iTagNd= valMArea.indexOf(tgEnd);		// index of the end-tag  [/jsCrypt]
			var tailStr= valMArea.substr(iTagNd);
			ptMArea.value= headStr + srcVal + tailStr;
				
			// change the left display
			var ptSpan= document.getElementById(curNm);
			ptSpan.innerHTML= srcVal;					
			globLineComment[curNm]= srcVal;	// stored the value that was just edited
		}else{
			pleaseWYP();
		}
	} // lineChangeComment
	
	
	/**
		Crypt directly in the area box.
	*/
	function boxCryptArea(areaName){
		var pw= getJsPassword();
		if (pw!==false){
			srcVal= document.getElementById(areaName).value;
			var str= readableStrToStoredStr(srcVal,fCharSet,pw);
			document.getElementById(areaName).value= str;
		}else{
			pleaseWYP();
		}	
	} // boxCryptArea
		

	/**
		Manual uncryption of the area.using its own string.
	*/
	function manUncrptArea(areaName){
		var pw= getJsPassword();
		var areaStr= document.getElementById(areaName).value;
		if (pw!==false){
			var str= storedStrToReadableStr(areaStr,fCharSet,pw);
			document.getElementById(areaName).value= str;
		}else{
			pleaseWYP();
		}	
	}; // manUncrptArea	
	

	/**
		Uncrypt the writing area previously loaded with a pure storage string!
	*/
	function uncryptArea(storedStr,areaName){
		var pw= getJsPassword();
		if (pw!==false){
			var str= storedStrToReadableStr(storedStr,fCharSet,pw);
			document.getElementById(areaName).value= str;
		}else{
			pleaseWYP();
		}	
	} // uncryptArea
	

	/**	
	*/
	function pwDelete(){
		localStorage.jsCryptPwBoxStr= '';
		
		if (localStorage.externalXorMode!=='ok'){
			document.getElementById('jsCryptPwBoxNm').value= '';
			pwWatch();
			hideAll();
			pleaseWYP();
		}
		
		globBinShufflPw= null;
		globBinXorPw= null;
		
		
		globLineComment= new Array();	// no point to delete someting accessible without password
		globCryptVar= new Array();		// never delete it otherwise need to reload the whole php page even when the password is given!!!
		
		gbRdblCrpVar= new Array();
		globCryptPw= '';
		
		strXorShift.fullBufferDel();
	}; // pwDelete

	/**
	*/
	function storesPw(){
		getJsPassword();
		pleaseWYP();
		pwWatch();
	}; // storesPw
	
	/*
		End tags Php calls :
	-------------------------------------------------------------------------- */
	

	/**
	*/
	function pwIndications(){
		var pt= document.getElementById('pwIndications');
		// Keep it : complement message in case of implementation + ' (twice as many for Shffle&Xor)';
		pt.innerHTML= 'Info : password minimum size='+minPasswdSz+' Optimum password size>= '+ optimPasswdSz + ' (sup. chars not taken)';
	}; // pwIndications
	
	
	/**
		Not used !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	*/
	function messWatch(){
		var pt= document.getElementById('lnCryptMessWatch');
		
		pt.innerHTML= 'Actual crypted password is: '+ valDspl;
	}; // messWatch
	
	
	/**
	*/
	function plsWYP(blink){
		var ptPwBox= document.getElementById('jsCryptPwBoxNm');
		if (ptPwBox===null){
			return '';
		}
		
		var pt= document.getElementById('jsCPwMssBxNm');
				
		var locStPwBoxStr= localStorage.jsCryptPwBoxStr;
		if (locStPwBoxStr!==null && kk.isNotUndef(locStPwBoxStr)){
			var local= locStPwBoxStr.length >= minPasswdSz;
		}else{
			var local= false;
		}
			
		var pwBoxVal= ptPwBox.value;
		var inBox= pwBoxVal.length >= minPasswdSz;
		
		if ( local || inBox ){
			pt.innerHTML= '';
			document.getElementById('pwBlinker').innerHTML= '';
		}else{
			pt.innerHTML= ' Please write your password!';
			if (blink){
				pwBlinker();
			}
		}
	}; // pleaseWYP

	// with blinker
	function pleaseWYP(){
		plsWYP(true);
	}
	
	// without blinker
	function plsNoBlinkWYP(){
		plsWYP(false);
	} // plsNoBlinkWYP
	

	/**
		Password proximity blinker.
	*/
	function pwBlinker(){
		var ptBlink= document.getElementById('pwBlinker');
		if (globLongBlinker){
			ptBlink.innerHTML= '&lt;&lt;&lt;';
			globLongBlinker= false;
		}else{
			ptBlink.innerHTML= '&lt;';
			globLongBlinker= true;
		}
	}; // pwBlinker
	
	
	/**
		Get the JS crypt password And return it on a binary vector form.
		Input :
			newPw : eventual new password
			
		Output :
			return the given pasword in a 128 bits form with addition of zeros or truncation.			
	*/
	function getJsPassword(newPw){
	
		if (localStorage.externalXorMode==='ok'){
			return localStorage.externalCurrentPw;
		}	
	
		if (kk.isNotUndef(newPw) && newPw.length>=minPasswdSz){
			var pw= newPw;
		}else{
			// classical password management / not new			
			var uniqPrfx= genAsa.globUniqPrefix;
			var sessKeyVal= localStorage.getItem(uniqPrfx+'sessionKey');			
			
			var pt= document.getElementById('jsCryptPwBoxNm');
			if (pt===null){
				return false;
			}
					
			var pw= pt.value;		
			var jsStore= document.getElementById('inlCryptStoringMd').value==='ok';
			if (pw.length>=minPasswdSz){
				if (jsStore){
					globCryptPw= pw; // taking granted that this transient readable pw variable is not likely to be written on the hard disk!
				
					// CRYPT AND STORES THE PASSWORD! (so can change php/html pages)
					strXorShift.scOn();
					var storedPw= readableStrToStoredStr(pw,fCharSet,sessKeyVal);
					localStorage.jsCryptPwBoxStr= storedPw;
					strXorShift.scOff();
				}
			}else{
				// uses the stored password
				if (jsStore){
					var storedPw= localStorage.jsCryptPwBoxStr;				
					if (storedPw!==null && kk.isNotUndef(storedPw) && storedPw.length>=minPasswdSz){
						if (globCryptPw===''){
							strXorShift.scOn();
							pw= storedStrToReadableStr(storedPw,fCharSet,sessKeyVal);
							globCryptPw= pw;
							strXorShift.scOff();
						}else{
							pw= globCryptPw;	// short cut
						}
					}else{
						return false;
					}
				}
			}
		}
		
		if (pw!==null && kk.isNotUndef(pw) && pw.length>=minPasswdSz){
			pwWatch();	// watch the stored pw if any	
			
			// alert('pw=' + pw);
			
			return pw;
		}else{
			return false;
		}
	}; // getJsPassword
		
		
	/**
	*/
	function actualiseNewPw(mainAreaNm){	
		var pt= document.getElementById('jsCryptPwBoxNm');
		if (pt===null) return false;
		if (pt.value.length<minPasswdSz) return false;
		// So at this stage, we know that the old passwor is going to be taken from the 1st password input box
	
		var iMax= largestCryptedIndex;
		var pw= getJsPassword();	
		if (pw!==false){
			for (i=1;i<=iMax;i++){
				var idxN= 'smallCrypt'+i;
				if (kk.isNotUndef(globCryptVar[idxN])){
					writeLn('# gbRdblCrpVar[idxN]='+gbRdblCrpVar[idxN]);
				
					if (kk.isNotUndef(gbRdblCrpVar[idxN]) && gbRdblCrpVar[idxN]!==''){
						var rdStr= storedStrToReadableStr(globCryptVar[idxN],fCharSet,pw);
						
						
						document.getElementById(idxN).innerHTML= rdStr;
						gbRdblCrpVar[idxN]= rdStr;
					}
					
					var newPw= document.getElementById('jsCryptNewPwBoxNm').value;				
					var tagName= 'lineCrypt';
					var tgHeader= '['+tagName+' i="'+i+'"]';
					var tgEnd= '[/'+tagName+i+']';				
					lineCryptArea(gbRdblCrpVar[idxN], mainAreaNm, tgHeader,tgEnd, idxN, newPw);
				}
			} // for
		}else{
			pleaseWYP();
		}
	}; // actualiseNewPw

		
	/**
		Display a part of the crypted password as an eyeball check to avoid password leaks.
	*/
	function pwWatch(){
		var pt= document.getElementById('pwIndic2');	
		var val= localStorage.jsCryptPwBoxStr;
		if (val==='' || val===false || kk.isUndef(val) || val===null){
			// probabls not all of these values possible
			valDspl= ' Not defined!';
		}else{
			// valDspl= String(val.slice(0,6)) + ' ..... ';
			valDspl= val;
		}		
		pt.innerHTML= ' The actual stored crypted password is : '+ valDspl;
	}; // pwWatch
		

	/**
		Translates each chars on 6 bits, but real length for the last char.
		
		Input :
			binStr : binary string
			oBinStor : global onject holding conversion vectors. (o.bWordSz; o.chrToBin; o.binToChr)
			
		Output:
			binStr
	*/
	function storeCodeToBinCode(storedStr){
		var o= oBinStor;;
		var binStr= '';
		
		var sz= storedStr.length;
		var lastCharM= storedStr[sz-2]; 	// last char minus one
		
		var beforeLast= 0;	// to stop the loop before the last character
		if (lastCharM==='#'){
			beforeLast= 3;	// need to stop 3 chars before the end to analyse the before last char.
		}		
		
		for (var i=0; i<sz-beforeLast; i++){
			var curChar= storedStr[i];
			binStr+= o.chrToBin[curChar];
		}
		
		if (lastCharM==='#'){
			var nZ= Number(storedStr[sz-1]);	// number of zeros to remove
			var lastSigChar= storedStr[sz-3];	// last significative character			
			var sLastSigChar= o.chrToBin[lastSigChar];
			binStr+= sLastSigChar.slice(nZ);
		}		
		return stringMath.strToVect(binStr);
	}; // storeCodeToBinCode also see binCodeToStorCode

		
	/**
		Translates the binary code into the stored code!	
		Input :
			binV : binary vector
			oBinStor : global onject holding conversion vectors. (o.bWordSz; o.chrToBin; o.binToChr)
			
		Output:
			storedCode
	*/
	function binCodeToStorCode(binV){
		var o= oBinStor;;
		var sz= binV.length;
		var storedCode= '';
		
		var cWP= 0;			// cur Word Position / countor
		var cBinWrd= '';	// current binary word
		for (var i=0; i<sz; i++){
			cBinWrd+= binV[i];
			cWP++;
			if (cWP===o.bWordSz){
				storedCode+= o.binToChr[cBinWrd];
				cWP= 0;
				cBinWrd= '';
			}
		} // for
		
		if (cWP!==0){
			// partial word
			var nbZeros= o.bWordSz-cWP;	// up to 5, because 6 bits are used to express the storage base.
			cBinWrdTwo= stringMath.strOFiller(nbZeros) + cBinWrd;	// so get a standard binaray identificator
			
			var cBinWrdT= o.binToChr[cBinWrdTwo];
			storedCode+= cBinWrdT + '#' + nbZeros;
		}
		return storedCode;
	}; // binCodeToStorCode also see storeCodeToBinCode	
		
		
	/**
		Small link action : transfers the link content (globCryptVar[lnName]) inside the main area box jsCrypTAN1 
		(nb one the main one if there are many).
		Input :
			rootName + idx : anchor/link Name.
			tagName ; name of the php tag.
			
			// infoStr : a 4th param udea : would be stored in inline JS with no easy modofication possibilities
			So ---> need an assoc array bottom page loaded.
	*/
	function smllLinkAct(rootName,idx,tagName){
		var lnName= rootName+idx;
		globCurLinkCateg= tagName;
		globCurrentInlineNm= lnName;
		globCurrentInlineIdx= idx;	// not used for now

		globLnTagHeader= '['+tagName+' i="'+idx+'"]';
		globLnTagEnd= '[/'+tagName+idx+']';
		
		var ptArea= document.getElementById('jsCrypTAN1');	// pt small crypt area

		/*	
			Loads the small working area (globLineComment is loade with processSmallArea()).
				Warning : can't recover the value directly from :	# var ptLn= document.getElementById(lnName);
					# ptArea.value= ptLn.innerHTML;  Because it remove the / of any html <tag /> so eg: <br/> becomes <br> !!
				Why? Probably because some browsers still use old html internally and convert only one way!
		*/
		if (rootName==='smallCrypt'){
			var pw= getJsPassword();
			if (pw!==false){
				var readableStr= gbRdblCrpVar[lnName];
				if (readableStr===''){
					gbRdblCrpVar[lnName]= storedStrToReadableStr(globCryptVar[lnName],fCharSet,pw);
				}
				ptArea.value= readableStr;
			}else{
				pleaseWYP();
			}
		}else{
			if (globLineComment!==null){
				ptArea.value= globLineComment[lnName];
			}
		}
	}; // smllLinkAct
		

	/**
		Uncrypt / decypher the small link on the spot (end-onload), from a php "initative" + ...
		must also load globCryptVar[...]
	*/
	function unCryptSmallLn(storedStr,widgetName){
		var pw= getJsPassword();
		if (pw!==false){
			var str= storedStrToReadableStr(storedStr,fCharSet,pw);
						
			document.getElementById(widgetName).innerHTML= str;
			gbRdblCrpVar[widgetName]= str;
		}else{
			var pt= document.getElementById(widgetName);
			if (pt!==null){
				pt.innerHTML= '**********';
				gbRdblCrpVar[widgetName]= '';
				pleaseWYP();
			}
		}	
		
		globCryptVar[widgetName]= storedStr;
	}; // unCryptSmallLn						

		
	/**
		Just after a php loads, javascript loads the global line comment
			Input :
				i : index.
				gStr : given string.
	*/
	function gloLnCom($fName,gStr){
		globLineComment[$fName]= gStr;
	}; // gloLnCom
		
				
	/**
	*/
	function hideAll(){
		var iMax= largestCryptedIndex;		
		for (i=1;i<=iMax;i++){
			var pt= document.getElementById('smallCrypt'+i);
			if (pt!==null){
				pt.innerHTML = '**********';
				gbRdblCrpVar['smallCrypt'+i]= '';
			}
			// Note : globCryptVar[...] // is always "hidden" ---> Crypted
		} // for
		
		document.getElementById('jsCrypTAN1').value= '';
	}; // hideAll


	/**
	*/
	function uncryptAll(){
		var iMax= largestCryptedIndex;
		var pt= document.getElementById('jsCryptPwBoxNm');
		var jsStore= document.getElementById('inlCryptStoringMd').value==='ok';		
		var pw= getJsPassword();
		if (pw!==false){
			if (jsStore){
				pt.value= '';	// remove the password entry so don't have to compute/store it each time!
			}
		
			for (i=1;i<=iMax;i++){
				if (kk.isNotUndef(globCryptVar['smallCrypt'+i])){
					// calls too many xorShifts vectors --> need to set a glob var
					var rdStr= storedStrToReadableStr(globCryptVar['smallCrypt'+i],fCharSet,pw);
					document.getElementById('smallCrypt'+i).innerHTML= rdStr;
					gbRdblCrpVar['smallCrypt'+i]= rdStr;
				}
			} // for
		}else{
			pleaseWYP();
			pt.value= '';	// delete the password, because it did not work
		}
	}; // uncryptAll
	
	/**
		Adds a new couple comment / Crypt only into the xml[] panel! Then got tp send it for actualisation.
	*/
	function newCouple(mainAreaNm,srcAreaNm){
		var pw= getJsPassword();
		if (pw!==false){
			var origIdx= largestCryptedIndex;
			var idx= origIdx+1;
			largestCryptedIndex= idx; // nee to check overflow when scanning the left side !!!!!!!!!!!!!
			
			var commentStr = '<br/><br/>The Comment Part';
			var tagNameCo= 'lineCom';
			var tgHeaderCo= '['+tagNameCo+' i="'+idx+'"]';
			var tgEndCo= '[/'+tagNameCo+idx+']';
			
			var cryptedStr= readableStrToStoredStr('The Crypted Part',fCharSet,pw);
			var tagNameCr= 'lineCrypt';
			var tgHeaderCr= '['+tagNameCr+' i="'+idx+'"]';
			var tgEndCr= '[/'+tagNameCr+idx+']';
						
			// last actual end tag (for remplacement search)		
			var lastEndTag= '[/'+tagNameCr+origIdx+']';
									
			// Builds the xml string		
			var xStr= crLn+crLn+crLn;
			xStr+= tgHeaderCo+commentStr+tgEndCo;
			xStr+= crLn;
			xStr+= tgHeaderCr+cryptedStr+tgEndCr;
					
			var ptMArea= document.getElementById(mainAreaNm);
			var valMArea= ptMArea.value;
			var indxLast= valMArea.indexOf(lastEndTag);		// index position of the end-tag  [/jsCrypt]			
			indxLast+= lastEndTag.length;
			
			var headStr= valMArea.substr(0,indxLast);
			var tailStr= valMArea.substr(indxLast);
			ptMArea.value= headStr+xStr+tailStr+crLn+crLn; // Loads the xml[] area
			var ptDspl= document.getElementById(srcAreaNm);
			ptDspl.value= 'Just wrote a new comment/crypt couple in the right panel'+crLn+ptDspl.value;
		}else{
			pleaseWYP();
		}
	}; // newCouple
	
	/**
	*/
	function initLrgstCrptIndx(val){
		largestCryptedIndex= Number(val);
	}; // initLrgstCrptIndx
		
	/**
		Enter button
	*/
	function processSmallArea(srcAreaNm,mainAreaNm){
		var srcVal= document.getElementById(srcAreaNm).value;
		if (globCurLinkCateg==='lineCrypt'){
			lineCryptArea(srcVal,mainAreaNm, globLnTagHeader,globLnTagEnd,globCurrentInlineNm);
		}else{
			lineChangeComment(srcVal,mainAreaNm, globLnTagHeader,globLnTagEnd,globCurrentInlineNm);
		}
	}; // processSmallArea
	
	/**
	*/
	function saveAll(srcAreaNm,mainAreaNm){
		document.getElementById('mainSubmitBut').click();
		document.getElementById('jsCrypTAN1').value= 'Please wait : Saving the Modifications!';
	}; // saveAll

	/**
	*/
	function winOpen(dsplStr){
		var win=window.open('','','width=700,height=500');	
		win.document.write(dsplStr);
		win.focus();
	}; // winOpen
	
	/* --------------------------------------------
		"Cloned" methods for usage easiness 
	*/
	function getString(){return kk.getString();}
	function write(s){kk.write(s);}
	function writeLn(s){kk.writeLn(s);}
	function clearWrite(s){kk.clearWrite(s);}
	function clear(){kk.clearWrite('');}

	/**
		Write a Vector content.
	*/
	function wect(lbl,v){
		stringMath.wect(lbl,v);
	}; // wect

	/**
	*/
	function wectLn(lbl,v){
		stringMath.wectLn(lbl,v);
	}; // wectLn

	/**
	*/
	function chabl(){

	}; // chabl

} // jsEncryption
