/**
	File name : strXorShift.js
	Author of this file : Oscar Fawer until 2019-08-15
	Contact : www.ataox.org 
	Web sites : ataox.org / ataox.com / ataox.info
	Copyrights open source GPL, GNU General Public License V3 : see ./aMainDr/code/LICENSE.txt


*/

/*
	A xor shift implemented with pseudo string (in fact arrays).
*/
function strXorShift(){
	
	/* ------------------------------------------------------------------------------------------------
		Public Methods 
	*/
		
	this.init= init;
	this.smplNxtScaled= smplNxtScaled;		// Just for testing the string xorshift with the graphical display
	
	this.nextScaled= nextScaled;			// Partially vectorised, not a problem with 32 pseudo bits registries.
	
	
	this.iniwBigBin= iniwBigBin;
	this.buildPairsVect= buildPairsVect;	// idem nextScaled, but fully vectorised --> too slow
	this.binaryVect= binaryVect;		
	this.xor= xor;		
	this.fullBufferDel= fullBufferDel;
	this.scOn= scOn;		
	this.scOff= scOff;		
	
	/* ------------------------------------------------------------------------------------------------
		Private global variable to store the 32 bits registries 
		----> independance on the javacsript machine implementation.
	*/
	var x= new Array();
	var y= new Array();
	var z= new Array();
	var w= new Array();
	
	/* -------------------------------------------------------
		Global buffer object : acceleration with buffer
		Exmpl :
			gBfO[globCurKind].v[]
			.calculated
	*/
	var gBfO = new Array();
	gBfO['xor']= new Object();
	gBfO['shf']= new Object();
		
	ldsBfAtribV(gBfO['xor'])
	ldsBfAtribV(gBfO['shf'])
	
	// Current global kind and size of the current asked for vector
	var globCurKind= '';
	var globCurSz= 0;
	var globBuffUsage= false;
	var globNextShortCut= false; // if true, call directly nextCore


	/* ------------------------------------------------------------------------------------------------
		Body Methods 
	*/
	
	/**
		For real random, each paramneter can be initialised with for exmpl :
			Math.floor(1000000000*Math.random())
			
		For crypting problems, each paramneter can be initialised with a numerisation of the password
	*/
	function init(a,b,c,d){
		x= bTenToVectBinTT(a);
		y= bTenToVectBinTT(b);
		z= bTenToVectBinTT(c);
		w= bTenToVectBinTT(d);
	} // init


	/**
		Just for testing the string xorshift with the graphical display.
	*/
	function smplNxtScaled(scale){
		var vInX= nextCore();
		var nbTen= stringMath.baseXAluTen(vInX,'01');
		return nbTen % scale;
	} // smplNxtScaled

	
	/**
		Return the next scaled number.
		Uses a vector buffer to avoid recalculation if unecessary
		
		Input :
			scale : the conversion scale (side of the squarre)
			step : the current step.
	*/
	function nextScaled(scale,step){
		var vInX= next(step);	
		var nbTen= stringMath.baseXAluTen(vInX,'01');
		var rem= nbTen % scale;
		return rem;
	} // nextScaled

	

	/**
		Short cut on
	*/
	function scOn(){
		globNextShortCut= true;
	} // scOn
	
	function scOff(){
		globNextShortCut= false;
	} // scOff
	

	/**
		Loads the current kind of buffer vector and attributes.
		Remark don't need to define the kind, because if it can reach the 1st level array the kind is known.
	*/
	function ldsBfAtribV(pt){
		pt.v= new Array();	// the xor buffer vector
		pt.sz= 0;			// size of v
		pt.calculatedNb= 0;	// count the exact number of calculated elements
		pt.x= new Array();
		pt.y= new Array();
		pt.z= new Array();
		pt.w= new Array();	
	} // ldsBfAtribV


	/**
		Delete / initialise (after creation, goes in reverse order) the whole buffer
	*/
	function fullBufferDel(){
		// liberation part
		ldsBfAtribV(gBfO['xor']);
		ldsBfAtribV(gBfO['shf']);
		gBfO['xor']= new Object();
		gBfO['shf']= new Object();
		
		// Initialisation part (reverse order for vertically dependant elements)
		gBfO = new Array();		
		gBfO['xor']= new Object();
		gBfO['shf']= new Object();
		ldsBfAtribV(gBfO['xor']);
		ldsBfAtribV(gBfO['shf']);
		
		
		globCurKind= '';
		globCurSz= 0;
		globBuffUsage= false;
	} // fullBufferDel
	
	
	/**
		Direct Initialisation With a large 128 bits binary vector.
		Input :
			v ; large binary vector of 128 bits.
			
			For the acceleration :
				kind : from what kind of call, so a xor 'xor' or a shuffle 'shf' --> the kind determine the whole serie
				sz : the size of the vector to be built!
			
		Output :
			Loads the 4 global shift registries.
	*/
	function iniwBigBin(v, kind,sz){
		globCurKind= kind;
		
		if (kind==='shf'){
			globCurSz= 2*sz;	// shuffling : times 2 because line & column
		}else{
			globCurSz= sz;
		}
		
		globBuffUsage= false;	// not yet! So to memorise if the global xorShift buffer was used, so we know if we have to recover the last initialisation values.
	
		x= v.slice(0,31);
		y= v.slice(32,63);
		z= v.slice(64,95);
		w= v.slice(96,127);
	} // iniwBigBin


	/**
	*/
	function bTenToVectBinTT(a){
		return stringMath.bTenToVectBin(a,32);
	} // bTenToVectBinTT


	/**
		Returns the next step.
		
		!!!! Should integrate the compact next() as a call of this one!
		+ uses the compact directly as a shunt for password crypting with localStorage.sessionKey
	*/
	function next(step){
		if (globNextShortCut){
			return nextCore();
		}
	
		if (step < gBfO[globCurKind].calculatedNb){
			globBuffUsage= true;			
			return gBfO[globCurKind].v[step];
		}else{
			if (globBuffUsage){
				// recover the last values
				var pt= gBfO[globCurKind];
				x= cloneArray(pt.x);
				y= cloneArray(pt.y);
				z= cloneArray(pt.z);
				w= cloneArray(pt.w);
				globBuffUsage= false; // so does the initialisation only once!
			}
			
			w= nextCore();
			gBfO[globCurKind].calculatedNb++;
			
			gBfO[globCurKind].v[step]= cloneArray(w);	// stores one more random number
			
			if (step===globCurSz-1){
				// saves last step (FINAL STATE) for next connection			
				var pt= gBfO[globCurKind];
				pt.x= cloneArray(x);
				pt.y= cloneArray(y);
				pt.z= cloneArray(z);
				pt.w= cloneArray(w);
			}
			
			return w;		
		}		
	} // next
		
	/**
		The core part of the xor shift algo
	*/		
	function nextCore(){
		var r = cloneArray(x);
		var s = cloneArray(w);
						
		// var t = r ^ (r << 11);			
		var rElev= lSh(r,11);
		var t = xor(r,rElev);
		
	  	x= cloneArray(y);
	  	y= cloneArray(z);
	  	z= cloneArray(w);
	  	
		// return w = (s ^ (s >> 19)) ^ (t ^ (t >> 8));		
		var sNteen= rSh(s,19);
		var a= xor(s,sNteen);
		
		var tEig= rSh(t,8);
		var b= xor(t,tEig);		
		w = xor(a,b);
		return w;		
	} // nextCore	
	

	/**
		Return a binary vector of size sz. Especially useful for xoring strings.
		Remark : could go by chunk of 32 bits instead of one by one !!!
	*/
	function binaryVect(sz){
		var vect= new Array();
		for (var i=0; i<sz; i++){
			vect[i]= String(nextScaled(2,i));
		}
		return vect;
	} // binaryVect
	
		
	/**
		Build the random pair vector, mainly for exchanges
		Input :
			sz : size of the requested vector.
			scale : the stored numbers span.
		Output : return the object vector
	*/
	function buildPairsVect(sz,scale){
		var exV= new Object();	// exchange object
		exV.x= new Array();
		exV.y= new Array();
		
		var j=0;
		for (var i=0; i<sz; i++){
			exV.x[i]= nextScaled(scale,j);
			j++;
			exV.y[i]= nextScaled(scale,j);
			j++;
		}
		
		exV.sz= sz;
		return exV;
	}; // buildPairsVect
		
		
	/* ---------------------------------------------------------------------------
		Left and right shift abstraction to test different sort of shifts.
		<< Shifts a from n positions.
	*/
	function lSh(a,n){
		return zLftShift(a,n);
	}


	// >> Shifts a from n positions.
	function rSh(a,n){		// >>
		return sRgtShift(a,n);
	}


	/**
	*/
	function xor(a,b){
		var c= new Array();
	
		var sz= a.length;
		for (var i=0; i<sz; i++){
			c[i]= boolXor(a[i],b[i]);
		}
		
		return c;
	} // xor




	/* -------------------------------------------------------------------------------
		Close replica of the Javascript << and >> except that uses more than 32 bits
	*/
		
	/**
		<<	Zero propagating left shift
		It goes toward the left, but fills the zeros from the right.
	*/
	function zLftShift(a,n){
		var b= cloneArray(a);
		for (var i=0; i<n; i++){
			b.push('0');
			b.shift();
		}
		return b;
	} // zLftShift

	/**
		>> Sign propagating right shift.
		It Goes toward the right and one or zeros (the sign) are fed from the left
	*/
	function sRgtShift(a,n){
		var b= cloneArray(a);
		for (var i=0; i<n; i++){
			b.unshift(a[0]);
			b.pop();
		}	
		return b;
	} // sRgtShift

	/* End Javascript replica.
	---------------------------- */



	/**
		Boolean Xor for '0' and '1'
	*/
	function boolXor(ai,bi){
		var e= bitToBool(ai);
		var f= bitToBool(bi);
		bol= (e && !f) || (!e && f);		// idem : return (!e || f) && (e || !f);
		return boolToBit(bol);
	} // boolXor


	/**
		('0','1') ---> (false,true)
	*/
	function bitToBool(bit){
		return bit==='1';
	} // bitToBool

	/**
		(false,true) ---> ('0','1') 
	*/
	function boolToBit(bol){
		if (bol){
			return '1';
		}else{
			return '0';
		}
	} // boolToBit


	/**
		Indispensable to clone the vectors for the xor shift assignation
	*/
	function cloneArray(a){
		return stringMath.cloneArray(a);
	} // cloneArray



	function write(s){kk.write(s);}
	function writeLn(s){kk.writeLn(s);}
	


	/**
	*/
	function chablon(){
	
	} // chablon

} // strXorShift
