<?php
/**
	File name : genHash.php
	Author of this file : Oscar Fawer until 2019-08-15
	Contact : www.ataox.org 
	Web sites : ataox.org / ataox.com / ataox.info
	Copyrights open source GPL, GNU General Public License V3 : see ./aMainDr/code/LICENSE.txt



	A general hash method for circular storage of hashes and their associated value.
	
	[cenPlugin name="genHash" /]					[_ r="PLUGIN DECLARATION"/]
	[testPlug display="Plugin trial of : " /]		[_ r="A CALL TO tg_testplug()"/]
	
	Direct calls explicit central or external :
			$tagsFu->instExplitCentralCls('classN');
			$tagsFu->instExplitExternalCls('classN');
			
	
	Keep going with :
		genHash::setHash()
		genHash::getHash()
*/
class genHash extends oTop{
	
	// CONSTRUCTOR
	function __construct(&$tagsFu){
		parent::__construct();	
		$this->glob= &$tagsFu->glob;
		$this->caps= &$tagsFu->caps;
		$this->tools= &$tagsFu->tools;
		$this->fls= &$tagsFu->fls;
		$this->dtb= &$tagsFu->dtb;
		$this->hPwC= &$tagsFu->hPwC;
		
		
		define('_c_genHashIdMin',1000);
		define('_c_genHashIdMax',1300);
		
		
		// Remark : need an ordinal id to have a circular buffer
		define('_c_genHashTbl','genHash');
			define('_c_genHashId',_c_genId);		// Id
			define('_c_genHashH','hash');			// The Hash itself
			define('_c_genHashInfo','info');		// Whatever information
														
		$tagsFu->includeSupFiles('genHash','debugDisplay',false);
		$this->gotASuccessfulCaptcha= false;
		
		
		$this->unitySessionHashStr= false;
	} // construct


	/**
	*/
	function tags(){
		return array();
	} // tags

	/**
	*/
	function captchaImgDfn($hash){
		return _c_captchaImgDir.$hash.'.jpg';
	} // captchaImgDfn

		
	/**
		Get the info associated with $hash
		Rem : should be used only once, so just remove the old record once it has been accessed.
	*/
	function getInfo($hash){
		$dtb= &$this->dtb;
		return $dtb->getElem(_c_genHashTbl, _c_genHashH,$hash, _c_genHashInfo);
	} // getInfo


	function getId($hash){
		$dtb= &$this->dtb;
		return $dtb->getElem(_c_genHashTbl, _c_genHashH,$hash, _c_genHashId);
	} // getId

		
	/**
	*/
	function deleteHashRec($id){
		$dtb= &$this->dtb;
		$dtb->deleteTableRecord(_c_genHashTbl,_c_genHashId,$id);
	} // deleteHashRec
		


		
	/**
		Set a new hash record.with the matching information, and returns the generated hash value.
	*/
	function setNewHash($info){
		$fls= &$this->fls;
		$dtb= &$this->dtb;
			
		$freeHashId= $dtb->getGenVar('genHashFreeId');
		$hash= $this->calculatesNewHashStr();
		
		if ($dtb->elementExists(_c_genHashId, $freeHashId, _c_genHashTbl)){
			$oldHash= $dtb->getElem(_c_genHashTbl, _c_genHashId,$freeHashId, _c_genHashH);
			$oldImgDfn= $this->captchaImgDfn($oldHash);
			
			$fls->secUnlink($oldImgDfn);
		
			// Ecrases actual hash/infos with new information.
			$dtb->setElem(_c_genHashTbl,  _c_genHashH,$hash,  _c_genHashId,$freeHashId);		
			$dtb->setElem(_c_genHashTbl,  _c_genHashInfo,$info,  _c_genHashId,$freeHashId);	
		}else{
			$rec= array($freeHashId,$hash,$info);
			$dtb->newRecFromV(_c_genHashTbl,$rec);
		}
		
		$this->incrFreeHashPos();
		return $hash;
	} // setNewHash


	/**
		Calculates (or recover from memory) a single NewH ash String per session-unity
	*/
	function calculatesNewHashStr(){
		$hPwC= &$this->hPwC;
		
		if ($this->unitySessionHashStr===false){
			$this->unitySessionHashStr= $hPwC->newHashStr(20);
		}
		
		return $this->unitySessionHashStr;
	} // calculatesNewHashStr


	/**
		Increment of one position the genHashFreeVal general variable.
	*/
	function incrFreeHashPos(){
		$dtb= &$this->dtb;

		$freeId= $dtb->getGenVar('genHashFreeId');
		
		if ($freeId<_c_genHashIdMax){
			$newFreeId= $freeId+1;
		}else{
			$newFreeId= _c_genHashIdMin;
		}
	
		$dtb->setGenVar('genHashFreeId',$newFreeId);
	} // incrFreeHashPos

		
	/**
		Return true if the concerned captcha is correct.
	*/
	function captchaIsCorrect($convert1ToL=false){
		$tools= &$this->tools;
		$fls= &$this->fls;
	
		if ( !$this->gotASuccessfulCaptcha ){
			$userAnswer= strtolower($tools->postVal(_c_captchaTxtNm));
			if ($convert1ToL){
				$userAnswer= str_replace('1','l',$userAnswer);
			}
			
			$userHash= $tools->postVal(_c_genCaptHidHashNm);	// user side hash
			$srvStoredAnswer= $this->getInfo($userHash);	// server stored info
			
			if ($srvStoredAnswer!==false && strtolower($srvStoredAnswer)===strtolower($userAnswer) ){
				// captcha was solved correctly, just got to delete the hash storage and see if other fields are ok.
				$usedHashId= $this->getId($userHash);	// user side, but checked!
				$this->deleteHashRec($usedHashId);		// delete the circular hash at the id position
				
				$imgDfn= $this->captchaImgDfn($userHash);
				$fls->secUnlink($imgDfn);
				$this->gotASuccessfulCaptcha= true;
				return true;
			}else{
				// Captcha uncorrect
				return false;	// can't have a form succeed if the captcha was asked for and not solved
			}
		}else{
			return true;
		}
	} // captchaIsCorrect
	
		
	/**
	*/
	function tg_tmptag(&$v,&$bd){
		$caps= &$this->caps;
	
	} // tg_tmptag

		
	/**
	*/
	function tmp(){
	
	} // tmp
		
} // genHash
?>
