<?php
/**
	File name : polyglot.php
	Author of this file : Oscar Fawer until 2019-08-15
	Contact : www.ataox.org 
	Web sites : ataox.org / ataox.com / ataox.info
	Copyrights open source GPL, GNU General Public License V3 : see ./aMainDr/code/LICENSE.txt



	To deal with language switching from one language to another.
*/
class polyglot extends oTop{
	

	function __construct(&$tagsFu){
		parent::__construct();	
		$this->tagsFu= &$tagsFu;
		
		$this->glob= &$tagsFu->glob;
		$glob= &$this->glob;
		
		$this->acc= &$tagsFu->acc;
		
		
		$this->tools= &$tagsFu->tools;
		$this->dtb= &$tagsFu->dtb;
		
		// bit of over normalized style, but much easier to keep it up to date on the long run!
		$dtb= &$this->dtb;
		
		$this->okToWrite= true;
		
		$tagsFu->supFilesExplCntr('polyglot');	
		
		$this->lgVect= $this->getLgVect();
		$lgVect= &$this->lgVect;
		$this->lgVectSz= count($lgVect);
		
		
		
		// $lgVectPlus= &$lgVect;
		$lgVectPlus= $this->getLgVect();
		
		// tg_lgswitch() : displays the language switches
		array_push($lgVectPlus,'fontdw');
		array_push($lgVectPlus,'fontup');
		
	
		/** -------------------------------------------------------------------------------------------
			tg_glot() xml stored in c_PolyglotTbl table to switch between language related pages like :
				[glot fr="bonjour" en="hello"/]
				define('_c_tgPolyglot','glot'); 

		*/
		$tagsFu->attribVect[_c_tgPolyglot]= $lgVect;
		
		/* A version of tg_glot without global implication and freely available to the forum user (not that
		they have an interest to use it, but it can't do an harm). Flot for Free Glot.  */
		$tagsFu->attribVect['flot']= $lgVect;
		
		
		$tagsFu->attribVect['lgswitch']= $lgVectPlus;		// Tag to define the language switch buttons
		$tagsFu->attribVect['edtlgswitch']= $lgVectPlus;
		
		$tagsFu->attribVect['connectlgpg']= $lgVectPlus;	// got the whole set of available language 
	} // construct
	
	
	/**
		This associative array must hold all the class's tags names, so the cms can declare them
	*/
	function tags(){
		return array(_c_tgPolyglot, 'flot', 'lgswitch', 'edtlgswitch', 'connectlgpg');
	} // tags
	
		
	/**
	*/
	function getLgVect(){
		$acc= &$this->acc;
		$tools= &$this->tools;
		$dtb= &$this->dtb;
	
		$lgList= $dtb->getGenVar('possibleSurfLang');
		$lgList= strtolower( $acc->toNoAccents($lgList) );
		return $tools->trimStrToV($lgList);
	} // getLgVect
	
		
	/**
		Generate a complete multilingual pages set even if the concerned page is
		language incomplete or mono linguistic.
	*/
	function synthLgPgToVect($pg){
		$syntVect= array();	// synthetised full language vector
		$lgVect= $this->lgVect;
		reset($lgVect);
		foreach ($lgVect as $key=>$lg){
			$syntVect[$lg]= $pg;
		}
		return $syntVect;
	} // synthLgPgToVect
	
		
	/**
		Complete the current language vector with the full synthetized one
	*/
	function polyglotizeGivenPg($hookerPg,$lgPgSet){
		$syntVect= $this->synthLgPgToVect($hookerPg);
		$mixVect= array();
		reset($syntVect);
		foreach ($syntVect as $key=>$pageName){
			if (array_key_exists($key,$lgPgSet)){
				$mixVect[$key]= $lgPgSet[$key];
			}else{
				$mixVect[$key]= $pageName;
			}
		}
		return $mixVect; // the polyglotized vect
	} // polyglotizeGivenPg
	
	
	
	/**
		Transform an array into an array indexed with its own values and having capitalized values (identical values will merge).
	*/
	function alphaCapitalize(&$v){
		reset($v);
		$u= array();
		foreach ($v as $key=>$val){
			$u[strtolower($val)]= strtoupper($val);
		}
		return $u;
	} // alphaCapitalize
	


	/**
		To process polyglot strings stored in _c_PolyglotTbl. Call by tg_lgswitch().only on the active page!
			Example : [glot fr="bonjour" en="hello"/] (storage :  _c_PolyglotTbl)
	*/
	function tg_glot(&$v,&$bd){
		$glob= &$this->glob;
		$glob->currentLanguageSet= $this->glotCore($v);
	} // tg_glot

	/**
		flot holds for free glot, so a tag freely available to forum users.
	*/
	function tg_flot(&$v,&$bd){
		$glob= &$this->glob;
		$glob->freeLanguageSet= $this->glotCore($v);
	} // tg_flot
	
	/**
		Eliminates non mentioned languages inside the tags glot and flot!
		Practically it allows to not bother translating some pages!
	*/
	function glotCore(&$v){
		reset($v);
		foreach ($v as $lg=>$pg){
			if ( $pg!==false && $lg[0]!=='_' ){
				// if ( used parameter && NOT a system parameter )
				$realSet[$lg]=$pg;
			}
		}
		return $realSet;
	} // glotCore
	


	/**
		Display the language switch. [lgSwitch fr="français" en="English" de="Deutsch" /]
			Remark : the pages are language categorised at building time, storing all the necessary language relationship
			inside the _c_PolyglotTbl table.
	*/
	function tg_lgswitch(&$v,&$bd){
		return $this->coreLgswitch($v);
	} // tg_lgswitch
	

	/**
		To display the eventual languages buttons.
	*/
	function languageSwitch(){
		$tagsFu= &$this->tagsFu;
		$dtb= &$this->dtb;
	
		$ls= $dtb->getGenVar('tagLgSwitch');
		if ($ls===false || $ls===_c_ko || $ls===''){
			// Automatical presentation
			$lgVect= $this->lgVect;	// idem keep comment $lgVect= $this->getLgVect();
			
			if ($lgVect===false){
				return '';
			}else{
				$v= $this->alphaCapitalize($lgVect);
				return $this->coreLgswitch($v);
			}
		}else{
			// Expand the given tag(s)
			return $tagsFu->expandX($ls);
		}
	} // languageSwitch
	
	
	/**
		[edtLgSwitch /]
	*/
	function tg_edtlgswitch(&$bidV,&$bidBd){
		reset($this->lgVect);
		foreach ($this->lgVect as $key=>$lg){
			$v[$lg]= strtoupper($lg);
		}
		
		$v['fontdw']= 'langSwitchDown';
		$v['fontup']= 'langSwitchUp';
		return $this->coreLgswitch($v);
	} // tg_edtlgswitch


	/**
	*/
	function coreLgswitch(&$v){
		$tagsFu= &$this->tagsFu;
		$tools= &$tagsFu->tools;
		$caps= &$tagsFu->caps;
		$glob= &$tagsFu->glob;
		$dtb= &$tagsFu->dtb;
		
		if (array_key_exists('fontdw',$v) && $v['fontdw']!==false){
			$fontdw= $v['fontdw'];
		}else{
			$fontdw= 'langSwitchDown'; // defined in zCms/plugUsrData/explicit/central/polyglot/polyglot.css 
		}
		
		if (array_key_exists('fontup',$v) && $v['fontup']!==false){
			$fontup= $v['fontup'];
		}else{
			$fontup= 'langSwitchUp'; // defined in zCms/plugUsrData/explicit/central/polyglot/polyglot.css
		}
		
		$curMthPg= $tools->curMothrPg();
		if ($curMthPg!==false){
			$curPgName= $curMthPg;
		}else{
			$curPgName= $tools->getUrlPageName();
		}
		
		$curLg= $dtb->currentLg($curPgName);
		$lgSet= $dtb->languageFamVect($curPgName);
		
		
		// Displays the language menu
		$displ= '';
		$nbLg= 0;
		reset($v);
		foreach ($v as $keyLg=>$dsplVal){
			if ($dsplVal!==false && $keyLg!=='fontdw' && $keyLg!=='fontup'){
				if ($lgSet===false || !array_key_exists($keyLg,$lgSet) ){
					$pg= _c_permanDefPage;
					$defPgLgSet= $dtb->languageFamVect($pg);
					if ( $defPgLgSet!==false && array_key_exists($keyLg,$defPgLgSet) ){
						$pg= $defPgLgSet[$keyLg]; // so when the actual page has no translation go back to the default page with the CORRECT language
					}
				}else{
					$pg= $lgSet[$keyLg];
				}
				
				// print('$keyLg='.$keyLg._c_);print('$dsplVal='.$dsplVal._c_._c_);
				
				$arCmd[_c_mothrPage]= _c_ko;
				$url= $tools->urlGivenPage($pg,$arCmd);
				if ($keyLg===$curLg){
					$displ.= $caps->linkC($url,$dsplVal,$fontdw);
				}else{
					$displ.= $caps->linkC($url,$dsplVal,$fontup);
				}
				
				$nbLg++;

				/*
				if ($nbLg>2) return $displ;
				*/
				
			}
		}
		
		// print('stop'._c_._c_);
		
		if ($nbLg>1){
			return $displ;
		}else{
			return '';
		}
	} // coreLgswitch	

	/**
		Returns true if the given page is inside the accross home-pages set!
	*/
	function inHomePgSet($pg){
		$dtb= &$this->dtb;

		$defPgLgSet= $dtb->languageFamVect(_c_permanDefPage);
		if ($defPgLgSet!==false){
			return in_array ($pg,$defPgLgSet);
		}else{
			// No available set --> single language
			if ($pg===_c_permanDefPage){
				return true;
			}else{
				return false;
			}
		}
	} // inHomePgSet


	/**
		Connect the same topic language page loading _c_PolyglotTbl table.
		Example : [connectLgPg fr="bonjour" en="hello" de="Guten tag" /]
	*/
	function tg_connectlgpg(&$v,&$bd){
		$tagsFu= &$this->tagsFu;
		$glob= &$tagsFu->glob;		
		$acc= &$tagsFu->acc;
		$dtb= &$tagsFu->dtb;
		
		$taggedStr= $tagsFu->rebuildTagStr(_c_tgPolyglot,$v);
		
		$mstLg= $glob->gMasterLanguage;
		$masterPage= $acc->safeIdStr($v[$mstLg]);
		reset($v);
		foreach ($v as $keyLg=>$pageId){
			if ($pageId!==false){ // necessary because got the whole bunch of languages
				$pageId= $acc->safeIdStr($pageId);
				
				if ($dtb->elementExists(_c_polyglotId, $pageId, _c_PolyglotTbl)){
					$dtb->delete(_c_PolyglotTbl,$pageId);
				}
				
				if ($keyLg===$mstLg){
					$dtb->createPolyDefCell($pageId,$taggedStr);
				}else{
					$dtb->createPolyDefCell($pageId,$masterPage);
				}
			}
		}
	} // tg_connectlgpg
	
	/**
	*/
	function isInSurfLgVect($lg){
		return in_array($lg,$this->lgVect);
	} // isInSurfLgVect
	
	/**
	*/
	function surfLgNb(){
		return count($this->lgVect);
	} // surfLgNb
	

} // poly
?>