<?php
/**
	File name : textOnImages.php
	Author of this file : Oscar Fawer until 2019-08-15
	Contact : www.ataox.org 
	Web sites : ataox.org / ataox.com / ataox.info
	Copyrights open source GPL, GNU General Public License V3 : see ./aMainDr/code/LICENSE.txt



	[cenPlugin name="textOnImages" /]				[_ r="PLUGIN DECLARATION"/]
	
	Direct calls explicit central or external :
			$tagsFu->instExplitCentralCls('textOnImages');
*/
class textOnImages extends oTop{
	
	function __construct(&$tagsFu){
		parent::__construct();	
		$this->glob= &$tagsFu->glob;
		$this->acc= &$tagsFu->acc;
		$this->caps= &$tagsFu->caps;
		$this->tools= &$tagsFu->tools;
		$this->hPwC= &$tagsFu->hPwC;
		$this->fls= &$tagsFu->fls;
		$this->dtb= &$tagsFu->dtb;
		$this->imgP= &$tagsFu->imgP;
														
		$this->genHash= $tagsFu->instExplitCentralCls('genHash');
		
		// Includes a few default files, such as the current .js and .css files, for more infos, see the method tagsFu::includeSupFiles() in tagsFu.php
		$tagsFu->includeSupFiles('textOnImages','debugDisplay',false);	
	} // construct


	/**
	*/
	function tags(){
		return array();
	} // tags

	
	/**
		Creates a text based captcha image.
		Input : ...
			$pureReading : true --> the image is done for a pure reading test (lazy users)
	*/
	function textToImage($text,$width=600,$height=120,$pureReading=false){
		$acc= &$this->acc;
		$tools= &$this->tools;
		$hPwC= &$this->hPwC;
		$dtb= &$this->dtb;
	
		$genHash= &$this->genHash;
		
		// Create the image
		// ... the height can be adapted to the text size !!!
		$im = imagecreatetruecolor($width,$height);

		// Create some colors
		/*
		$grey = imagecolorallocate($im, 128, 128, 128);
		$red = imagecolorallocate($im, 255, 0, 0);
		$yellow = imagecolorallocate($im, 255, 255, 0);
		*/
		
		$white = imagecolorallocate($im, 255, 255, 255);
		$black = imagecolorallocate($im, 0, 0, 0);
		imagefilledrectangle($im, 0, 0, $width-1, $height-1, $white);
		
		imagesetthickness ($im,2);	// for threads (could be not used)
		
		$this->fillNRandomEllipse($im,$width-1,$height-1);

		// Add the text
		$fontDfn = _c_mainFontDfn.'courbd.ttf';
		if ($pureReading){
			$textV= $tools->mbStrToAry($text,$nbChars);
			
			// $nbChars= (int)$dtb->getGenVar('simpleCaptchaSz'); ... computed in gen2Captcha.php
			$thickness= (int)$dtb->getGenVar('captchaLineThickness',10); 
			$lineNb= (int)$dtb->getGenVar('captchaLineNb',3); 
			$fontSz= (int)$dtb->getGenVar('captchaFontSz',50); 
			$spacer= (int)$dtb->getGenVar('captchaCharSpace',$fontSz);
				
			for ($i=0; $i<$lineNb; $i++){
				$red= rand(40,145);
				$green= rand(40,145);
				$blue= rand(40,145);
				$color= imagecolorallocate($im, $red,$green,$blue);
				$this->drawNRandomThickLine($im,$color,$width-1,$height-1,$thickness);
			} // for
			
			
			$xMax= $width - $spacer*$nbChars;
			if ($xMax < 10){
				$xMax= 10;
			}
			if ($xMax > $width-$fontSz){
				$xMax= $width-$fontSz;
			}
			$xStart= rand(10, $xMax);
			
			$i=0;
			reset($textV);
			foreach ($textV as $k=>$curChar){
				$set= rand(0,1);
				if ($set===0){
					$angle= rand(0,30);
				}else{
					$angle= rand(329,359);
				}
			
				$red= rand(40,145);
				$green= rand(40,145);
				$blue= rand(40,145);
				
				$color= imagecolorallocate($im, $red,$green,$blue);
				
				imagettftext($im, $fontSz, $angle, $xStart+$i*$spacer, $height/2+20, $color, $fontDfn, $curChar);
				$i++;
			}
			
		}else{
			$text= $acc->fromHtmlToAccents($text);
			imagettftext($im, 16, 0, 10, 20, $black, $fontDfn, $text);
		}
		

		$newHashStr= $genHash->calculatesNewHashStr();
		$imgDfn= $genHash->captchaImgDfn($newHashStr);
		
		$this->erasesOldFiles(_c_captchaImgDir);
		
		$ok= imagejpeg($im,$imgDfn,10);		// creates the image file
		imagedestroy($im);	
		return $imgDfn;
	} // textToImage

		
	/**
		Erases all files of the given directpry $gDir that are older than $deltaTm seconds from the current time stamp
		$deltaTm default === 360 sec === 6 min to avoid accumulation in case of extatic button pusher!
	*/
	function erasesOldFiles($gDir,$deltaTm=360){
		$fls= &$this->fls;
	
		$curTm= time();
		$limitTm= $curTm-$deltaTm;
	
		$fls->firstLevelFiles($v,$gDir)	;
		reset($v);
		foreach ($v as $key=>$file){
			$dfn= $gDir.$file;
			$fModTm= filemtime($dfn);
			if ($fModTm<$limitTm){
				if ($fls->isWritable($dfn)){
					unlink($dfn);
					// print('unlink='.$dfn._c_);
				}
			}
		}
	} // erasesOldFiles

		
	/**
		Draw N random lines
		$this->drawNRandomLine(449,99)
	*/
	function drawNRandomLine(&$im,$color,$xMax,$yMax,$N=10,$xMin=0,$yMin=0){
		for ($i=0; $i<$N; $i++){
			$x1= rand($xMin,$xMax);
			$y1= rand($yMin,$yMax);
		
			$x2= rand($xMin,$xMax);
			$y2= rand($yMin,$yMax);
			imageline($im,$x1,$y1,$x2,$y2,$color);
		}
	} // drawNRandomLine
	
	
	function drawNRandomThickLine(&$im,$color,$xMax,$yMax,$thickness=10,$xMin=0,$yMin=0){
		$imgP= &$this->imgP;

		$x1= rand($xMin,$xMax);
		$y1= rand($yMin,$yMax);
	
		$x2= rand($xMin,$xMax);
		$y2= rand($yMin,$yMax);
		
		$imgP->traceThickLine($im,$color,$x1,$y1,$x2,$y2,$thickness);
	} // drawNRandomThickLine

		
	/**
	*/
	function fillNRandomEllipse(&$im,$xMax,$yMax,$N=20,$xMin=0,$yMin=0){
		for ($i=0; $i<$N; $i++){
		
			$color = imagecolorallocate($im, rand(125,255), rand(125,255), rand(125,255));
			
			$xC= rand($xMin,$xMax);
			$yC= rand($yMin,$yMax);
		
			$width= rand($xMin,$xMax);
			$height= rand($yMin,$yMax);
			imagefilledellipse($im,$xC,$yC,$width,$height,$color);
		}
	} // fillNRandomEllipse

		
} // textOnImages
?>
