/**
	File name : calcBaseN.js
	Author of this file : Oscar Fawer until 2019-08-15
	Contact : www.ataox.org 
	Web sites : ataox.org / ataox.com / ataox.info
	Copyrights open source GPL, GNU General Public License V3 : see ./aMainDr/code/LICENSE.txt


*/

/**
*/
function calcBaseN(){


	/* ---------------------------------------------------------------
		Global variable for the "class"
	*/
	
	var crLn= "\r\n";
	var brTag= '<br class="flux" />';
	var globFreqTbl= new Array();
	var oGlobFreq= new Object();
	var tableSideSz= 8;	// for estethical and fast colorstests


	/* ---------------------------------------------------------------
		Methods interface.
	*/	
	this.testATHT= testATHT;
	this.callXorkNext= callXorkNext;
	this.countChars= countChars;
	this.spacer= spacer;
	this.clearAll= clearAll;
	this.setBaseDsplOne= setBaseDsplOne;
	this.setBaseDsplTwo= setBaseDsplTwo;
	this.setBaseFRDsplX= setBaseFRDsplX;	
	this.strgOperBsN= strgOperBsN;		
	this.baseXToBaseY= baseXToBaseY;
	

	/* -----------------------------------------------------------------------
		Methods definitions
	*/	
	
	/**
	*/
	function testATHT(bitXor){
		for (var i=0; i<tableSideSz; i++){
			globFreqTbl[i]= new Array();;
			for (var j=0; j<tableSideSz; j++){
				globFreqTbl[i][j]= 0;	// Initial frenquency				
			}
		}
			
		globFreqTbl= xorShiftFreqTbl(globFreqTbl,200,tableSideSz, bitXor);		
		var oFreq= maxFrequency(globFreqTbl);		
		var htmlTbl= arrToHtmlTbl(globFreqTbl, oFreq.maxF, false, 300);
		
		var htmlInfTbl= '<span class="frequency">Color Scale</span>'+brTag+colorScale();
		document.getElementById('substDiv').innerHTML= htmlTbl;
		document.getElementById('substInfDiv').innerHTML= htmlInfTbl;
	}; // testATHT

	
	/**
		Call (Only from outside) xorNext! Added k to avoid editor search confusion.
	*/
	function callXorkNext(bitXor){
		for (var i=0; i<100; i++){
			xorNext(globFreqTbl,tableSideSz,bitXor);
		}
		
		var oFreq= maxFrequency(globFreqTbl);
		
		var htmlTbl= arrToHtmlTbl(globFreqTbl, oGlobFreq.maxF, false, 300);
		document.getElementById('substDiv').innerHTML= htmlTbl;
	}; // callXorkNext
	

	/**
	*/
	function countChars(){
		initCDspl();
		var str= getInputOne();	
		writeLn('Got '+str.length+' characters in the Operand A text area.');
	}; // countChars
	
	
	/**
	*/
	function spacer(){
		initCDspl();
		var str= document.getElementById('inputOne').value; // don't want the trim
		var acr= document.getElementById('inputTwo').value; // don't want the trim
		
		if (acr===''){
			acr= ' ';	// a space
		}
		
		var sz= str.length;
		var spacedStr= '';
		if (sz>=2){
			for (var i=0; i<sz-1; i++){
				spacedStr+= str.charAt(i) + acr;
			};
			spacedStr+= str.charAt(sz-1);
		}else{
			spacedStr= str;
		}
		writeLn(spacedStr);
	}; // spacer
	
	/**
	*/
	function clearAll(){
		initCDspl();
		clear();
		clrInput();
		
		document.getElementById('substDiv').innerHTML= '';
		document.getElementById('substInfDiv').innerHTML= '';
	} // clearAll


	/**
	*/
	function setBaseDsplOne(s){
		document.getElementById('baseOneDspl').value= s;
	}; // setBaseDsplOne
	

	function setBaseFRDsplX(x){
		document.getElementById(x).value= '(fr)'+stringMath.getBaseFr();
	}; // setBaseFRDspl

	
	/**
	*/
	function strgOperBsN(op){
		initCDspl();
		var base= getBaseOne();	
		var a= getInputOne();
		var b= getInputTwo();	
		clear();
				
		if (op==='/' && vGotOnly(b,base.charAt(0)) ){
			writeLn('Division by zero!');
			return;
		}		
							
		var result= stringMath.baseConvAndSolve(a,b,base,op);
		
		if (op==='/'){
			writeLn('Q='+explicitZero(result.quotient,base));
			writeLn('R='+explicitZero(result.residue,base));
		}else{
			writeLn(explicitZero(result,base));
		}				
	}; // strgOperBsN
	
	/**
		transform a natural zero (empty chain) into an explicit zero base[0]
	*/
	function explicitZero(x,base){
		if (x===''){
			return base.charAt(0);
		}else{
			return x;
		}
	}; // explicitZero

	
	/**
		Returns true if the vector v got only the element e.
	*/
	function vGotOnly(v,e){
		for (var i in v) {
			if (v[i]!==e){
				return false;
			}
		}
		return true
	}; // vGotOnly

	
	/**
	*/
	function baseXToBaseY(){
		initCDspl();
		var nbX= getInputOne();
		var baseX= getBaseOne();	
		var baseY= getBaseTwo();	
		clear();

		var nbY= stringMath.baseXToY(nbX,baseX,baseY);
		writeLn(stringMath.vectToStr(nbY));
	}; // baseXToBaseY	
	
	
	/**
		Calculates the frequency table.
	*/
	function xorShiftFreqTbl(freqTbl, szHalfRndVect, sideTbl, bitXor){
		var a= Math.random();
		
		var b= rndI();
		var c= rndI();
		var d= rndI();
		var e= rndI();
				
		if (bitXor==='ok'){
			stdXorShift.init(b,c,d,e);
		}else{
			stdXorShift.init(b,c,d,e);
		}
		
		for (var i=0; i<szHalfRndVect; i++){
			xorNext(freqTbl,sideTbl);
		}
		return freqTbl;
	}; // xorShiftFreqTbl


	/**
		One step of frequency table filling, just for graphism purpose.
		Input:
			sideTbl : side of the table.
			allow the program to independant from ALU limitations.
		
		Output:
			freqTbl : the frequency table.
	*/
	function xorNext(freqTbl,sideTbl){
		var k= stdXorShift.smplNxtScaled(sideTbl);
		var l= stdXorShift.smplNxtScaled(sideTbl);
		freqTbl[k][l]+= 1;	
	}; // xorNext
	

	/**
	*/
	function colorScale(){
		var k= 0;
		var freqTbl= new Array();
		for (var i=0; i<3; i++){	
			freqTbl[i]= new Array();
			for (var j=0; j<6; j++){
				freqTbl[i][j]= k;			
				k++;
			}
		}
		return arrToHtmlTbl(freqTbl,17,true,100);
	}; // colorScale

	
	/**
		Build an html table to hold a given array :
			1) print the values (c<yle of letters).
			2) use background colors.
			
			<table><tr><td>...</td>...</tr>...</table>
			
		Input :
			freqTbl : the frequency table.
			maxF the maximum frequency in freqTbl.
			dsplFreq : true if we got to display the frequency numbers inside the table.
			tableSz : table size in pixels.
		Output :
			htm : the html table with the frequencies expressed as whished.
	*/
	function arrToHtmlTbl(freqTbl,maxF,dsplFreq,tableSz){
		var squaresNb= freqTbl.length;
		var squaresSz= Math.round(tableSz/squaresNb);
		tableSz= squaresSz*squaresNb;	// adjust
		
		var htmHead= '<table  border="0" cellspacing="0" cellpadding="0"'+crLn;
		
		var htmBody= '';
		var htmLine= '';		
		for (var l in freqTbl) {
			var curLine= freqTbl[l];			
			for (var c in curLine) {
				var curFreq= curLine[c];	// current element of the array
				
				var h= (curFreq/maxF)*0.9 + 0.1; // 0.9 + 0.1 so get rid of the 10% bottom colors too similar to the 10% sup.
				var rgb= hsvToRgb(h,1,1);
				
				var r= Math.round(rgb[0]);
				var g= Math.round(rgb[1]);
				var b= Math.round(rgb[2]);
				
								
				var r= baseTenToHexa(r,2);
				var g= baseTenToHexa(g,2);
				var b= baseTenToHexa(b,2);							
				var curStyle= ' style="background-color:#'+r+g+b+'; width:'+squaresSz+'px; height:'+squaresSz+'px;"';					
							
				if (dsplFreq){
					var freq= '<div class="frequency">'+curFreq+'</div>';					
					htmLine+= '<td'+curStyle+'>'+freq+'</td>'+crLn;
				}else{
					htmLine+= '<td'+curStyle+'></td>'+crLn;
				}
			}
			htmBody+= '<tr>'+crLn+htmLine+'</tr>'+crLn;
			htmLine= '';
		}	
		
		htmTot= htmHead + htmBody + '</table">'+crLn;	
		return 	htmTot;
	}; // arrToHtmlTbl
	

	/**
		Converts an HSV color value to RGB. 
		Assumes h, s, and v are contained in the set [0, 1] and returns r, g, and b in the set [0, 255].
		Input :
			h : hue
			s : saturation
			v : value
		return 
			Array RGB representation
	 */
	function hsvToRgb(h,s,v){
	    var r,g,b;
	    var i = Math.floor(h*6);
	    var f = h*6 - i;
	    var p = v*(1 - s);
	    var q = v*(1 - f*s);
	    var t = v*(1 - (1 - f)*s);
	    switch(i % 6){
	        case 0: r = v, g = t, b = p; break;
	        case 1: r = q, g = v, b = p; break;
	        case 2: r = p, g = v, b = t; break;
	        case 3: r = p, g = q, b = v; break;
	        case 4: r = t, g = p, b = v; break;
	        case 5: r = v, g = p, b = q; break;
	    }
	    return [r*255, g*255, b*255];
	} // hsvToRgb


	/**
	*/
	function rndI(){
		return Math.floor(1000000000*Math.random());
	}; // rndInt


	/**
		Returns the maximum frenquency
	*/
	function maxFrequency(freqTbl){
		var maxF= 0;
		var minF= 999999999;
		for (var i in freqTbl){
			var curLine=freqTbl[i];			
			for (var j in curLine) {
				var curF= curLine[j];
				if (curF>maxF){
					maxF= curF;
				}
				if (curF<minF){
					minF= curF;
				}				
			}
		}
		
		oGlobFreq.minF= minF;
		oGlobFreq.maxF= maxF;		
		return oGlobFreq;
	}; // maxFrequency
	
	/**
		Input :
			nbTen : number in base ten
		Output :
			hexadecimal code.
			minD : minimum number of digits
	*/
	function baseTenToHexa(nbTen,minD){
		var hexa= baseTenAluX(nbTen,'0123456789ABCDEF');
		
		if (minD!==0){
			var curD= hexa.length;	// current digits
			if (curD<minD){
				hexa= zeroPad(minD-curD) + hexa;
			}
		}
		
		return hexa;
	} // baseTenToHexa

	/**
	*/
	function zeroPad(sz){
		var b= '';
		for (var i=0; i<sz; i++){
			b+= '0';
		}
		return b;
	}; // zeroPad


	/**
		Take a string in base A0 and return the equivalent in base X.
		Input :
			nbTen : number to transform given in base ten
			x : decimal numerical span of the new base (binary=2, hexa=16 etc..)
			charSet : char set of the new base.
		-------> To ---> Alu (arith logic unit), meanning that calcultion is done with Alu
		math routines and not stringMath routines!
	*/
	function baseTenAluX(nbTen, charSet){
		var x= charSet.length;
		var buffStr= '';
		do{
			var rem= nbTen % x; 		// remainder
			var nbTen= Math.floor(nbTen/x);	// whole quotient
			buffStr= charSet.charAt(rem) + buffStr;
		}while (nbTen!==0);
		return buffStr;
	} // baseTenAluX


	function getInputOne(){
		return kk.trim(document.getElementById('inputOne').value);
	}; // getInputOne

	function getInputTwo(){
		return kk.trim(document.getElementById('inputTwo').value);
	}; // getInputTwo
	

	/**
	*/
	function setBaseDsplTwo(s){
		document.getElementById('baseTwoDspl').value= s;
	}; // setBaseDsplTwo


	/**
	*/
	function ptInput(){
		return document.getElementById('inputTwo');
	}; // ptInput
	
	function clrInput(){
		var pt= ptInput();
		pt.value= '';
		
		document.getElementById('inputOne').value='';
	}; // clrInput


	/**
		string.substring(from, to)	*/
	
	function getBaseOne(){
		return getBaseN('baseOneDspl');
	}; // getBaseOne

	function getBaseTwo(){
		return getBaseN('baseTwoDspl');
	}; // getBaseTwo

	/**
		Get the base N with input area test to avoid eventual html area
		transformations / differences on \? chars (can't risk to have different bases),
		so bases using special char need a special prefix (in trhat case (fr) ) to take only one source
		
		cmpTwoBaseFr
	*/
	function getBaseN(id){
		var str= document.getElementById(id).value;	
		var sz= str.length;
		if (sz>6){
			// 6 as a min would imply a french char set in base B
			var prfx= str.substring(0,4);
			writeLn('prfx='+prfx);
			
			if (prfx==='(fr)'){
				writeLn('Got the (fr) prefix!');
				return stringMath.getBaseFr()
			}
		}	
		return str;
	}; // getBaseN


	/**
	*/
	function initCDspl(){
		kk.initCurrentDisplay('calcBaseNDspl');
	}; // chabl


	/* --------------------------------------------
		"Cloned" methods for usage easiness 
	*/
	function getString(){return kk.getString();}
	function write(s){kk.write(s);}
	function writeLn(s){kk.writeLn(s);}
	function clearWrite(s){kk.clearWrite(s);}
	function clear(){kk.clearWrite('');}


} // calcBaseN
