/**
	File name : ariStack.js
	Author of this file : Oscar Fawer until 2019-08-15
	Contact : www.ataox.org 
	Web sites : ataox.org / ataox.com / ataox.info
	Copyrights open source GPL, GNU General Public License V3 : see ./aMainDr/code/LICENSE.txt


*/

/**
	Manages a stack for arithmetic () infixed operations.
	Note : It's not necessary to use persistant variable such as localStorage or cookies, because :
		1) clicking javascript button does not change the session
		2) seessions stable variable use only the string type!
*/
function ariStack(){

	var crLn= "\r\n";
	var lookLB= 'lookLB';	// "constant" value for the look for left bracket state.
	var calculator;
	var gStack= newStack();	// Global Stack : holds the operanD and operaTor Stack.
	
	/* ---------------------------------------------------------------
		Methods interface.
	*/
	this.newStack= newStack;
	this.setNewStack= setNewStack;
	this.roundXAfterComma= roundXAfterComma;
	this.callSolvStackTop= callSolvStackTop;
	
	this.pushD= pushD;
	this.popD= popD;
	this.pushT= pushT;
	this.popT= popT;
	
	this.isAnExpression= isAnExpression;

	/* -----------------------------------------------------------------------
		Methods definitions
	 ----------------------------------------------------------------------- */

	/**
		Generates a new stack of operands and operators! JS restrictions :
			localStorage must be used to maintain
	*/
	function newStack(){
		var s= new Object();
		s.d= new Array();	// stack of operanDs
		s.t= new Array();	// stack of operaTors
		return s;
	} // newStack
		
	/**
	*/
	function setNewStack(){
		gStack= newStack();
	} // setNewStack
	
	/**
		Pushes the operand d on the operand stack aStack.d[]
	*/
	function pushD(d){
		gStack.d.push(d);
	} // pushD
	
	/**
		Pop an operand.
	*/
	function popD(){
		return	gStack.d.pop();
	} // popD

	/**
		Pushes the operator t on the operator stack aStack.t[].
	*/
	function pushT(t){
		gStack.t.push(t);
	} // pushT
	
	/**
		Pop an operator..
	*/
	function popT(){
		return	gStack.t.pop();
	} // popT


	/**
		Pop the element just below the top operator
	*/
	function subPopT(){
		sz= gStack.t.length;
		gStack.t.splice(sz-2,1);	
	} // subPopT


	/**
		Try to solve the whole stack (operators and operands) starting from the top.		
	*/
	function solveStackTop(){
		var returnedVal= false;
		var szDstack= gStack.d.length;
		var szTstack= gStack.t.length;
		
		if ( szDstack===1 && szTstack<=1 && gStack.t[0]!=='=' ){
			// got a binary operator with only one operand, so need to wait the next operand input
			return true;
		}				
		
		if (szTstack===0){
			var finalVal= popD();
			return finalVal;	// returns what sould be the final value
		}
		
		var topT= gStack.t[szTstack-1];	// operator or sort of bracket on the top of the operator stack.
		if (szTstack>1){
			var uT= gStack.t[szTstack-2];	// under top operator
		}else{
			var uT= topT;		
		}
					
		/*	Just got to deal with solvable binary operators and ')' bracket.
			Note : 
				- to be solvable, the under operaTor must be topped by an equal or lower priority operator.
				- never got )), because the inside one can always be solved before.
				- ) imply tha the below operator is going to be solved, so ')' got the same stack solving priority
				  than + and -, except that we look for the matching ')'.
		*/
		if ( solvableTopStackT(uT,topT) ){
			if (uT==='(' && topT===')'){
				// May have to display the top stack value
				var partialRes= popD();
				setSmallScreenVal(partialRes);	// going to be pushed back with the next operator
				
				// pops the 2 useless brackets
				popT();
				popT();				
			}else{
				// ok, uT is solvable with 2 top of the stack operands, so replaces the two operands by the result and remove the operator.
				var q= popD();	// must have been stack from the small screen
				var p= popD();	// 2nd operand.
				var result= sop.applyBinOprt(uT,p,q);
				if (topT==='=' && szTstack===2){
					popT();
					popT();
					return result;	// FINAL RESULT
				}
				
				pushD(result);
				setSmallScreenVal(result);
				
				if (szTstack>1){
					subPopT();	// remove the sub top operator, because it has been used
				}else{
					popT();	// pop the last operator (used above)
					popD(); // pop the last operand
					return result;	// pushed just before
				}				
				
				if (szTstack>=1){
					returnedVal= solveStackTop();	// still got some stacked operators
				}								
								
			}
		}
		
		if (returnedVal!==false){
			return returnedVal;
		}else{
			return true;	// waiting for a user action
		}
	} // solveStackTop

	
	/**
		Round x=6 digits after the comma!
		So result like that 7.1+2.3= 9.399999999999999 (from a copy/paste) are avoided!
	*/
	function roundXAfterComma(nb){
		if (nb===true){
			return true;
		}
	
		var rf= 1000000; // rounding factor
		return Math.round(nb*rf)/rf;
	} // roundXAfterComma

	/**
		So got a clear outside place to examine the state of the stack after each opereation.
	*/
	function callSolvStackTop(){
		var val= roundXAfterComma(solveStackTop());
		
		// displayStacks();
		if (val===true){
			// just wait for a user input
		}else{
			setSmallScreenVal(val);		
		}
		
		return val;
	} // callSolvStackTop


	/**
		Returns true if the top of the operator stack is solvable : standard operators
		or ')'.
	*/
	function solvableTopStackT(uT,topT){
		if ( topT===')' || uT==='(' && topT===')' || topT==='=' ){
			return true;
		}else{
			if ( oprtPriority(uT)!==false && oprtPriority(topT)!==false ){
				return oprtPriority(uT) >= oprtPriority(topT);
			}else{
				return false;
			}
		}
	} // solvableTopStackT


	/**
		Stack operator ponderation
	*/
	function oprtPriority(t){
		switch (t){
			case '+':
			case '-':
				return 1;
			break;			
			
			case '*':
			case '/':
				return 2;
			break;
			
			case 'powN':
			case 'rootN':
			case '^':
			case 'r':
				return 3;
			break;
			
			default:
				return false;
		} // switch
	} // oprtPriority


	/**
	*/
	function displayStacks(){
		var tStr= gStack.t.toString();
		var dStr= gStack.d.toString();
		totStr= 'Operators : '+ tStr + crLn + ' Operands : '+ dStr + crLn + crLn;
		writeLn(totStr);
	} // displayStacks	
	

	/**
		Returns true if the string is a math expression :
			- Does not bother about detecting (unary operator) trigi/log functions, also because solving the expression
			  implies the introduction of unary operator as a part of the full stack resolution instead of simply
			  solving the displayed result. The main goal being a verification of the actual stack algo and a verification
			  of the LCEB results.
			Algo : detect if any of the 4 operators are in the string after a trim and a first char removal (avoid
			+/- case as a sign for number).	 
	*/
	function isAnExpression(s){
		return ( kk.gotChar(s,'+') || kk.gotChar(s,'-') || 
			kk.gotChar(s,'*') || kk.gotChar(s,'/') ||  kk.gotChar(s,'^') || 
			kk.gotChar(s,'(') || kk.gotChar(s,')') ) && !kk.gotChar(s,'e') &&
			s.charAt(0)!=='+' && s.charAt(0)!=='-';
		// e ---> does not take exponent numbers in expressions
	} // isAnExpression


	/* --------------------------------------------
		Cloned methods for usage easiness 
	*/
	function getSmallScreenVal(){return sop.getSmallScreenVal();}	
	function setSmallScreenVal(s){sop.setSmallScreenVal(s);}	
	function write(s){sop.write(s);}
	function writeLn(s){sop.writeLn(s);}

} // ariStack
