/**
	File name : stackedOperM.js
	Author of this file : Oscar Fawer until 2019-08-15
	Contact : www.ataox.org 
	Web sites : ataox.org / ataox.com / ataox.info
	Copyrights open source GPL, GNU General Public License V3 : see ./aMainDr/code/LICENSE.txt


*/

/**

	File name : stackedOperM (stacked operator method), so a set of all methods needed by ariStack.js and calculator.js

	Instanciation :
		var stackedOperM= new stackedOperM();

*/
function stackedOperM(){

	var crLn= "\r\n";
	
	var smllScreenBffOn= false;
	var smllScreenBuff= '';
	var writeOnFlag= true;
	
	
	
	/* ---
		Compulsory interface for all methods call from outside !!!!!!!!!!
	*/
	this.applyBinOprt= applyBinOprt;
	this.uniqueOper= uniqueOper;


	this.getSmallScreenVal= getSmallScreenVal;
	this.setSmallScreenVal= setSmallScreenVal;	
	this.getSmallScreenBuff= getSmallScreenBuff;	
	
			
	this.smllScrnBffOn= smllScrnBffOn;
	this.smllScrnBffOff= smllScrnBffOff;
	
		
		
	this.write= write;
	this.writeLn= writeLn;
	
	
	this.writeNl= writeNl;
	this.clearWrite= clearWrite;
	this.writeBefore= writeBefore;
	
	
	
	this.writeOff= writeOff;	
	this.writeOn= writeOn;	



	/* ---
		Methods bodies :
	*/	

	/**
		Does the calculation with the stored info and display the result.
	*/
	function applyBinOprt(op,p,q){
	
		p= parseFloat(p);
		q= parseFloat(q);
	
		switch (op){
			case '*':
				var r= p*q;
			break;
			
			case '+':
				var r= p+q;
			break;
			
			case '-':
				var r= p-q;
			break;
			
			case '/':
				var r= p/q;
			break;
			
			case 'powN':
			case '^':
				var r= Math.pow(p,q);
			break;
			
			case 'rootN':
			case 'r':
				var r= calcRootN(p,q);
			break;
									
			case '=':	// happens when one pushes = without any other operator in the stack
				var r= q;
			break;
									
			default: alert('Error applyBinOprt() op='+op);			
		} // switch
		
		return r;
	} // applyBinOprt


	/**
		Calculates the nth root of x,
	*/
	function calcRootN(x,n){
		var lgx= Math.log(x);
		return Math.exp(lgx/n); 
	} // calcRootN



	
	/**
		Operator with unique operand (got only one operand) : appl	
			Input :
				op : operator
				a : operand
				
		In stand by for now, but later on test it on the global stack.
	*/
	function uniqueOper(op,a){
		var a= screenInput();			// operand	
		var aS= getSortOfAngle();		// angle sort
		var f= factToRadians(aS);		// angle factor
		var r= 0;
		
		switch (op){
			case '-':
				r= -a;
			break;
			
			case 'invert':
				r= 1/a;
			break;
			
			case 'p2':
				r= Math.pow(a,2); ;
			break;
			
			case 'sqrt':
				r= Math.sqrt(a);
			break;
			
			case 'sin':
				r= Math.sin(a*f);
			break;
			
			case 'cos':
				r= Math.cos(a*f);
			break;
			
			case 'tan':
				r= Math.tan(a*f);
			break;
			
			case 'asin':
				r= Math.asin(a) / f;
			break;
			
			case 'acos':
				r= Math.acos(a) / f;
			break;
			
			case 'atan':
				r= Math.atan(a) / f;
			break;
			
			
			case 'ln':
				r= Math.log(a);
			break;
			
			case 'en':
				r= Math.exp(a);
			break;
			
			case 'l10':
				r= calcLogTen(a);
			break;
			
			case '10n':
				r= tenPowerN(a);
			break;
			
			case 'fact':
				r= calcFact(a);
			break;
			
			
			
			case 'sigma':
				var strNb= document.getElementById('calcLnExtDispl').value;
				var o=new Object();
				r= sigmaSum(strNb,o);
			break;
			
			case 'average':
				var strNbs= document.getElementById('calcLnExtDispl').value;
				r= average(strNbs);
			break;
			
			
			default: alert('Error uniqueOper() op='+op);
		} // switch
		
		return r;
	} // uniqueOper



	/**
		log10		
	*/
	function calcLogTen(a){
		return Math.log(a) / Math.LN10;
	} // calcLogTen


	/**
		10^N		
	*/
	function tenPowerN(n){
		return Math.pow(10,n); 
	} // tenPowerN


	/**
		Random number of size n		
	*/
	function calcRandomN(n){
		var f= tenPowerN(n);
		var botLim= tenPowerN(n-1);
		var r= Math.floor(Math.random()*f); 

		if (r<botLim){
			r= r+botLim;
		}
		
		return r;
	} // calcRandomN

	/**
	*/
	function calcFact(n){

		if (n<0){
			return Number.NaN;
		}

		tot=1;
		for(i=1; i<=n && tot!==Number.POSITIVE_INFINITY; i++){
			tot= tot * i;
		}
		
		return tot;
	} // calcFact



	/* ----------------------------------------------------------------------------------
		Shared input / output
	*/
	

	/**
		Returns the pointor of the calculator small screen.
	*/
	function smallScreenPt(){
		return document.getElementById('tbCalcNumber');
	} // smallScreenPt
	
	/**
		Returns the pointor of the side extension screen.
	*/
	function sideExtScreenDsplPt(){
		return document.getElementById('calcLnExtDispl');
	} // sideExtScreenDsplPt
	
	/**
		Returns the value of the calculator small screen. (reverse below)
	*/
	function getSmallScreenVal(){
		if (smllScreenBffOn){
			return smllScreenBuff;	// invisible buffer for off eyes stack calculation
		}else{
			var d= smallScreenPt();
			return d.value;
		}
	} // getSmallScreenVal
	
	function setSmallScreenVal(s){
		if (smllScreenBffOn){
			smllScreenBuff= s;
		}else{
			var d= smallScreenPt();
			d.value= s;
		}
	} // setSmallScreenVal
	
	
	/**
	*/
	function getSmallScreenBuff(){
		return smllScreenBuff;
	} // getSmallScreenBuff
	

	/**
		If called, we'll use the small Screen Buffer after that as a buffer 
		instead of the small screen 'tbCalcNumber'.
	*/
	function smllScrnBffOn(){
		smllScreenBffOn= true;
	} // smllScrnBffOn
	
	function smllScrnBffOff(){
		smllScreenBffOn= false;
	} // smllScrnBffOff
	
	
	/* ---------------------------------------------
		Different writing functions
	*/
	
	function writeLn(s){
		write(s+crLn);
	} // writeLn

	function writeNl(s){
		writeBefore(s+crLn);
	} // writeNl
	


	/* ---------------------------------------------
		Base methods
	*/
	function write(s){
		if (writeOnFlag){
			var d= sideExtScreenDsplPt();
			d.value+= s;
		}
	} // write
	
	function clearWrite(s){
		if (writeOnFlag){
			var d= sideExtScreenDsplPt();
			d.value= s;
		}
	} // clearWrite
	
	function writeBefore(s){
		if (writeOnFlag){
			var d= sideExtScreenDsplPt();
			d.value= s + d.value;
		}
	} // writeBefore

	/**
	*/
	function writeOff(){
		writeOnFlag= false;
	} // writeOff

	/**
	*/
	function writeOn(){
		writeOnFlag= true;
	} // writeOn
	
} // stackedOperM
