/**
	File name : forum.js
	Author of this file : Oscar Fawer until 2019-08-15
	Contact : www.ataox.org 
	Web sites : ataox.org / ataox.com / ataox.info
	Copyrights open source GPL, GNU General Public License V3 : see ./aMainDr/code/LICENSE.txt


*/

/**
*/
function forum(){

	/* -----------------------------------
		Public variables
	*/
	this.newRegistrationWdgt= '';		// new registration widgets
	
	/* -----------------------------------
		Methods interface.
	*/
	this.genInit= genInit;
	
	this.onclickCrypt= onclickCrypt;
	this.cryptForAAUser= cryptForAAUser;	
	
	this.regWgtDealer= regWgtDealer;
	this.unlog= unlog;


	/* -----------------------------------
		Global variable for the "class"
	*/
	var crLn= "\r\n";
	
	// Xhars allowed for forum user Ids
	var allowedIdChars= 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789';
	
	/* -----------------------------------
		Methods definitions
	*/	
	
	/**
		General initialisations!
	*/
	function genInit(){
		kk.initCurrentDisplay('debugDisplay');
	} // genInit
	

	/**
		Return str purged of any non allowedIdChars and force it to all lower cases.
	*/
	function safeStr(str){		
		var buff= '';
		var sz= str.length;
		for (var i=0; i<sz; i++){
			var c= str.charAt(i);
			var pos= allowedIdChars.indexOf(c);
			if (pos!==-1){
				buff+= c;
			}
		}
			
		return buff.toLowerCase();
	} // safeStr
	
	
	/**
		Crypt the password following the onclick event :
			- crypt the password : auto-crypt * hash0 * hash1  (has0 and hash1 sent together)
			- generates a localStorage variable to hold the session crypted identifier.
			
		Input :
			passwordId	: to access the password box
			pwHash : to do a 1st hash on the password for local hard disk storage.
			netHash : to do a 12ndst hash on the password for network transportation. 
			sentMode : the mode it WAS send and interpreted by the php program / the server (already the previous state!!!)
			userId : the user Id epured from funny letters, spaces etc.
	*/
	function onclickCrypt(passwordId, pwHash, netHash, sentMode, userId){
		var ptUsrNm= document.getElementById('usrNm');

		if ( (userId==='ko' || userId==='userNOP') && ptUsrNm!==null){
			userId= safeStr(ptUsrNm.value);	// also need to use a JS function to purge unauthorized chars (program also a new one simpler in php)
		}
		
		if (userId!=='ko'){
			// UserId php init is done with the php Post when available, otherwise from the cookie with a JS cookie renewed on any valid userId
			genAsa.createCookie('fumUsrId',userId,1);
		}
		
		// alert('onclickCrypt sentMode='+sentMode);
		writeLn('onclickCrypt sentMode='+sentMode);
		
		switch (sentMode){
			case 'ko':
			case 'pwKo':
			case 'regKo':
				/* ko --> Just pop on the form, so it was not sent just before, so the current status 
				 (how to crypt etc.) must be deduced from actual widgets and/or long term variables. */
				var ptCB= document.getElementById('cbNewReg');		// check box
			
				if (ptCB!==null){
					// early registration process
					localStorage.forumUsrId= userId;
										
					if (ptCB.checked===true){
						cryptForNewUser(passwordId,pwHash,netHash);
					}else{
						// alert('onclickCrypt 2');
						cryptForRegistredUser(passwordId,pwHash,netHash);
					}
				}else{
					cryptForAAUser(netHash);	// returning on the form after some surf on other pages
				}
			break;
					
			case 'newUser':
			case 'oldUser':
				cryptForAAUser(netHash);
				
				// Saves the actual state of the combo boxes
				localStorage.comboFumFomt= kk.getSelVal('comboFumFomt');
				localStorage.comboFumWeight= kk.getSelVal('comboFumWeight');
				localStorage.comboFumSz= kk.getSelVal('comboFumSz');
				localStorage.comboFumColor= kk.getSelVal('comboFumColor');
			break;
									
			case 'anomNewUsr':
			case 'captKo':
				cryptForNewUser(passwordId,pwHash,netHash);
			break;
						
			case 'usrTaken':
				// alert('Error : the chosen User Name is already taken!');	// uses a php message instead!
			break;
						
			default:
				alert('Error in onclickCrypt');
		} // switch
		
	} // onclickCrypt
	
					
	/**
		At the start : crypt the password for a new user
	*/
	function cryptForNewUser(passwordId,pwHash,netHash){
		// alert('cryptForNewUser');
		var ptPw= document.getElementById(passwordId);
		var pw= ptPw.value;
		
		writeLn('pw='+pw);
		
		var cpw= genAsa.doCrypt(pw,pw);		// pw crypted with pw
		writeLn('cpw='+cpw);
		
		// Straight forward crypting with transmitted hash keys, otherwise impossible to uncrypt on the server side (so weak security on 1st encryption, but it's only a forum, and it's only once!)
		var c2pw= genAsa.doCrypt(cpw,pwHash);		// the stored password crypt with hash key, so can decipher on the server
		writeLn('c2pw='+c2pw);
		
		var netPw= genAsa.doCrypt(c2pw,netHash);
		writeLn('netPw='+netPw);
		
		// prepare for the registred user case, idem below, just to save localStorage.forumC2pw
		// got to do a little fu()
		// cpw= genAsa.doCrypt(pw,pw);		// pw crypted with pw
		
		c2pw= genAsa.doCrypt(pwHash,cpw);	// the stored password crypt
		writeLn('c2pw='+c2pw);
		
		localStorage.forumC2pw= c2pw;	// ---> the localStorage assignment
		
		ptPw.value= netPw;	// so can send it to the server
	} // cryptForNewUser


	/**
		At the start : crypt the password for a registred user
	*/
	function cryptForRegistredUser(passwordId,pwHash,netHash){
		var ptPw= document.getElementById(passwordId);
		var pw= ptPw.value;
		
		var cpw= genAsa.doCrypt(pw,pw);		// pw crypted with pw
		var c2pw= genAsa.doCrypt(pwHash,cpw);	// the stored password crypt
		localStorage.forumC2pw= c2pw;		// ---> the localStorage assignment
		
		var netPw= genAsa.doCrypt(netHash,c2pw);	// the network / traveling crypted pw.
		ptPw.value= netPw;			// so can send it to the server through the password box
		
		cryptForAAUser(netHash);	// so set the cookie 'fumUsrRight' straight away
	} // cryptForRegistredUser

	/**
		Crypt for already accepted/logged user
	*/
	function cryptForAAUser(netHash){
		var fc2pw= localStorage.forumC2pw;
		if (fc2pw!==null){
			var crptHash= genAsa.doCrypt(netHash,fc2pw);
			genAsa.createCookie('fumUsrRight',crptHash,1);	// the password multicrypted
		}
	}
		 

	function unlog(){
		genAsa.createCookie('fumUsrRight','ko',-1);
		genAsa.createCookie('fumUsrId','ko',-1);
		localStorage.forumUsrId= null;
		localStorage.forumC2pw= null;
		// localStorage.prevSentMode= 'ko';	// not used !!
	} // unlog


	/**
		Deal with the registration supplementary widgets (activated from the check box).
	*/
	function regWgtDealer(){		
		var ptCB= document.getElementById('cbNewReg');		// check box
		if (ptCB!==null){	
			var ptDiv= document.getElementById('apOnClick');	// Div to fill
			if (ptDiv!==null){
				if (ptCB.checked===true){
					ptDiv.innerHTML= this.newRegistrationWdgt;
				}else{
					ptDiv.innerHTML= '';
				}
			}
		}
	} // regWgtDealer


	/* --------------------------------------------
		"Cloned" methods for usage easiness 
	*/
	function write(s){kk.write(s);}
	function writeLn(s){kk.writeLn(s);}
	function clearWrite(s){kk.clearWrite(s);}

} // forum
