/**
	File name : projector.js
	Author of this file : Oscar Fawer until 2019-08-15
	Contact : www.ataox.org 
	Web sites : ataox.org / ataox.com / ataox.info
	Copyrights open source GPL, GNU General Public License V3 : see ./aMainDr/code/LICENSE.txt


*/

/* ------------------------------------------------------------------------
	File name : projector.js
*/

function projector(){

	// Glob methods
	this.initProjectorVar= initProjectorVar;
	this.loopOnImg= loopOnImg;
	this.chgSamePgImg= chgSamePgImg;
	this.managesTO= managesTO;
	
		
	
	// Glob plublic var
	this.bigImgVect;
	this.bigImgDir;
	
	
	// Glob var
	var runningProjector;
	var procTO;
	var currentTmBreak;
	var currentIndex;
	var bigImageId;
	var curImgPFN;
	
	


	/* ---------------------------------------------------------------------------------
		Php-Call : loops with a timeBreak on each image of the given vector and set the
		image match. 
		Called by PHP method imageTags::tg_projector(&$v,&$bd).
			Input :
				// image vector ( bigImgVect[] ).   ...global
				timeBreak : time in ms
	*/
	function loopOnImg(timeBreak){
		if (runningProjector===true){
			clearTimeout(procTO);
			runningProjector= false;
		}else{
			currentTmBreak= timeBreak;	// time break in milliseconds
			currentIndex= indexCurImg(projector.bigImgVect, curImgPFN);
			bigImageId= 'idZoom';
			runningProjector= true;
			managesTO();
		}
	}; // loopOnImg



	/* ---------------------------------------------------------------------------------
		All projector initialisation that can easily be specified within this file.
		Rem : the bigImgVect[ ] series could not be initialised here because specific to the page.
		Called by the PHP function  filesMg::loadsZoomVect( )
	*/
	function initProjectorVar(){
		runningProjector= false;
		
		var lastIndx= projector.bigImgVect.length-1;
		curImgPFN= projector.bigImgDir + projector.bigImgVect[lastIndx];
	}; // initProjectorVar



	/* ---------------------------------------------------------------------------------
		Change an image inside the same windows (Pg).
		Input :
			imgId : xhtml identification if the image to change
			newImgPFN : new full image access, path + file name  

		Call for parent :
			onclick="window.parent.chgSamePgImg()";
		Called by filesMg::jsChgParentZoom()  ++
	*/
	function chgSamePgImg(imgId, newImgPFN){
		if (runningProjector===true){
			clearTimeout(procTO);	// 1st action to do otherwise may just have time to change the image again !!
			runningProjector= false;
		}

		coreChgSmePgImg(imgId,newImgPFN);
	}; // chgSamePgImg


	/* ---------------------------------------------------------------------------------
	*/
	function coreChgSmePgImg(imgId, newImgPFN){
		document.getElementById(imgId).src= newImgPFN;
		
		// Supress alt and title of the zoomed image.
		document.getElementById(imgId).alt= '';
		document.getElementById(imgId).title= '';
		
		if (document.getElementById('idZmLink') !== null){
			document.getElementById('idZmLink').title= '';
		}
		
		if (document.getElementById('idTextZoomer') !== null){
			document.getElementById('idTextZoomer').innerHTML= '';	// suppress the text explanation
		}
			
		curImgPFN= newImgPFN;
	}; // coreChgSmePgImg


	/* ---------------------------------------------------------------------------------
		Returns the index of the current image/value.
			Input :
				vect : image vector.
				totPath : image path and file name.
	*/
	function indexCurImg(vect,curImgPFN){
		var szPath= projector.bigImgDir.length;
		var szImgPFN= curImgPFN.length;
		var imgFN= curImgPFN.substring(szPath,szImgPFN);

		var sz= vect.length;	
		var i=0;	
		while(i<sz && vect[i]!==imgFN){
			i++;
		}

		if (vect[i]===imgFN){
			return i;
		}else{
			return false;
		}
	}; // indexCurImg


	/* ---------------------------------------------------------------------------------
		Manages setTimeOut() to loads the zoomed images.
	*/
	function managesTO(){
		var i= nextImgIdx(currentIndex);
		var newImgPFN= projector.bigImgDir + projector.bigImgVect[i];
		currentIndex= i;	
		coreChgSmePgImg(bigImageId, newImgPFN);	// Change the big image
		procTO=  setTimeout( "projector.managesTO()", currentTmBreak);	// TIMER
	}; // managesTO


	/* ---------------------------------------------------------------------------------
		Gives the next image index.
			Input :
				curIdx : current index.
	*/
	function nextImgIdx(curIdx){
		var idMax= projector.bigImgVect.length-1;
		if (curIdx<idMax){
			return curIdx+1;
		}else{
			return 0;
		}
	}; // nextImgIdx


} // projector

