<?php
/**
	File name : cssTransform.php
	Author of this file : Oscar Fawer until 2019-08-15
	Contact : www.ataox.org 
	Web sites : ataox.org / ataox.com / ataox.info
	Copyrights open source GPL, GNU General Public License V3 : see ./aMainDr/code/LICENSE.txt


*/
class cssTransform extends capsHtml{
	

	function __construct(&$tagsFu){
		parent::__construct($tagsFu->tools);	
		$this->tagsFu= &$tagsFu;
		$this->tools= &$tagsFu->tools;		
		
		// [rotElem mainW="" mainH="" vtShift="" hzShift=""] ... [/rotElem]
		$tagsFu->attribVect['rotelem']= array('angle','mainh','mainw','vtshift','hzshift','space', 'id','border');
		
		
		// [varDiv width="" height=""  id="1" border='ok'] ... [/varDiv]
		$tagsFu->attribVect['vardiv']= array('width','height','id','border');
	} // construct


	/**
		Rotates any text, especially to have some vertical text.
			[rotElem mainW="" mainH="" vtShift="" hzShift=""] ... [/rotElem]
			Uses inline style to set the exact measurments each time without hassles of a css file.
		
				Graphical model :
				     	 _
					   ¦[a]¦				a) to vertical <div> : $vtshift / rTopInRect
					[ he(b)llo ]			b) the "horizontal" word : $hzshift / rHzShift;		rBotRotInR
					   ¦   ¦		
					     -
					     d					d) surrounding <div> (mainH,mainW) : rMainRect
					     
		Uses just the rotation method and deal with the translation using the surrounding divs! Would
		be nicer to program the translation with a transform/tranlation method and keep the 2 possibilities for
		comparaison purpose.
	*/
	function tg_rotelem(&$v,&$bd){
		$tools= &$this->tools;
		$tagsFu= &$this->tagsFu;
		
		$angle= $tools->binLoads($v['angle'],'270');
		
		$mainW= $v['mainw'];
		$mainH= $v['mainh'];
		$vtShift= $v['vtshift'];	// non rotated vertical shift. 		
		$hzShift= $v['hzshift'];	// vertical shift rotated, so end up being the horizontal shift.
		$space= $tools->binLoads($v['space'], $mainW);	// space for the text along the writing direction
		$id= $tools->binLoads($v['id'],'1');
		$border= $this->getBorder($v['border']);		
		
		// Inline styles : <style type="text/css"> ...  </style>
		$bss= '.rMainRect'.$id.' { width:'.$mainW.'px; height: '.$mainH.'px; float:left;'.$border.'}'._c_R;		// Main rectangle
		$bss.= '.rTopInRect'.$id.' { width:'.$mainW.'px; height: '.$vtShift.'px; float:left;'.$border.'}'._c_R;	// Top included rectangle
		$bss.= '.rBotRotInR'.$id.' { width:'.$space.'px; height: '.$mainW.'px; float:left;'.$border.''._c_R;	// A square so don't bother to pass the height
		$bss.=  $this->cssRot($angle).'}'._c_R;
		$bss.= '.rHzShift'.$id.' { position:relative; top: '.$hzShift.'px;'.$border.'}'._c_R;
		$bss= $this->cssTag($bss)._c_R;
		
		$topPack = $this->div('','rTopInRect'.$id);
		$hzPackage= $this->div($bd,'rHzShift'.$id);		// a div to set the limits for the horizontal shift
		$botBoxPack= $this->div($hzPackage,'rBotRotInR'.$id);	// rotation done here inside the bottom div
		return $bss.$this->div($topPack.$botBoxPack,'rMainRect'.$id);
	} // tg_rotelem
	

	/**
		Generates a div element with variable dimentions.
		[varDiv width="" height=""  id="1" border='ok'] ... [/varDiv]
	*/
	function tg_vardiv(&$v,&$bd){
		$tools= &$this->tools;
		$tagsFu= &$this->tagsFu;

		$width= $v['width'];
		$height= $v['height'];
		$id= $v['id'];
		$border= $this->getBorder($v['border']);
		
		$bss= '.varDiv'.$id.' {float:left; width:'.$width.'px; height: '.$height.'px; '.$border.'}'._c_R;
		$bss= $this->cssTag($bss)._c_R;
		return $bss.$this->div($bd,'varDiv'.$id);
	} // tg_myTagOne
	
	
	/**
		Only by 90 degr rot only for text, because of explorer.
	*/
	function cssRot($angle){
		$nbRot= (int)round($angle / 90);
		$b= '';
		$b.= '-webkit-transform: rotate('.$angle.'deg);'._c_R;		// Safari
		$b.= '-moz-transform: rotate('.$angle.'deg);'._c_R;			// Firefox
		$b.= '-ms-transform: rotate('.$angle.'deg);'._c_R;			// IE
		$b.= '-o-transform: rotate('.$angle.'deg);'._c_R;			// Opera
		return $b;
	} // cssRot


	/**
	*/
	function cssGenRot($angle){
		return $this->genTransform($angle.'deg');
	} // cssGenRot

	
	/**
		Manages the CSS transform syntax.
	*/
	function genTransform($operand='270deg',$operator='rotate'){
		$b= '';
		$b.= '-ms-transform: '.$operator.'('.$operand.');'._c_R; 
		$b.= '-webkit-transform: '.$operator.'('.$operand.');'._c_R;
		$b.= '-o-transform: '.$operator.'('.$operand.');'._c_R;
		$b.= '-moz-transform: '.$operator.'('.$operand.');';
		return $b;
	} // genTransform

		
	/**
	*/
	function getBorder($border){
		if ($border!==false){
			return ' border: 1px solid red;';
		}else{
			return '';
		}
	} // getBorder


	/**
	*/
	function tg_myTagOne(&$v,&$bd){
		$tools= &$this->tools;
		$tagsFu= &$this->tagsFu;
	} // tg_myTagOne
	
} // cssTransform

?>