<?php
/**
	File name : filesMg.php
	Author of this file : Oscar Fawer until 2019-08-15
	Contact : www.ataox.org 
	Web sites : ataox.org / ataox.com / ataox.info
	Copyrights open source GPL, GNU General Public License V3 : see ./aMainDr/code/LICENSE.txt


	General file manager to look at file and directories.
	filesMg::cpyFileWithPossRsz()
	filesMg::domainName()
*/
class filesMg {
	

	function __construct(&$top){
		$this->top= &$top;
		$this->glot= &$top->glot;
		$this->caps= &$top->caps;
		
		define('_c_mbStrAtom',100);	// _c_mbString "Atom" size --> if bigger got splitted in 2 parts


		// special chars ... space/white need special treatment because don't accept contiguous.
		$this->allAccentsLow= array(
		'à','á','â','ä',
		'è','é','ê','ë',
		'ì','í','î','ï',
		'ò','ó','ô','ö',
		'ù','ú','û','ü');
			
		$this->allAccentsUp= array(
		'À','Á','Â','Ä',
		'È','É','Ê','Ë',
		'Ì','Í','Î','Ï',
		'Ò','Ó','Ô','Ö',
		'Ù','Ú','Û','Ü');
		
		$cedillaArray= array('ç','°','ß');	
		$this->upperAccents= array_combine ($this->allAccentsLow,$this->allAccentsUp);	// To Gives upper accents from lower accents
		$this->allAccents= array_merge($this->allAccentsLow,$this->allAccentsUp,$cedillaArray);
		$this->forSmplFnSymb= array('°','%','&','@','?','!');	// For simpler file names symbols
		$this->special= array_merge($this->allAccents,$this->forSmplFnSymb);



		// simple chars ...
		$this->noAccents= array(
		'a','a','a','a',
		'e','e','e','e',
		'i','i','i','i',
		'o','o','o','o',
		'u','u','u','u',

		'A','A','A','A',
		'E','E','E','E',
		'I','I','I','I',
		'O','O','O','O',
		'U','U','U','U','c','o','ss');
		
		$this->forSmplFnChars= array('o','pc','and','at','','');
		$this->simple= array_merge($this->noAccents,$this->forSmplFnChars);	


		$this->zArrCons= array(
		'C','j','k','l','m','T','V','W','X','B','3','h','r','2','Z','p','q',
		'G','H','I','N','0','1','9','F','P','J','K','L','M','n','Q','R','S','D', 
		'b','c','d','f','g','4','8','s','t','6','7','v','w','5','x','z');
		$this->zLnArrCons= sizeof($this->zArrCons);

			
		$this->userMess= '';
	} // construct

	/**
		Generates a consonance string with numbers!
	*/
	function consonStr($hSz=10){
		$hash='';
		for($i=0;$i<$hSz;$i++){
			$idx= rand(0,$this->zLnArrCons-1);
			$hash.= $this->zArrCons[$idx];
		}
		return $hash;
	} // consonStr

	/**
		$this->addMess
	*/
	function addMess($mess){
		$this->userMess= $this->userMess . $mess;
	} // addMess
	
	
	function getMess(){
		return $this->userMess;
	} // getMess
	

	/**
	*/
	function atHome(){
		$sName= $_SERVER['SERVER_NAME'];
		return $sName === _c_localhost;
	} // atHome


	/**
		Split on the last apparition of $c. ($c is non inclusive.)
		Exmpl : $this->splitOnLastC($str,$left,$right,'/');
	*/
	function splitOnLastC($str,&$left,&$right,$c='/'){
		$sz= strlen($str);
	
		$pos= strrpos($str,$c);
		if ($pos===false){
			$left= $str;
			$right= '';
			return false;	// did not find $c
		}		
		
		$left= substr($str,0,$pos);
		if ($pos<$sz-1){
			$right= substr($str,$pos+1);
		}else{
			$right= '';
		}
		
		return true;	// got $c
	} // splitOnLastC
	
	
	/**
	*/
	function locOpDir($d){
		if (is_readable($d)){
			return opendir($d);
		}else{
			return false;
		}
	} // locOpDir
	
	
	/**
		Give the 1st level of directories and files.
			Output :
				$v : the associative array holding all the 1st level files of $dir
			Input :
				$dir : directory to analyse with a  / suffix.
	*/
	function firstLevelFD(&$v,$dir){
		if (!is_readable($dir)){
			$this->addMess($caps->span('The directory '.$dir.' is not readable'._c_,'zipError'));	
			return false;
		}
	
		$v = array();	// initialization		
		if ( $handle = $this->locOpDir($dir) ){
			$i= 0;
			while ($file = readdir($handle)){
				$dfn= $dir.$file;
				if ($file!='.' && $file!='..'){
					$v[$i]= $file;	// got a file to load (just loads the dir name == $file, not the path)
					$i++;
				}
			} // while
			closedir($handle);
		}
		return isset($v[0]); // return true if got at least one file
	} // firstLevelFD
	
	
	/**
		Look for a vector of candidate starters, hoppefully a single one!
		--> if find many just take the 1st one and indicate that all other are potential candidate!
		Should ne used in a context where if (!is_readable($dir)) has already been checked !!
	*/
	function firstLevelStarters(&$v,$dir){
		$v = array();	// initialization		
		if ( $handle = $this->locOpDir($dir) ){
			$i= 0;
			while ($file = readdir($handle)){
				$dfn= $dir.$file;
				if ($file!='.' && $file!='..' && is_file($dfn)){
					$this->splitOnLastC($file,$fBdy,$ext,'.');
					if ($ext==='php' && $fBdy!=='unzip'){
						$v[$i]= $file;	// got a file to load (just loads the dir name == $file, not the path)
						$i++;
					}
				}
			} // while
			closedir($handle);
		}
		return isset($v[0]); // return true if got at least one file
	} // firstLevelStarters


	/**
		Extract one archive content.
			Input :
				$destDir : destination directory.
				$pathFN : the archive path.
			Output :
				Inflates the archive content in $destDir
	*/
	function openOneArch($destDir,$pathFN){
		$zip = new ZipArchive;
				
		$res= $zip->open($pathFN);
		$this->analyseArchIO($res);
		if ($res===true) {
		    $this->zipExtractTo($zip,$destDir);
		    $res= $zip->close();
		    $this->analyseArchIO($res,'Close');
		    return true;
		} else {
		    return false;
		}	
	} // openOneArch
	
	
	/**
	*/
	function zipExtractTo(&$zip, $destDir){
		error_reporting(NULL);
		$bool= $zip->extractTo($destDir);
		error_reporting(E_ALL);
		return $bool;
	} // zipExtractTo

	
	/**
		Standard simple file copying with mode adaptation.
	*/
	function copyFile($srcPF,$destPF){
		$r= copy($srcPF,$destPF);
		$this->chgMode($destPF);
		return $r;
	} // copyFile	
	
	/**
	*/
	function strToFile($fName,$gStr,$mode="w"){		
		$fd= fopen($fName,$mode);
		$res= fwrite($fd, $gStr);
		fclose($fd);
		$this->chgMode($fName);
		return $res; // for an eventual checking
	} // strToFile
	
	
	/**
	*/
	function fileToStr($fn){
		if ($this->fileExist($fn)){
			return file_get_contents($fn);
		}
	} // fileToStr
	
	/**
	*/
	function flWrite($hdl, $s){
		return fwrite($hdl, $s);
	} // flWrite
	


	/**
		Unzip the full content of the given directory
			Input :
				$destDir : destination directory.
				$archDir : the directory holding all potential archives.
	*/
	function unzipFullDir($destDir,$archDir){
		$glot= &$this->glot;
		$caps= &$this->caps;
		

		$gotOneZip=false;	// True if got at least one file to unzip
		$this->firstLevelFD($vect,$archDir);
		reset($vect);
		foreach ($vect as $key=>$file){
			$this->splitOnLastC($file,$flBody,$ext,'.');
			
			$frozFN= $flBody.'.froz';
			$pathFrzDFN= $archDir.$frozFN;
			
			if ($ext==='zip'){
				$pathDFN= $archDir.$file;
				if ( !$this->fileExist($pathFrzDFN) ){
				
					$success= $this->openOneArch($destDir,$pathDFN);	// UNZIP !!!!!
					
					if ($success){
						$gotOneZip= true;
						$this->addMess($caps->span($glot->sl('Success to unzip : ','kaba').$pathDFN.$glot->sl('(can\'t be done anymore until the .froz file is deleted)','kabb')._c_,'zipInfo'));
						
						if ($this->okToCreate($pathFrzDFN)){
							$this->strToFile($pathFrzDFN,'');	// Freeze it !!!
						}else{
							/*
								Can't write the frozen file because hasn't got the rights --> must prompt the user to
								remove the unzipper with FTP before they log into the cms !! --> direct check at the loggin console!!
								--> Please (*1)FTP-remove the unzipper you have used to unzip this CMS. (*1) FTP or what ever software you used!
							*/
						}
						
					}else{
						$this->addMess($caps->span($glot->sl('Failure to unzip : ','kabc').$pathDFN._c_,'zipError'));
					}
					
				}else{
					$this->addMess($caps->span($glot->sl('Can\'t unzip : ','kabd').$pathDFN.$glot->sl(' because it\'s frozen by ','kabe').$frozFN._c_,'zipError'));
				}
			}
		}
		
		if ($gotOneZip){
			// $this->firstLevelStarters($starterV,$archDir);
			// return $starterV;
			return true;
		}else{
			$this->addMess($caps->span($glot->sl('Was not able to unzip any file!','kabf')._c_,'zipError'));
			return false;
		}
	} // unzipFullDir
	
	
	function fileExist($fn){
		return file_exists($fn);
	} // fileExist
	
	
	function secUnlink($pF){
		if ($this->fileExist($pF)){
			unlink($pF);
		}
	} // secUnlink


	/**
	*/
	function currentDirDsplStr($dir){
		$glot= &$this->glot;
		$caps= &$this->caps;
		
		$this->firstLevelFD($vect,_c_flTchHomeSlsh);
		$mess=  $caps->span($glot->sl('List of files and directories :','kabi')._c_._c_,'zipSmallTitle');;
		reset($vect);
		foreach ($vect as $key=>$fileN){
			$this->splitOnLastC($fileN,$flBdy,$ext,'.');
			if ($ext==='php' || $ext==='htm' || $ext==='html'){
				$url= _c_flTchHomeSlsh.$fileN;
				$mess.= $caps->hL($url,$fileN._c_,'zipLink','_blank');
			}else{
				$mess.= $caps->span($fileN._c_,'zipInfo');
			}
		}
		
		return $mess;
	} // currentDirDsplStr	
	
	
	/**
		Generates a character safe identifier (file/url/field/variable etc,) string from a given string.
	*/
	function safIdStr($sOrig){	
		// Makes sure we have the proper set of characters : Strict identificator fitness
		$vect= $this->toSimpleChars($sOrig,$n);
		$lastAddedChar='';
		$lastForbidChar='';
		$i=0;
		$newS= '';
		while($i<$n){
			$c= $vect[$i];
		
			if ($this->allowedChar($c) && $c!=='-'){
				if($lastForbidChar==='\''){
					// special patch, mainly for French, so eg "collection d'articles" becomes "collection articles" 
					$newS= rtrim($newS,_c_snglQ);
				}
			
				$newS.= $c;	// build the new string
				$lastAddedChar= $c;
			}else{
				$lastForbidChar= $c;
			}
		
			$i++;
		} // while
		
		return $newS;
	} // safIdStr	
	

	/**
		Mainly used to generate file,url,record names the most compatible
		with diverse automation algorithm (search engines, dtb record names etc.).
	*/
	function toSimpleChars(&$str,&$sz){
		$vect= $this->strToVect($str,$sz);
		$replStr= $this->vectReplace($this->special,$this->simple,$vect);
		return $replStr;
	} // toSimpleChars
	

	/*
		Allowed identificator chars : True if the given char belong to the allowed char set to create a 
		file/record/tag/variable name,and incidentally a good chunk of email characters!
	*/
	function allowedChar($c){
		return $this->idChar($c) || $c==='-';
	} // allowedChar

	/**
		Strict identificator chars.
	*/
	function idChar($c){
		return ($c>='a' && $c<='z') || ($c>='A' && $c<='Z')  || ($c>='0' && $c<='9') || $c==='_';
	} // idChar
	
	
	/**
		Convert a UFT8 into an array.
	*/
	function strToVect(&$str,&$sz){
		if ($str==='' || $str===null){
			return array();
		}
		
		$vect= $this->mbStrToA($str);
		$sz= count($vect);
		return $vect;
	} // strToVect

	function mbStrToA($str){
		$sz= mb_strlen($str,'UTF-8');
		if ($sz>_c_mbStrAtom){
			$sz1= round($sz/2);
			$sz2= $sz-$sz1;
			
			$str1= mb_substr($str,0,$sz1,'UTF-8');
			$str2= mb_substr($str,$sz1,$sz2,'UTF-8');
			
			$vect1= $this->mbStrToA($str1);
			$vect2= $this->mbStrToA($str2);
			
			$vect= array_merge($vect1,$vect2);
		}else{
			$vect= array();
			for ($i=0; $i<$sz; $i++){
				$vect[$i]= mb_substr($str,$i,1,'UTF-8');
			} // for
		}
		return $vect;
	} // mbStrToA	

	
	
	
	function vectReplace(&$searchV,&$replaceV,$vect){
		reset($searchV);
		foreach ($searchV as $key=>$vlSrch){
			$vlRpl= $replaceV[$key];
			$this->vectChrReplace($vlSrch,$vlRpl,$vect);
		}
		return $vect;
	} // vectReplace

	function vectChrReplace($srch,$rplc,&$vect){
		reset($vect);
		foreach ($vect as $key=>$val){
			if ($srch===$val){
				$vect[$key]= $rplc;
			}
		}
	} // vectChrReplace
	
	
	function firstLevelFiles(&$v,$dir){	
		$v = array();	// initialization		
		if ( $handle = $this->locOpDir($dir) ){
			$i= 0;
			while ($file = readdir($handle)){
				$dfn= $dir.$file;
				if ($file!='.' && $file!='..' && is_file($dfn)){
					$v[$i]= $file;	// got a file to load (just loads the dir name == $file, not the path)
					$i++;
				}
			} // while
			closedir($handle);
		}
		return isset($v[0]); // return true if got at least one file
	} // firstLevelFiles
	
	
	function frstLvlDir(&$v,$dir){	
		$v = array();	// initialization		
		if ( $handle = $this->locOpDir($dir) ){
			$i= 0;
			while ($file = readdir($handle)){
				$dfn= $dir.$file;
				if ($file!='.' && $file!='..' && is_dir($dfn)){
					$v[$i]= $file;	// got a file to load (just loads the dir name == $file, not the path)
					$i++;
				}
			} // while
			closedir($handle);
		}
		return isset($v[0]); // return true if got at least one file
	} // frstLvlDir



	/**
	*/
	function emailChar($c){
		return $this->allowedChar($c) || $c=='@' || $c=='.';
	} // emailChar
	
	/**
		True if the string $s holds only characters allowed into an email address
	*/
	function emailChars($s){
		$sz= strlen($s);
		$i=0;
		while ($i<$sz && $this->emailChar($s[$i])){
			$i++;
		}
		return $i===$sz;
	} // emailChars
	

	/**
		Catches most wrong email addresses.
		$regRes= preg_match( "^[a-z0-9._%+-]+@[a-z0-9.-]+\.[a-z]{2,6}$",$eMail);  // stall ?
	*/
	function isValidEmail($eMail){
		if (!$this->emailChars($eMail)){
			return false;
		}
	
		$gotAt= false;
		$got2ndPartDot= false;
		$k=0;
		$validEmail= false;
		$szStr= strlen($eMail);
		for($i=0; $i<$szStr; $i++){
			if ($gotAt && $eMail[$i]==='@'){
				break; // because got a second @
			}
		
			if (!$gotAt && $i>0 && $eMail[$i]==='@'){
				$gotAt= true;
			}

			if ($gotAt && $eMail[$i]==='.'){
				$got2ndPartDot= true;
			}

			if ($got2ndPartDot){
				$k++;
			}
			
			if ($i<$szStr-1 && $eMail[$i]==='.' && $eMail[$i+1]==='.'){
				break; // // because consequtive .
			}

			if ($k>2)
				$validEmail= true;
		}
		return $validEmail;
	} // isValidEmail	


	/**
		To test on parent directory of a file to create or overwrite : 
			everyBodyWritingRights || isOwner($fn) && ownerWritingRights.
		Input :
			$dfn : the file or directory to create.
	*/
	function okToCreate($dfn){
		if ( file_exists($dfn) ){
			// the file exists so it depends on the file itself and the directory
			$fOk= $this->isWritable($dfn);
			$dOk= $this->holdDirOkToChg($dfn);
			return $fOk && $dOk;
		}else{
			// the file does not exists so it depends only on the surrounding directory
			return $this->holdDirOkToChg($dfn);
		}		
	} // okToCreate
	
	/**
		Returns true if the holding diectory can be written in!
	*/
	function holdDirOkToChg($dfn){
		$this->splitOnLastC($dfn,$dirDfn,$right);
		return $this->isWritable($dirDfn);
	} // holdDirOkToChg
	
	
	/**
		Returns true if we can change the gicen pathed file or directory.
	*/
	function isWritable($dfn){
		return is_writable($dfn);
	} // isWritable
	
	/**
		Unspecified category writing rights / general writing rights.
	*/
	function genWRights($g){
		return $g=='2' || $g=='3' || $g=='6' || $g=='7';
	} // genWRights
	
	/**
	*/
	function filepermStr($dfn){
		return substr(sprintf('%o', fileperms($dfn)), -4);
	} // filepermStr
	
	
	/**
		Remark : wortk in most cases even if the DTB is still not connected!
	*/
	function domainName(){
		if (array_key_exists('HTTP_HOST',$_SERVER)){
			$sName= $_SERVER['HTTP_HOST'];
		}else if (array_key_exists('SERVER_NAME',$_SERVER)){
			$sName= $_SERVER['SERVER_NAME'];
		}else{
			$sName= '.';	// try relative
		}
		return $sName;
	} // domainName

	

	/**
		Getting there because the php doc indicates :
			Avoid using umask()in multithreaded webservers. It is better to change the 
			file permissions with chmod()
	*/
	function chgMode($fn, $mode= _c_stdFoldersRights){
		if ( $this->isOwner($fn)){
			// changes the mode only if the script user is the file owner
			return chmod($fn,$mode); 
		}else{
			$this->appendOnLogFile($fn.' - '.$mode, 'chgMode');
			return false;
		}
	} // chgMode
	
	
	/**
		Returns true if the current user is the owner of the file.
	*/
	function isOwner($dfn){
		return $this->scriptUserId()===$this->fileOwnerId($dfn);
	} // isOwner
	
	function fileOwnerId($dfn){
		return fileowner($dfn);
	} // fileOwnerId
	
	// scriptUserId
	function scriptUserId(){
		return getmyuid();
	} // scriptUserId
	

	/*	newDir, makeDir, etc.
	*/
	function wMkDir($pathFN,$mode=_c_stdFoldersRights){
		$r= false;
		if (!$this->fileExist($pathFN)){
			$this->splitOnLastC($pathFN,$left,$right);
		
			if ($this->fileExist($left)){
				$r= mkdir($pathFN,$mode);
				
				/*
					Really got to call chmod() below to get a real change of mode, because
					the mode imposed by mkdir() does not seem to change anything. or may be 
					is dependant of umask() when chmod() is not !!!
				*/
				$this->chgMode($pathFN);
			}else{
				print($left.' Does not exists!'._c_);
			}
		}
		
		return $r;
	} // wMkDir


	/**
	*/
	function alertSafeMode(&$message){
		$glot= &$this->glot;
		
		if ( $this->isWritable(_c_ataoxPathNS) ){
			// We may have a case of "safe" mode, need to try further
			
			$dN= 'testFmUnZ';	// test saFe mode for the unzipper.
			$dfn1= _c_ataoxPath.$dN;
			
			$dfn2= $dfn1.'/'.$dN;
			$dfn3= $dfn2.'/'.$dN;
			$dfn4= $dfn3.'/text.txt';
			
			error_reporting(NULL);	// don't want an error blur for our new user !!!!
			$this->wMkDir($dfn1);
			
			// Probably exessive testing, but need to be sure that already created -TP-tranferred files do not interfer with the error messages			
			if ($this->isWritable($dfn1)){
				$this->wMkDir($dfn2);
			}
			
			if ($this->isWritable($dfn2)){
				$this->wMkDir($dfn3);
			}
			
			if ($this->fileExist($dfn4)){
				unlink($dfn4);
			}
			
			if ($this->isWritable($dfn3)){
				$this->strToFile($dfn4,'Test!');
			}
			error_reporting(E_ALL);
			
			if ( $this->fileExist($dfn4) && $this->isWritable($dfn4) ){
				// The only transélation to do, because other won't generally appear (except if the server get security paranoïd!) becasue the system was installed
				$message= $glot->sl('The system has enough rights','kaxv');
				return false; // fine no safe mode alert, nothing to write		
			}else{
				$message= $glot->sl('Alert : we could not create and read : ','kaxw')._c_._c_. $dfn4._c_._c_.$glot->sl(' so you probably have a strong freedom restriction','kaxx');
				return true;
			}
		
		}else{
			// Can't write insiude the main directory
			$message= $glot->sl('Warning : you do not have writing rights on ','kaxy')._c_ataoxPathNS.$glot->sl(' please allow all writing rights "777" ','kaxz');
			return true;	// we have an alert wrong rights on files, may be safe mode too, but tackled on second round!
		}
		
	} // detectSafeMode	
	
	/**
	*/
	function analyseArchIO($code,$contextLbl='Open'){
		if ($code===true){
			return true;	// formal return : no error
		}
		
		if ($code===false){
			return false;	// formal return
		}
		
		$dspl= $contextLbl.' ZipArchive Error : ';
		$codeDspl= $code.' - ';
		switch ($code){
		
			/* ------------------------------------------------------------------
				Constants strictely specified under ZipArchive::open() doc.
			*/
			case ZipArchive::ER_EXISTS:
				$dspl.= $codeDspl.'ER_EXISTS';
			break;
			
			case ZipArchive::ER_INCONS:
				$dspl.= $codeDspl.'ER_INCONS';
			break;
			
			case ZipArchive::ER_INVAL:
				$dspl.= $codeDspl.'ER_INVAL';
			break;
			
			case ZipArchive::ER_MEMORY:
				$dspl.= $codeDspl.'ER_MEMORY';
			break;
			
			case ZipArchive::ER_NOENT:
				$dspl.= $codeDspl.'ER_NOENT';
			break;
			
			case ZipArchive::ER_NOZIP:
				$dspl.= $codeDspl.'ER_NOZIP';
			break;
			
			case ZipArchive::ER_OPEN:
				$dspl.= $codeDspl.'ER_OPEN';
			break;
			
			case ZipArchive::ER_READ:
				$dspl.= $codeDspl.'ER_READ';
			break;
			
			case ZipArchive::ER_SEEK:
				$dspl.= $codeDspl.'ER_SEEK';
			break;
			
			/* ------------------------------------------------------------------
				Supplement values see http://php.net/manual/en/zip.constants.php
				Suspicious about ZipArchive::close() returning non boolean values
			*/
			case ZipArchive::ER_TMPOPEN:
				$dspl.= $codeDspl.'ER_TMPOPEN';
			break;
			
			case ZipArchive::ER_CLOSE:
				$dspl.= $codeDspl.'ER_CLOSE';
			break;
			
			case ZipArchive::ER_ZIPCLOSED:
				$dspl.= $codeDspl.'ER_ZIPCLOSED';
			break;
			
			
			
			default:
				$dspl.= $codeDspl.'Unknown Error code!';
			break;
		} // switch
		
		$this->addMess($dspl);
	} // analyseArchIO	


} // filesMg
?>
