/**
	File name : mbCrypt.js
	Author of this file : Oscar Fawer until 2019-08-15
	Contact : www.ataox.org 
	Web sites : ataox.org / ataox.com / ataox.info
	Copyrights open source GPL, GNU General Public License V3 : see ./aMainDr/code/LICENSE.txt


*/

/**
	Multibyte crypting. trimmed version for the uncompressed unzipper.
*/
function mbCrypt(){

	this.xbCrpt= xbCrpt;		// Php compatible done with utf8 convertion
	this.jsPhpCryptCompatible= jsPhpCryptCompatible;
	this.setSortOfInteger= setSortOfInteger;


	this.write= write;
	this.writeLn= writeLn;
	this.clearWrite= clearWrite;

	
	var largeTopInt= 'ok';
	
	
	/**
		A special Utf8 cryptor for the compatibility with Php that uses only Utf8.
	*/
	function xbCrpt(mbStr,crptK){
		var vect= toUTF8Array(mbStr);
		var hxStr= hexEncodeMbV(vect);
		var crptKV= toUTF8Array(crptK);
		var hxCrptK= hexEncodeMbV(crptKV);		
		return cryptHxToHx(hxStr,hxCrptK);
	} // xbCrpt
		

	/**
		Set the given widget (normally a link) to Ok 
		if a correct crypt match comparison was obtained with the php hexa crypt given string.
		Input :
			wdgtId : the widget Id used for the display.
	*/
	function jsPhpCryptCompatible(wdgtId,congratStr,sorryStr, infoStr,keyStr,phpHxCrypt){
		var jsHxCrypt= xbCrpt(infoStr,keyStr);
		var dspl= '';
		if (jsHxCrypt===phpHxCrypt){
			dspl= congratStr;
		}else{
			dspl= sorryStr;
		}
		
		document.getElementById(wdgtId).innerHTML= dspl;
	} // jsPhpCryptCompatible
	
	
	/**
		Called for a general initialisation inside the php class capsHtml::formatAll()
	*/
	function setSortOfInteger(iSort){
		largeTopInt= iSort;
	}  // setSortOfInteger

	

	/**
		Source for hexEncode() hexDecode()
			http://stackoverflow.com/questions/21647928/javascript-unicode-string-to-hex
	*/
	function hexEncode(mbStr){
		var hex;
		var result= '';
		for (var i=0; i<mbStr.length; i++) {
			hex = mbStr.charCodeAt(i).toString(16);
			result += ('000'+hex).slice(-4);
		}

		return result
	} // hexEncode


	function hexDecode(hxStr){
		var hexes= hxStr.match(/.{1,4}/g) || [];		// [] defensive programming in case of no match, give res empty array!
		
		var res= '';
		for(var j=0; j<hexes.length; j++) {
			res += String.fromCharCode(parseInt(hexes[j], 16));
		}
		return res;
	} // hexDecode
	
	/**
		Input :
			vect : a vector of integers to be converted into hexadecimal code.
		Output :
			return the matching hexadecimal string.
	*/
	function hexEncodeMbV(vect){
		var hex;
		var result= '';
		var sz= vect.length;
		for (var i=0; i<sz; i++) {
			var curNb= new Number(vect[i]);
			hex = curNb.toString(16);
			result += ('0'+hex).slice(-2);
		}

		return result
	} // hexEncodeMbV
	
	
	
	/**
		Input :
			hxStr : the hexadecimal string to crypt.or decyphered
			hxCrptStr : the hexadecimal un / crypting key.
		Output :
			return the hexadecimal crypted or decyphered string
	*/
	function cryptHxToHx(hxStr,hxCrptStr){
		var sz= hxStr.length;
		var seed= sumHexStr(hxCrptStr);
		var pseudoRndV= serieMCG(seed,sz);
		var curHx= '';
		var curInt= 0;
		var rndNb= 0;
		var curXor= 0;
		var newHx= '';
		var newHxStr= '';
		for (var i=0; i<sz; i++) {
			curHx= hxStr.charAt(i);
			curInt= parseInt(curHx,16);
			rndNb= pseudoRndV[i];
			curXor= curInt ^ rndNb;
			newHx= Number(curXor).toString(16);
			newHxStr += newHx;
		}
		return newHxStr;
	} // cryptHxToHx	


	/* ----------------------------------------------------------------
		Power Sum the characters of a string s!
		Should set the base as large as possible while staying below 2^53 = maximum integer in javascript.
		--> y=(2^53)^(1/N); if(y>128){return 128)}else{return y}

		Input :
			hxStr : a hexadecimal string to extract a key from it.
		Output :
			A matching number to be used as a seed for a pseudo random number generator.
			!! Must check the match with the php code !!
	*/
	function sumHexStr(hxStr){
		var sz= hxStr.length;		
		var sum= 0;
		
		var cSp=16;	// character space for hexadecimal code
		
		/**
			y1 : maximum size for a crypting seed
		*/		
		if (largeTopInt==='ok'){
			if (sz>34){
				sz= 34;
			}
			var y1= Math.pow(2,40) / cSp;
		}else{
			if (sz>11){
				sz= 11;
			}
			var y1= 127773 / cSp;
		}
		
		var y= Math.pow(y1,(1/sz));
		if (y>=cSp){
			var base= cSp;
		}else{
			var base= Math.floor(y);	// a floor because already at the limit
		}		
		
		var spacer= base;			
		for(var i=0;i<sz;i++){
			curNb= parseInt( hxStr.charAt(i) , 16);	// hexadecimal to integer
			sum= sum + spacer*curNb;
			spacer= spacer*base;
		}

		return sum;
	} // sumHexStr
	

	function serieMCG(seed,sz){
		if (largeTopInt==='ok'){
			return serMCGLrgInt(seed,sz);
		}else{
			return serMCGSmlInt(seed,sz);
		}
	}  // serieMCG
	
	function serMCGSmlInt(seed,sz){
		var v= new Array();
		var z= seed;
		var cSp=16;
		for (var i=0; i<sz; i++){
			var z= (211*z) % 9870967;
			if (z===0){z= 1;}
			v[i]= z % cSp;
		}
		return v;
	}  // serMCGSmlInt
	
	function serMCGLrgInt(seed,sz){
		var v= new Array();
		var z= seed;
		var cSp=16;
		for (var i=0; i<sz; i++){
			var z= (16807*z) % 2147483647;
			if (z===0){z= 1;}
			v[i]= z % cSp;
		}
		return v;
	}  // serMCGLrgInt
		
	/**
		From UTF-16 to UTF-8… in JavaScript
			http://jonisalonen.com/2012/from-utf-16-to-utf-8-in-javascript/
	*/
	function toUTF8Array(str) {
		var utf8 = [];
		for (var i=0; i < str.length; i++) {
			var charcode = str.charCodeAt(i);
			if (charcode < 0x80) utf8.push(charcode);
			else if (charcode < 0x800) {
				utf8.push(0xc0 | (charcode >> 6),
					0x80 | (charcode & 0x3f));
			} else if (charcode < 0xd800 || charcode >= 0xe000) {
				utf8.push(0xe0 | (charcode >> 12),
					0x80 | ((charcode>>6) & 0x3f),
					0x80 | (charcode & 0x3f));
			} else {
				i++;
				charcode = 0x10000 + (((charcode & 0x3ff)<<10)
					| (str.charCodeAt(i) & 0x3ff))
				utf8.push(0xf0 | (charcode >>18),
					0x80 | ((charcode>>12) & 0x3f),
					0x80 | ((charcode>>6) & 0x3f),
					0x80 | (charcode & 0x3f));
			}
		}
		return utf8;
	} 
	
	/**
	*/
	function intToHexString(n){
		return n.toString(16);
	} // intToHexString

	
	/**
		Convert a small hexadecimal string into an integer
	*/
	function hexStrToInt(s){
		return parseInt(s,16);
	} // hexStrToInt


	/* --------------------------------------------
		"Cloned" methods for usage easiness 
	*/
	function write(s){kk.write(s);}
	function writeLn(s){kk.writeLn(s);}
	function clearWrite(s){kk.clearWrite(s);}


} // mbCrypt
